@@ -29,13 +29,7 @@ int main(int argc, char **argv) {
     memset(lib, 0, PATHLEN);
     snprintf(lib, PATHLEN, "%s/lib", base);
 
-    /* qt-at-spi causes crashes and performance issues in various distros, so disable it */
-    SET("QT_ACCESSIBILITY", "0")
-    memset(buf, 0, PATHLEN);
-    ldp = getenv("QT_PLUGIN_PATH");
-    if (ldp == NULL) snprintf(buf, PATHLEN, "%s/qt_plugins", lib);
-    else snprintf(buf, PATHLEN, "%s/qt_plugins:%s", lib, ldp);
-    SET("QT_PLUGIN_PATH", buf);
+    SET("CALIBRE_QT_PREFIX", base)
 
     memset(buf, 0, PATHLEN);
     ldp = getenv("LD_LIBRARY_PATH");
