@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -21,8 +21,13 @@ from polyglot.builtins import iteritems
 class EPUBHelpBuilder(EpubBuilder):
     name = 'myepub'
 
-    def build_epub(self, outdir, outname):
-        EpubBuilder.build_epub(self, outdir, outname)
+    def build_epub(self, outdir=None, outname=None):
+        if outdir:
+            EpubBuilder.build_epub(self, outdir, outname)
+        else:
+            EpubBuilder.build_epub(self)
+            outdir = self.outdir
+            outname = self.config.epub_basename + '.epub'
         container = get_container(os.path.join(outdir, outname))
         self.fix_epub(container)
         container.commit()
@@ -72,6 +77,13 @@ class EPUBHelpBuilder(EpubBuilder):
         cover_id = rmap['_static/' + self.config.epub_cover[0]]
         for item in container.opf_xpath('//opf:item[@id="{}"]'.format(cover_id)):
             item.set('properties', 'cover-image')
+        for item in container.opf_xpath('//opf:item[@href="epub-cover.xhtml"]'):
+            item.set('properties', 'svg calibre:title-page')
+        for item in container.opf_xpath('//opf:package'):
+            prefix = item.get('prefix') or ''
+            if prefix:
+                prefix += ' '
+            item.set('prefix', prefix + 'calibre: https://calibre-ebook.com')
 
         # Remove any <meta cover> tag as it is not needed in epub 3
         for meta in container.opf_xpath('//opf:meta[@name="cover"]'):
