@@ -23,8 +23,10 @@ DOWNLOADS = '/srv/main/downloads'
 HTML2LRF = "calibre/ebooks/lrf/html/demo"
 TXT2LRF = "src/calibre/ebooks/lrf/txt/demo"
 STAGING_HOST = 'download.calibre-ebook.com'
-STAGING_USER = 'root'
+BACKUP_HOST = 'code.calibre-ebook.com'
+STAGING_USER = BACKUP_USER = 'root'
 STAGING_DIR = '/root/staging'
+BACKUP_DIR = '/binaries'
 
 
 def installers(include_source=True):
@@ -71,7 +73,7 @@ def upload_signatures():
                 raw = f.read()
             fingerprint = hashlib.sha512(raw).hexdigest()
             sha512 = os.path.join(tdir, os.path.basename(installer + '.sha512'))
-            with open(sha512, 'wb') as f:
+            with open(sha512, 'w') as f:
                 f.write(fingerprint)
             scp.append(sha512)
         for srv in 'code main'.split():
@@ -108,7 +110,7 @@ class ReUpload(Command):  # {{{
 # Data {{{
 def get_github_data():
     with open(os.environ['PENV'] + '/github-token', 'rb') as f:
-        un, pw = f.read().strip().split(':')
+        un, pw = f.read().decode('utf-8').strip().split(':')
     return {'username': un, 'password': pw}
 
 
@@ -123,11 +125,21 @@ def get_fosshub_data():
 
 def send_data(loc):
     subprocess.check_call([
-        'rsync', '--inplace', '--delete', '-r', '-z', '-h', '--progress', '-e',
+        'rsync', '--inplace', '--delete', '-r', '-zz', '-h', '--info=progress2', '-e',
         'ssh -x', loc + '/', '%s@%s:%s' % (STAGING_USER, STAGING_HOST, STAGING_DIR)
     ])
 
 
+def send_to_backup(loc):
+    host = f'{BACKUP_USER}@{BACKUP_HOST}'
+    dest = f'{BACKUP_DIR}/{__version__}'
+    subprocess.check_call(['ssh', '-x', host, 'mkdir', '-p', dest])
+    subprocess.check_call([
+        'rsync', '--inplace', '--delete', '-r', '-zz', '-h', '--info=progress2', '-e',
+        'ssh -x', loc + '/', f'{host}:{dest}/'
+    ])
+
+
 def gh_cmdline(ver, data):
     return [
         __appname__, ver, 'fmap', 'github', __appname__, data['username'],
@@ -149,7 +161,7 @@ def run_remote_upload(args):
     print('Running remotely:', ' '.join(args))
     subprocess.check_call([
         'ssh', '-x', '%s@%s' % (STAGING_USER, STAGING_HOST), 'cd', STAGING_DIR, '&&',
-        'python2', 'hosting.py'
+        'python', 'hosting.py'
     ] + args)
 
 
@@ -206,8 +218,10 @@ def upload_to_fosshub():
         'publish': True,
         'isOldRelease': False,
     }
-    # print(json.dumps(jq, indent=2))
-    if not request('projects/{}/releases/'.format(project_id), data=json.dumps(jq)):
+    data = json.dumps(jq)
+    # print(data)
+    data = data.encode('utf-8')
+    if not request('projects/{}/releases/'.format(project_id), data=data):
         raise SystemExit('Failed to queue publish job with fosshub')
 
 
@@ -234,11 +248,8 @@ class UploadInstallers(Command):  # {{{
         sizes = {os.path.basename(x): os.path.getsize(x) for x in files}
         self.record_sizes(sizes)
         tdir = mkdtemp()
-        backup = os.path.join('/mnt/external/calibre/%s' % __version__)
-        if not os.path.exists(backup):
-            os.mkdir(backup)
         try:
-            self.upload_to_staging(tdir, backup, files)
+            self.upload_to_staging(tdir, files)
             self.upload_to_calibre()
             if opts.replace:
                 upload_signatures()
@@ -257,7 +268,7 @@ class UploadInstallers(Command):  # {{{
         ]
         check_call(['ssh', 'code', '/usr/local/bin/dist_sizes'] + args)
 
-    def upload_to_staging(self, tdir, backup, files):
+    def upload_to_staging(self, tdir, files):
         os.mkdir(tdir + '/dist')
         hosting = os.path.join(
             os.path.dirname(os.path.abspath(__file__)), 'hosting.py'
@@ -265,7 +276,7 @@ class UploadInstallers(Command):  # {{{
         shutil.copyfile(hosting, os.path.join(tdir, 'hosting.py'))
 
         for f in files:
-            for x in (tdir + '/dist', backup):
+            for x in (tdir + '/dist',):
                 dest = os.path.join(x, os.path.basename(f))
                 shutil.copy2(f, x)
                 os.chmod(
@@ -274,7 +285,7 @@ class UploadInstallers(Command):  # {{{
 
         with open(os.path.join(tdir, 'fmap'), 'wb') as fo:
             for f, desc in iteritems(files):
-                fo.write('%s: %s\n' % (f, desc))
+                fo.write(('%s: %s\n' % (f, desc)).encode('utf-8'))
 
         while True:
             try:
@@ -285,6 +296,15 @@ class UploadInstallers(Command):  # {{{
             else:
                 break
 
+        while True:
+            try:
+                send_to_backup(tdir)
+            except:
+                print('\nUpload to backup failed, retrying in a minute')
+                time.sleep(60)
+            else:
+                break
+
     def upload_to_github(self, replace):
         data = get_github_data()
         args = gh_cmdline(__version__, data)
@@ -336,7 +356,7 @@ class UploadUserManual(Command):  # {{{
         srcdir = self.j(gettempdir(), 'user-manual-build', 'en', 'html') + '/'
         check_call(
             ' '.join(
-                ['rsync', '-zrl', '--info=progress2', srcdir, 'main:/srv/manual/']
+                ['rsync', '-zz', '-rl', '--info=progress2', srcdir, 'main:/srv/manual/']
             ),
             shell=True
         )
