@@ -34,24 +34,11 @@ def option_parser_for(cmd, args=()):
     return cmd_option_parser
 
 
-def send_message(msg=''):
-    prints('Notifying calibre of the change')
-    from calibre.utils.ipc import RC
-    t = RC(print_error=False)
-    t.start()
-    t.join(3)
-    if t.done:
-        t.conn.send('refreshdb:' + msg)
-        t.conn.close()
-
-
 def run_cmd(cmd, opts, args, dbctx):
     m = module_for_cmd(cmd)
     if dbctx.is_remote and getattr(m, 'no_remote', False):
         raise SystemExit(_('The {} command is not supported with remote (server based) libraries').format(cmd))
     ret = m.main(opts, args, dbctx)
-    # if not dbctx.is_remote and not opts.dont_notify_gui and not getattr(m, 'readonly', False):
-    #     send_message()
     return ret
 
 
@@ -97,6 +84,13 @@ def get_parser(usage):
                ' for your shell.').format(
                    '<stdin>', '<f:C:/path/to/file>' if iswindows else '<f:/path/to/file>')
     )
+    go.add_option(
+        '--timeout',
+        type=float,
+        default=120,
+        help=_('The timeout, in seconds, when connecting to a calibre library over the network. The default is'
+               ' two minutes.')
+    )
 
     return parser
 
@@ -135,6 +129,7 @@ class DBCtx(object):
 
     def __init__(self, opts):
         self.library_path = opts.library_path or prefs['library_path']
+        self.timeout = opts.timeout
         self.url = None
         if self.library_path is None:
             raise SystemExit(
@@ -207,7 +202,7 @@ class DBCtx(object):
         rq = Request(url, data=msgpack_dumps(args),
                      headers={'Accept': MSGPACK_MIME, 'Content-Type': MSGPACK_MIME})
         try:
-            res = self.br.open_novisit(rq)
+            res = self.br.open_novisit(rq, timeout=self.timeout)
             ans = msgpack_loads(res.read())
         except HTTPError as err:
             self.interpret_http_error(err)
@@ -222,7 +217,7 @@ class DBCtx(object):
         from mechanize import HTTPError
         url = self.url + '/ajax/library-info'
         try:
-            res = self.br.open_novisit(url)
+            res = self.br.open_novisit(url, timeout=self.timeout)
             ans = json.loads(res.read())
         except HTTPError as err:
             self.interpret_http_error(err)
