@@ -1,8 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
-# from future_builtins import map
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -20,12 +18,17 @@ from calibre.utils.config_base import tweaks
 from calibre.utils.icu import sort_key
 from calibre.utils.date import UNDEFINED_DATE, clean_date_for_sort, parse_date
 from calibre.utils.localization import calibre_langcode_to_name
+from polyglot.builtins import iteritems, unicode_type
 
 
 def bool_sort_key(bools_are_tristate):
     return (lambda x:{True: 1, False: 2, None: 3}.get(x, 3)) if bools_are_tristate else lambda x:{True: 1, False: 2, None: 2}.get(x, 2)
 
 
+def numeric_sort_key(x):
+    return x or 0
+
+
 IDENTITY = lambda x: x
 
 
@@ -57,6 +60,7 @@ class Field(object):
 
         if dt in {'int', 'float', 'rating'}:
             self._default_sort_key = 0
+            self._sort_key = numeric_sort_key
         elif dt == 'bool':
             self._default_sort_key = None
             self._sort_key = bool_sort_key(bools_are_tristate)
@@ -69,6 +73,8 @@ class Field(object):
                 elif self.metadata['is_custom']:
                     fmt = self.metadata.get('display', {}).get('date_format', None)
                 self._sort_key = partial(clean_date_for_sort, fmt=fmt)
+        elif dt == 'comments' or name == 'identifiers':
+            self._default_sort_key = ''
 
         if self.name == 'languages':
             self._sort_key = lambda x:sort_key(calibre_langcode_to_name(x))
@@ -79,7 +85,7 @@ class Field(object):
             self._sort_key = lambda x: sort_key(author_to_author_sort(x))
             self.sort_sort_key = False
         self.default_value = {} if name == 'identifiers' else () if self.is_multiple else None
-        self.category_formatter = type(u'')
+        self.category_formatter = unicode_type
         if dt == 'rating':
             if self.metadata['display'].get('allow_half_stars', False):
                 self.category_formatter = lambda x: rating_to_stars(x, True)
@@ -150,7 +156,7 @@ class Field(object):
 
         id_map = self.table.id_map
         special_sort = hasattr(self, 'category_sort_value')
-        for item_id, item_book_ids in self.table.col_book_map.iteritems():
+        for item_id, item_book_ids in iteritems(self.table.col_book_map):
             if book_ids is not None:
                 item_book_ids = item_book_ids.intersection(book_ids)
             if item_book_ids:
@@ -184,13 +190,20 @@ class OneToOneField(Field):
         return {item_id}
 
     def __iter__(self):
-        return self.table.book_col_map.iterkeys()
+        return iter(self.table.book_col_map)
 
     def sort_keys_for_books(self, get_metadata, lang_map):
         bcmg = self.table.book_col_map.get
         dk = self._default_sort_key
         sk = self._sort_key
         if sk is IDENTITY:
+            if dk is not None:
+                def none_safe_key(book_id):
+                    ans = bcmg(book_id, dk)
+                    if ans is None:
+                        ans = dk
+                    return ans
+                return none_safe_key
             return lambda book_id:bcmg(book_id, dk)
         return lambda book_id:sk(bcmg(book_id, dk))
 
@@ -315,7 +328,7 @@ class CompositeField(OneToOneField):
             for v in vals:
                 if v:
                     val_map[v].add(book_id)
-        for val, book_ids in val_map.iteritems():
+        for val, book_ids in iteritems(val_map):
             yield val, book_ids
 
     def get_composite_categories(self, tag_class, book_rating_map, book_ids,
@@ -328,7 +341,7 @@ class CompositeField(OneToOneField):
             for val in vals:
                 if val:
                     id_map[val].add(book_id)
-        for item_id, item_book_ids in id_map.iteritems():
+        for item_id, item_book_ids in iteritems(id_map):
             ratings = tuple(r for r in (book_rating_map.get(book_id, 0) for
                                         book_id in item_book_ids) if r > 0)
             avg = sum(ratings)/len(ratings) if ratings else 0
@@ -409,7 +422,7 @@ class OnDeviceField(OneToOneField):
         val_map = defaultdict(set)
         for book_id in candidates:
             val_map[self.for_book(book_id, default_value=default_value)].add(book_id)
-        for val, book_ids in val_map.iteritems():
+        for val, book_ids in iteritems(val_map):
             yield val, book_ids
 
 
@@ -456,7 +469,7 @@ class ManyToOneField(Field):
         return self.table.col_book_map.get(item_id, set())
 
     def __iter__(self):
-        return self.table.id_map.iterkeys()
+        return iter(self.table.id_map)
 
     def sort_keys_for_books(self, get_metadata, lang_map):
         sk_map = LazySortMap(self._default_sort_key, self._sort_key, self.table.id_map)
@@ -466,7 +479,7 @@ class ManyToOneField(Field):
     def iter_searchable_values(self, get_metadata, candidates, default_value=None):
         cbm = self.table.col_book_map
         empty = set()
-        for item_id, val in self.table.id_map.iteritems():
+        for item_id, val in iteritems(self.table.id_map):
             book_ids = cbm.get(item_id, empty).intersection(candidates)
             if book_ids:
                 yield val, book_ids
@@ -475,7 +488,7 @@ class ManyToOneField(Field):
     def book_value_map(self):
         try:
             return {book_id:self.table.id_map[item_id] for book_id, item_id in
-                self.table.book_col_map.iteritems()}
+                iteritems(self.table.book_col_map)}
         except KeyError:
             raise InvalidLinkTable(self.name)
 
@@ -507,7 +520,7 @@ class ManyToManyField(Field):
         return self.table.col_book_map.get(item_id, set())
 
     def __iter__(self):
-        return self.table.id_map.iterkeys()
+        return iter(self.table.id_map)
 
     def sort_keys_for_books(self, get_metadata, lang_map):
         sk_map = LazySortMap(self._default_sort_key, self._sort_key, self.table.id_map)
@@ -524,7 +537,7 @@ class ManyToManyField(Field):
     def iter_searchable_values(self, get_metadata, candidates, default_value=None):
         cbm = self.table.col_book_map
         empty = set()
-        for item_id, val in self.table.id_map.iteritems():
+        for item_id, val in iteritems(self.table.id_map):
             book_ids = cbm.get(item_id, empty).intersection(candidates)
             if book_ids:
                 yield val, book_ids
@@ -534,14 +547,14 @@ class ManyToManyField(Field):
         cbm = self.table.book_col_map
         for book_id in candidates:
             val_map[len(cbm.get(book_id, ()))].add(book_id)
-        for count, book_ids in val_map.iteritems():
+        for count, book_ids in iteritems(val_map):
             yield count, book_ids
 
     @property
     def book_value_map(self):
         try:
             return {book_id:tuple(self.table.id_map[item_id] for item_id in item_ids)
-                for book_id, item_ids in self.table.book_col_map.iteritems()}
+                for book_id, item_ids in iteritems(self.table.book_col_map)}
         except KeyError:
             raise InvalidLinkTable(self.name)
 
@@ -561,7 +574,7 @@ class IdentifiersField(ManyToManyField):
         'Sort by identifier keys'
         bcmg = self.table.book_col_map.get
         dv = {self._default_sort_key:None}
-        return lambda book_id: tuple(sorted(bcmg(book_id, dv).iterkeys()))
+        return lambda book_id: tuple(sorted(bcmg(book_id, dv)))
 
     def iter_searchable_values(self, get_metadata, candidates, default_value=()):
         bcm = self.table.book_col_map
@@ -573,7 +586,7 @@ class IdentifiersField(ManyToManyField):
     def get_categories(self, tag_class, book_rating_map, lang_map, book_ids=None):
         ans = []
 
-        for id_key, item_book_ids in self.table.col_book_map.iteritems():
+        for id_key, item_book_ids in iteritems(self.table.col_book_map):
             if book_ids is not None:
                 item_book_ids = item_book_ids.intersection(book_ids)
             if item_book_ids:
@@ -610,6 +623,9 @@ class FormatsField(ManyToManyField):
     def format_fname(self, book_id, fmt):
         return self.table.fname_map[book_id][fmt.upper()]
 
+    def format_size(self, book_id, fmt):
+        return self.table.size_map.get(book_id, {}).get(fmt.upper(), None)
+
     def iter_searchable_values(self, get_metadata, candidates, default_value=None):
         val_map = defaultdict(set)
         cbm = self.table.book_col_map
@@ -618,13 +634,13 @@ class FormatsField(ManyToManyField):
             for val in vals:
                 val_map[val].add(book_id)
 
-        for val, book_ids in val_map.iteritems():
+        for val, book_ids in iteritems(val_map):
             yield val, book_ids
 
     def get_categories(self, tag_class, book_rating_map, lang_map, book_ids=None):
         ans = []
 
-        for fmt, item_book_ids in self.table.col_book_map.iteritems():
+        for fmt, item_book_ids in iteritems(self.table.col_book_map):
             if book_ids is not None:
                 item_book_ids = item_book_ids.intersection(book_ids)
             if item_book_ids:
@@ -665,7 +681,7 @@ class SeriesField(ManyToOneField):
             return ssk(ts(val, order=sso, lang=lang))
         sk_map = LazySeriesSortMap(self._default_sort_key, sk, self.table.id_map)
         bcmg = self.table.book_col_map.get
-        lang_map = {k:v[0] if v else None for k, v in lang_map.iteritems()}
+        lang_map = {k:v[0] if v else None for k, v in iteritems(lang_map)}
 
         def key(book_id):
             lang = lang_map.get(book_id, None)
@@ -694,8 +710,8 @@ class SeriesField(ManyToOneField):
         sso = tweaks['title_series_sorting']
         ts = title_sort
         empty = set()
-        lang_map = {k:v[0] if v else None for k, v in lang_map.iteritems()}
-        for item_id, val in self.table.id_map.iteritems():
+        lang_map = {k:v[0] if v else None for k, v in iteritems(lang_map)}
+        for item_id, val in iteritems(self.table.id_map):
             book_ids = cbm.get(item_id, empty).intersection(candidates)
             if book_ids:
                 lang_counts = Counter()
@@ -712,7 +728,7 @@ class TagsField(ManyToManyField):
     def get_news_category(self, tag_class, book_ids=None):
         news_id = None
         ans = []
-        for item_id, val in self.table.id_map.iteritems():
+        for item_id, val in iteritems(self.table.id_map):
             if val == _('News'):
                 news_id = item_id
                 break
@@ -724,7 +740,7 @@ class TagsField(ManyToManyField):
             news_books = news_books.intersection(book_ids)
         if not news_books:
             return ans
-        for item_id, item_book_ids in self.table.col_book_map.iteritems():
+        for item_id, item_book_ids in iteritems(self.table.col_book_map):
             item_book_ids = item_book_ids.intersection(news_books)
             if item_book_ids:
                 name = self.category_formatter(self.table.id_map[item_id])
