@@ -265,7 +265,8 @@ class NumericSearch(object):  # {{{
                 cast = lambda x: 0 if x is None else int(x)
                 adjust = lambda x: x // 2
             else:
-                cast = float if dt in ('float', 'composite', 'half-rating') else int
+                # Datatype is empty if the source is a tempate. Assume float
+                cast = float if dt in ('float', 'composite', 'half-rating', '') else int
 
             mult = 1.0
             if len(query) > 1:
@@ -423,7 +424,11 @@ class SavedSearchQueries(object):  # {{{
             db._set_pref(self.opt_name, self.queries)
 
     def lookup(self, name):
-        return self.queries.get(self.force_unicode(name), None)
+        sn = self.force_unicode(name).lower()
+        for n, q in self.queries.items():
+            if sn == n.lower():
+                return q
+        return None
 
     def delete(self, name):
         db = self.db
