@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -11,12 +10,10 @@ import numbers
 from datetime import datetime, timedelta
 from collections import defaultdict
 
-from calibre.constants import plugins
 from calibre.utils.date import parse_date, UNDEFINED_DATE, utc_tz
 from calibre.ebooks.metadata import author_to_author_sort
 from polyglot.builtins import iteritems, itervalues, range
-
-_c_speedup = plugins['speedup'][0].parse_date
+from calibre_extensions.speedup import parse_date as _c_speedup
 
 
 def c_parse(val):
@@ -574,8 +571,8 @@ class AuthorsTable(ManyToManyTable):
 
         return ret
 
-    def remove_items(self, item_ids, db):
-        raise ValueError('Direct removal of authors is not allowed')
+    def remove_items(self, item_ids, db, restrict_to_book_ids=None):
+        raise NotImplementedError('Direct removal of authors is not allowed')
 
 
 class FormatsTable(ManyToManyTable):
