@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,6 +9,8 @@ __docformat__ = 'restructuredtext en'
 import traceback, re
 
 from calibre.constants import iswindows
+from polyglot.builtins import iteritems
+
 
 class DeviceDefaults(object):
 
@@ -18,8 +19,8 @@ class DeviceDefaults(object):
                 # Amazon devices
                 ({'vendor':0x1949}, {
                     'format_map': ['azw3', 'mobi', 'azw',
-                                    'azw1', 'azw4', 'pdf'],
-                    'send_to': ['documents', 'books', 'kindle'],
+                                    'azw1', 'azw4', 'kfx', 'pdf'],
+                    'send_to': ['documents', 'kindle', 'books'],
                     }
                 ),
                 # B&N devices
@@ -46,7 +47,7 @@ class DeviceDefaults(object):
         for rule in self.rules:
             tests = rule[0]
             matches = True
-            for k, v in tests.iteritems():
+            for k, v in iteritems(tests):
                 if k == 'vendor' and v != vid:
                     matches = False
                     break
@@ -60,5 +61,3 @@ class DeviceDefaults(object):
                 return ans
 
         return {}
-
-
