@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john at nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
@@ -8,9 +9,7 @@ __docformat__ = 'restructuredtext en'
 Device driver for Barns and Nobel's Nook
 '''
 
-import os, errno
-
-import cStringIO
+import io, os, errno
 
 from calibre import fsync, prints
 from calibre.constants import DEBUG
@@ -49,20 +48,16 @@ class NOOK(USBMS):
     SUPPORTS_SUB_DIRS = True
 
     def upload_cover(self, path, filename, metadata, filepath):
-        try:
-            from PIL import Image, ImageDraw
-            Image, ImageDraw
-        except ImportError:
-            import Image, ImageDraw
+        from PIL import Image, ImageDraw
 
         coverdata = getattr(metadata, 'thumbnail', None)
         if coverdata and coverdata[2]:
-            cover = Image.open(cStringIO.StringIO(coverdata[2]))
+            cover = Image.open(io.BytesIO(coverdata[2]))
         else:
             coverdata = lopen(I('library.png'), 'rb').read()
 
             cover = Image.new('RGB', (96, 144), 'black')
-            im = Image.open(cStringIO.StringIO(coverdata))
+            im = Image.open(io.BytesIO(coverdata))
             im.thumbnail((96, 144), Image.ANTIALIAS)
 
             x, y = im.size
@@ -72,7 +67,7 @@ class NOOK(USBMS):
             draw.text((1, 15), metadata.get('title', _('Unknown')).encode('ascii', 'ignore'))
             draw.text((1, 115), metadata.get('authors', _('Unknown')).encode('ascii', 'ignore'))
 
-        data = cStringIO.StringIO()
+        data = io.BytesIO()
         cover.save(data, 'JPEG')
         coverdata = data.getvalue()
 
@@ -93,6 +88,7 @@ class NOOK_COLOR(NOOK):
         0x005,  # Nook HD+
         0x007,  # Glowlight from 2013
         0xb,    # Glowlight from 2017
+        0xc,    # Glowlight from 2019
     ]
     BCD         = [0x216, 0x9999]
 
@@ -109,7 +105,7 @@ class NOOK_COLOR(NOOK):
         product_id = self.device_being_opened[1]
         if DEBUG:
             prints('Opened NOOK with product id:', product_id)
-        if product_id == 0xb:
+        if product_id in (0xb, 0xc):
             if DEBUG:
                 prints('Setting Nook upload directory to NOOK/My Files')
             self.EBOOK_DIR_MAIN = 'NOOK/My Files'
