@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john at nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
@@ -20,20 +21,21 @@ from calibre.devices.usbms.cli import CLI
 from calibre.devices.usbms.device import Device
 from calibre.devices.usbms.books import BookList, Book
 from calibre.ebooks.metadata.book.json_codec import JsonCodec
-
-BASE_TIME = None
+from polyglot.builtins import itervalues, unicode_type, string_or_bytes, zip
 
 
-def debug_print(*args):
-    global BASE_TIME
-    if BASE_TIME is None:
-        BASE_TIME = time.time()
+def debug_print(*args, **kw):
+    base_time = getattr(debug_print, 'base_time', None)
+    if base_time is None:
+        debug_print.base_time = base_time = time.monotonic()
     if DEBUG:
-        prints('DEBUG: %6.1f'%(time.time()-BASE_TIME), *args)
+        prints('DEBUG: %6.1f'%(time.monotonic()-base_time), *args, **kw)
 
 
-def safe_walk(top, topdown=True, onerror=None, followlinks=False):
+def safe_walk(top, topdown=True, onerror=None, followlinks=False, maxdepth=128):
     ' A replacement for os.walk that does not die when it encounters undecodeable filenames in a linux filesystem'
+    if maxdepth < 0:
+        return
     islink, join, isdir = os.path.islink, os.path.join, os.path.isdir
 
     # We may not have read permission for top, in which case we can't
@@ -43,7 +45,7 @@ def safe_walk(top, topdown=True, onerror=None, followlinks=False):
     # left to visit.  That logic is copied here.
     try:
         names = os.listdir(top)
-    except os.error as err:
+    except OSError as err:
         if onerror is not None:
             onerror(err)
         return
@@ -66,7 +68,7 @@ def safe_walk(top, topdown=True, onerror=None, followlinks=False):
     for name in dirs:
         new_path = join(top, name)
         if followlinks or not islink(new_path):
-            for x in safe_walk(new_path, topdown, onerror, followlinks):
+            for x in safe_walk(new_path, topdown, onerror, followlinks, maxdepth-1):
                 yield x
     if not topdown:
         yield top, dirs, nondirs
@@ -105,14 +107,14 @@ class USBMS(CLI, Device):
         if not isinstance(dinfo, dict):
             dinfo = {}
         if dinfo.get('device_store_uuid', None) is None:
-            dinfo['device_store_uuid'] = unicode(uuid.uuid4())
+            dinfo['device_store_uuid'] = unicode_type(uuid.uuid4())
         if dinfo.get('device_name', None) is None:
             dinfo['device_name'] = self.get_gui_name()
         if name is not None:
             dinfo['device_name'] = name
         dinfo['location_code'] = location_code
         dinfo['last_library_uuid'] = getattr(self, 'current_library_uuid', None)
-        dinfo['calibre_version'] = '.'.join([unicode(i) for i in numeric_version])
+        dinfo['calibre_version'] = '.'.join([unicode_type(i) for i in numeric_version])
         dinfo['date_last_connected'] = isoformat(now())
         dinfo['prefix'] = prefix.replace('\\', '/')
         return dinfo
@@ -127,13 +129,19 @@ class USBMS(CLI, Device):
                     driveinfo = None
                 driveinfo = self._update_driveinfo_record(driveinfo, prefix,
                                                           location_code, name)
+            data = json.dumps(driveinfo, default=to_json)
+            if not isinstance(data, bytes):
+                data = data.encode('utf-8')
             with lopen(os.path.join(prefix, self.DRIVEINFO), 'wb') as f:
-                f.write(json.dumps(driveinfo, default=to_json))
+                f.write(data)
                 fsync(f)
         else:
             driveinfo = self._update_driveinfo_record({}, prefix, location_code, name)
+            data = json.dumps(driveinfo, default=to_json)
+            if not isinstance(data, bytes):
+                data = data.encode('utf-8')
             with lopen(os.path.join(prefix, self.DRIVEINFO), 'wb') as f:
-                f.write(json.dumps(driveinfo, default=to_json))
+                f.write(data)
                 fsync(f)
         return driveinfo
 
@@ -242,7 +250,7 @@ class USBMS(CLI, Device):
                     import traceback
                     traceback.print_exc()
             return changed
-        if isinstance(ebook_dirs, basestring):
+        if isinstance(ebook_dirs, string_or_bytes):
             ebook_dirs = [ebook_dirs]
         for ebook_dir in ebook_dirs:
             ebook_dir = self.path_to_unicode(ebook_dir)
@@ -280,7 +288,7 @@ class USBMS(CLI, Device):
         # Remove books that are no longer in the filesystem. Cache contains
         # indices into the booklist if book not in filesystem, None otherwise
         # Do the operation in reverse order so indices remain valid
-        for idx in sorted(bl_cache.itervalues(), reverse=True):
+        for idx in sorted(itervalues(bl_cache), reverse=True, key=lambda x: -1 if x is None else x):
             if idx is not None:
                 need_sync = True
                 del bl[idx]
@@ -310,7 +318,7 @@ class USBMS(CLI, Device):
         metadata = iter(metadata)
 
         for i, infile in enumerate(files):
-            mdata, fname = metadata.next(), names.next()
+            mdata, fname = next(metadata), next(names)
             filepath = self.normalize_path(self.create_upload_path(path, mdata, fname))
             if not hasattr(infile, 'read'):
                 infile = self.normalize_path(infile)
@@ -328,7 +336,7 @@ class USBMS(CLI, Device):
 
         self.report_progress(1.0, _('Transferring books to device...'))
         debug_print('USBMS: finished uploading %d books'%(len(files)))
-        return zip(paths, cycle([on_card]))
+        return list(zip(paths, cycle([on_card])))
 
     def upload_cover(self, path, filename, metadata, filepath):
         '''
@@ -347,9 +355,10 @@ class USBMS(CLI, Device):
         debug_print('USBMS: adding metadata for %d books'%(len(metadata)))
 
         metadata = iter(metadata)
+        locations = tuple(locations)
         for i, location in enumerate(locations):
             self.report_progress((i+1) / float(len(locations)), _('Adding books to device metadata listing...'))
-            info = metadata.next()
+            info = next(metadata)
             blist = 2 if location[1] == 'cardb' else 1 if location[1] == 'carda' else 0
 
             # Extract the correct prefix from the pathname. To do this correctly,
