@@ -1,4 +1,5 @@
-from __future__ import with_statement
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
@@ -12,6 +13,7 @@ import shutil, textwrap, codecs, os
 from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
 from calibre import CurrentDir
 from calibre.ptempfile import PersistentTemporaryDirectory
+from polyglot.builtins import getcwd, map
 
 
 class ComicInput(InputFormatPlugin):
@@ -19,7 +21,7 @@ class ComicInput(InputFormatPlugin):
     name        = 'Comic Input'
     author      = 'Kovid Goyal'
     description = 'Optimize comic files (.cbz, .cbr, .cbc) for viewing on portable devices'
-    file_types  = {'cbz', 'cbr', 'cbc'}
+    file_types  = {'cbz', 'cbr', 'cb7', 'cbc'}
     is_image_collection = True
     commit_name = 'comic_input'
     core_usage = -1
@@ -101,7 +103,8 @@ class ComicInput(InputFormatPlugin):
                     '%s is not a valid comic collection'
                     ' no comics.txt was found in the file')
                         %stream.name)
-            raw = open('comics.txt', 'rb').read()
+            with open('comics.txt', 'rb') as f:
+                raw = f.read()
             if raw.startswith(codecs.BOM_UTF16_BE):
                 raw = raw.decode('utf-16-be')[1:]
             elif raw.startswith(codecs.BOM_UTF16_LE):
@@ -137,8 +140,8 @@ class ComicInput(InputFormatPlugin):
                     %comic)
         if self.opts.no_process:
             n2 = []
-            for page in new_pages:
-                n2.append(os.path.join(tdir2, os.path.basename(page)))
+            for i, page in enumerate(new_pages):
+                n2.append(os.path.join(tdir2, '{} - {}' .format(i, os.path.basename(page))))
                 shutil.copyfile(page, n2[-1])
             new_pages = n2
         else:
@@ -173,24 +176,29 @@ class ComicInput(InputFormatPlugin):
             comics_ = [['Comic', os.path.abspath(stream.name)]]
         stream.close()
         comics = []
+        num_pages_per_comic = []
         for i, x in enumerate(comics_):
             title, fname = x
-            cdir = u'comic_%d'%(i+1) if len(comics_) > 1 else u'.'
+            cdir = 'comic_%d'%(i+1) if len(comics_) > 1 else '.'
             cdir = os.path.abspath(cdir)
             if not os.path.exists(cdir):
                 os.makedirs(cdir)
             pages = self.get_pages(fname, cdir)
             if not pages:
                 continue
-            wrappers = self.create_wrappers(pages)
-            comics.append((title, pages, wrappers))
+            num_pages_per_comic.append(len(pages))
+            if self.for_viewer:
+                comics.append((title, pages, [self.create_viewer_wrapper(pages, cdir)]))
+            else:
+                wrappers = self.create_wrappers(pages)
+                comics.append((title, pages, wrappers))
 
         if not comics:
             raise ValueError('No comic pages found in %s'%stream.name)
 
         mi  = MetaInformation(os.path.basename(stream.name).rpartition('.')[0],
             [_('Unknown')])
-        opf = OPFCreator(os.getcwdu(), mi)
+        opf = OPFCreator(getcwd(), mi)
         entries = []
 
         def href(x):
@@ -198,10 +206,13 @@ class ComicInput(InputFormatPlugin):
                 return os.path.basename(x)
             return '/'.join(x.split(os.sep)[-2:])
 
+        cover_href = None
         for comic in comics:
             pages, wrappers = comic[1:]
-            entries += [(w, None) for w in map(href, wrappers)] + \
-                    [(x, None) for x in map(href, pages)]
+            page_entries = [(x, None) for x in map(href, pages)]
+            entries += [(w, None) for w in map(href, wrappers)] + page_entries
+            if cover_href is None and page_entries:
+                cover_href = page_entries[0][0]
         opf.create_manifest(entries)
         spine = []
         for comic in comics:
@@ -210,33 +221,51 @@ class ComicInput(InputFormatPlugin):
         for comic in comics:
             self._images.extend(comic[1])
         opf.create_spine(spine)
+        if self.for_viewer and cover_href:
+            if os.path.isabs(cover_href):
+                cover_href = os.path.relpath(cover_href).replace(os.sep, '/')
+            opf.guide.set_cover(cover_href)
         toc = TOC()
         if len(comics) == 1:
             wrappers = comics[0][2]
-            for i, x in enumerate(wrappers):
-                toc.add_item(href(x), None, _('Page')+' %d'%(i+1),
-                        play_order=i)
+            if self.for_viewer:
+                wrapper_page_href = href(wrappers[0])
+                for i in range(num_pages_per_comic[0]):
+                    toc.add_item('{}#page_{}'.format(wrapper_page_href, i+1), None,
+                        _('Page')+' %d'%(i+1), play_order=i)
+
+            else:
+                for i, x in enumerate(wrappers):
+                    toc.add_item(href(x), None, _('Page')+' %d'%(i+1),
+                            play_order=i)
         else:
             po = 0
-            for comic in comics:
+            for num_pages, comic in zip(num_pages_per_comic, comics):
                 po += 1
                 wrappers = comic[2]
                 stoc = toc.add_item(href(wrappers[0]),
                         None, comic[0], play_order=po)
                 if not opts.dont_add_comic_pages_to_toc:
-                    for i, x in enumerate(wrappers):
-                        stoc.add_item(href(x), None,
-                                _('Page')+' %d'%(i+1), play_order=po)
-                        po += 1
+                    if self.for_viewer:
+                        wrapper_page_href = href(wrappers[0])
+                        for i in range(num_pages):
+                            stoc.add_item('{}#page_{}'.format(wrapper_page_href, i+1), None,
+                                    _('Page')+' %d'%(i+1), play_order=po)
+                            po += 1
+                    else:
+                        for i, x in enumerate(wrappers):
+                            stoc.add_item(href(x), None,
+                                    _('Page')+' %d'%(i+1), play_order=po)
+                            po += 1
         opf.set_toc(toc)
-        m, n = open(u'metadata.opf', 'wb'), open('toc.ncx', 'wb')
-        opf.render(m, n, u'toc.ncx')
-        return os.path.abspath(u'metadata.opf')
+        with open('metadata.opf', 'wb') as m, open('toc.ncx', 'wb') as n:
+            opf.render(m, n, 'toc.ncx')
+        return os.path.abspath('metadata.opf')
 
     def create_wrappers(self, pages):
         from calibre.ebooks.oeb.base import XHTML_NS
         wrappers = []
-        WRAPPER = textwrap.dedent(u'''\
+        WRAPPER = textwrap.dedent('''\
         <html xmlns="%s">
             <head>
                 <meta charset="utf-8"/>
@@ -257,8 +286,43 @@ class ComicInput(InputFormatPlugin):
         dir = os.path.dirname(pages[0])
         for i, page in enumerate(pages):
             wrapper = WRAPPER%(XHTML_NS, i+1, os.path.basename(page), i+1)
-            page = os.path.join(dir, u'page_%d.xhtml'%(i+1))
+            page = os.path.join(dir, 'page_%d.xhtml'%(i+1))
             with open(page, 'wb') as f:
                 f.write(wrapper.encode('utf-8'))
             wrappers.append(page)
         return wrappers
+
+    def create_viewer_wrapper(self, pages, cdir):
+        from calibre.ebooks.oeb.base import XHTML_NS
+
+        def page(pnum, src):
+            return '<img id="page_{}" src="{}"></img>'.format(pnum + 1, os.path.basename(src))
+
+        pages = '\n'.join(page(i, src) for i, src in enumerate(pages))
+        base = os.path.dirname(pages[0])
+        wrapper = '''
+        <html xmlns="%s">
+            <head>
+                <meta charset="utf-8"/>
+                <style type="text/css">
+                html, body, img { height: 100vh; display: block; margin: 0; padding: 0; border-width: 0; }
+                img {
+                    width: 100%%; height: 100%%;
+                    object-fit: contain;
+                    margin-left: auto; margin-right: auto;
+                    max-width: 100vw; max-height: 100vh;
+                    top: 50vh; transform: translateY(-50%%);
+                    position: relative;
+                    page-break-after: always;
+                }
+                </style>
+            </head>
+            <body>
+            %s
+            </body>
+        </html>
+        ''' % (XHTML_NS, pages)
+        path = os.path.join(base, cdir, 'wrapper.xhtml')
+        with open(path, 'wb') as f:
+            f.write(wrapper.encode('utf-8'))
+        return path
