@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
 
-import os
-
 from calibre.customize.conversion import InputFormatPlugin
+from polyglot.builtins import getcwd
 
 
 class PDBInput(InputFormatPlugin):
@@ -32,6 +32,6 @@ class PDBInput(InputFormatPlugin):
         log.debug('Detected ebook format as: %s with identity: %s' % (IDENTITY_TO_NAME[header.ident], header.ident))
 
         reader = Reader(header, stream, log, options)
-        opf = reader.extract_content(os.getcwdu())
+        opf = reader.extract_content(getcwd())
 
         return opf
