@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 __license__ = 'GPL 3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
@@ -7,6 +8,7 @@ __docformat__ = 'restructuredtext en'
 import os
 
 from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
+from polyglot.builtins import as_bytes, getcwd
 
 
 class PDFInput(InputFormatPlugin):
@@ -14,9 +16,10 @@ class PDFInput(InputFormatPlugin):
     name        = 'PDF Input'
     author      = 'Kovid Goyal and John Schember'
     description = 'Convert PDF files to HTML'
-    file_types  = set(['pdf'])
+    file_types  = {'pdf'}
+    commit_name = 'pdf_input'
 
-    options = set([
+    options = {
         OptionRecommendation(name='no_images', recommended_value=False,
             help=_('Do not extract images from the document')),
         OptionRecommendation(name='unwrap_factor', recommended_value=0.45,
@@ -25,18 +28,18 @@ class PDFInput(InputFormatPlugin):
             'default is 0.45, just below the median line length.')),
         OptionRecommendation(name='new_pdf_engine', recommended_value=False,
             help=_('Use the new PDF conversion engine. Currently not operational.'))
-    ])
+    }
 
     def convert_new(self, stream, accelerators):
         from calibre.ebooks.pdf.pdftohtml import pdftohtml
         from calibre.utils.cleantext import clean_ascii_chars
         from calibre.ebooks.pdf.reflow import PDFDocument
 
-        pdftohtml(os.getcwdu(), stream.name, self.opts.no_images, as_xml=True)
-        with open(u'index.xml', 'rb') as f:
+        pdftohtml(getcwd(), stream.name, self.opts.no_images, as_xml=True)
+        with lopen('index.xml', 'rb') as f:
             xml = clean_ascii_chars(f.read())
         PDFDocument(xml, self.opts, self.log)
-        return os.path.join(os.getcwdu(), u'metadata.opf')
+        return os.path.join(getcwd(), 'metadata.opf')
 
     def convert(self, stream, options, file_ext, log,
                 accelerators):
@@ -48,32 +51,32 @@ class PDFInput(InputFormatPlugin):
         self.opts, self.log = options, log
         if options.new_pdf_engine:
             return self.convert_new(stream, accelerators)
-        pdftohtml(os.getcwdu(), stream.name, options.no_images)
+        pdftohtml(getcwd(), stream.name, options.no_images)
 
         from calibre.ebooks.metadata.meta import get_metadata
         log.debug('Retrieving document metadata...')
         mi = get_metadata(stream, 'pdf')
-        opf = OPFCreator(os.getcwdu(), mi)
+        opf = OPFCreator(getcwd(), mi)
 
-        manifest = [(u'index.html', None)]
+        manifest = [('index.html', None)]
 
-        images = os.listdir(os.getcwdu())
+        images = os.listdir(getcwd())
         images.remove('index.html')
         for i in images:
             manifest.append((i, None))
         log.debug('Generating manifest...')
         opf.create_manifest(manifest)
 
-        opf.create_spine([u'index.html'])
+        opf.create_spine(['index.html'])
         log.debug('Rendering manifest...')
-        with open(u'metadata.opf', 'wb') as opffile:
+        with lopen('metadata.opf', 'wb') as opffile:
             opf.render(opffile)
-        if os.path.exists(u'toc.ncx'):
+        if os.path.exists('toc.ncx'):
             ncxid = opf.manifest.id_for_path('toc.ncx')
             if ncxid:
-                with open(u'metadata.opf', 'r+b') as f:
-                    raw = f.read().replace(b'<spine', b'<spine toc="%s"' % bytes(ncxid))
+                with lopen('metadata.opf', 'r+b') as f:
+                    raw = f.read().replace(b'<spine', b'<spine toc="%s"' % as_bytes(ncxid))
                     f.seek(0)
                     f.write(raw)
 
-        return os.path.join(os.getcwdu(), u'metadata.opf')
+        return os.path.join(getcwd(), 'metadata.opf')
