@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 
+
 __license__ = 'GPL 3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
 
-from cStringIO import StringIO
+from io import BytesIO
 
 from calibre.customize.conversion import InputFormatPlugin
 
@@ -14,7 +15,7 @@ class TCRInput(InputFormatPlugin):
     name        = 'TCR Input'
     author      = 'John Schember'
     description = 'Convert TCR files to HTML'
-    file_types  = set(['tcr'])
+    file_types  = {'tcr'}
     commit_name = 'tcr_input'
 
     def convert(self, stream, options, file_ext, log, accelerators):
@@ -24,7 +25,7 @@ class TCRInput(InputFormatPlugin):
         raw_txt = decompress(stream)
 
         log.info('Converting text to OEB...')
-        stream = StringIO(raw_txt)
+        stream = BytesIO(raw_txt)
 
         from calibre.customize.ui import plugin_for_input_format
 
