@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -529,9 +529,9 @@ OptionRecommendation(name='read_metadata_from_opf',
 
 OptionRecommendation(name='asciiize',
         recommended_value=False, level=OptionRecommendation.LOW,
-        help=(_('Transliterate unicode characters to an ASCII '
+        help=(_('Transliterate Unicode characters to an ASCII '
             'representation. Use with care because this will replace '
-            'unicode characters with ASCII. For instance it will replace "%s" '
+            'Unicode characters with ASCII. For instance it will replace "%s" '
             'with "Mikhail Gorbachiov". Also, note that in '
             'cases where there are multiple representations of a character '
             '(characters shared by Chinese and Japanese for instance) the '
@@ -1191,7 +1191,8 @@ OptionRecommendation(name='search_replace',
             self.log('Structured HTML written to:', out_dir)
 
         if self.opts.extra_css and os.path.exists(self.opts.extra_css):
-            self.opts.extra_css = open(self.opts.extra_css, 'rb').read()
+            with open(self.opts.extra_css, 'rb') as f:
+                self.opts.extra_css = f.read()
 
         oibl = self.opts.insert_blank_line
         orps  = self.opts.remove_paragraph_spacing
