@@ -694,22 +694,16 @@ static PyMethodDef bzzdec_methods[] = {
     {NULL, NULL, 0, NULL}
 };
 
-#define INITMODULE
-static struct PyModuleDef bzzdec_module = {
-    /* m_base     */ PyModuleDef_HEAD_INIT,
-    /* m_name     */ "bzzdec",
-    /* m_doc      */ bzzdec_doc,
-    /* m_size     */ -1,
-    /* m_methods  */ bzzdec_methods,
-    /* m_slots    */ 0,
-    /* m_traverse */ 0,
-    /* m_clear    */ 0,
-    /* m_free     */ 0,
+static int
+exec_module(PyObject *module) { return 0; }
+static PyModuleDef_Slot slots[] = { {Py_mod_exec, exec_module}, {0, NULL} };
+
+static struct PyModuleDef module_def = {
+    .m_base     = PyModuleDef_HEAD_INIT,
+    .m_name     = "bzzdec",
+    .m_doc      = bzzdec_doc,
+    .m_methods  = bzzdec_methods,
+    .m_slots    = slots,
 };
-CALIBRE_MODINIT_FUNC PyInit_bzzdec(void) {
-    PyObject *m = PyModule_Create(&bzzdec_module);
-    if (m == NULL) {
-        return NULL;
-    }
-    return m;
-}
+
+CALIBRE_MODINIT_FUNC PyInit_bzzdec(void) { return PyModuleDef_Init(&module_def); }
