@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,7 +8,8 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 import re
 
 from calibre.ebooks.docx.index import process_index, polish_index_markup
-from calibre.ebooks.docx.names import XPath, get, namespaces
+from polyglot.builtins import iteritems, native_string_type
+
 
 class Field(object):
 
@@ -22,7 +22,9 @@ class Field(object):
         self.name = None
 
     def add_instr(self, elem):
-        raw = elem.text
+        self.add_raw(elem.text)
+
+    def add_raw(self, raw):
         if not raw:
             return
         if self.name is None:
@@ -38,6 +40,7 @@ class Field(object):
         self.instructions = ''.join(self.buf)
         del self.buf
 
+
 WORD, FLAG = 0, 1
 scanner = re.Scanner([
     (r'\\\S{1}', lambda s, t: (t, FLAG)),  # A flag of the form \x
@@ -48,8 +51,6 @@ scanner = re.Scanner([
 
 null = object()
 
-def WORD(x):
-    return '{%s}%s' % (namespaces['w'], x)
 
 def parser(name, field_map, default_field_name=None):
 
@@ -74,10 +75,11 @@ def parser(name, field_map, default_field_name=None):
         ans.pop(null, None)
         return ans
 
-    parse.__name__ = str('parse_' + name)
+    parse.__name__ = native_string_type('parse_' + name)
 
     return parse
 
+
 parse_hyperlink = parser('hyperlink',
     'l:anchor m:image-map n:target o:title t:target', 'url')
 
@@ -98,22 +100,26 @@ parse_noteref = parser('noteref',
 
 class Fields(object):
 
-    def __init__(self):
+    def __init__(self, namespace):
+        self.namespace = namespace
         self.fields = []
         self.index_bookmark_counter = 0
         self.index_bookmark_prefix = 'index-'
 
     def __call__(self, doc, log):
-        all_ids = frozenset(XPath('//*/@w:id')(doc))
+        all_ids = frozenset(self.namespace.XPath('//*/@w:id')(doc))
         c = 0
         while self.index_bookmark_prefix in all_ids:
             c += 1
             self.index_bookmark_prefix = self.index_bookmark_prefix.replace('-', '%d-' % c)
         stack = []
-        for elem in XPath(
-            '//*[name()="w:p" or name()="w:r" or name()="w:instrText" or (name()="w:fldChar" and (@w:fldCharType="begin" or @w:fldCharType="end"))]')(doc):
+        for elem in self.namespace.XPath(
+            '//*[name()="w:p" or name()="w:r" or'
+            ' name()="w:instrText" or'
+            ' (name()="w:fldChar" and (@w:fldCharType="begin" or @w:fldCharType="end") or'
+            ' name()="w:fldSimple")]')(doc):
             if elem.tag.endswith('}fldChar'):
-                typ = get(elem, 'w:fldCharType')
+                typ = self.namespace.get(elem, 'w:fldCharType')
                 if typ == 'begin':
                     stack.append(Field(elem))
                     self.fields.append(stack[-1])
@@ -125,6 +131,14 @@ class Fields(object):
             elif elem.tag.endswith('}instrText'):
                 if stack:
                     stack[-1].add_instr(elem)
+            elif elem.tag.endswith('}fldSimple'):
+                field = Field(elem)
+                instr = self.namespace.get(elem, 'w:instr')
+                if instr:
+                    field.add_raw(instr)
+                    self.fields.append(field)
+                    for r in self.namespace.XPath('descendant::w:r')(elem):
+                        field.contents.append(r)
             else:
                 if stack:
                     stack[-1].contents.append(elem)
@@ -193,6 +207,8 @@ class Fields(object):
         if xe:
             # We insert a synthetic bookmark around this index item so that we
             # can link to it later
+            def WORD(x):
+                return self.namespace.expand('w:' + x)
             self.index_bookmark_counter += 1
             bmark = xe['anchor'] = '%s%d' % (self.index_bookmark_prefix, self.index_bookmark_counter)
             p = field.start.getparent()
@@ -210,7 +226,7 @@ class Fields(object):
         if not field.contents:
             return
         idx = parse_func(field.instructions, log)
-        hyperlinks, blocks = process_index(field, idx, self.xe_fields, log)
+        hyperlinks, blocks = process_index(field, idx, self.xe_fields, log, self.namespace.XPath, self.namespace.expand)
         if not blocks:
             return
         for anchor, run in hyperlinks:
@@ -221,11 +237,12 @@ class Fields(object):
     def polish_markup(self, object_map):
         if not self.index_fields:
             return
-        rmap = {v:k for k, v in object_map.iteritems()}
+        rmap = {v:k for k, v in iteritems(object_map)}
         for idx, blocks in self.index_fields:
             polish_index_markup(idx, [rmap[b] for b in blocks])
 
-def test_parse_fields():
+
+def test_parse_fields(return_tests=False):
     import unittest
 
     class TestParseFields(unittest.TestCase):
@@ -250,7 +267,10 @@ def test_parse_fields():
             ae(r'\b \c 1', {'bookmark':None, 'columns-per-page': '1'})
 
     suite = unittest.TestLoader().loadTestsFromTestCase(TestParseFields)
+    if return_tests:
+        return suite
     unittest.TextTestRunner(verbosity=4).run(suite)
 
+
 if __name__ == '__main__':
     test_parse_fields()
