@@ -1,12 +1,12 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 from collections import OrderedDict
+from polyglot.builtins import iteritems, unicode_type
 
 
 class Note(object):
@@ -52,15 +52,14 @@ class Footnotes(object):
         if note is not None and note.type == 'normal':
             self.counter += 1
             anchor = 'note_%d' % self.counter
-            self.notes[anchor] = (type('')(self.counter), note)
-            return anchor, type('')(self.counter)
+            self.notes[anchor] = (unicode_type(self.counter), note)
+            return anchor, unicode_type(self.counter)
         return None, None
 
     def __iter__(self):
-        for anchor, (counter, note) in self.notes.iteritems():
+        for anchor, (counter, note) in iteritems(self.notes):
             yield anchor, counter, note
 
     @property
     def has_notes(self):
         return bool(self.notes)
-
