@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,12 +9,12 @@ import os
 import posixpath
 from collections import namedtuple
 from functools import partial
-from future_builtins import map
+from polyglot.builtins import iteritems, itervalues, map, unicode_type
 
 from lxml import etree
 
 from calibre import fit_image
-from calibre.ebooks.oeb.base import urlunquote
+from calibre.ebooks.oeb.base import urlunquote, urlquote
 from calibre.ebooks.docx.images import pt_to_emu
 from calibre.utils.filenames import ascii_filename
 from calibre.utils.imghdr import identify
@@ -35,7 +34,7 @@ def get_image_margins(style):
     ans = {}
     for edge in 'Left Right Top Bottom'.split():
         val = as_num(getattr(style, 'padding' + edge)) + as_num(getattr(style, 'margin' + edge))
-        ans['dist' + edge[0]] = str(pt_to_emu(val))
+        ans['dist' + edge[0]] = unicode_type(pt_to_emu(val))
     return ans
 
 
@@ -51,8 +50,9 @@ class ImagesManager(object):
 
     def read_image(self, href):
         if href not in self.images:
-            item = self.oeb.manifest.hrefs.get(href)
+            item = self.oeb.manifest.hrefs.get(href) or self.oeb.manifest.hrefs.get(urlquote(href))
             if item is None or not isinstance(item.data, bytes):
+                self.log.warning('Failed to find image:', href)
                 return
             try:
                 fmt, width, height = identify(item.data)
@@ -127,11 +127,11 @@ class ImagesManager(object):
             makeelement(parent, 'wp:simplePos', x='0', y='0')
             makeelement(makeelement(parent, 'wp:positionH', relativeFrom='margin'), 'wp:align').text = floating
             makeelement(makeelement(parent, 'wp:positionV', relativeFrom='line'), 'wp:align').text = 'top'
-        makeelement(parent, 'wp:extent', cx=str(width), cy=str(height))
+        makeelement(parent, 'wp:extent', cx=unicode_type(width), cy=unicode_type(height))
         if fake_margins:
             # DOCX does not support setting margins for inline images, so we
             # fake it by using effect extents to simulate margins
-            makeelement(parent, 'wp:effectExtent', **{k[-1].lower():v for k, v in get_image_margins(style).iteritems()})
+            makeelement(parent, 'wp:effectExtent', **{k[-1].lower():v for k, v in iteritems(get_image_margins(style))})
         else:
             makeelement(parent, 'wp:effectExtent', l='0', r='0', t='0', b='0')
         if floating is not None:
@@ -145,7 +145,7 @@ class ImagesManager(object):
 
     def create_docx_image_markup(self, parent, name, alt, img_rid, width, height):
         makeelement, namespaces = self.document_relationships.namespace.makeelement, self.document_relationships.namespace.namespaces
-        makeelement(parent, 'wp:docPr', id=str(self.count), name=name, descr=alt)
+        makeelement(parent, 'wp:docPr', id=unicode_type(self.count), name=name, descr=alt)
         makeelement(makeelement(parent, 'wp:cNvGraphicFramePr'), 'a:graphicFrameLocks', noChangeAspect="1")
         g = makeelement(parent, 'a:graphic')
         gd = makeelement(g, 'a:graphicData', uri=namespaces['pic'])
@@ -158,7 +158,7 @@ class ImagesManager(object):
         makeelement(makeelement(bf, 'a:stretch'), 'a:fillRect')
         spPr = makeelement(pic, 'pic:spPr')
         xfrm = makeelement(spPr, 'a:xfrm')
-        makeelement(xfrm, 'a:off', x='0', y='0'), makeelement(xfrm, 'a:ext', cx=str(width), cy=str(height))
+        makeelement(xfrm, 'a:off', x='0', y='0'), makeelement(xfrm, 'a:ext', cx=unicode_type(width), cy=unicode_type(height))
         makeelement(makeelement(spPr, 'a:prstGeom', prst='rect'), 'a:avLst')
 
     def create_filename(self, href, fmt):
@@ -169,13 +169,13 @@ class ImagesManager(object):
         base = fname
         while fname.lower() in self.seen_filenames:
             num += 1
-            fname = base + str(num)
+            fname = base + unicode_type(num)
         self.seen_filenames.add(fname.lower())
         fname += os.extsep + fmt.lower()
         return fname
 
     def serialize(self, images_map):
-        for img in self.images.itervalues():
+        for img in itervalues(self.images):
             images_map['word/' + img.fname] = partial(self.get_data, img.item)
 
     def get_data(self, item):
@@ -184,9 +184,16 @@ class ImagesManager(object):
         finally:
             item.unload_data_from_memory(False)
 
-    def create_cover_markup(self, img, width, height):
+    def create_cover_markup(self, img, preserve_aspect_ratio, width, height):
         self.count += 1
         makeelement, namespaces = self.document_relationships.namespace.makeelement, self.document_relationships.namespace.namespaces
+        if preserve_aspect_ratio:
+            if img.width >= img.height:
+                ar = img.height / img.width
+                height = ar * width
+            else:
+                ar = img.width / img.height
+                width = ar * height
 
         root = etree.Element('root', nsmap=namespaces)
         ans = makeelement(root, 'w:drawing', append=False)
@@ -197,7 +204,7 @@ class ImagesManager(object):
         makeelement(makeelement(parent, 'wp:positionH', relativeFrom='page'), 'wp:align').text = 'center'
         makeelement(makeelement(parent, 'wp:positionV', relativeFrom='page'), 'wp:align').text = 'center'
         width, height = map(pt_to_emu, (width, height))
-        makeelement(parent, 'wp:extent', cx=str(width), cy=str(height))
+        makeelement(parent, 'wp:extent', cx=unicode_type(width), cy=unicode_type(height))
         makeelement(parent, 'wp:effectExtent', l='0', r='0', t='0', b='0')
         makeelement(parent, 'wp:wrapTopAndBottom')
         self.create_docx_image_markup(parent, 'cover.jpg', _('Cover'), img.rid, width, height)
