@@ -7,11 +7,7 @@ LZX compression/decompression wrapper.
 __license__   = 'GPL v3'
 __copyright__ = '2008, Marshall T. Vandegrift <llasram@gmail.com>'
 
-from calibre.constants import plugins
-
-_lzx, _error = plugins['lzx']
-if _lzx is None:
-    raise RuntimeError('Failed to load the lzx plugin: %s' % _error)
+from calibre_extensions import lzx as _lzx
 
 __all__ = ['Compressor', 'Decompressor', 'LZXError']
 
