@@ -1,6 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -10,7 +11,7 @@ All fields must have a NULL value represented as None for simple types,
 an empty list/dictionary for complex types and (None, None) for cover_data
 '''
 
-SOCIAL_METADATA_FIELDS = frozenset([
+SOCIAL_METADATA_FIELDS = frozenset((
     'tags',             # Ordered list
     'rating',           # A floating point number between 0 and 10
     'comments',         # A simple HTML enabled string
@@ -19,17 +20,17 @@ SOCIAL_METADATA_FIELDS = frozenset([
     # Of the form { scheme1:value1, scheme2:value2}
     # For example: {'isbn':'123456789', 'doi':'xxxx', ... }
     'identifiers',
-])
+))
 
 '''
 The list of names that convert to identifiers when in get and set.
 '''
 
-TOP_LEVEL_IDENTIFIERS = frozenset([
+TOP_LEVEL_IDENTIFIERS = frozenset((
     'isbn',
-])
+))
 
-PUBLICATION_METADATA_FIELDS = frozenset([
+PUBLICATION_METADATA_FIELDS = frozenset((
     'title',            # title must never be None. Should be _('Unknown')
     # Pseudo field that can be set, but if not set is auto generated
     # from title and languages
@@ -58,28 +59,27 @@ PUBLICATION_METADATA_FIELDS = frozenset([
     # image_path which is the path to an image file, encoded
     # in filesystem_encoding
     'thumbnail',
-    ])
+))
 
-BOOK_STRUCTURE_FIELDS = frozenset([
+BOOK_STRUCTURE_FIELDS = frozenset((
     # These are used by code, Null values are None.
     'toc', 'spine', 'guide', 'manifest',
-    ])
+))
 
-USER_METADATA_FIELDS = frozenset([
+USER_METADATA_FIELDS = frozenset((
     # A dict of dicts similar to field_metadata. Each field description dict
     # also contains a value field with the key #value#.
     'user_metadata',
-])
+))
 
-DEVICE_METADATA_FIELDS = frozenset([
+DEVICE_METADATA_FIELDS = frozenset((
     'device_collections',   # Ordered list of strings
     'lpath',                # Unicode, / separated
     'size',                 # In bytes
     'mime',                 # Mimetype of the book file being represented
+))
 
-])
-
-CALIBRE_METADATA_FIELDS = frozenset([
+CALIBRE_METADATA_FIELDS = frozenset((
     'application_id',   # An application id, currently set to the db_id.
     'db_id',            # the calibre primary key of the item.
     'formats',          # list of formats (extensions) for this book
@@ -88,9 +88,7 @@ CALIBRE_METADATA_FIELDS = frozenset([
     'user_categories',
     # a dict of author to an associated hyperlink
     'author_link_map',
-
-    ]
-)
+))
 
 ALL_METADATA_FIELDS =      SOCIAL_METADATA_FIELDS.union(
                            PUBLICATION_METADATA_FIELDS).union(
@@ -107,13 +105,13 @@ STANDARD_METADATA_FIELDS = SOCIAL_METADATA_FIELDS.union(
                            CALIBRE_METADATA_FIELDS)
 
 # Metadata fields that smart update must do special processing to copy.
-SC_FIELDS_NOT_COPIED =     frozenset(['title', 'title_sort', 'authors',
+SC_FIELDS_NOT_COPIED =     frozenset(('title', 'title_sort', 'authors',
                                       'author_sort', 'author_sort_map',
                                       'cover_data', 'tags', 'languages',
-                                      'identifiers'])
+                                      'identifiers'))
 
 # Metadata fields that smart update should copy only if the source is not None
-SC_FIELDS_COPY_NOT_NULL =  frozenset(['device_collections', 'lpath', 'size', 'comments', 'thumbnail'])
+SC_FIELDS_COPY_NOT_NULL =  frozenset(('device_collections', 'lpath', 'size', 'comments', 'thumbnail'))
 
 # Metadata fields that smart update should copy without special handling
 SC_COPYABLE_FIELDS =       SOCIAL_METADATA_FIELDS.union(
@@ -129,6 +127,6 @@ SERIALIZABLE_FIELDS =      SOCIAL_METADATA_FIELDS.union(
                            PUBLICATION_METADATA_FIELDS).union(
                            CALIBRE_METADATA_FIELDS).union(
                            DEVICE_METADATA_FIELDS) - \
-                           frozenset(['device_collections', 'formats',
-                               'cover_data'])
+                           frozenset(('device_collections', 'formats',
+                               'cover_data'))
 # these are rebuilt when needed
