@@ -1,4 +1,5 @@
-from __future__ import print_function
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Ashish Kulkarni <kulkarni.ashish@gmail.com>'
 '''Read meta information from IMP files'''
@@ -6,8 +7,9 @@ __copyright__ = '2008, Ashish Kulkarni <kulkarni.ashish@gmail.com>'
 import sys
 
 from calibre.ebooks.metadata import MetaInformation, string_to_authors
+from polyglot.builtins import unicode_type
 
-MAGIC = ['\x00\x01BOOKDOUG', '\x00\x02BOOKDOUG']
+MAGIC = (b'\x00\x01BOOKDOUG', b'\x00\x02BOOKDOUG')
 
 
 def get_metadata(stream):
@@ -17,18 +19,18 @@ def get_metadata(stream):
     stream.seek(0)
     try:
         if stream.read(10) not in MAGIC:
-            print(u'Couldn\'t read IMP header from file', file=sys.stderr)
+            print('Couldn\'t read IMP header from file', file=sys.stderr)
             return mi
 
         def cString(skip=0):
-            result = ''
+            result = b''
             while 1:
                 data = stream.read(1)
-                if data == '\x00':
+                if data == b'\x00':
                     if not skip:
-                        return result
+                        return result.decode('utf-8')
                     skip -= 1
-                    result, data = '', ''
+                    result, data = b'', b''
                 result += data
 
         stream.read(38)  # skip past some uninteresting headers
@@ -43,6 +45,6 @@ def get_metadata(stream):
         if category:
             mi.category = category
     except Exception as err:
-        msg = u'Couldn\'t read metadata from imp: %s with error %s'%(mi.title, unicode(err))
+        msg = 'Couldn\'t read metadata from imp: %s with error %s'%(mi.title, unicode_type(err))
         print(msg.encode('utf8'), file=sys.stderr)
     return mi
