@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,12 +8,10 @@ __docformat__ = 'restructuredtext en'
 
 import time, re
 from datetime import datetime
-from Queue import Queue, Empty
 from threading import Thread
-from io import BytesIO
+from io import StringIO
 from operator import attrgetter
-from urlparse import urlparse
-from urllib import quote
+from polyglot.urllib import urlparse, quote
 
 from calibre.customize.ui import metadata_plugins, all_metadata_plugins
 from calibre.ebooks.metadata import check_issn, authors_to_sort_string
@@ -27,6 +24,8 @@ from calibre.utils.html2text import html2text
 from calibre.utils.icu import lower
 from calibre.utils.date import UNDEFINED_DATE
 from calibre.utils.formatter import EvalFormatter
+from polyglot.builtins import iteritems, itervalues, unicode_type
+from polyglot.queue import Queue, Empty
 
 # Download worker {{{
 
@@ -39,7 +38,7 @@ class Worker(Thread):
 
         self.plugin, self.kwargs, self.rq = plugin, kwargs, Queue()
         self.abort = abort
-        self.buf = BytesIO()
+        self.buf = StringIO()
         self.log = create_log(self.buf)
 
     def run(self):
@@ -98,7 +97,7 @@ class ISBNMerge(object):
 
     def isbn_in_pool(self, isbn):
         if isbn:
-            for isbns, pool in self.pools.iteritems():
+            for isbns, pool in iteritems(self.pools):
                 if isbn in isbns:
                     return pool
         return None
@@ -146,7 +145,7 @@ class ISBNMerge(object):
 
     def finalize(self):
         has_isbn_result = False
-        for results in self.pools.itervalues():
+        for results in itervalues(self.pools):
             if results:
                 has_isbn_result = True
                 break
@@ -191,7 +190,7 @@ class ISBNMerge(object):
 
         if len(groups) != len(self.results):
             self.results = []
-            for rgroup in groups.itervalues():
+            for rgroup in itervalues(groups):
                 rel = [r.average_source_relevance for r in rgroup]
                 if len(rgroup) > 1:
                     result = self.merge(rgroup, None, do_asr=False)
@@ -205,7 +204,7 @@ class ISBNMerge(object):
             groups, empty = {}, []
             for result in self.results:
                 key = set()
-                for typ, val in result.identifiers.iteritems():
+                for typ, val in iteritems(result.identifiers):
                     if typ and val:
                         key.add((typ, val))
                 if key:
@@ -226,7 +225,7 @@ class ISBNMerge(object):
 
             if len(groups) != len(self.results):
                 self.results = []
-                for rgroup in groups.itervalues():
+                for rgroup in itervalues(groups):
                     rel = [r.average_source_relevance for r in rgroup]
                     if len(rgroup) > 1:
                         result = self.merge(rgroup, None, do_asr=False)
@@ -243,7 +242,7 @@ class ISBNMerge(object):
     def merge_isbn_results(self):
         self.results = []
         sources = set()
-        for min_year, results in self.pools.itervalues():
+        for min_year, results in itervalues(self.pools):
             if results:
                 for r in results:
                     sources.add(r.identify_plugin)
@@ -361,7 +360,7 @@ class ISBNMerge(object):
 
 def merge_identify_results(result_map, log):
     isbn_merge = ISBNMerge(log)
-    for plugin, results in result_map.iteritems():
+    for plugin, results in iteritems(result_map):
         for result in results:
             isbn_merge.add_result(result)
 
@@ -438,12 +437,12 @@ def identify(log, abort,  # {{{
         pass
 
     sort_kwargs = dict(kwargs)
-    for k in list(sort_kwargs.iterkeys()):
+    for k in list(sort_kwargs):
         if k not in ('title', 'authors', 'identifiers'):
             sort_kwargs.pop(k)
 
     longest, lp = -1, ''
-    for plugin, presults in results.iteritems():
+    for plugin, presults in iteritems(results):
         presults.sort(key=plugin.identify_results_keygen(**sort_kwargs))
 
         # Throw away lower priority results from the same source that have exactly the same
@@ -471,7 +470,7 @@ def identify(log, abort,  # {{{
         for r in presults:
             log('\n\n---')
             try:
-                log(unicode(r))
+                log(unicode_type(r))
             except TypeError:
                 log(repr(r))
         if plog:
@@ -541,7 +540,7 @@ def identify(log, abort,  # {{{
 
 
 def urls_from_identifiers(identifiers):  # {{{
-    identifiers = {k.lower():v for k, v in identifiers.iteritems()}
+    identifiers = {k.lower():v for k, v in iteritems(identifiers)}
     ans = []
     keys_left = set(identifiers)
 
@@ -552,9 +551,9 @@ def urls_from_identifiers(identifiers):  # {{{
     rules = msprefs['id_link_rules']
     if rules:
         formatter = EvalFormatter()
-        for k, val in identifiers.iteritems():
+        for k, val in iteritems(identifiers):
             val = val.replace('|', ',')
-            vals = {'id':quote(val if isinstance(val, bytes) else val.encode('utf-8')).decode('ascii')}
+            vals = {'id':unicode_type(quote(val if isinstance(val, bytes) else val.encode('utf-8')))}
             items = rules.get(k) or ()
             for name, template in items:
                 try:
@@ -591,7 +590,7 @@ def urls_from_identifiers(identifiers):  # {{{
         add(issn, 'issn', issn,
             'https://www.worldcat.org/issn/'+issn)
     q = {'http', 'https', 'file'}
-    for k, url in identifiers.iteritems():
+    for k, url in iteritems(identifiers):
         if url and re.match(r'ur[il]\d*$', k) is not None:
             url = url[:8].replace('|', ':') + url[8:].replace('|', ',')
             if url.partition(':')[0].lower() in q:
