@@ -1,5 +1,6 @@
-#!/usr/bin/env  python2
-from __future__ import print_function
+#!/usr/bin/env python
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -11,20 +12,15 @@ from lxml.builder import ElementMaker
 
 from calibre.constants import __appname__, __version__
 from calibre.ebooks.chardet import xml_to_unicode
+from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.utils.cleantext import clean_xml_chars
-from polyglot.builtins import unicode_type
+from polyglot.builtins import unicode_type, getcwd
 from polyglot.urllib import unquote, urlparse
 
 NCX_NS = "http://www.daisy.org/z3986/2005/ncx/"
 CALIBRE_NS = "http://calibre.kovidgoyal.net/2009/metadata"
-NSMAP = {
-            None: NCX_NS,
-            'calibre':CALIBRE_NS
-            }
-
-
+NSMAP = {None: NCX_NS, 'calibre':CALIBRE_NS}
 E = ElementMaker(namespace=NCX_NS, nsmap=NSMAP)
-
 C = ElementMaker(namespace=CALIBRE_NS, nsmap=NSMAP)
 
 
@@ -51,7 +47,7 @@ def parse_html_toc(data):
 class TOC(list):
 
     def __init__(self, href=None, fragment=None, text=None, parent=None,
-            play_order=0, base_path=os.getcwdu(), type='unknown', author=None,
+            play_order=0, base_path=getcwd(), type='unknown', author=None,
             description=None, toc_thumbnail=None):
         self.href = href
         self.fragment = fragment
@@ -69,7 +65,7 @@ class TOC(list):
     def __str__(self):
         lines = ['TOC: %s#%s %s'%(self.href, self.fragment, self.text)]
         for child in self:
-            c = str(child).splitlines()
+            c = unicode_type(child).splitlines()
             for l in c:
                 lines.append('\t'+l)
         return '\n'.join(lines)
@@ -122,19 +118,16 @@ class TOC(list):
             for i in obj.flat():
                 yield i
 
-    @dynamic_property
+    @property
     def abspath(self):
-        doc='Return the file this toc entry points to as a absolute path to a file on the system.'
+        'Return the file this toc entry points to as a absolute path to a file on the system.'
 
-        def fget(self):
-            if self.href is None:
-                return None
-            path = self.href.replace('/', os.sep)
-            if not os.path.isabs(path):
-                path = os.path.join(self.base_path, path)
-            return path
-
-        return property(fget=fget, doc=doc)
+        if self.href is None:
+            return None
+        path = self.href.replace('/', os.sep)
+        if not os.path.isabs(path):
+            path = os.path.join(self.base_path, path)
+        return path
 
     def read_from_opf(self, opfreader):
         toc = opfreader.soup.find('spine', toc=True)
@@ -182,10 +175,10 @@ class TOC(list):
     def read_ncx_toc(self, toc, root=None):
         self.base_path = os.path.dirname(toc)
         if root is None:
-            raw  = xml_to_unicode(open(toc, 'rb').read(), assume_utf8=True,
-                    strip_encoding_pats=True)[0]
-            root = etree.fromstring(raw, parser=etree.XMLParser(recover=True,
-                no_network=True))
+            with open(toc, 'rb') as f:
+                raw  = xml_to_unicode(f.read(), assume_utf8=True,
+                        strip_encoding_pats=True)[0]
+            root = safe_xml_fromstring(raw)
         xpn = {'re': 'http://exslt.org/regular-expressions'}
         XPath = functools.partial(etree.XPath, namespaces=xpn)
 
@@ -210,10 +203,10 @@ class TOC(list):
             nl = nl_path(np)
             if nl:
                 nl = nl[0]
-                text = u''
+                text = ''
                 for txt in txt_path(nl):
                     text += etree.tostring(txt, method='text',
-                            encoding=unicode_type, with_tail=False)
+                            encoding='unicode', with_tail=False)
                 content = content_path(np)
                 if content and text:
                     content = content[0]
@@ -236,7 +229,9 @@ class TOC(list):
 
     def read_html_toc(self, toc):
         self.base_path = os.path.dirname(toc)
-        for href, fragment, txt in parse_html_toc(lopen(toc, 'rb').read()):
+        with lopen(toc, 'rb') as f:
+            parsed_toc = parse_html_toc(f.read())
+        for href, fragment, txt in parsed_toc:
             add = True
             for i in self.flat():
                 if i.href == href and i.fragment == fragment:
@@ -248,8 +243,8 @@ class TOC(list):
     def render(self, stream, uid):
         root = E.ncx(
                 E.head(
-                    E.meta(name='dtb:uid', content=str(uid)),
-                    E.meta(name='dtb:depth', content=str(self.depth())),
+                    E.meta(name='dtb:uid', content=unicode_type(uid)),
+                    E.meta(name='dtb:depth', content=unicode_type(self.depth())),
                     E.meta(name='dtb:generator', content='%s (%s)'%(__appname__,
                         __version__)),
                     E.meta(name='dtb:totalPageCount', content='0'),
@@ -274,7 +269,7 @@ class TOC(list):
                     E.content(src=unicode_type(np.href)+(('#' + unicode_type(np.fragment))
                         if np.fragment else '')),
                     id=item_id,
-                    playOrder=str(np.play_order)
+                    playOrder=unicode_type(np.play_order)
             )
             au = getattr(np, 'author', None)
             if au:
