@@ -1,4 +1,5 @@
-from __future__ import with_statement
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -7,6 +8,8 @@ import os
 from calibre.utils.zipfile import ZipFile
 from calibre.ptempfile import TemporaryDirectory
 from calibre import CurrentDir
+from polyglot.builtins import getcwd
+
 
 def get_metadata(stream):
     from calibre.ebooks.metadata.meta import get_metadata
@@ -46,7 +49,7 @@ def zip_opf_metadata(opfpath, zf):
     from calibre.ebooks.metadata.opf2 import OPF
     if hasattr(opfpath, 'read'):
         f = opfpath
-        opfpath = getattr(f, 'name', os.getcwdu())
+        opfpath = getattr(f, 'name', getcwd())
     else:
         f = open(opfpath, 'rb')
     opf = OPF(f, os.path.dirname(opfpath))
@@ -64,4 +67,3 @@ def zip_opf_metadata(opfpath, zf):
             data = zf.read(covername)
             mi.cover_data = (fmt, data)
     return mi
-
