@@ -1,21 +1,23 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import struct, datetime, os
+import struct, datetime, os, numbers, binascii
 
 from calibre.utils.date import utc_tz
 from calibre.ebooks.mobi.reader.headers import NULL_INDEX
 from calibre.ebooks.mobi.langcodes import main_language, sub_language
 from calibre.ebooks.mobi.debug import format_bytes
 from calibre.ebooks.mobi.utils import get_trailing_data
+from polyglot.builtins import iteritems, range, unicode_type
 
 # PalmDB {{{
+
+
 class PalmDOCAttributes(object):
 
     class Attr(object):
@@ -26,6 +28,7 @@ class PalmDOCAttributes(object):
 
         def __str__(self):
             return '%s: %s'%(self.name, bool(self.val))
+        __unicode__ = __str__
 
     def __init__(self, raw):
         self.val = struct.unpack(b'<H', raw)[0]
@@ -40,8 +43,10 @@ class PalmDOCAttributes(object):
                 self.val))
 
     def __str__(self):
-        attrs = '\n\t'.join([str(x) for x in self.attributes])
+        attrs = '\n\t'.join([unicode_type(x) for x in self.attributes])
         return 'PalmDOC Attributes: %s\n\t%s'%(bin(self.val), attrs)
+    __unicode__ = __str__
+
 
 class PalmDB(object):
 
@@ -81,7 +86,7 @@ class PalmDB(object):
     def __str__(self):
         ans = ['*'*20 + ' PalmDB Header '+ '*'*20]
         ans.append('Name: %r'%self.name)
-        ans.append(str(self.attributes))
+        ans.append(unicode_type(self.attributes))
         ans.append('Version: %s'%self.version)
         ans.append('Creation date: %s (%s)'%(self.creation_date.isoformat(),
             self.creation_date_raw))
@@ -99,8 +104,10 @@ class PalmDB(object):
         ans.append('Number of records: %s'%self.number_of_records)
 
         return '\n'.join(ans)
+    __unicode__ = __str__
 # }}}
 
+
 class Record(object):  # {{{
 
     def __init__(self, raw, header):
@@ -114,6 +121,8 @@ class Record(object):  # {{{
 # }}}
 
 # EXTH {{{
+
+
 class EXTHRecord(object):
 
     def __init__(self, type_, data, length):
@@ -201,11 +210,12 @@ class EXTHRecord(object):
             else:
                 self.data, = struct.unpack(b'>L', self.data)
         elif self.type in {209, 300}:
-            self.data = bytes(self.data.encode('hex'))
+            self.data = binascii.hexlify(self.data)
 
     def __str__(self):
         return '%s (%d): %r'%(self.name, self.type, self.data)
 
+
 class EXTHHeader(object):
 
     def __init__(self, raw):
@@ -217,7 +227,7 @@ class EXTHHeader(object):
 
         pos = 12
         self.records = []
-        for i in xrange(self.count):
+        for i in range(self.count):
             pos = self.read_record(pos)
         self.records.sort(key=lambda x:x.type)
         self.rmap = {x.type:x for x in self.records}
@@ -248,10 +258,13 @@ class EXTHHeader(object):
         ans.append('Number of EXTH records: %d'%self.count)
         ans.append('EXTH records...')
         for r in self.records:
-            ans.append(str(r))
+            ans.append(unicode_type(r))
         return '\n'.join(ans)
+    __unicode__ = __str__
+
 # }}}
 
+
 class MOBIHeader(object):  # {{{
 
     def __init__(self, record0, offset):
@@ -488,7 +501,7 @@ class MOBIHeader(object):  # {{{
         ans = '\n'.join(ans)
 
         if self.has_exth:
-            ans += '\n\n' + str(self.exth)
+            ans += '\n\n' + unicode_type(self.exth)
             ans += '\n\nBytes after EXTH (%d bytes): %s'%(
                     len(self.bytes_after_exth),
                     format_bytes(self.bytes_after_exth))
@@ -500,6 +513,7 @@ class MOBIHeader(object):  # {{{
         return ans
 # }}}
 
+
 class MOBIFile(object):
 
     def __init__(self, stream):
@@ -508,7 +522,7 @@ class MOBIFile(object):
 
         self.record_headers = []
         self.records = []
-        for i in xrange(self.palmdb.number_of_records):
+        for i in range(self.palmdb.number_of_records):
             pos = 78 + i * 8
             offset, a1, a2, a3, a4 = struct.unpack(b'>LBBBB', self.raw[pos:pos+8])
             flags, val = a1, a2 << 16 | a3 << 8 | a4
@@ -548,7 +562,7 @@ class MOBIFile(object):
             from calibre.ebooks.mobi.huffcdic import HuffReader
 
             def huffit(off, cnt):
-                huffman_record_nums = list(xrange(off, off+cnt))
+                huffman_record_nums = list(range(off, off+cnt))
                 huffrecs = [self.records[r].raw for r in huffman_record_nums]
                 huffs = HuffReader(huffrecs)
                 return huffman_record_nums, huffs.unpack
@@ -571,6 +585,7 @@ class MOBIFile(object):
 
         self.decompress6, self.decompress8 = d6, d8
 
+
 class TextRecord(object):  # {{{
 
     def __init__(self, idx, record, extra_data_flags, decompress):
@@ -586,9 +601,9 @@ class TextRecord(object):  # {{{
             self.trailing_data['uncrossable_breaks'] = self.trailing_data.pop(2)
         self.trailing_data['raw_bytes'] = raw_trailing_bytes
 
-        for typ, val in self.trailing_data.iteritems():
-            if isinstance(typ, int):
-                print ('Record %d has unknown trailing data of type: %d : %r'%
+        for typ, val in iteritems(self.trailing_data):
+            if isinstance(typ, numbers.Integral):
+                print('Record %d has unknown trailing data of type: %d : %r'%
                         (idx, typ, val))
 
         self.idx = idx
@@ -598,7 +613,7 @@ class TextRecord(object):  # {{{
         with open(os.path.join(folder, name+'.txt'), 'wb') as f:
             f.write(self.raw)
         with open(os.path.join(folder, name+'.trailing_data'), 'wb') as f:
-            for k, v in self.trailing_data.iteritems():
+            for k, v in iteritems(self.trailing_data):
                 raw = '%s : %r\n\n'%(k, v)
                 f.write(raw.encode('utf-8'))
 
@@ -606,5 +621,3 @@ class TextRecord(object):  # {{{
         return len(self.raw)
 
 # }}}
-
-
