@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -12,39 +11,45 @@ from collections import OrderedDict, namedtuple
 
 from calibre.ebooks.mobi.utils import (decint, count_set_bits,
         decode_string)
+from polyglot.builtins import iteritems, range, zip
 
 TagX = namedtuple('TagX', 'tag num_of_values bitmask eof')
 PTagX = namedtuple('PTagX', 'tag value_count value_bytes num_of_values')
 INDEX_HEADER_FIELDS = (
             'len', 'nul1', 'type', 'gen', 'start', 'count', 'code',
             'lng', 'total', 'ordt', 'ligt', 'nligt', 'ncncx'
-    ) + tuple('unknown%d'%i for i in xrange(27)) + ('ocnt', 'oentries',
+    ) + tuple('unknown%d'%i for i in range(27)) + ('ocnt', 'oentries',
             'ordt1', 'ordt2', 'tagx')
 
 
 class InvalidFile(ValueError):
     pass
 
+
 def check_signature(data, signature):
     if data[:len(signature)] != signature:
         raise InvalidFile('Not a valid %r section'%signature)
 
+
 class NotAnINDXRecord(InvalidFile):
     pass
 
+
 class NotATAGXSection(InvalidFile):
     pass
 
+
 def format_bytes(byts):
     byts = bytearray(byts)
     byts = [hex(b)[2:] for b in byts]
     return ' '.join(byts)
 
+
 def parse_indx_header(data):
     check_signature(data, b'INDX')
     words = INDEX_HEADER_FIELDS
     num = len(words)
-    values = struct.unpack(bytes('>%dL' % num), data[4:4*(num+1)])
+    values = struct.unpack('>%dL' % num, data[4:4*(num+1)])
     ans = dict(zip(words, values))
     ordt1, ordt2 = ans['ordt1'], ans['ordt2']
     ans['ordt1_raw'], ans['ordt2_raw'] = [], []
@@ -68,7 +73,7 @@ def parse_indx_header(data):
             # ascii character. If we cannot, we map to the ? char.
 
             parsed = bytearray(ans['oentries'])
-            for i in xrange(0, 2*ans['oentries'], 2):
+            for i in range(0, 2*ans['oentries'], 2):
                 parsed[i//2] = raw[i+1] if 0x20 < raw[i+1] < 0x7f else ord(b'?')
             ans['ordt_map'] = bytes(parsed).decode('ascii')
         else:
@@ -99,7 +104,7 @@ class CNCX(object):  # {{{
                     except:
                         byts = raw[pos:]
                         r = format_bytes(byts)
-                        print ('CNCX entry at offset %d has unknown format %s'%(
+                        print('CNCX entry at offset %d has unknown format %s'%(
                             pos+record_offset, r))
                         self.records[pos+record_offset] = r
                         pos = len(raw)
@@ -117,9 +122,13 @@ class CNCX(object):  # {{{
     __nonzero__ = __bool__
 
     def iteritems(self):
-        return self.records.iteritems()
+        return iteritems(self.records)
+
+    def items(self):
+        return iteritems(self.records)
 # }}}
 
+
 def parse_tagx_section(data):
     check_signature(data, b'TAGX')
 
@@ -127,11 +136,12 @@ def parse_tagx_section(data):
     first_entry_offset, = struct.unpack_from(b'>L', data, 4)
     control_byte_count, = struct.unpack_from(b'>L', data, 8)
 
-    for i in xrange(12, first_entry_offset, 4):
+    for i in range(12, first_entry_offset, 4):
         vals = list(bytearray(data[i:i+4]))
         tags.append(TagX(*vals))
     return control_byte_count, tags
 
+
 def get_tag_map(control_byte_count, tagx, data, strict=False):
     ptags = []
     ans = {}
@@ -170,7 +180,7 @@ def get_tag_map(control_byte_count, tagx, data, strict=False):
         values = []
         if x.value_count is not None:
             # Read value_count * values_per_entry variable width values.
-            for _ in xrange(x.value_count * x.num_of_values):
+            for _ in range(x.value_count * x.num_of_values):
                 byts, consumed = decint(data)
                 data = data[consumed:]
                 values.append(byts)
@@ -202,17 +212,18 @@ def get_tag_map(control_byte_count, tagx, data, strict=False):
 
     return ans
 
+
 def parse_index_record(table, data, control_byte_count, tags, codec,
         ordt_map, strict=False):
     header = parse_indx_header(data)
     idxt_pos = header['start']
     if data[idxt_pos:idxt_pos+4] != b'IDXT':
-        print ('WARNING: Invalid INDX record')
+        print('WARNING: Invalid INDX record')
     entry_count = header['count']
 
     # loop through to build up the IDXT position starts
     idx_positions= []
-    for j in xrange(entry_count):
+    for j in range(entry_count):
         pos, = struct.unpack_from(b'>H', data, idxt_pos + 4 + (2 * j))
         idx_positions.append(pos)
     # The last entry ends before the IDXT tag (but there might be zero fill
@@ -221,7 +232,7 @@ def parse_index_record(table, data, control_byte_count, tags, codec,
 
     # For each entry in the IDXT build up the tag map and any associated
     # text
-    for j in xrange(entry_count):
+    for j in range(entry_count):
         start, end = idx_positions[j:j+2]
         rec = data[start:end]
         # Sometimes (in the guide table if the type attribute has non ascii
@@ -241,6 +252,7 @@ def parse_index_record(table, data, control_byte_count, tags, codec,
         table[ident] = tag_map
     return header
 
+
 def read_index(sections, idx, codec):
     table, cncx = OrderedDict(), CNCX([], codec)
 
@@ -257,10 +269,9 @@ def read_index(sections, idx, codec):
     tag_section_start = indx_header['tagx']
     control_byte_count, tags = parse_tagx_section(data[tag_section_start:])
 
-    for i in xrange(idx + 1, idx + 1 + indx_count):
+    for i in range(idx + 1, idx + 1 + indx_count):
         # Index record
         data = sections[i][0]
         parse_index_record(table, data, control_byte_count, tags, codec,
                 indx_header['ordt_map'])
     return table, cncx
-
