@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -11,21 +10,40 @@ import struct, string, zlib, os
 from collections import OrderedDict
 from io import BytesIO
 
-from calibre.utils.img import save_cover_data_to, scale_image, image_to_data, image_from_data, resize_image
+from calibre.utils.img import save_cover_data_to, scale_image, image_to_data, image_from_data, resize_image, png_data_to_gif_data
 from calibre.utils.imghdr import what
 from calibre.ebooks import normalize
+from polyglot.builtins import unicode_type, range, as_bytes, map
 from tinycss.color3 import parse_color_string
 
 IMAGE_MAX_SIZE = 10 * 1024 * 1024
 RECORD_SIZE = 0x1000  # 4096 (Text record size (uncompressed))
 
 
-def decode_string(raw, codec='utf-8', ordt_map=''):
-    length, = struct.unpack(b'>B', raw[0])
+class PolyglotDict(dict):
+
+    def __setitem__(self, key, val):
+        if isinstance(key, unicode_type):
+            key = key.encode('utf-8')
+        dict.__setitem__(self, key, val)
+
+    def __getitem__(self, key):
+        if isinstance(key, unicode_type):
+            key = key.encode('utf-8')
+        return dict.__getitem__(self, key)
+
+    def __contains__(self, key):
+        if isinstance(key, unicode_type):
+            key = key.encode('utf-8')
+        return dict.__contains__(self, key)
+
+
+def decode_string(raw, codec='utf-8', ordt_map=None):
+    length, = struct.unpack(b'>B', raw[0:1])
     raw = raw[1:1+length]
     consumed = length+1
     if ordt_map:
-        return ''.join(ordt_map[ord(x)] for x in raw), consumed
+        return ''.join(ordt_map[x] for x in bytearray(raw)), consumed
     return raw.decode(codec), consumed
 
 
@@ -46,7 +64,7 @@ def decode_hex_number(raw, codec='utf-8'):
 
 
 def encode_string(raw):
-    ans = bytearray(bytes(raw))
+    ans = bytearray(as_bytes(raw))
     ans.insert(0, len(ans))
     return bytes(ans)
 
@@ -59,7 +77,7 @@ def encode_number_as_hex(num):
     The bytes that follow are simply the hexadecimal representation of the
     number.
     '''
-    num = bytes(hex(num)[2:].upper())
+    num = hex(num)[2:].upper().encode('ascii')
     nlen = len(num)
     if nlen % 2 != 0:
         num = b'0'+num
@@ -205,7 +223,7 @@ def get_trailing_data(record, extra_data_flags):
     if extra_data_flags & 0b1:
         # Only the first two bits are used for the size since there can
         # never be more than 3 trailing multibyte chars
-        sz = (ord(record[-1]) & 0b11) + 1
+        sz = (ord(record[-1:]) & 0b11) + 1
         consumed = 1
         if sz > consumed:
             data[0] = record[-sz:-consumed]
@@ -241,7 +259,7 @@ def encode_fvwi(val, flags, flag_size=4):
     bytestring.
     '''
     ans = val << flag_size
-    for i in xrange(flag_size):
+    for i in range(flag_size):
         ans |= (flags & (1 << i))
     return encint(ans)
 
@@ -253,7 +271,7 @@ def decode_fvwi(byts, flag_size=4):
     arg, consumed = decint(bytes(byts))
     val = arg >> flag_size
     flags = 0
-    for i in xrange(flag_size):
+    for i in range(flag_size):
         flags |= (arg & (1 << i))
     return val, flags, consumed
 
@@ -280,7 +298,7 @@ def decode_tbs(byts, flag_size=4):
         extra[0b0010] = x
         consumed += consumed2
     if flags & 0b0100:
-        extra[0b0100] = ord(byts[0])
+        extra[0b0100] = ord(byts[0:1])
         byts = byts[1:]
         consumed += 1
     if flags & 0b0001:
@@ -317,7 +335,7 @@ def utf8_text(text):
     '''
     if text and text.strip():
         text = text.strip()
-        if not isinstance(text, unicode):
+        if not isinstance(text, unicode_type):
             text = text.decode('utf-8', 'replace')
         text = normalize(text).encode('utf-8')
     else:
@@ -399,13 +417,8 @@ def to_base(num, base=32, min_num_digits=None):
 def mobify_image(data):
     'Convert PNG images to GIF as the idiotic Kindle cannot display some PNG'
     fmt = what(None, data)
-
     if fmt == 'png':
-        from PIL import Image
-        im = Image.open(BytesIO(data))
-        buf = BytesIO()
-        im.save(buf, 'gif')
-        data = buf.getvalue()
+        data = png_data_to_gif_data(data)
     return data
 
 # Font records {{{
@@ -461,7 +474,7 @@ def read_font_record(data, extent=1040):
         extent = len(font_data) if extent is None else extent
         extent = min(extent, len(font_data))
 
-        for n in xrange(extent):
+        for n in range(extent):
             buf[n] ^= key[n%xor_len]  # XOR of buf and key
 
         font_data = bytes(buf)
@@ -505,7 +518,7 @@ def write_font_record(data, obfuscate=True, compress=True):
         xor_key = os.urandom(key_len)
         key = bytearray(xor_key)
         data = bytearray(data)
-        for i in xrange(1040):
+        for i in range(1040):
             data[i] ^= key[i%key_len]
         data = bytes(data)
 
@@ -588,7 +601,7 @@ class CNCX(object):  # {{{
         offset = 0
         buf = BytesIO()
         RECORD_LIMIT = 0x10000 - 1024  # kindlegen appears to use 1024, PDB limit is 0x10000
-        for key in self.strings.iterkeys():
+        for key in self.strings:
             utf8 = utf8_text(key[:self.MAX_STRING_LENGTH])
             l = len(utf8)
             sz_bytes = encint(l)
@@ -625,7 +638,7 @@ def is_guide_ref_start(ref):
 
 
 def convert_color_for_font_tag(val):
-    rgba = parse_color_string(unicode(val or ''))
+    rgba = parse_color_string(unicode_type(val or ''))
     if rgba is None or rgba == 'currentColor':
         return val
     clamp = lambda x: min(x, max(0, x), 1)
