@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -11,10 +10,11 @@ import re
 from struct import pack
 from io import BytesIO
 
-from calibre.constants import iswindows, isosx
+from calibre.constants import iswindows, ismacos
 from calibre.ebooks.mobi.utils import (utf8_text, to_base)
 from calibre.utils.localization import lang_as_iso639_1
 from calibre.ebooks.metadata import authors_to_sort_string
+from polyglot.builtins import iteritems, unicode_type
 
 EXTH_CODES = {
     'creator': 100,
@@ -62,14 +62,14 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
         items = metadata[term]
         if term == 'creator':
             if prefer_author_sort:
-                creators = [authors_to_sort_string([unicode(c)]) for c in
+                creators = [authors_to_sort_string([unicode_type(c)]) for c in
                             items]
             else:
-                creators = [unicode(c) for c in items]
+                creators = [unicode_type(c) for c in items]
             items = creators
         elif term == 'rights':
             try:
-                rights = utf8_text(unicode(metadata.rights[0]))
+                rights = utf8_text(unicode_type(metadata.rights[0]))
             except:
                 rights = b'Unknown'
             exth.write(pack(b'>II', EXTH_CODES['rights'], len(rights) + 8))
@@ -78,7 +78,7 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
             continue
 
         for item in items:
-            data = unicode(item)
+            data = unicode_type(item)
             if term != 'description':
                 data = COLLAPSE_RE.sub(' ', data)
             if term == 'identifier':
@@ -102,14 +102,14 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
     from calibre.ebooks.oeb.base import OPF
     for x in metadata['identifier']:
         if (x.get(OPF('scheme'), None).lower() == 'uuid' or
-                unicode(x).startswith('urn:uuid:')):
-            uuid = unicode(x).split(':')[-1]
+                unicode_type(x).startswith('urn:uuid:')):
+            uuid = unicode_type(x).split(':')[-1]
             break
     if uuid is None:
         from uuid import uuid4
-        uuid = str(uuid4())
+        uuid = unicode_type(uuid4())
 
-    if isinstance(uuid, unicode):
+    if isinstance(uuid, unicode_type):
         uuid = uuid.encode('utf-8')
     if not share_not_sync:
         exth.write(pack(b'>II', 113, len(uuid) + 8))
@@ -137,14 +137,14 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
 
     # Add a publication date entry
     if metadata['date']:
-        datestr = str(metadata['date'][0])
+        datestr = unicode_type(metadata['date'][0])
     elif metadata['timestamp']:
-        datestr = str(metadata['timestamp'][0])
+        datestr = unicode_type(metadata['timestamp'][0])
 
     if datestr is None:
         raise ValueError("missing date or timestamp")
 
-    datestr = bytes(datestr)
+    datestr = datestr.encode('utf-8')
     exth.write(pack(b'>II', EXTH_CODES['pubdate'], len(datestr) + 8))
     exth.write(datestr)
     nrecs += 1
@@ -154,7 +154,7 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
         nrecs += 1
 
     if be_kindlegen2:
-        mv = 200 if iswindows else 202 if isosx else 201
+        mv = 200 if iswindows else 202 if ismacos else 201
         vals = {204:mv, 205:2, 206:9, 207:0}
     elif is_periodical:
         # Pretend to be amazon's super secret periodical generator
@@ -162,7 +162,7 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
     else:
         # Pretend to be kindlegen 1.2
         vals = {204:201, 205:1, 206:2, 207:33307}
-    for code, val in vals.iteritems():
+    for code, val in iteritems(vals):
         exth.write(pack(b'>III', code, 12, val))
         nrecs += 1
     if be_kindlegen2:
@@ -178,7 +178,7 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
     if thumbnail_offset is not None:
         exth.write(pack(b'>III', EXTH_CODES['thumboffset'], 12,
             thumbnail_offset))
-        thumbnail_uri_str = bytes('kindle:embed:%s' %(to_base(thumbnail_offset, base=32, min_num_digits=4)))
+        thumbnail_uri_str = ('kindle:embed:%s' %(to_base(thumbnail_offset, base=32, min_num_digits=4))).encode('utf-8')
         exth.write(pack(b'>II', EXTH_CODES['kf8_thumbnail_uri'], len(thumbnail_uri_str) + 8))
         exth.write(thumbnail_uri_str)
         nrecs += 2
@@ -216,7 +216,7 @@ def build_exth(metadata, prefer_author_sort=False, is_periodical=False,
         nrecs += 1
 
     if page_progression_direction in {'rtl', 'ltr', 'default'}:
-        ppd = bytes(page_progression_direction)
+        ppd = page_progression_direction.encode('ascii')
         exth.write(pack(b'>II', EXTH_CODES['page_progression_direction'], len(ppd) + 8))
         exth.write(ppd)
         nrecs += 1
