@@ -1,25 +1,26 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import posixpath, os, time, types
-from collections import namedtuple, defaultdict, Counter
+from collections import namedtuple, defaultdict
 from itertools import chain
 
 from calibre import prepare_string_for_xml, force_unicode
 from calibre.ebooks.oeb.base import XPath, xml2text
 from calibre.ebooks.oeb.polish.container import OEB_DOCS, OEB_STYLES, OEB_FONTS
-from calibre.ebooks.oeb.polish.spell import get_all_words
-from calibre.utils.icu import numeric_sort_key, ord_string, safe_chr
-from calibre.utils.magick.draw import identify
+from calibre.ebooks.oeb.polish.spell import get_all_words, count_all_chars
+from calibre.utils.icu import numeric_sort_key, safe_chr
+from calibre.utils.imghdr import identify
 from css_selectors import Select, SelectorError
+from polyglot.builtins import iteritems
 
 File = namedtuple('File', 'name dir basename size category')
 
+
 def get_category(name, mt):
     category = 'misc'
     if mt.startswith('image/'):
@@ -40,46 +41,54 @@ def get_category(name, mt):
         category = 'toc'
     return category
 
+
 def safe_size(container, name):
     try:
         return os.path.getsize(container.name_to_abspath(name))
     except Exception:
         return 0
 
+
 def safe_img_data(container, name, mt):
     if 'svg' in mt:
         return 0, 0
     try:
-        width, height, fmt = identify(container.name_to_abspath(name))
+        fmt, width, height = identify(container.name_to_abspath(name))
     except Exception:
         width = height = 0
     return width, height
 
+
 def files_data(container, *args):
-    for name, path in container.name_path_map.iteritems():
+    for name, path in iteritems(container.name_path_map):
         yield File(name, posixpath.dirname(name), posixpath.basename(name), safe_size(container, name),
                    get_category(name, container.mime_map.get(name, '')))
 
+
 Image = namedtuple('Image', 'name mime_type usage size basename id width height')
 
 LinkLocation = namedtuple('LinkLocation', 'name line_number text_on_line')
 
+
 def sort_locations(container, locations):
     nmap = {n:i for i, (n, l) in enumerate(container.spine_names)}
+
     def sort_key(l):
         return (nmap.get(l.name, len(nmap)), numeric_sort_key(l.name), l.line_number)
     return sorted(locations, key=sort_key)
 
+
 def safe_href_to_name(container, href, base):
     try:
         return container.href_to_name(href, base)
     except ValueError:
         pass  # Absolute path on windows
 
+
 def images_data(container, *args):
     image_usage = defaultdict(set)
     link_sources = OEB_STYLES | OEB_DOCS
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in link_sources:
             for href, line_number, offset in container.iterlinks(name):
                 target = safe_href_to_name(container, href, name)
@@ -89,12 +98,13 @@ def images_data(container, *args):
                         image_usage[target].add(LinkLocation(name, line_number, href))
 
     image_data = []
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt.startswith('image/') and container.exists(name):
             image_data.append(Image(name, mt, sort_locations(container, image_usage.get(name, set())), safe_size(container, name),
                                     posixpath.basename(name), len(image_data), *safe_img_data(container, name, mt)))
     return tuple(image_data)
 
+
 def description_for_anchor(elem):
     def check(x, min_len=4):
         if x:
@@ -121,6 +131,7 @@ def description_for_anchor(elem):
         if desc is not None:
             return desc
 
+
 def create_anchor_map(root, pat, name):
     ans = {}
     for elem in pat(root):
@@ -129,8 +140,11 @@ def create_anchor_map(root, pat, name):
             ans[anchor] = (LinkLocation(name, elem.sourceline, anchor), description_for_anchor(elem))
     return ans
 
+
 Anchor = namedtuple('Anchor', 'id location text')
 L = namedtuple('Link', 'location text is_external href path_ok anchor_ok anchor ok')
+
+
 def Link(location, text, is_external, href, path_ok, anchor_ok, anchor):
     if is_external:
         ok = None
@@ -138,12 +152,13 @@ def Link(location, text, is_external, href, path_ok, anchor_ok, anchor):
         ok = path_ok and anchor_ok
     return L(location, text, is_external, href, path_ok, anchor_ok, anchor, ok)
 
+
 def links_data(container, *args):
     anchor_map = {}
     links = []
     anchor_pat = XPath('//*[@id or @name]')
     link_pat = XPath('//h:a[@href]')
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_DOCS:
             root = container.parsed(name)
             anchor_map[name] = create_anchor_map(root, anchor_pat, name)
@@ -179,35 +194,27 @@ def links_data(container, *args):
                 link = Link(location, text, False, dest, False, False, Anchor(frag, None, None))
         yield link
 
+
 Word = namedtuple('Word', 'id word locale usage')
 
+
 def words_data(container, book_locale, *args):
     count, words = get_all_words(container, book_locale, get_word_count=True)
-    return (count, tuple(Word(i, word, locale, v) for i, ((word, locale), v) in enumerate(words.iteritems())))
+    return (count, tuple(Word(i, word, locale, v) for i, ((word, locale), v) in enumerate(iteritems(words))))
+
 
 Char = namedtuple('Char', 'id char codepoint usage count')
 
-def chars_data(container, *args):
-    chars = defaultdict(set)
-    counter = Counter()
-    def count(codepoint):
-        counter[codepoint] += 1
-
-    for name, is_linear in container.spine_names:
-        if container.mime_map.get(name) not in OEB_DOCS:
-            continue
-        raw = container.raw_data(name)
-        counts = Counter(ord_string(raw))
-        counter.update(counts)
-        for codepoint in counts:
-            chars[codepoint].add(name)
 
+def chars_data(container, book_locale, *args):
+    cc = count_all_chars(container, book_locale)
     nmap = {n:i for i, (n, l) in enumerate(container.spine_names)}
+
     def sort_key(name):
         return nmap.get(name, len(nmap)), numeric_sort_key(name)
 
-    for i, (codepoint, usage) in enumerate(chars.iteritems()):
-        yield Char(i, safe_chr(codepoint), codepoint, sorted(usage, key=sort_key), counter[codepoint])
+    for i, (codepoint, usage) in enumerate(iteritems(cc.chars)):
+        yield Char(i, safe_chr(codepoint), codepoint, sorted(usage, key=sort_key), cc.counter[codepoint])
 
 
 CSSRule = namedtuple('CSSRule', 'selector location')
@@ -220,6 +227,7 @@ ClassEntry = namedtuple('ClassEntry', 'cls num_of_matches matched_files sort_key
 ClassFileMatch = namedtuple('ClassFileMatch', 'file_name class_elements sort_key')
 ClassElement = namedtuple('ClassElement', 'name line_number text_on_line tag matched_rules')
 
+
 def css_data(container, book_locale, result_data, *args):
     import tinycss
     from tinycss.css21 import RuleSet, ImportRule
@@ -244,7 +252,7 @@ def css_data(container, book_locale, result_data, *args):
     spine_names = {name for name, is_linear in container.spine_names}
     style_path, link_path = XPath('//h:style'), XPath('//h:link/@href')
 
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_STYLES:
             importable_sheets[name] = css_rules(name, parser.parse_stylesheet(container.raw_data(name)).rules)
         elif mt in OEB_DOCS and name in spine_names:
@@ -274,6 +282,7 @@ def css_data(container, book_locale, result_data, *args):
                 yield sheet
 
     tt_cache = {}
+
     def tag_text(elem):
         ans = tt_cache.get(elem)
         if ans is None:
@@ -298,7 +307,7 @@ def css_data(container, book_locale, result_data, *args):
 
     class_map = defaultdict(lambda : defaultdict(list))
 
-    for name, inline_sheets in html_sheets.iteritems():
+    for name, inline_sheets in iteritems(html_sheets):
         root = container.parsed(name)
         cmap = defaultdict(lambda : defaultdict(list))
         for elem in root.xpath('//*[@class]'):
@@ -308,21 +317,21 @@ def css_data(container, book_locale, result_data, *args):
         for sheet in chain(sheets_for_html(name, root), inline_sheets):
             for rule in rules_in_sheet(sheet):
                 rule_map[rule][name].extend(matches_for_selector(rule.selector, select, cmap, rule))
-        for cls, elem_map in cmap.iteritems():
+        for cls, elem_map in iteritems(cmap):
             class_elements = class_map[cls][name]
-            for elem, usage in elem_map.iteritems():
+            for elem, usage in iteritems(elem_map):
                 class_elements.append(
                     ClassElement(name, elem.sourceline, elem.get('class'), tag_text(elem), tuple(usage)))
 
     result_data['classes'] = ans = []
-    for cls, name_map in class_map.iteritems():
-        la = tuple(ClassFileMatch(name, tuple(class_elements), numeric_sort_key(name)) for name, class_elements in name_map.iteritems() if class_elements)
+    for cls, name_map in iteritems(class_map):
+        la = tuple(ClassFileMatch(name, tuple(class_elements), numeric_sort_key(name)) for name, class_elements in iteritems(name_map) if class_elements)
         num_of_matches = sum(sum(len(ce.matched_rules) for ce in cfm.class_elements) for cfm in la)
         ans.append(ClassEntry(cls, num_of_matches, la, numeric_sort_key(cls)))
 
     ans = []
-    for rule, loc_map in rule_map.iteritems():
-        la = tuple(CSSFileMatch(name, tuple(locations), numeric_sort_key(name)) for name, locations in loc_map.iteritems() if locations)
+    for rule, loc_map in iteritems(rule_map):
+        la = tuple(CSSFileMatch(name, tuple(locations), numeric_sort_key(name)) for name, locations in iteritems(loc_map) if locations)
         count = sum(len(fm.locations) for fm in la)
         ans.append(CSSEntry(rule, count, la, numeric_sort_key(rule.selector)))
 
