@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -13,15 +12,18 @@ from functools import partial
 from lxml.etree import tostring
 import regex
 
-from calibre.ebooks.oeb.base import XHTML
+from calibre.ebooks.oeb.base import XHTML, css_text
 from calibre.ebooks.oeb.polish.cascade import iterrules, resolve_styles, iterdeclaration
 from calibre.utils.icu import ord_string, safe_chr
+from polyglot.builtins import iteritems, itervalues, range, unicode_type
+from tinycss.fonts3 import parse_font_family
+
 
 def normalize_font_properties(font):
     w = font.get('font-weight', None)
     if not w and w != 0:
         w = 'normal'
-    w = unicode(w)
+    w = unicode_type(w)
     w = {'normal':'400', 'bold':'700'}.get(w, w)
     if w not in {'100', '200', '300', '400', '500', '600', '700',
             '800', '900'}:
@@ -41,11 +43,13 @@ def normalize_font_properties(font):
     font['font-stretch'] = val
     return font
 
+
 widths = {x:i for i, x in enumerate(('ultra-condensed',
         'extra-condensed', 'condensed', 'semi-condensed', 'normal',
         'semi-expanded', 'expanded', 'extra-expanded', 'ultra-expanded'
         ))}
 
+
 def get_matching_rules(rules, font):
     matches = []
 
@@ -87,10 +91,10 @@ def get_matching_rules(rules, font):
     elif fw == 500:
         q = [500, 400, 300, 200, 100, 600, 700, 800, 900]
     elif fw < 400:
-        q = [fw] + list(xrange(fw-100, -100, -100)) + list(xrange(fw+100,
+        q = [fw] + list(range(fw-100, -100, -100)) + list(range(fw+100,
             100, 1000))
     else:
-        q = [fw] + list(xrange(fw+100, 100, 1000)) + list(xrange(fw-100,
+        q = [fw] + list(range(fw+100, 100, 1000)) + list(range(fw-100,
             -100, -100))
     for wt in q:
         m = [f for f in matches if f['weight'] == wt]
@@ -98,21 +102,24 @@ def get_matching_rules(rules, font):
             return m
     return []
 
+
 def get_css_text(elem, resolve_pseudo_property, which='before'):
     text = resolve_pseudo_property(elem, which, 'content')[0].value
     if text and len(text) > 2 and text[0] == '"' and text[-1] == '"':
         return text[1:-1]
     return ''
 
+
 caps_variants = {'smallcaps', 'small-caps', 'all-small-caps', 'petite-caps', 'all-petite-caps', 'unicase'}
 
+
 def get_element_text(elem, resolve_property, resolve_pseudo_property, capitalize_pat, for_pseudo=None):
     ans = []
     before = get_css_text(elem, resolve_pseudo_property)
     if before:
         ans.append(before)
     if for_pseudo is not None:
-        ans.append(tostring(elem, method='text', encoding=unicode, with_tail=False))
+        ans.append(tostring(elem, method='text', encoding='unicode', with_tail=False))
     else:
         if elem.text:
             ans.append(elem.text)
@@ -143,6 +150,7 @@ def get_element_text(elem, resolve_property, resolve_pseudo_property, capitalize
                 ans += icu_upper(m.group())
     return ans
 
+
 def get_font_dict(elem, resolve_property, pseudo=None):
     ans = {}
     if pseudo is None:
@@ -153,20 +161,23 @@ def get_font_dict(elem, resolve_property, pseudo=None):
     for p in 'weight', 'style', 'stretch':
         p = 'font-' + p
         rp = resolve_property(elem, p) if pseudo is None else resolve_property(elem, pseudo, p)
-        ans[p] = type('')(rp[0].value)
+        ans[p] = unicode_type(rp[0].value)
     normalize_font_properties(ans)
     return ans
 
+
 bad_fonts = {'serif', 'sans-serif', 'monospace', 'cursive', 'fantasy', 'sansserif', 'inherit'}
 exclude_chars = frozenset(ord_string('\n\r\t'))
 skip_tags = {XHTML(x) for x in 'script style title meta link'.split()}
 font_keys = {'font-weight', 'font-style', 'font-stretch', 'font-family'}
 
+
 def prepare_font_rule(cssdict):
     cssdict['font-family'] = frozenset(cssdict['font-family'][:1])
     cssdict['width'] = widths[cssdict['font-stretch']]
     cssdict['weight'] = int(cssdict['font-weight'])
 
+
 class StatsCollector(object):
 
     first_letter_pat = capitalize_pat = None
@@ -191,7 +202,7 @@ class StatsCollector(object):
                 cssdict = {}
                 for prop in iterdeclaration(rule.style):
                     if prop.name == 'font-family':
-                        cssdict['font-family'] = [icu_lower(x.value) for x in prop.propertyValue]
+                        cssdict['font-family'] = [icu_lower(x) for x in parse_font_family(css_text(prop.propertyValue))]
                     elif prop.name.startswith('font-'):
                         cssdict[prop.name] = prop.propertyValue[0].value
                     elif prop.name == 'src':
@@ -202,7 +213,7 @@ class StatsCollector(object):
                                 cssdict['src'] = fname
                                 break
                         else:
-                            container.log.warn('The @font-face rule refers to a font file that does not exist in the book: %s' % prop.propertyValue.cssText)
+                            container.log.warn('The @font-face rule refers to a font file that does not exist in the book: %s' % css_text(prop.propertyValue))
                 if 'src' not in cssdict:
                     continue
                 ff = cssdict.get('font-family')
@@ -223,7 +234,7 @@ class StatsCollector(object):
                 return
             ff = [icu_lower(x) for x in font.get('font-family', ())]
             if ff and ff[0] not in bad_fonts:
-                key = frozenset(((k, ff[0] if k == 'font-family' else v) for k, v in font.iteritems() if k in font_keys))
+                key = frozenset(((k, ff[0] if k == 'font-family' else v) for k, v in iteritems(font) if k in font_keys))
                 val = font_usage_map.get(key)
                 if val is None:
                     val = font_usage_map[key] = {'text': set()}
@@ -288,11 +299,12 @@ class StatsCollector(object):
             self.font_usage_map[name] = {}
             self.font_spec_map[name] = set()
             self.get_font_usage(container, name, resolve_property, resolve_pseudo_property, font_face_rules, do_embed)
-        self.font_stats = {k:{safe_chr(x) for x in v} for k, v in self.font_stats.iteritems()}
-        for fum in self.font_usage_map.itervalues():
-            for v in fum.itervalues():
+        self.font_stats = {k:{safe_chr(x) for x in v} for k, v in iteritems(self.font_stats)}
+        for fum in itervalues(self.font_usage_map):
+            for v in itervalues(fum):
                 v['text'] = {safe_chr(x) for x in v['text']}
 
+
 if __name__ == '__main__':
     from calibre.ebooks.oeb.polish.container import get_container
     from calibre.utils.logging import default_log
