@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
-from __future__ import print_function
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Gerendi Sandor Attila'
@@ -22,7 +21,7 @@ class tokenDelimitatorStart():
         pass
 
     def toRTF(self):
-        return b'{'
+        return '{'
 
     def __repr__(self):
         return '{'
@@ -34,7 +33,7 @@ class tokenDelimitatorEnd():
         pass
 
     def toRTF(self):
-        return b'}'
+        return '}'
 
     def __repr__(self):
         return '}'
@@ -367,19 +366,15 @@ class RtfTokenizer():
 if __name__ == "__main__":
     import sys
     if len(sys.argv) < 2:
-        print ("Usage %prog rtfFileToConvert")
+        print("Usage %prog rtfFileToConvert")
         sys.exit()
-    f = open(sys.argv[1], 'rb')
-    data = f.read()
-    f.close()
+    with open(sys.argv[1], 'rb') as f:
+        data = f.read()
 
     tokenizer = RtfTokenizer(data)
     parsedTokens = RtfTokenParser(tokenizer.tokens)
 
     data = parsedTokens.toRTF()
 
-    f = open(sys.argv[1], 'w')
-    f.write(data)
-    f.close()
-
-
+    with open(sys.argv[1], 'w') as f:
+        f.write(data)
