@@ -1,4 +1,4 @@
-from __future__ import unicode_literals, absolute_import, print_function, division
+
 #########################################################################
 #                                                                       #
 #                                                                       #
@@ -26,11 +26,13 @@ from calibre.ebooks.rtf2xml import headings_to_sections, \
     body_styles, preamble_rest, group_styles, \
     inline
 from calibre.ebooks.rtf2xml.old_rtf import OldRtf
+from polyglot.builtins import unicode_type
+
 from . import open_for_read, open_for_write
 
 """
 Here is an example script using the ParseRTF module directly
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 def Handle_Main():
     # Handles options and creates a parse object
@@ -248,7 +250,7 @@ class ParseRtf:
             enc = encode_obj.get_codepage()
             # TODO: to check if cp is a good idea or if I should use a dict to convert
             enc = 'cp' + enc
-            msg = '%s\nException in token processing' % str(msg)
+            msg = '%s\nException in token processing' % unicode_type(msg)
             if check_encoding_obj.check_encoding(self.__file, enc):
                 file_name = self.__file if isinstance(self.__file, bytes) \
                                     else self.__file.encode('utf-8')
