@@ -1,7 +1,10 @@
-#!/usr/bin/env python2
-from __future__ import unicode_literals, absolute_import, print_function, division
+#!/usr/bin/env python
+
+
 import sys
 
+from polyglot.builtins import unicode_type
+
 
 class CheckEncoding:
 
@@ -15,7 +18,7 @@ class CheckEncoding:
             try:
                 char.decode(encoding)
             except ValueError as msg:
-                sys.stderr.write('line: %s char: %s\n%s\n' %  (line_num, char_position, str(msg)))
+                sys.stderr.write('line: %s char: %s\n%s\n' %  (line_num, char_position, unicode_type(msg)))
 
     def check_encoding(self, path, encoding='us-ascii', verbose=True):
         line_num = 0
