@@ -1,3 +1,4 @@
+
 #########################################################################
 #                                                                       #
 #                                                                       #
@@ -14,6 +15,9 @@ import sys, os
 
 from calibre.ebooks.rtf2xml import copy
 from calibre.ptempfile import better_mktemp
+from polyglot.builtins import unicode_type
+
+from . import open_for_read, open_for_write
 
 
 class Sections:
@@ -27,19 +31,19 @@ class Sections:
     logic
     ---------------
     The tags for the first section breaks have already been written.
-    RTF stores section breaks with the \sect tag. Each time this tag is
+    RTF stores section breaks with the \\sect tag. Each time this tag is
     encountered, add one to the counter.
-    When I encounter the \sectd tag, I want to collect all the appropriate tokens
-    that describe the section. When I reach a \pard, I know I an stop collecting
+    When I encounter the \\sectd tag, I want to collect all the appropriate tokens
+    that describe the section. When I reach a \\pard, I know I an stop collecting
     tokens and write the section tags.
     The exception to this method occurs when sections occur in field blocks, such
     as the index. Normally, two section break occur within the index and other
-    field-blocks. (If less or more section breaks occurr, this code may not work.)
+    field-blocks. (If less or more section breaks occur, this code may not work.)
     I want the sections to occur outside of the index. That is, the index
     should be nested inside one section tag. After the index is complete, a new
     section should begin.
     In order to write the sections outside of the field blocks, I have to store
-    all of the field block as a string. When I ecounter the \sect tag, add one to
+    all of the field block as a string. When I ecounter the \\sect tag, add one to
     the section counter, but store this number in a list. Likewise, store the
     information describing the section in another list.
     When I reach the end of the field block, choose the first item from the
@@ -243,7 +247,7 @@ class Sections:
             nothing
         Logic:
             Text or control words indicating text have been found
-            before \pard. This shoud indicate older RTF. Reset the state
+            before \\pard. This shoud indicate older RTF. Reset the state
             Write the section defintion. Insert a paragraph definition.
             Insert {} to mark the end of a paragraph defintion
         """
@@ -272,8 +276,8 @@ class Sections:
             my_string += 'mi<tg<close_____<section\n'
         else:
             self.__found_first_sec = 1
-        my_string += 'mi<tg<open-att__<section<num>%s' % str(self.__section_num)
-        my_string += '<num-in-level>%s' % str(self.__section_num)
+        my_string += 'mi<tg<open-att__<section<num>%s' % unicode_type(self.__section_num)
+        my_string += '<num-in-level>%s' % unicode_type(self.__section_num)
         my_string += '<type>rtf-native'
         my_string += '<level>0'
         keys = self.__section_values.keys()
@@ -289,7 +293,7 @@ class Sections:
             self.__handle_sec_def(my_string)
         elif self.__run_level > 3:
             msg = 'missed a flag\n'
-            raise self.__bug_handler, msg
+            raise self.__bug_handler(msg)
 
     def __handle_sec_def(self, my_string):
         """
@@ -355,7 +359,7 @@ class Sections:
                     '<num-in-level>%s'
                     '<type>rtf-native'
                     '<level>0\n'
-                    % (str(self.__section_num), str(self.__section_num))
+                    % (unicode_type(self.__section_num), unicode_type(self.__section_num))
                     )
             self.__found_first_sec = 1
         elif self.__token_info == 'tx<nu<__________':
@@ -366,7 +370,7 @@ class Sections:
                     '<num-in-level>%s'
                     '<type>rtf-native'
                     '<level>0\n'
-                    % (str(self.__section_num), str(self.__section_num))
+                    % (unicode_type(self.__section_num), unicode_type(self.__section_num))
                     )
             self.__write_obj.write(
                 'cw<pf<par-def___<true\n'
@@ -459,7 +463,7 @@ class Sections:
         self.__field_num = self.__field_num[1:]
         self.__write_obj.write(
         'mi<tg<close_____<section\n'
-        'mi<tg<open-att__<section<num>%s' % str(num)
+        'mi<tg<open-att__<section<num>%s' % unicode_type(num)
         )
         if self.__list_of_sec_values:
             keys =  self.__list_of_sec_values[0].keys()
@@ -469,7 +473,7 @@ class Sections:
             self.__list_of_sec_values = self.__list_of_sec_values[1:]
         self.__write_obj.write('<level>0')
         self.__write_obj.write('<type>rtf-native')
-        self.__write_obj.write('<num-in-level>%s' % str(self.__section_num))
+        self.__write_obj.write('<num-in-level>%s' % unicode_type(self.__section_num))
         self.__write_obj.write('\n')
         # Look here
 
@@ -513,8 +517,8 @@ class Sections:
             If the state is body, send the line to the body method.
         """
         self.__initiate_values()
-        read_obj = open(self.__file, 'r')
-        self.__write_obj = open(self.__write_to, 'w')
+        read_obj = open_for_read(self.__file)
+        self.__write_obj = open_for_write(self.__write_to)
         line_to_read = 1
         while line_to_read:
             line_to_read = read_obj.readline()
