@@ -1,4 +1,5 @@
-from functions import textile, textile_restricted, Textile
+
+from .functions import textile, textile_restricted, Textile
 
 if False:
     textile, textile_restricted, Textile
