@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 __license__ = 'GPL 3'
 __copyright__ = '2011, Leigh Parry <leighparry@blueyonder.co.uk>'
 __docformat__ = 'restructuredtext en'
@@ -34,7 +35,7 @@ class TextileMLizer(OEB2HTML):
         self.in_a_link = False
         self.our_ids = []
         self.images = {}
-        self.id_no_text = u''
+        self.id_no_text = ''
         self.style_embed = []
         self.remove_space_after_newline = False
         self.base_hrefs = [item.href for item in oeb_book.spine]
@@ -56,7 +57,7 @@ class TextileMLizer(OEB2HTML):
         return txt
 
     def mlize_spine(self, oeb_book):
-        output = [u'']
+        output = ['']
         for item in oeb_book.spine:
             self.log.debug('Converting %s to Textile formatted TXT...' % item.href)
             self.rewrite_ids(item.data, item)
@@ -199,7 +200,7 @@ class TextileMLizer(OEB2HTML):
 
     def check_id_tag(self, attribs):
         txt = ''
-        if attribs.has_key('id'):  # noqa
+        if 'id' in attribs:
             txt = '(#'+attribs['id']+ ')'
             self.our_ids.append('#'+attribs['id'])
             self.id_no_text = u'\xa0'
@@ -336,12 +337,12 @@ class TextileMLizer(OEB2HTML):
             tags.append('pre\n')
         elif tag == 'a':
             if self.opts.keep_links:
-                if attribs.has_key('href'):  # noqa
+                if 'href' in attribs:
                     text.append('"')
                     tags.append('a')
                     tags.append('":' + attribs['href'])
                     self.our_links.append(attribs['href'])
-                    if attribs.has_key('title'):  # noqa
+                    if 'title' in attribs:
                         tags.append('(' + attribs['title'] + ')')
                     self.in_a_link = True
                 else:
@@ -353,7 +354,7 @@ class TextileMLizer(OEB2HTML):
                 txt += self.check_valign(style)
                 txt += attribs['src']
                 text.append(txt)
-                if attribs.has_key('alt'):  # noqa
+                if 'alt' in attribs:
                     txt = attribs['alt']
                     if txt != '':
                         text.append('(' + txt + ')')
@@ -405,9 +406,9 @@ class TextileMLizer(OEB2HTML):
             txt = ''
             txt += self.check_halign(style)
             txt += self.check_valign(style)
-            if attribs.has_key('colspan'):  # noqa
+            if 'colspan' in attribs:
                 txt += '\\' + attribs['colspan']
-            if attribs.has_key('rowspan'):  # noqa
+            if 'rowspan' in attribs:
                 txt += '/' + attribs['rowspan']
             txt += self.check_styles(style)
             if txt != '':
@@ -432,7 +433,7 @@ class TextileMLizer(OEB2HTML):
                         text.append(txt)
                         tags.append('%')
 
-        if self.opts.keep_links and attribs.has_key('id'):  # noqa
+        if self.opts.keep_links and 'id' in attribs:
             if tag not in ('body', 'div', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'span', 'table'):
                 text.append(self.check_id_tag(attribs))
 
