@@ -124,9 +124,9 @@ class MovedDialog(QDialog):  # {{{
         self.cd.clicked.connect(self.choose_dir)
         l.addWidget(self.cd, l.rowCount() - 1, 1, 1, 1)
         self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Abort)
-        b = self.bb.addButton(_('Library moved'), self.bb.AcceptRole)
+        b = self.bb.addButton(_('Library moved'), QDialogButtonBox.ButtonRole.AcceptRole)
         b.setIcon(QIcon(I('ok.png')))
-        b = self.bb.addButton(_('Forget library'), self.bb.RejectRole)
+        b = self.bb.addButton(_('Forget library'), QDialogButtonBox.ButtonRole.RejectRole)
         b.setIcon(QIcon(I('edit-clear.png')))
         b.clicked.connect(self.forget_library)
         self.bb.accepted.connect(self.accept)
@@ -167,7 +167,7 @@ class BackupStatus(QDialog):  # {{{
         self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
-        b = bb.addButton(_('Queue &all books for backup'), bb.ActionRole)
+        b = bb.addButton(_('Queue &all books for backup'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.mark_all_dirty)
         b.setIcon(QIcon(I('lt.png')))
         l.addWidget(bb)
