@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -14,6 +13,7 @@ from calibre import force_unicode
 from calibre.gui2 import gprefs
 from calibre.gui2.actions import InterfaceAction
 
+
 class EmbedAction(InterfaceAction):
 
     name = 'Embed Metadata'
@@ -37,7 +37,7 @@ class EmbedAction(InterfaceAction):
     def drop_event(self, event, mime_data):
         mime = 'application/calibre+from_library'
         if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, str(mime_data.data(mime)).split()))
+            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
             QTimer.singleShot(1, self.do_drop)
             return True
         return False
@@ -80,7 +80,7 @@ class EmbedAction(InterfaceAction):
     def do_embed(self, book_ids, only_fmts=None):
         pd = QProgressDialog(_('Embedding updated metadata into book files...'), _('&Stop'), 0, len(book_ids), self.gui)
         pd.setWindowTitle(_('Embedding metadata...'))
-        pd.setWindowModality(Qt.WindowModal)
+        pd.setWindowModality(Qt.WindowModality.WindowModal)
         errors = []
         self.job_data = (0, tuple(book_ids), pd, only_fmts, errors)
         self.pd_timer.start()
@@ -97,7 +97,8 @@ class EmbedAction(InterfaceAction):
             self.job_data = None
             self.gui.library_view.model().refresh_ids(book_ids)
             if i > 0:
-                self.gui.status_bar.show_message(_('Embedded metadata in %d books') % i, 5000)
+                self.gui.status_bar.show_message(ngettext(
+                    'Embedded metadata in one book', 'Embedded metadata in {} books', i).format(i), 5000)
             if errors:
                 det_msg = '\n\n'.join([_('The {0} format of {1}:\n\n{2}\n').format(
                     (fmt or '').upper(), force_unicode(mi.title), force_unicode(tb)) for mi, fmt, tb in errors])
@@ -120,6 +121,7 @@ class EmbedAction(InterfaceAction):
         pd.setValue(i)
         db = self.gui.current_db.new_api
         book_id = book_ids[i]
+
         def report_error(mi, fmt, tb):
             mi.book_id = book_id
             errors.append((mi, fmt, tb))
