@@ -7,7 +7,7 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 from functools import partial
 
-from PyQt5.Qt import QTimer, QApplication, Qt
+from PyQt5.Qt import QTimer, QApplication, Qt, QEvent
 
 from calibre.gui2 import error_dialog
 from calibre.gui2.actions import InterfaceAction
@@ -79,7 +79,7 @@ class MarkBooksAction(InterfaceAction):
                 continue
 
     def eventFilter(self, obj, ev):
-        if ev.type() == ev.MouseButtonPress and ev.button() == Qt.MouseButton.LeftButton:
+        if ev.type() == QEvent.Type.MouseButtonPress and ev.button() == Qt.MouseButton.LeftButton:
             mods = QApplication.keyboardModifiers()
             if mods & Qt.KeyboardModifier.ControlModifier or mods & Qt.KeyboardModifier.ShiftModifier:
                 self.show_marked()
