@@ -1,6 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -8,12 +9,13 @@ __docformat__ = 'restructuredtext en'
 
 from calibre.gui2.actions import InterfaceAction
 
+
 class OpenFolderAction(InterfaceAction):
 
     name = 'Open Folder'
     action_spec = (_('Open containing folder'), 'document_open.png',
                    _('Open the folder containing the current book\'s files'), _('O'))
-    dont_add_to = frozenset(['context-menu-device'])
+    dont_add_to = frozenset(('context-menu-device',))
     action_type = 'current'
 
     def genesis(self):
@@ -22,5 +24,4 @@ class OpenFolderAction(InterfaceAction):
     def location_selected(self, loc):
         enabled = loc == 'library'
         self.qaction.setEnabled(enabled)
-
-
+        self.menuless_qaction.setEnabled(enabled)
