@@ -39,9 +39,9 @@ class ShowBookDetailsAction(InterfaceAction):
         if index.isValid():
             d = BookInfo(self.gui, self.gui.library_view, index,
                     self.gui.book_details.handle_click)
-            d.open_cover_with.connect(self.gui.bd_open_cover_with, type=Qt.QueuedConnection)
+            d.open_cover_with.connect(self.gui.bd_open_cover_with, type=Qt.ConnectionType.QueuedConnection)
             self.memory.append(d)
-            d.closed.connect(self.closed, type=Qt.QueuedConnection)
+            d.closed.connect(self.closed, type=Qt.ConnectionType.QueuedConnection)
             d.show()
 
     def closed(self, d):
