@@ -3,6 +3,7 @@
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
 
+from PyQt5.Qt import QDialog
 from polyglot.builtins import iteritems, map, range
 
 from calibre.gui2 import gprefs
@@ -42,7 +43,7 @@ class TagMapAction(InterfaceAction):
             'The changes will be applied to <b>one book in the library</b>',
             'The changes will be applied to <b>{} books in the library</b>', len(book_ids))
         d.edit_widget.msg_label.setText(d.edit_widget.msg_label.text() + '<p>' + txt.format(len(book_ids)))
-        if d.exec_() != d.Accepted:
+        if d.exec_() != QDialog.DialogCode.Accepted:
             return
         with BusyCursor():
             rules = d.rules
