@@ -1,17 +1,18 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from PyQt5.Qt import Qt, QAbstractListModel, QModelIndex
+from PyQt5.Qt import Qt, QAbstractListModel, QModelIndex, QItemSelectionModel
 
 from calibre.gui2.convert.page_setup_ui import Ui_Form
 from calibre.gui2.convert import Widget
 from calibre.customize.ui import input_profiles, output_profiles
 from calibre.ebooks.conversion.config import OPTIONS
+from polyglot.builtins import unicode_type
 
 
 class ProfileModel(QAbstractListModel):
@@ -25,9 +26,11 @@ class ProfileModel(QAbstractListModel):
 
     def data(self, index, role):
         profile = self.profiles[index.row()]
-        if role == Qt.DisplayRole:
-            return (profile.name)
-        if role in (Qt.ToolTipRole, Qt.StatusTipRole, Qt.WhatsThisRole):
+        if role == Qt.ItemDataRole.DisplayRole:
+            if profile.name.startswith('Default '):
+                return _('Default profile')
+            return profile.name
+        if role in (Qt.ItemDataRole.ToolTipRole, Qt.ItemDataRole.StatusTipRole, Qt.ItemDataRole.WhatsThisRole):
             w, h = profile.screen_size
             if w >= 10000:
                 ss = _('unlimited')
@@ -60,13 +63,13 @@ class PageSetupWidget(Widget, Ui_Form):
             x.setMouseTracking(True)
             x.entered[(QModelIndex)].connect(self.show_desc)
         self.initialize_options(get_option, get_help, db, book_id)
-        it = unicode(self.opt_input_profile.toolTip())
+        it = unicode_type(self.opt_input_profile.toolTip())
         self.opt_input_profile.setToolTip('<p>'+it.replace('t.','t.\n<br>'))
-        it = unicode(self.opt_output_profile.toolTip())
+        it = unicode_type(self.opt_output_profile.toolTip())
         self.opt_output_profile.setToolTip('<p>'+it.replace('t.','ce.\n<br>'))
 
     def show_desc(self, index):
-        desc = unicode(index.model().data(index, Qt.StatusTipRole) or '')
+        desc = unicode_type(index.model().data(index, Qt.ItemDataRole.StatusTipRole) or '')
         self.profile_description.setText(desc)
 
     def connect_gui_obj_handler(self, g, slot):
@@ -83,7 +86,7 @@ class PageSetupWidget(Widget, Ui_Form):
             idx = g.model().index(idx)
             sm = g.selectionModel()
             g.setCurrentIndex(idx)
-            sm.select(idx, sm.SelectCurrent)
+            sm.select(idx, QItemSelectionModel.SelectionFlag.SelectCurrent)
             return True
         return False
 
