@@ -33,16 +33,15 @@ class PluginWidget(Widget, Ui_Form):
         except TypeError:
             pass  # link already localized
 
-        for x in get_option('paper_size').option.choices:
-            self.opt_paper_size.addItem(x)
+        self.opt_paper_size.initialize(get_option('paper_size').option.choices)
         for x in get_option('unit').option.choices:
             self.opt_unit.addItem(x)
         for x in get_option('pdf_standard_font').option.choices:
             self.opt_pdf_standard_font.addItem(x)
 
         self.initialize_options(get_option, get_help, db, book_id)
-        self.layout().setFieldGrowthPolicy(self.layout().ExpandingFieldsGrow)
-        self.template_box.layout().setFieldGrowthPolicy(self.layout().AllNonFixedFieldsGrow)
+        self.layout().setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
+        self.template_box.layout().setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
         self.profile_size_toggled()
 
     def profile_size_toggled(self):
