@@ -8,7 +8,7 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 import time
 
 from PyQt5.Qt import (
-    QApplication, QMainWindow, QVBoxLayout, Qt, QKeySequence, QAction,
+    QApplication, QMainWindow, QVBoxLayout, Qt, QKeySequence, QAction, QEvent, QStyle,
     QActionGroup, QMenu, QPushButton, QWidget, QTimer, QMessageBox, pyqtSignal)
 
 from calibre.gui2.dbus_export.utils import setup_for_cli_run
@@ -41,19 +41,21 @@ class MainWindow(QMainWindow):
         m.aboutToShow.connect(self.about_to_show_one)
         s = self.style()
         self.q = q = QAction('&Quit', self)
-        q.setShortcut(QKeySequence.Quit), q.setIcon(s.standardIcon(s.SP_DialogCancelButton))
+        q.setShortcut(QKeySequence.StandardKey.Quit), q.setIcon(s.standardIcon(QStyle.StandardPixmap.SP_DialogCancelButton))
         q.triggered.connect(QApplication.quit)
         self.addAction(q)
-        QApplication.instance().setWindowIcon(s.standardIcon(s.SP_ComputerIcon))
-        for i, icon in zip(range(3), map(s.standardIcon, (s.SP_DialogOkButton, s.SP_DialogHelpButton, s.SP_ArrowUp))):
+        QApplication.instance().setWindowIcon(s.standardIcon(QStyle.StandardPixmap.SP_ComputerIcon))
+        for i, icon in zip(range(3), map(s.standardIcon, (
+                QStyle.StandardPixmap.SP_DialogOkButton, QStyle.StandardPixmap.SP_DialogHelpButton, QStyle.StandardPixmap.SP_ArrowUp))):
             ac = m.addAction('One - &%d' % (i + 1))
-            ac.setShortcut(QKeySequence(Qt.CTRL | (Qt.Key_1 + i), Qt.SHIFT | (Qt.Key_1 + i)))
+            ac.setShortcut(QKeySequence(Qt.Modifier.CTRL | (Qt.Key.Key_1 + i), Qt.Modifier.SHIFT | (Qt.Key.Key_1 + i)))
             ac.setIcon(icon)
         m.addSeparator()
         self.menu_two = m2 = m.addMenu('A &submenu')
-        for i, icon in zip(range(3), map(s.standardIcon, (s.SP_DialogOkButton, s.SP_DialogCancelButton, s.SP_ArrowUp))):
+        for i, icon in zip(range(3), map(s.standardIcon, (
+                QStyle.StandardPixmap.SP_DialogOkButton, QStyle.StandardPixmap.SP_DialogCancelButton, QStyle.StandardPixmap.SP_ArrowUp))):
             ac = m2.addAction('Two - &%d' % (i + 1))
-            ac.setShortcut(QKeySequence(Qt.CTRL | (Qt.Key_A + i)))
+            ac.setShortcut(QKeySequence(Qt.Modifier.CTRL | (Qt.Key.Key_A + i)))
             ac.setIcon(icon)
         m2.aboutToShow.connect(self.about_to_show_two)
         m2.addSeparator(), m.addSeparator()
@@ -111,9 +113,8 @@ class MainWindow(QMainWindow):
 
     def change_icon(self):
         import random
-        s = self.style()
-        num = s.SP_ComputerIcon
-        while num == s.SP_ComputerIcon:
+        num = QStyle.StandardPixmap.SP_ComputerIcon
+        while num == QStyle.StandardPixmap.SP_ComputerIcon:
             num = random.choice(range(20))
         self.systray.setIcon(self.style().standardIcon(num))
 
@@ -153,8 +154,8 @@ class MainWindow(QMainWindow):
         QMessageBox.information(self, 'A test dialog', 'While this dialog is shown, the global menu should be hidden')
 
     def event(self, ev):
-        if ev.type() in (ev.WindowBlocked, ev.WindowUnblocked):
-            if ev.type() == ev.WindowBlocked:
+        if ev.type() in (QEvent.Type.WindowBlocked, QEvent.Type.WindowUnblocked):
+            if ev.type() == QEvent.Type.WindowBlocked:
                 self.window_blocked.emit()
             else:
                 self.window_unblocked.emit()
@@ -162,7 +163,7 @@ class MainWindow(QMainWindow):
 
 
 app=QApplication([])
-app.setAttribute(Qt.AA_DontUseNativeMenuBar, False)
+app.setAttribute(Qt.ApplicationAttribute.AA_DontUseNativeMenuBar, False)
 app.setApplicationName('com.calibre-ebook.DBusExportDemo')
 mw=MainWindow()
 mw.show()
