@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,7 +9,9 @@ import sys, array, re, os, errno
 
 import dbus
 
-from PyQt5.Qt import QSize, QImage, Qt, QKeySequence, QBuffer, QByteArray
+from PyQt5.Qt import QSize, QImage, Qt, QKeySequence, QBuffer, QByteArray, QIODevice
+
+from polyglot.builtins import unicode_type, iteritems
 
 
 def log(*args, **kw):
@@ -18,6 +19,7 @@ def log(*args, **kw):
     print('DBusExport:', *args, **kw)
     kw['file'].flush()
 
+
 from calibre.ptempfile import PersistentTemporaryDirectory
 
 
@@ -58,6 +60,7 @@ class IconCache(object):
         # dir to decide whether it should look for new icons in the theme dir.
         os.utime(self.icon_theme_path, None)
 
+
 _icon_cache = None
 
 
@@ -78,7 +81,7 @@ def qicon_to_sni_image_list(qicon):
         for size in sizes:
             # Convert to DBUS struct of width, height, and image data in ARGB32
             # in network endianness
-            i = qicon.pixmap(size).toImage().convertToFormat(QImage.Format_ARGB32)
+            i = qicon.pixmap(size).toImage().convertToFormat(QImage.Format.Format_ARGB32)
             w, h = i.width(), i.height()
             data = i.constBits().asstring(4 * w * h)
             if socket.htonl(1) != 1:
@@ -103,13 +106,14 @@ def swap_mnemonic_char(text, from_char='&', to_char='_'):
 
 def key_sequence_to_dbus_shortcut(qks):
     for key in qks:
-        if key == -1 or key == Qt.Key_unknown:
+        if key == -1 or key == Qt.Key.Key_unknown:
             continue
         items = []
-        for mod, name in {Qt.META:'Super', Qt.CTRL:'Control', Qt.ALT:'Alt', Qt.SHIFT:'Shift'}.iteritems():
+        for mod, name in iteritems({Qt.Modifier.META:'Super', Qt.Modifier.CTRL:'Control', Qt.Modifier.ALT:'Alt', Qt.Modifier.SHIFT:'Shift'}):
             if key & mod == mod:
                 items.append(name)
-        key &= int(~(Qt.ShiftModifier | Qt.ControlModifier | Qt.AltModifier | Qt.MetaModifier | Qt.KeypadModifier))
+        key &= int(~(Qt.KeyboardModifier.ShiftModifier | Qt.KeyboardModifier.ControlModifier |
+                     Qt.KeyboardModifier.AltModifier | Qt.KeyboardModifier.MetaModifier | Qt.KeyboardModifier.KeypadModifier))
         text = QKeySequence(key).toString()
         if text:
             text = {'+':'plus', '-':'minus'}.get(text, text)
@@ -123,9 +127,9 @@ def icon_to_dbus_menu_icon(icon, size=32):
         return None
     ba = QByteArray()
     buf = QBuffer(ba)
-    buf.open(QBuffer.WriteOnly)
+    buf.open(QIODevice.OpenModeFlag.WriteOnly)
     icon.pixmap(32).save(buf, 'PNG')
-    return dbus.ByteArray(bytes((ba.data())))
+    return dbus.ByteArray(ba)
 
 
 def setup_for_cli_run():
@@ -142,10 +146,10 @@ def set_X_window_properties(win_id, **properties):
     conn = xcb.connect()
     atoms = {name:conn.core.InternAtom(False, len(name), name) for name in properties}
     utf8_string_atom = None
-    for name, val in properties.iteritems():
+    for name, val in iteritems(properties):
         atom = atoms[name].reply().atom
         type_atom = xcb.xproto.Atom.STRING
-        if isinstance(val, unicode):
+        if isinstance(val, unicode_type):
             if utf8_string_atom is None:
                 utf8_string_atom = conn.core.InternAtom(True, len(b'UTF8_STRING'), b'UTF8_STRING').reply().atom
             type_atom = utf8_string_atom
