@@ -135,9 +135,9 @@ class IgnoredFolders(QDialog):
                                    QDialogButtonBox.StandardButton.Cancel)
         self.bb.accepted.connect(self.accept)
         self.bb.rejected.connect(self.reject)
-        self.sab = self.bb.addButton(_('Select &all'), self.bb.ActionRole)
+        self.sab = self.bb.addButton(_('Select &all'), QDialogButtonBox.ButtonRole.ActionRole)
         self.sab.clicked.connect(self.select_all)
-        self.snb = self.bb.addButton(_('Select &none'), self.bb.ActionRole)
+        self.snb = self.bb.addButton(_('Select &none'), QDialogButtonBox.ButtonRole.ActionRole)
         self.snb.clicked.connect(self.select_none)
         l.addWidget(self.bb)
         self.setWindowTitle(_('Choose folders to scan'))
