@@ -1,4 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
+
+
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
 __license__   = 'GPL v3'
@@ -38,7 +40,7 @@ class AddEmptyBookDialog(QDialog):
 
         self.authors_combo = EditWithComplete(self)
         self.authors_combo.setSizeAdjustPolicy(
-                self.authors_combo.AdjustToMinimumContentsLengthWithIcon)
+                QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
         self.authors_combo.setEditable(True)
         self._layout.addWidget(self.authors_combo, 3, 0, 1, 1)
         self.initialize_authors(db, author)
@@ -54,7 +56,7 @@ class AddEmptyBookDialog(QDialog):
 
         self.series_combo = EditWithComplete(self)
         self.series_combo.setSizeAdjustPolicy(
-                self.authors_combo.AdjustToMinimumContentsLengthWithIcon)
+                QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
         self.series_combo.setEditable(True)
         self._layout.addWidget(self.series_combo, 5, 0, 1, 1)
         self.initialize_series(db, series)
@@ -103,12 +105,12 @@ class AddEmptyBookDialog(QDialog):
         cf.setChecked(gprefs.get('create_empty_copy_dup_formats', False))
         self._layout.addWidget(cf, 10, 0, 1, -1)
 
-        button_box = self.bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        button_box = self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         button_box.accepted.connect(self.accept)
         button_box.rejected.connect(self.reject)
         self._layout.addWidget(button_box, 11, 0, 1, -1)
         if dup_title:
-            self.dup_button = b = button_box.addButton(_('&Duplicate current book'), button_box.ActionRole)
+            self.dup_button = b = button_box.addButton(_('&Duplicate current book'), QDialogButtonBox.ButtonRole.ActionRole)
             b.clicked.connect(self.do_duplicate_book)
             b.setIcon(QIcon(I('edit-copy.png')))
             b.setToolTip(_(
