@@ -1,19 +1,20 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, sys, importlib
+import os, sys, importlib, weakref
 
-from PyQt5.Qt import QDialog, QCoreApplication, QSize, QScrollArea
+from PyQt5.Qt import QDialog, QCoreApplication, QSize, QScrollArea, QApplication, QDialogButtonBox
 
 from calibre.customize.ui import config
 from calibre.gui2.dialogs.catalog_ui import Ui_Dialog
 from calibre.gui2 import dynamic, info_dialog
 from calibre.customize.ui import catalog_plugins
+from polyglot.builtins import unicode_type
 
 
 class Catalog(QDialog, Ui_Dialog):
@@ -21,7 +22,7 @@ class Catalog(QDialog, Ui_Dialog):
     ''' Catalog Dialog builder'''
 
     def __init__(self, parent, dbspec, ids, db):
-        import re, cStringIO
+        import re
         from calibre import prints as info
         from PyQt5.uic import compileUi
 
@@ -30,7 +31,7 @@ class Catalog(QDialog, Ui_Dialog):
         self.dbspec, self.ids = dbspec, ids
 
         # Display the number of books we've been passed
-        self.count.setText(unicode(self.count.text()).format(len(ids)))
+        self.count.setText(unicode_type(self.count.text()).format(len(ids)))
 
         # Display the last-used title
         self.title.setText(dynamic.get('catalog_last_used_title',
@@ -47,6 +48,7 @@ class Catalog(QDialog, Ui_Dialog):
                 try:
                     catalog_widget = importlib.import_module('calibre.gui2.catalog.'+name)
                     pw = catalog_widget.PluginWidget()
+                    pw.parent_ref = weakref.ref(self)
                     pw.initialize(name, db)
                     pw.ICON = I('forward.png')
                     self.widgets.append(pw)
@@ -65,13 +67,14 @@ class Catalog(QDialog, Ui_Dialog):
 
                     # Compile the .ui form provided in plugin.zip
                     if not os.path.exists(compiled_form):
+                        from polyglot.io import PolyglotStringIO
                         # info('\tCompiling form', form)
-                        buf = cStringIO.StringIO()
+                        buf = PolyglotStringIO()
                         compileUi(form, buf)
                         dat = buf.getvalue()
                         dat = re.compile(r'QtGui.QApplication.translate\(.+?,\s+"(.+?)(?<!\\)",.+?\)',
                                          re.DOTALL).sub(r'_("\1")', dat)
-                        open(compiled_form, 'wb').write(dat)
+                        open(compiled_form, 'wb').write(dat.encode('utf-8'))
 
                     # Import the dynamic PluginWidget() from .py file provided in plugin.zip
                     try:
@@ -91,10 +94,10 @@ class Catalog(QDialog, Ui_Dialog):
                 else:
                     info("No dynamic tab resources found for %s" % name)
 
-        self.widgets = sorted(self.widgets, cmp=lambda x,y:cmp(x.TITLE, y.TITLE))
+        self.widgets = sorted(self.widgets, key=lambda x: x.TITLE)
 
         # Generate a sorted list of installed catalog formats/sync_enabled pairs
-        fmts = sorted([x[0] for x in self.fmts])
+        fmts = sorted((x[0] for x in self.fmts))
 
         self.sync_enabled_formats = []
         for fmt in self.fmts:
@@ -121,15 +124,19 @@ class Catalog(QDialog, Ui_Dialog):
         self.add_to_library.setChecked(dynamic.get('catalog_add_to_library', True))
 
         self.format.currentIndexChanged.connect(self.show_plugin_tab)
-        self.buttonBox.button(self.buttonBox.Apply).clicked.connect(self.apply)
-        self.buttonBox.button(self.buttonBox.Help).clicked.connect(self.help)
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).clicked.connect(self.apply)
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Help).clicked.connect(self.help)
         self.show_plugin_tab(None)
 
         geom = dynamic.get('catalog_window_geom', None)
         if geom is not None:
-            self.restoreGeometry(bytes(geom))
+            QApplication.instance().safe_restore_geometry(self, bytes(geom))
         else:
             self.resize(self.sizeHint())
+        d = QCoreApplication.instance().desktop()
+        g = d.availableGeometry(d.screenNumber(self))
+        self.setMaximumWidth(g.width() - 50)
+        self.setMaximumHeight(g.height() - 50)
 
     def sizeHint(self):
         desktop = QCoreApplication.instance().desktop()
@@ -145,7 +152,7 @@ class Catalog(QDialog, Ui_Dialog):
         return ans
 
     def show_plugin_tab(self, idx):
-        cf = unicode(self.format.currentText()).lower()
+        cf = unicode_type(self.format.currentText()).lower()
         while self.tabs.count() > 1:
             self.tabs.removeTab(1)
         for pw in self.widgets:
@@ -158,12 +165,12 @@ class Catalog(QDialog, Ui_Dialog):
                     self.tabs.addTab(s, pw.TITLE)
                 break
         if hasattr(self.options_widget, 'show_help'):
-            self.buttonBox.button(self.buttonBox.Help).setVisible(True)
+            self.buttonBox.button(QDialogButtonBox.StandardButton.Help).setVisible(True)
         else:
-            self.buttonBox.button(self.buttonBox.Help).setVisible(False)
+            self.buttonBox.button(QDialogButtonBox.StandardButton.Help).setVisible(False)
 
     def format_changed(self, idx):
-        cf = unicode(self.format.currentText())
+        cf = unicode_type(self.format.currentText())
         if cf in self.sync_enabled_formats:
             self.sync.setEnabled(True)
         else:
@@ -174,8 +181,8 @@ class Catalog(QDialog, Ui_Dialog):
         '''
         When title/format change, invalidate Preset in E-book options tab
         '''
-        cf = unicode(self.format.currentText()).lower()
-        if cf in ['azw3', 'epub', 'mobi'] and hasattr(self.options_widget, 'settings_changed'):
+        cf = unicode_type(self.format.currentText()).lower()
+        if cf in ('azw3', 'epub', 'mobi') and hasattr(self.options_widget, 'settings_changed'):
             self.options_widget.settings_changed("title/format")
 
     @property
@@ -187,9 +194,9 @@ class Catalog(QDialog, Ui_Dialog):
         return ans
 
     def save_catalog_settings(self):
-        self.catalog_format = unicode(self.format.currentText())
+        self.catalog_format = unicode_type(self.format.currentText())
         dynamic.set('catalog_preferred_format', self.catalog_format)
-        self.catalog_title = unicode(self.title.text())
+        self.catalog_title = unicode_type(self.title.text())
         dynamic.set('catalog_last_used_title', self.catalog_title)
         self.catalog_sync = bool(self.sync.isChecked())
         dynamic.set('catalog_sync_to_device', self.catalog_sync)
