@@ -1,4 +1,6 @@
-#!/usr/bin/env  python2
+#!/usr/bin/env python
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net' \
                 '2010, John Schember <john@nachtimwald.com>'
@@ -17,7 +19,7 @@ class Dialog(QDialog, Ui_Dialog):
         self.loc = None
         self.msg.setText(msg)
         self.name = name
-        self.buttonBox.setFocus(Qt.OtherFocusReason)
+        self.buttonBox.setFocus(Qt.FocusReason.OtherFocusReason)
         connect_lambda(self.button_lib.clicked, self, lambda self: self.set_loc('lib'))
         connect_lambda(self.button_device.clicked, self, lambda self: self.set_loc('dev'))
         connect_lambda(self.button_both.clicked, self, lambda self: self.set_loc('both'))
@@ -45,6 +47,6 @@ def confirm_location(msg, name, parent=None, pixmap='dialog_warning.png'):
     d.resize(d.sizeHint())
     ret = d.exec_()
     d.break_cycles()
-    if ret == d.Accepted:
+    if ret == QDialog.DialogCode.Accepted:
         return d.choice()
     return None
