@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -58,7 +57,7 @@ class SmartdeviceDialog(QDialog, Ui_Dialog):
         self.fixed_port.setToolTip('<p>' +
             _('Try 9090. If calibre says that it fails to connect '
               'to the port, try another number. You can use any number between '
-              '8,000 and 32,000.') + '</p>')
+              '8,000 and 65,535.') + '</p>')
 
         self.ip_addresses.setToolTip('<p>' +
             _('These are the IP addresses for this computer. If you decide to have your device connect to '
@@ -101,11 +100,11 @@ class SmartdeviceDialog(QDialog, Ui_Dialog):
         self.resize(self.sizeHint())
 
     def use_fixed_port_changed(self, state):
-        self.fixed_port.setEnabled(state == Qt.Checked)
+        self.fixed_port.setEnabled(state == Qt.CheckState.Checked)
 
     def toggle_password(self, state):
-        self.password_box.setEchoMode(QLineEdit.Password if state ==
-                Qt.Unchecked else QLineEdit.Normal)
+        self.password_box.setEchoMode(QLineEdit.EchoMode.Password if state ==
+                Qt.CheckState.Unchecked else QLineEdit.EchoMode.Normal)
 
     def accept(self):
         port = unicode_type(self.fixed_port.text())
@@ -117,12 +116,12 @@ class SmartdeviceDialog(QDialog, Ui_Dialog):
             port = int(port)
         except:
             error_dialog(self, _('Invalid port number'),
-                _('The port must be a number between 8000 and 32000.'), show=True)
+                _('The port must be a number between 8000 and 65535.'), show=True)
             return
 
-        if port < 8000 or port > 32000:
+        if port < 8000 or port > 65535:
             error_dialog(self, _('Invalid port number'),
-                _('The port must be a number between 8000 and 32000.'), show=True)
+                _('The port must be a number between 8000 and 65535.'), show=True)
             return
 
         self.device_manager.set_option('smartdevice', 'password',
