@@ -1,6 +1,8 @@
-from __future__ import print_function
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
+
 import sys, collections, operator, copy, re, numbers
 
 from PyQt5.Qt import (
@@ -11,7 +13,7 @@ from calibre.ebooks.lrf.fonts import LIBERATION_FONT_MAP
 from calibre.ebooks.hyphenate import hyphenate_word
 from polyglot.builtins import unicode_type, string_or_bytes
 
-WEIGHT_MAP = lambda wt : int((wt/10.)-1)
+WEIGHT_MAP = lambda wt : int((wt/10)-1)
 NULL       = lambda a, b: a
 COLOR      = lambda a, b: QColor(*a)
 WEIGHT     = lambda a, b: WEIGHT_MAP(a)
@@ -21,19 +23,19 @@ class PixmapItem(QGraphicsPixmapItem):
 
     def __init__(self, data, encoding, x0, y0, x1, y1, xsize, ysize):
         p = QPixmap()
-        p.loadFromData(data, encoding, Qt.AutoColor)
+        p.loadFromData(data, encoding, Qt.ImageConversionFlag.AutoColor)
         w, h = p.width(), p.height()
         p = p.copy(x0, y0, min(w, x1-x0), min(h, y1-y0))
         if p.width() != xsize or p.height() != ysize:
-            p = p.scaled(xsize, ysize, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+            p = p.scaled(xsize, ysize, Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
         QGraphicsPixmapItem.__init__(self, p)
         self.height, self.width = ysize, xsize
-        self.setTransformationMode(Qt.SmoothTransformation)
-        self.setShapeMode(QGraphicsPixmapItem.BoundingRectShape)
+        self.setTransformationMode(Qt.TransformationMode.SmoothTransformation)
+        self.setShapeMode(QGraphicsPixmapItem.ShapeMode.BoundingRectShape)
 
     def resize(self, width, height):
         p = self.pixmap()
-        self.setPixmap(p.scaled(width, height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation))
+        self.setPixmap(p.scaled(width, height, Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation))
         self.width, self.height = width, height
 
 
@@ -41,7 +43,7 @@ class Plot(PixmapItem):
 
     def __init__(self, plot, dpi):
         img = plot.refobj
-        xsize, ysize = dpi*plot.attrs['xsize']/720., dpi*plot.attrs['ysize']/720.
+        xsize, ysize = dpi*plot.attrs['xsize']/720, dpi*plot.attrs['ysize']/720
         x0, y0, x1, y1 = img.x0, img.y0, img.x1, img.y1
         data, encoding = img.data, img.encoding
         PixmapItem.__init__(self, data, encoding, x0, y0, x1, y1, xsize, ysize)
@@ -78,7 +80,7 @@ class FontLoader(object):
         if font in self.cache:
             rfont = self.cache[font]
         else:
-            italic = font[2] == QFont.StyleItalic
+            italic = font[2] == QFont.Style.StyleItalic
             rfont = QFont(font[0], font[3], font[1], italic)
             rfont.setPixelSize(font[3])
             rfont.setBold(wt>=69)
@@ -95,7 +97,7 @@ class Style(object):
     map = collections.defaultdict(lambda : NULL)
 
     def __init__(self, style, dpi):
-        self.fdpi = dpi/720.
+        self.fdpi = dpi/720
         self.update(style.as_dict())
 
     def update(self, *args, **kwds):
@@ -129,7 +131,7 @@ class TextStyle(Style):
 
     def __init__(self, style, font_loader, ruby_tags):
         self.font_loader = font_loader
-        self.fontstyle   = QFont.StyleNormal
+        self.fontstyle   = QFont.Style.StyleNormal
         for attr in ruby_tags:
             setattr(self, attr, ruby_tags[attr])
         Style.__init__(self, style, font_loader.dpi)
@@ -154,7 +156,7 @@ class ParSkip(object):
         self.height = parskip
 
     def __str__(self):
-        return 'Parskip: '+str(self.height)
+        return 'Parskip: '+unicode_type(self.height)
 
 
 class TextBlock(object):
@@ -258,7 +260,7 @@ class TextBlock(object):
                 self.create_link(i.attrs['refobj'])
             elif i.name == 'Italic':
                 open_containers.append((('current_style', self.current_style.copy()),))
-                self.current_style.update(fontstyle=QFont.StyleItalic)
+                self.current_style.update(fontstyle=QFont.Style.StyleItalic)
             elif i.name == 'Plot':
                 plot = Plot(i, self.font_loader.dpi)
                 if self.current_line is None:
@@ -289,7 +291,7 @@ class TextBlock(object):
                                                       self.current_style.linespace,
                                                       self.opts.visual_debug)
             if self.height > self.max_y+10:
-                raise TextBlock.HeightExceeded(str(self.current_line))
+                raise TextBlock.HeightExceeded(unicode_type(self.current_line))
             self.lines.append(self.current_line)
             self.current_line = None
 
@@ -307,7 +309,7 @@ class TextBlock(object):
 
     def process_text(self, raw):
         for ent, rep in TextBlock.XML_ENTITIES.items():
-            raw = raw.replace(u'&%s;'%ent, rep)
+            raw = raw.replace('&%s;'%ent, rep)
         while len(raw) > 0:
             if self.current_line is None:
                 self.create_line()
@@ -315,6 +317,8 @@ class TextBlock(object):
             raw = raw[pos:]
             if line_filled:
                 self.end_line()
+            if not pos:
+                break
 
     def __iter__(self):
         for line in self.lines:
@@ -323,7 +327,7 @@ class TextBlock(object):
     def __str__(self):
         s = ''
         for line in self:
-            s += str(line) + '\n'
+            s += unicode_type(line) + '\n'
         return s
 
 
@@ -336,8 +340,8 @@ class Link(QGraphicsRectItem):
         self.refobj = refobj
         self.slot = slot
         self.brush = self.__class__.inactive_brush
-        self.setPen(QPen(Qt.NoPen))
-        self.setCursor(Qt.PointingHandCursor)
+        self.setPen(QPen(Qt.PenStyle.NoPen))
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
         self.setAcceptHoverEvents(True)
 
     def hoverEnterEvent(self, event):
@@ -488,11 +492,11 @@ class Line(QGraphicsItem):
         x, y = 0, 0+self.height-self.descent
         if self.vdebug:
             painter.save()
-            painter.setPen(QPen(Qt.yellow, 1, Qt.DotLine))
+            painter.setPen(QPen(Qt.GlobalColor.yellow, 1, Qt.PenStyle.DotLine))
             painter.drawRect(self.boundingRect())
             painter.restore()
         painter.save()
-        painter.setPen(QPen(Qt.NoPen))
+        painter.setPen(QPen(Qt.PenStyle.NoPen))
         for c in self.children():
             painter.setBrush(c.brush)
             painter.drawRect(c.boundingRect())
@@ -505,8 +509,8 @@ class Line(QGraphicsItem):
                 painter.setFont(tok.font)
                 if tok.highlight:
                     painter.save()
-                    painter.setPen(QPen(Qt.NoPen))
-                    painter.setBrush(QBrush(Qt.yellow))
+                    painter.setPen(QPen(Qt.PenStyle.NoPen))
+                    painter.setBrush(QBrush(Qt.GlobalColor.yellow))
                     painter.drawRect(x, 0, tok.width, tok.height)
                     painter.restore()
                 painter.setPen(QPen(tok.text_color))
@@ -561,7 +565,7 @@ class Line(QGraphicsItem):
             return (textwidth-self.width)/2.
 
     def __unicode__(self):
-        s = u''
+        s = ''
         for tok in self.tokens:
             if isinstance(tok, numbers.Number):
                 s += ' '
