@@ -86,19 +86,19 @@ class ConfirmDialog(QDialog):
         self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Cancel)
         self.bb.rejected.connect(self.reject)
         b = self.bb.addButton(_('Download only &metadata'),
-                self.bb.AcceptRole)
+                QDialogButtonBox.ButtonRole.AcceptRole)
         b.clicked.connect(self.only_metadata)
         b.setIcon(QIcon(I('edit_input.png')))
         b = self.bb.addButton(_('Download only &covers'),
-                self.bb.AcceptRole)
+                QDialogButtonBox.ButtonRole.AcceptRole)
         b.clicked.connect(self.only_covers)
         b.setIcon(QIcon(I('default_cover.png')))
-        b = self.b = self.bb.addButton(_('&Configure download'), self.bb.ActionRole)
+        b = self.b = self.bb.addButton(_('&Configure download'), QDialogButtonBox.ButtonRole.ActionRole)
         b.setIcon(QIcon(I('config.png')))
         connect_lambda(b.clicked, self, lambda self: show_config(self))
         l.addWidget(self.bb, 1, 0, 1, 2)
         b = self.bb.addButton(_('Download &both'),
-                self.bb.AcceptRole)
+                QDialogButtonBox.ButtonRole.AcceptRole)
         b.clicked.connect(self.accept)
         b.setDefault(True)
         b.setAutoDefault(True)
