@@ -1,19 +1,21 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+import numbers
 import textwrap
-
-from PyQt5.Qt import (QWidget, QGridLayout, QGroupBox, QListView, Qt, QSpinBox,
-        QDoubleSpinBox, QCheckBox, QLineEdit, QComboBox, QLabel)
+from PyQt5.Qt import (
+    QCheckBox, QComboBox, QDoubleSpinBox, QGridLayout, QGroupBox, QLabel, QLineEdit,
+    QListView, QSpinBox, Qt, QVBoxLayout, QWidget
+)
 
 from calibre.gui2.preferences.metadata_sources import FieldsModel as FM
 from calibre.utils.icu import sort_key
+from polyglot.builtins import iteritems, unicode_type
 
 
 class FieldsModel(FM):  # {{{
@@ -36,8 +38,8 @@ class FieldsModel(FM):  # {{{
 
     def state(self, field, defaults=False):
         src = self.prefs.defaults if defaults else self.prefs
-        return (Qt.Unchecked if field in src['ignore_fields']
-                    else Qt.Checked)
+        return (Qt.CheckState.Unchecked if field in src['ignore_fields']
+                    else Qt.CheckState.Checked)
 
     def restore_defaults(self):
         self.beginResetModel()
@@ -45,51 +47,57 @@ class FieldsModel(FM):  # {{{
         self.endResetModel()
 
     def commit(self):
-        ignored_fields = set([x for x in self.prefs['ignore_fields'] if x not in
-            self.overrides])
-        changed = set([k for k, v in self.overrides.iteritems() if v ==
-            Qt.Unchecked])
+        ignored_fields = {x for x in self.prefs['ignore_fields'] if x not in
+            self.overrides}
+        changed = {k for k, v in iteritems(self.overrides) if v ==
+            Qt.CheckState.Unchecked}
         self.prefs['ignore_fields'] = list(ignored_fields.union(changed))
 
 # }}}
 
 
+class FieldsList(QListView):
+
+    def sizeHint(self):
+        return self.minimumSizeHint()
+
+
 class ConfigWidget(QWidget):
 
     def __init__(self, plugin):
         QWidget.__init__(self)
         self.plugin = plugin
 
-        self.l = l = QGridLayout()
-        self.setLayout(l)
-
-        self.gb = QGroupBox(_('Downloaded metadata fields'), self)
+        self.overl = l = QVBoxLayout(self)
+        self.gb = QGroupBox(_('Metadata fields to download'), self)
         if plugin.config_help_message:
             self.pchm = QLabel(plugin.config_help_message)
             self.pchm.setWordWrap(True)
             self.pchm.setOpenExternalLinks(True)
-            l.addWidget(self.pchm, 0, 0, 1, 2)
-        l.addWidget(self.gb, l.rowCount(), 0, 1, 2)
-        self.gb.l = QGridLayout()
-        self.gb.setLayout(self.gb.l)
-        self.fields_view = v = QListView(self)
-        self.gb.l.addWidget(v, 0, 0)
-        v.setFlow(v.LeftToRight)
+            l.addWidget(self.pchm, 10)
+        l.addWidget(self.gb)
+        self.gb.l = g = QVBoxLayout(self.gb)
+        g.setContentsMargins(0, 0, 0, 0)
+        self.fields_view = v = FieldsList(self)
+        g.addWidget(v)
+        v.setFlow(QListView.Flow.LeftToRight)
         v.setWrapping(True)
-        v.setResizeMode(v.Adjust)
+        v.setResizeMode(QListView.ResizeMode.Adjust)
         self.fields_model = FieldsModel(self.plugin)
         self.fields_model.initialize()
         v.setModel(self.fields_model)
-
         self.memory = []
         self.widgets = []
+        self.l = QGridLayout()
+        self.l.setContentsMargins(0, 0, 0, 0)
+        l.addLayout(self.l, 100)
         for opt in plugin.options:
             self.create_widgets(opt)
 
     def create_widgets(self, opt):
         val = self.plugin.prefs[opt.name]
         if opt.type == 'number':
-            c = QSpinBox if isinstance(opt.default, int) else QDoubleSpinBox
+            c = QSpinBox if isinstance(opt.default, numbers.Integral) else QDoubleSpinBox
             widget = c(self)
             widget.setValue(val)
         elif opt.type == 'string':
@@ -100,8 +108,8 @@ class ConfigWidget(QWidget):
             widget.setChecked(bool(val))
         elif opt.type == 'choices':
             widget = QComboBox(self)
-            items = list(opt.choices.iteritems())
-            items.sort(key=lambda (k, v): sort_key(v))
+            items = list(iteritems(opt.choices))
+            items.sort(key=lambda k_v: sort_key(k_v[1]))
             for key, label in items:
                 widget.addItem(label, (key))
             idx = widget.findData((val))
@@ -126,10 +134,10 @@ class ConfigWidget(QWidget):
             if isinstance(w, (QSpinBox, QDoubleSpinBox)):
                 val = w.value()
             elif isinstance(w, QLineEdit):
-                val = unicode(w.text())
+                val = unicode_type(w.text())
             elif isinstance(w, QCheckBox):
                 val = w.isChecked()
             elif isinstance(w, QComboBox):
                 idx = w.currentIndex()
-                val = unicode(w.itemData(idx) or '')
+                val = unicode_type(w.itemData(idx) or '')
             self.plugin.prefs[w.opt.name] = val
