@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -12,6 +12,7 @@ from PyQt5.Qt import QDialog, QVBoxLayout, QPlainTextEdit, QTimer, \
 
 from calibre.gui2 import error_dialog
 
+
 class DebugDevice(QDialog):
 
     def __init__(self, gui, parent=None):
@@ -31,8 +32,8 @@ class DebugDevice(QDialog):
         self.ok.setAutoDefault(False)
         self.ok.clicked.connect(self.accept)
         self.bbox = QDialogButtonBox(self)
-        self.bbox.addButton(self.copy, QDialogButtonBox.ActionRole)
-        self.bbox.addButton(self.ok, QDialogButtonBox.AcceptRole)
+        self.bbox.addButton(self.copy, QDialogButtonBox.ButtonRole.ActionRole)
+        self.bbox.addButton(self.ok, QDialogButtonBox.ButtonRole.AcceptRole)
         self._layout.addWidget(self.bbox)
         self.resize(750, 500)
         self.bbox.setEnabled(False)
@@ -51,7 +52,8 @@ class DebugDevice(QDialog):
         self.gui.debug_detection(self)
 
     def __call__(self, job):
-        if not self.isVisible(): return
+        if not self.isVisible():
+            return
         self.bbox.setEnabled(True)
         if job.failed:
             return error_dialog(self, _('Debugging failed'),
@@ -63,6 +65,7 @@ class DebugDevice(QDialog):
     def copy_to_clipboard(self):
         QApplication.clipboard().setText(self.log.toPlainText())
 
+
 if __name__ == '__main__':
     app = QApplication([])
     d = DebugDevice()
