@@ -1,16 +1,15 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import glob, os, string, shutil
+import glob, os, shutil
 from functools import partial
 from PyQt5.Qt import (
     QDialog, QVBoxLayout, QListWidget, QListWidgetItem, Qt, QIcon,
-    QApplication, QSize, QDialogButtonBox, QTimer, QLabel)
+    QApplication, QSize, QDialogButtonBox, QTimer, QLabel, QAbstractItemView, QListView)
 
 from calibre.constants import config_dir
 from calibre.gui2 import choose_files, error_dialog
@@ -45,12 +44,12 @@ class TextureChooser(QDialog):
         self.tdir = texture_dir()
 
         self.images = il = QListWidget(self)
-        il.itemDoubleClicked.connect(self.accept, type=Qt.QueuedConnection)
+        il.itemDoubleClicked.connect(self.accept, type=Qt.ConnectionType.QueuedConnection)
         il.setIconSize(QSize(256, 256))
-        il.setViewMode(il.IconMode)
-        il.setFlow(il.LeftToRight)
+        il.setViewMode(QListView.ViewMode.IconMode)
+        il.setFlow(QListView.Flow.LeftToRight)
         il.setSpacing(20)
-        il.setSelectionMode(il.SingleSelection)
+        il.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
         il.itemSelectionChanged.connect(self.update_remove_state)
         l.addWidget(il)
 
@@ -58,13 +57,13 @@ class TextureChooser(QDialog):
         ad.setOpenExternalLinks(True)
         ad.setWordWrap(True)
         l.addWidget(ad)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
-        b = self.add_button = bb.addButton(_('Add texture'), bb.ActionRole)
+        b = self.add_button = bb.addButton(_('Add texture'), QDialogButtonBox.ButtonRole.ActionRole)
         b.setIcon(QIcon(I('plus.png')))
         b.clicked.connect(self.add_texture)
-        b = self.remove_button = bb.addButton(_('Remove texture'), bb.ActionRole)
+        b = self.remove_button = bb.addButton(_('Remove texture'), QDialogButtonBox.ButtonRole.ActionRole)
         b.setIcon(QIcon(I('minus.png')))
         b.clicked.connect(self.remove_texture)
         l.addWidget(bb)
@@ -72,7 +71,7 @@ class TextureChooser(QDialog):
         images = [{
             'fname': ':'+os.path.basename(x),
             'path': x,
-            'name': ' '.join(map(string.capitalize, os.path.splitext(os.path.basename(x))[0].split('_')))
+            'name': ' '.join(map(lambda s: s.capitalize(), os.path.splitext(os.path.basename(x))[0].split('_')))
         } for x in glob.glob(I('textures/*.png'))] + [{
             'fname': os.path.basename(x),
             'path': x,
@@ -81,11 +80,12 @@ class TextureChooser(QDialog):
 
         images.sort(key=lambda x:sort_key(x['name']))
 
-        map(self.create_item, images)
+        for i in images:
+            self.create_item(i)
         self.update_remove_state()
 
         if initial:
-            existing = {unicode_type(i.data(Qt.UserRole) or ''):i for i in (self.images.item(c) for c in range(self.images.count()))}
+            existing = {unicode_type(i.data(Qt.ItemDataRole.UserRole) or ''):i for i in (self.images.item(c) for c in range(self.images.count()))}
             item = existing.get(initial, None)
             if item is not None:
                 item.setSelected(True)
@@ -96,8 +96,8 @@ class TextureChooser(QDialog):
     def create_item(self, data):
         x = data
         i = QListWidgetItem(QIcon(x['path']), x['name'], self.images)
-        i.setData(Qt.UserRole, x['fname'])
-        i.setData(Qt.UserRole+1, x['path'])
+        i.setData(Qt.ItemDataRole.UserRole, x['fname'])
+        i.setData(Qt.ItemDataRole.UserRole+1, x['path'])
         return i
 
     def update_remove_state(self):
@@ -116,7 +116,7 @@ class TextureChooser(QDialog):
         path = path[0]
         fname = os.path.basename(path)
         name = fname.rpartition('.')[0]
-        existing = {unicode_type(i.data(Qt.UserRole) or ''):i for i in (self.images.item(c) for c in range(self.images.count()))}
+        existing = {unicode_type(i.data(Qt.ItemDataRole.UserRole) or ''):i for i in (self.images.item(c) for c in range(self.images.count()))}
         dest = os.path.join(self.tdir, fname)
         with open(path, 'rb') as s, open(dest, 'wb') as f:
             shutil.copyfileobj(s, f)
@@ -135,7 +135,7 @@ class TextureChooser(QDialog):
     @property
     def selected_fname(self):
         try:
-            return unicode_type(self.selected_item.data(Qt.UserRole) or '')
+            return unicode_type(self.selected_item.data(Qt.ItemDataRole.UserRole) or '')
         except (AttributeError, TypeError):
             pass
 
@@ -145,7 +145,7 @@ class TextureChooser(QDialog):
         if self.selected_fname.startswith(':'):
             return error_dialog(self, _('Cannot remove'),
                                 _('Cannot remove builtin textures'), show=True)
-        os.remove(unicode_type(self.selected_item.data(Qt.UserRole+1) or ''))
+        os.remove(unicode_type(self.selected_item.data(Qt.ItemDataRole.UserRole+1) or ''))
         self.images.takeItem(self.images.row(self.selected_item))
 
 
