@@ -9,9 +9,9 @@ __docformat__ = 'restructuredtext en'
 from collections import namedtuple
 
 from PyQt5.Qt import (
-    QWidget, Qt, QLabel, QVBoxLayout, QDialogButtonBox, QApplication, QTimer, QPixmap,
+    QWidget, Qt, QLabel, QVBoxLayout, QDialogButtonBox, QApplication, QTimer, QPixmap, QEvent,
     QSize, pyqtSignal, QIcon, QPlainTextEdit, QCheckBox, QPainter, QHBoxLayout, QFontMetrics,
-    QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy, QImage)
+    QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy, QImage, QPalette)
 
 from calibre.constants import __version__
 from calibre.gui2.dialogs.message_box import ViewLog
@@ -45,12 +45,12 @@ class Icon(QWidget):
 
     def __init__(self, parent):
         QWidget.__init__(self, parent)
-        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         self.set_icon('dialog_question.png')
         self.default_icon = self.icon
         self._fraction = 0.0
         self.animation = a = QPropertyAnimation(self, b"fraction", self)
-        a.setDuration(2000), a.setEasingCurve(QEasingCurve.Linear)
+        a.setDuration(2000), a.setEasingCurve(QEasingCurve.Type.Linear)
         a.setStartValue(0.0), a.setEndValue(2.0), a.setLoopCount(10)
 
     def set_icon(self, icon):
@@ -101,7 +101,7 @@ class ProceedQuestion(QWidget):
 
         self._show_fraction = 0.0
         self.show_animation = a = QPropertyAnimation(self, b"show_fraction", self)
-        a.setDuration(1000), a.setEasingCurve(QEasingCurve.OutQuad)
+        a.setDuration(1000), a.setEasingCurve(QEasingCurve.Type.OutQuad)
         a.setStartValue(0.0), a.setEndValue(1.0)
         a.finished.connect(self.stop_show_animation)
         self.rendered_pixmap = None
@@ -114,24 +114,25 @@ class ProceedQuestion(QWidget):
         self.bb = QDialogButtonBox()
         self.bb.accepted.connect(self.accept)
         self.bb.rejected.connect(self.reject)
-        self.log_button = self.bb.addButton(_('View log'), self.bb.ActionRole)
+        self.log_button = self.bb.addButton(_('View log'), QDialogButtonBox.ButtonRole.ActionRole)
         self.log_button.setIcon(QIcon(I('debug.png')))
         self.log_button.clicked.connect(self.show_log)
         self.copy_button = self.bb.addButton(_('&Copy to clipboard'),
-                self.bb.ActionRole)
+                QDialogButtonBox.ButtonRole.ActionRole)
         self.copy_button.clicked.connect(self.copy_to_clipboard)
-        self.action_button = self.bb.addButton('', self.bb.ActionRole)
+        self.action_button = self.bb.addButton('', QDialogButtonBox.ButtonRole.ActionRole)
         self.action_button.clicked.connect(self.action_clicked)
         self.show_det_msg = _('Show &details')
         self.hide_det_msg = _('Hide &details')
-        self.det_msg_toggle = self.bb.addButton(self.show_det_msg, self.bb.ActionRole)
+        self.det_msg_toggle = self.bb.addButton(self.show_det_msg, QDialogButtonBox.ButtonRole.ActionRole)
         self.det_msg_toggle.clicked.connect(self.toggle_det_msg)
         self.det_msg_toggle.setToolTip(
                 _('Show detailed information about this error'))
         self.det_msg = PlainTextEdit(self)
         self.det_msg.setReadOnly(True)
-        self.bb.setStandardButtons(self.bb.Yes|self.bb.No|self.bb.Ok)
-        self.bb.button(self.bb.Yes).setDefault(True)
+        self.bb.setStandardButtons(
+            QDialogButtonBox.StandardButton.Yes | QDialogButtonBox.StandardButton.No | QDialogButtonBox.StandardButton.Ok)
+        self.bb.button(QDialogButtonBox.StandardButton.Yes).setDefault(True)
         self.title_label = title = QLabel('A dummy title')
         f = title.font()
         f.setBold(True)
@@ -150,16 +151,16 @@ class ProceedQuestion(QWidget):
         l.addWidget(self.bb)
 
         self.ask_question.connect(self.do_ask_question,
-                type=Qt.QueuedConnection)
-        self.setFocusPolicy(Qt.NoFocus)
+                type=Qt.ConnectionType.QueuedConnection)
+        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         for child in self.findChildren(QWidget):
-            child.setFocusPolicy(Qt.NoFocus)
+            child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         self.setFocusProxy(self.parent())
         self.resize_timer = t = QTimer(self)
         t.setSingleShot(True), t.setInterval(100), t.timeout.connect(self.parent_resized)
 
     def eventFilter(self, obj, ev):
-        if ev.type() == ev.Resize and self.isVisible():
+        if ev.type() == QEvent.Type.Resize and self.isVisible():
             self.resize_timer.start()
         return False
 
@@ -240,7 +241,7 @@ class ProceedQuestion(QWidget):
                     QIcon() if question.action_icon is None else question.action_icon)
             # Force the button box to relayout its buttons, as button text
             # might have changed
-            self.bb.setOrientation(Qt.Vertical), self.bb.setOrientation(Qt.Horizontal)
+            self.bb.setOrientation(Qt.Orientation.Vertical), self.bb.setOrientation(Qt.Orientation.Horizontal)
             self.det_msg.setPlainText(question.det_msg or '')
             self.det_msg.setVisible(False)
             self.det_msg_toggle.setVisible(bool(question.det_msg))
@@ -249,15 +250,15 @@ class ProceedQuestion(QWidget):
             if question.checkbox_msg is not None:
                 self.checkbox.setText(question.checkbox_msg)
                 self.checkbox.setChecked(question.checkbox_checked)
-            self.bb.button(self.bb.Ok).setVisible(question.show_ok)
-            self.bb.button(self.bb.Yes).setVisible(not question.show_ok)
-            self.bb.button(self.bb.No).setVisible(not question.show_ok)
+            self.bb.button(QDialogButtonBox.StandardButton.Ok).setVisible(question.show_ok)
+            self.bb.button(QDialogButtonBox.StandardButton.Yes).setVisible(not question.show_ok)
+            self.bb.button(QDialogButtonBox.StandardButton.No).setVisible(not question.show_ok)
             self.copy_button.setVisible(bool(question.show_copy_button))
             self.action_button.setVisible(question.action_callback is not None)
             self.toggle_det_msg() if question.show_det else self.do_resize()
             self.show_widget()
             button = self.action_button if question.focus_action and question.action_callback is not None else \
-                (self.bb.button(self.bb.Ok) if question.show_ok else self.bb.button(self.bb.Yes))
+                (self.bb.button(QDialogButtonBox.StandardButton.Ok) if question.show_ok else self.bb.button(QDialogButtonBox.StandardButton.Yes))
             button.setDefault(True)
             self.raise_()
             self.start_show_animation()
@@ -267,7 +268,7 @@ class ProceedQuestion(QWidget):
             return
 
         dpr = getattr(self, 'devicePixelRatioF', self.devicePixelRatio)()
-        p = QImage(dpr * self.size(), QImage.Format_ARGB32_Premultiplied)
+        p = QImage(dpr * self.size(), QImage.Format.Format_ARGB32_Premultiplied)
         p.setDevicePixelRatio(dpr)
         # For some reason, Qt scrolls the book view when rendering this widget,
         # for the very first time, so manually preserve its position
@@ -376,8 +377,8 @@ class ProceedQuestion(QWidget):
 
     def paintEvent(self, ev):
         painter = QPainter(self)
-        painter.setRenderHint(QPainter.Antialiasing, True)
-        painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
+        painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
+        painter.setRenderHint(QPainter.RenderHint.SmoothPixmapTransform, True)
         try:
             if self.rendered_pixmap is None:
                 self.paint_background(painter)
@@ -394,13 +395,13 @@ class ProceedQuestion(QWidget):
         br = 12  # border_radius
         bw = 1  # border_width
         pal = self.palette()
-        c = pal.color(pal.Window)
+        c = pal.color(QPalette.ColorRole.Window)
         c.setAlphaF(0.9)
         p = QPainterPath()
         p.addRoundedRect(QRectF(self.rect()), br, br)
         painter.fillPath(p, c)
         p.addRoundedRect(QRectF(self.rect()).adjusted(bw, bw, -bw, -bw), br, br)
-        painter.fillPath(p, pal.color(pal.WindowText))
+        painter.fillPath(p, pal.color(QPalette.ColorRole.WindowText))
 
 
 def main():
