@@ -80,7 +80,7 @@ class Saver(QObject):
         self._book_id_data = {}
         self.all_book_ids = frozenset(book_ids)
         self.pd = ProgressDialog(_('Saving %d books...') % len(self.all_book_ids), _('Collecting metadata...'), min=0, max=0, parent=parent, icon='save.png')
-        self.do_one_signal.connect(self.tick, type=Qt.QueuedConnection)
+        self.do_one_signal.connect(self.tick, type=Qt.ConnectionType.QueuedConnection)
         self.do_one = self.do_one_collect
         self.ids_to_collect = iter(self.all_book_ids)
         self.tdir = PersistentTemporaryDirectory('_save_to_disk')
@@ -153,7 +153,7 @@ class Saver(QObject):
             except Failure as err:
                 error_dialog(self.pd, _('Critical failure'), _(
                     'Could not save books to disk, click "Show details" for more information'),
-                    det_msg=unicode_type(err.failure_message) + '\n' + unicode_type(err.details), show=True)
+                    det_msg=force_unicode(err.failure_message) + '\n' + force_unicode(err.details), show=True)
                 self.pd.canceled = True
         self.do_one_signal.emit()
 
