@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,28 +9,29 @@ __docformat__ = 'restructuredtext en'
 from functools import partial
 
 from PyQt5.Qt import (
-    QAbstractListModel, Qt, QKeySequence, QListView, QVBoxLayout, QLabel,
-    QHBoxLayout, QWidget, QApplication, QStyledItemDelegate, QStyle, QIcon,
-    QTextDocument, QRectF, QFrame, QSize, QFont, QKeyEvent, QRadioButton, QPushButton, QToolButton
+    QAbstractListModel, Qt, QKeySequence, QListView, QVBoxLayout, QLabel, QAbstractItemView,
+    QHBoxLayout, QWidget, QApplication, QStyledItemDelegate, QStyle, QIcon, QAbstractItemDelegate,
+    QTextDocument, QRectF, QFrame, QSize, QFont, QKeyEvent, QRadioButton, QPushButton, QToolButton, QEvent
 )
 
 from calibre.gui2 import error_dialog
 from calibre.utils.config import XMLConfig
 from calibre.utils.icu import sort_key
+from polyglot.builtins import unicode_type
 
-DEFAULTS = Qt.UserRole
-DESCRIPTION = Qt.UserRole + 1
-CUSTOM = Qt.UserRole + 2
-KEY = Qt.UserRole + 3
+DEFAULTS = Qt.ItemDataRole.UserRole
+DESCRIPTION = Qt.ItemDataRole.UserRole + 1
+CUSTOM = Qt.ItemDataRole.UserRole + 2
+KEY = Qt.ItemDataRole.UserRole + 3
 
 
 class Customize(QFrame):
 
     def __init__(self, index, dup_check, parent=None):
         QFrame.__init__(self, parent)
-        self.setFrameShape(self.StyledPanel)
-        self.setFrameShadow(self.Raised)
-        self.setFocusPolicy(Qt.StrongFocus)
+        self.setFrameShape(QFrame.Shape.StyledPanel)
+        self.setFrameShadow(QFrame.Shadow.Raised)
+        self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
         self.setAutoFillBackground(True)
         self.l = l = QVBoxLayout(self)
         self.header = la = QLabel(self)
@@ -71,10 +72,10 @@ class Customize(QFrame):
         if self.capture == 0 or obj not in (self.button1, self.button2):
             return QFrame.eventFilter(self, obj, event)
         t = event.type()
-        if t == event.ShortcutOverride:
+        if t == QEvent.Type.ShortcutOverride:
             event.accept()
             return True
-        if t == event.KeyPress:
+        if t == QEvent.Type.KeyPress:
             self.key_press_event(event, 1 if obj is self.button1 else 2)
             return True
         return QFrame.eventFilter(self, obj, event)
@@ -82,7 +83,7 @@ class Customize(QFrame):
     def clear_button(self, which):
         b = getattr(self, 'button%d' % which)
         s = getattr(self, 'shortcut%d' % which, None)
-        b.setText(_('None') if s is None else s.toString(QKeySequence.NativeText))
+        b.setText(_('None') if s is None else s.toString(QKeySequence.SequenceFormat.NativeText))
         b.setFont(QFont())
 
     def clear_clicked(self, which=0):
@@ -100,25 +101,25 @@ class Customize(QFrame):
             self.clear_button(w)
         button = getattr(self, 'button%d'%which)
         button.setText(_('Press a key...'))
-        button.setFocus(Qt.OtherFocusReason)
+        button.setFocus(Qt.FocusReason.OtherFocusReason)
         font = QFont()
         font.setBold(True)
         button.setFont(font)
 
     def key_press_event(self, ev, which=0):
         code = ev.key()
-        if self.capture == 0 or code in (0, Qt.Key_unknown,
-                Qt.Key_Shift, Qt.Key_Control, Qt.Key_Alt, Qt.Key_Meta,
-                Qt.Key_AltGr, Qt.Key_CapsLock, Qt.Key_NumLock, Qt.Key_ScrollLock):
+        if self.capture == 0 or code in (0, Qt.Key.Key_unknown,
+                Qt.Key.Key_Shift, Qt.Key.Key_Control, Qt.Key.Key_Alt, Qt.Key.Key_Meta,
+                Qt.Key.Key_AltGr, Qt.Key.Key_CapsLock, Qt.Key.Key_NumLock, Qt.Key.Key_ScrollLock):
             return QWidget.keyPressEvent(self, ev)
-        sequence = QKeySequence(code|(int(ev.modifiers())&~Qt.KeypadModifier))
+        sequence = QKeySequence(code|(int(ev.modifiers()) & (~Qt.KeyboardModifier.KeypadModifier)))
         setattr(self, 'shortcut%d'%which, sequence)
         self.clear_button(which)
         self.capture = 0
         dup_desc = self.dup_check(sequence, self.key)
         if dup_desc is not None:
             error_dialog(self, _('Already assigned'),
-                    unicode(sequence.toString(QKeySequence.NativeText)) + ' ' +
+                    unicode_type(sequence.toString(QKeySequence.SequenceFormat.NativeText)) + ' ' +
                     _('already assigned to') + ' ' + dup_desc, show=True)
             self.clear_clicked(which=which)
 
@@ -154,8 +155,8 @@ class Delegate(QStyledItemDelegate):
         painter.save()
         painter.setClipRect(QRectF(option.rect))
         if hasattr(QStyle, 'CE_ItemViewItem'):
-            QApplication.style().drawControl(QStyle.CE_ItemViewItem, option, painter)
-        elif option.state & QStyle.State_Selected:
+            QApplication.style().drawControl(QStyle.ControlElement.CE_ItemViewItem, option, painter)
+        elif option.state & QStyle.StateFlag.State_Selected:
             painter.fillRect(option.rect, option.palette.highlight())
         painter.translate(option.rect.topLeft())
         self.to_doc(index).drawContents(painter)
@@ -169,12 +170,12 @@ class Delegate(QStyledItemDelegate):
 
     def setEditorData(self, editor, index):
         defs = index.data(DEFAULTS)
-        defs = _(' or ').join([unicode(x.toString(x.NativeText)) for x in defs])
-        editor.key = unicode(index.data(KEY))
+        defs = _(' or ').join([unicode_type(x.toString(QKeySequence.SequenceFormat.NativeText)) for x in defs])
+        editor.key = unicode_type(index.data(KEY))
         editor.default_shortcuts.setText(_('&Default') + ': %s' % defs)
         editor.default_shortcuts.setChecked(True)
         editor.header.setText('<b>%s: %s</b>'%(_('Customize shortcuts for'),
-            unicode(index.data(DESCRIPTION))))
+            unicode_type(index.data(DESCRIPTION))))
         custom = index.data(CUSTOM)
         if custom:
             editor.custom.setChecked(True)
@@ -182,11 +183,11 @@ class Delegate(QStyledItemDelegate):
                 button = getattr(editor, 'button%d'%(x+1))
                 if len(custom) > x:
                     seq = QKeySequence(custom[x])
-                    button.setText(seq.toString(seq.NativeText))
+                    button.setText(seq.toString(QKeySequence.SequenceFormat.NativeText))
                     setattr(editor, 'shortcut%d'%(x+1), seq)
 
     def setModelData(self, editor, model, index):
-        self.closeEditor.emit(editor, self.NoHint)
+        self.closeEditor.emit(editor, QAbstractItemDelegate.EndEditHint.NoHint)
         custom = []
         if editor.custom.isChecked():
             for x in ('1', '2'):
@@ -202,7 +203,7 @@ class Delegate(QStyledItemDelegate):
 
 class Shortcuts(QAbstractListModel):
 
-    TEMPLATE = u'''
+    TEMPLATE = '''
     <p><b>{0}</b><br>
     {2}: <code>{1}</code></p>
     '''
@@ -236,11 +237,11 @@ class Shortcuts(QAbstractListModel):
     def get_match(self, event_or_sequence, ignore=tuple()):
         q = event_or_sequence
         if isinstance(q, QKeyEvent):
-            q = QKeySequence(q.key()|(int(q.modifiers())&~Qt.KeypadModifier))
+            q = QKeySequence(q.key()|(int(q.modifiers()) & (~Qt.KeyboardModifier.KeypadModifier)))
         for key in self.order:
             if key not in ignore:
                 for seq in self.get_sequences(key):
-                    if seq.matches(q) == QKeySequence.ExactMatch:
+                    if seq.matches(q) == QKeySequence.SequenceMatch.ExactMatch:
                         return key
         return None
 
@@ -250,7 +251,7 @@ class Shortcuts(QAbstractListModel):
             return self.descriptions[key]
 
     def get_shortcuts(self, key):
-        return [unicode(x.toString(x.NativeText)) for x in
+        return [unicode_type(x.toString(QKeySequence.SequenceFormat.NativeText)) for x in
                 self.get_sequences(key)]
 
     def data(self, index, role):
@@ -258,10 +259,10 @@ class Shortcuts(QAbstractListModel):
         if row < 0 or row >= len(self.order):
             return None
         key = self.order[row]
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return self.TEMPLATE.format(self.descriptions[key],
                     _(' or ').join(self.get_shortcuts(key)), _('Keys'))
-        if role == Qt.ToolTipRole:
+        if role == Qt.ItemDataRole.ToolTipRole:
             return _('Double click to change')
         if role == DEFAULTS:
             return self.sequences[key]
@@ -279,14 +280,14 @@ class Shortcuts(QAbstractListModel):
     def set_data(self, index, custom):
         key = self.order[index.row()]
         if custom:
-            self.custom[key] = [unicode(x.toString(QKeySequence.PortableText)) for x in custom]
+            self.custom[key] = [unicode_type(x.toString(QKeySequence.SequenceFormat.PortableText)) for x in custom]
         elif key in self.custom:
             del self.custom[key]
 
     def flags(self, index):
         if not index.isValid():
-            return Qt.ItemIsEnabled
-        return QAbstractListModel.flags(self, index) | Qt.ItemIsEditable
+            return Qt.ItemFlag.ItemIsEnabled
+        return QAbstractListModel.flags(self, index) | Qt.ItemFlag.ItemIsEditable
 
 
 class ShortcutConfig(QWidget):
@@ -301,14 +302,14 @@ class ShortcutConfig(QWidget):
         self.delegate = Delegate()
         self.view.setItemDelegate(self.delegate)
         self.delegate.sizeHintChanged.connect(self.scrollTo,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
 
     def scrollTo(self, index):
-        self.view.scrollTo(index, self.view.EnsureVisible)
+        self.view.scrollTo(index, QAbstractItemView.ScrollHint.EnsureVisible)
 
     @property
     def is_editing(self):
-        return self.view.state() == self.view.EditingState
+        return self.view.state() == QAbstractItemView.State.EditingState
 
 
 if __name__ == '__main__':
