@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 
 __license__ = 'GPL 3'
@@ -68,7 +67,7 @@ class Matches(QAbstractItemModel):
         self.got_result_details_dispatcher = FunctionDispatcher(self.got_result_details)
 
         self.sort_col = 2
-        self.sort_order = Qt.AscendingOrder
+        self.sort_order = Qt.SortOrder.AscendingOrder
 
     def closing(self):
         self.cover_pool.abort()
@@ -184,10 +183,10 @@ class Matches(QAbstractItemModel):
         return len(self.HEADERS)
 
     def headerData(self, section, orientation, role):
-        if role != Qt.DisplayRole:
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
         text = ''
-        if orientation == Qt.Horizontal:
+        if orientation == Qt.Orientation.Horizontal:
             if section < len(self.HEADERS):
                 text = self.HEADERS[section]
             return (text)
@@ -199,7 +198,7 @@ class Matches(QAbstractItemModel):
         if row >= len(self.matches):
             return None
         result = self.matches[row]
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             if col == 1:
                 t = result.title if result.title else _('Unknown')
                 a = result.author if result.author else ''
@@ -209,7 +208,7 @@ class Matches(QAbstractItemModel):
             elif col == 4:
                 return ('<span>%s<br>%s</span>' % (result.store_name, result.formats))
             return None
-        elif role == Qt.DecorationRole:
+        elif role == Qt.ItemDataRole.DecorationRole:
             if col == 0 and result.cover_data:
                 p = QPixmap()
                 p.loadFromData(result.cover_data)
@@ -228,7 +227,7 @@ class Matches(QAbstractItemModel):
             if col == 6:
                 if result.affiliate:
                     return (self.DONATE_ICON)
-        elif role == Qt.ToolTipRole:
+        elif role == Qt.ItemDataRole.ToolTipRole:
             if col == 1:
                 return ('<p>%s</p>' % result.title)
             elif col == 2:
@@ -254,7 +253,7 @@ class Matches(QAbstractItemModel):
             elif col == 6:
                 if result.affiliate:
                     return ('<p>' + _('Buying from this store supports the calibre developer: %s.') % result.plugin_author + '</p>')
-        elif role == Qt.SizeHintRole:
+        elif role == Qt.ItemDataRole.SizeHintRole:
             return QSize(64, 64)
         return None
 
@@ -290,7 +289,7 @@ class Matches(QAbstractItemModel):
         self.sort_order = order
         if not self.matches:
             return
-        descending = order == Qt.DescendingOrder
+        descending = order == Qt.SortOrder.DescendingOrder
         self.all_matches.sort(
             key=lambda x: sort_key(unicode_type(self.data_as_text(x, col))),
             reverse=descending)
@@ -342,7 +341,7 @@ class SearchFilter(SearchQueryParser):
         self.srs.add(search_result)
 
     def clear_search_results(self):
-        self.srs = set([])
+        self.srs = set()
 
     def universal_set(self):
         return self.srs
@@ -391,8 +390,8 @@ class SearchFilter(SearchQueryParser):
             query = query.lower()
 
         if location not in self.USABLE_LOCATIONS:
-            return set([])
-        matches = set([])
+            return set()
+        matches = set()
         all_locs = set(self.USABLE_LOCATIONS) - {'all'}
         locations = all_locs if location == 'all' else [location]
         q = {
