@@ -9,7 +9,7 @@ import re
 from random import shuffle
 
 from PyQt5.Qt import (Qt, QDialog, QDialogButtonBox, QTimer, QCheckBox, QLabel,
-                      QVBoxLayout, QIcon, QWidget, QTabWidget, QGridLayout, QApplication)
+                      QVBoxLayout, QIcon, QWidget, QTabWidget, QGridLayout, QApplication, QStyle)
 
 from calibre.gui2 import JSONConfig, info_dialog, error_dialog
 from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
@@ -34,7 +34,7 @@ class SearchDialog(QDialog, Ui_Dialog):
         QDialog.__init__(self, parent)
         self.setupUi(self)
         s = self.style()
-        self.close.setIcon(s.standardIcon(s.SP_DialogCloseButton))
+        self.close.setIcon(s.standardIcon(QStyle.StandardPixmap.SP_DialogCloseButton))
 
         self.config = JSONConfig('store/search')
         self.search_title.initialize('store_search_search_title')
