@@ -1,15 +1,19 @@
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from __future__ import (unicode_literals, division, absolute_import, print_function)
-store_version = 4  # Needed for dynamic plugin loading
+store_version = 6  # Needed for dynamic plugin loading
 
 __license__ = 'GPL 3'
-__copyright__ = '2011-2016, Tomasz Długosz <tomek3d@gmail.com>'
+__copyright__ = '2011-2020, Tomasz Długosz <tomek3d@gmail.com>'
 __docformat__ = 'restructuredtext en'
 
 import re
-import urllib
 from contextlib import closing
+from base64 import standard_b64encode
+try:
+    from urllib.parse import quote_plus
+except ImportError:
+    from urllib import quote_plus
 
 from lxml import html
 
@@ -23,30 +27,41 @@ from calibre.gui2.store.search_result import SearchResult
 from calibre.gui2.store.web_store_dialog import WebStoreDialog
 
 
+def as_base64(data):
+    if not isinstance(data, bytes):
+        data = data.encode('utf-8')
+    ans = standard_b64encode(data)
+    if isinstance(ans, bytes):
+        ans = ans.decode('ascii')
+    return ans
+
+
 class NextoStore(BasicStoreConfig, StorePlugin):
 
     def open(self, parent=None, detail_item=None, external=False):
-        pid = '155711'
+        aff_root = 'https://www.a4b-tracking.com/pl/stat-click-text-link/35/58/'
+
+        url = 'http://www.nexto.pl/'
+
+        aff_url = aff_root + as_base64(url)
 
-        url = 'http://www.nexto.pl/ebooki_c1015.xml'
         detail_url = None
-
         if detail_item:
             book_id = re.search(r'p[0-9]*\.xml\Z', detail_item)
             book_id = book_id.group(0).replace('.xml','').replace('p','')
             if book_id:
-                detail_url = 'http://www.nexto.pl/rf/pr?p=' + book_id + '&pid=' + pid
+                detail_url = aff_root + as_base64('http://www.nexto.pl/rf/pr?p=' + book_id)
 
         if external or self.config.get('open_external', False):
-            open_url(QUrl(url_slash_cleaner(detail_url if detail_url else url)))
+            open_url(QUrl(url_slash_cleaner(detail_url if detail_url else aff_url)))
         else:
-            d = WebStoreDialog(self.gui, url, parent, detail_url)
+            d = WebStoreDialog(self.gui, url, parent, detail_url if detail_url else aff_url)
             d.setWindowTitle(self.name)
             d.set_tags(self.config.get('tags', ''))
             d.exec_()
 
     def search(self, query, max_results=10, timeout=60):
-        url = 'http://www.nexto.pl/szukaj.xml?search-clause=' + urllib.quote_plus(query) + '&scid=1015'
+        url = 'http://www.nexto.pl/szukaj.xml?search-clause=' + quote_plus(query) + '&scid=1015'
 
         br = browser()
         offset=0
@@ -54,7 +69,7 @@ class NextoStore(BasicStoreConfig, StorePlugin):
         counter = max_results
 
         while counter:
-            with closing(br.open(url + '&_offset=' + str(offset), timeout=timeout)) as f:
+            with closing(br.open(url + '&_offset={}'.format(offset), timeout=timeout)) as f:
                 doc = html.fromstring(f.read())
                 for data in doc.xpath('//ul[@class="productslist"]/li'):
                     if counter <= 0:
@@ -73,7 +88,7 @@ class NextoStore(BasicStoreConfig, StorePlugin):
                     title = re.sub(r' – ebook', '', title)
                     author = ', '.join(data.xpath('.//div[@class="col-7"]//h4//a/text()'))
                     formats = ', '.join(data.xpath('.//ul[@class="formats"]/li//b/text()'))
-                    DrmFree = re.search(r'znak', str(data.xpath('.//ul[@class="formats"]/li//b/@title')))
+                    DrmFree = data.xpath('.//ul[@class="formats"]/li//b[contains(@title, "znak")]')
 
                     counter -= 1
 
