@@ -11,7 +11,7 @@ from functools import partial
 from PyQt5.Qt import (
     QGridLayout, QToolButton, QIcon, QRadioButton, QMenu, QApplication, Qt,
     QSize, QWidget, QLabel, QStackedLayout, QPainter, QRect, QVBoxLayout,
-    QCursor, QEventLoop, QKeySequence, pyqtSignal, QTimer, QHBoxLayout)
+    QCursor, QEventLoop, QKeySequence, pyqtSignal, QTimer, QHBoxLayout, QDialogButtonBox)
 
 from calibre.ebooks.oeb.polish.container import Container
 from calibre.ebooks.oeb.polish.utils import guess_type
@@ -118,6 +118,8 @@ def get_decoded_raw(name):
                     enc = 'utf-8'
             else:
                 enc = force_encoding(raw, verbose=True)
+            if isinstance(enc, bytes):
+                enc = enc.decode('utf-8', 'ignore')
             try:
                 raw = raw.decode(enc)
             except (LookupError, ValueError):
@@ -297,13 +299,13 @@ class Diff(Dialog):
         self.names = QLabel('')
         self.hl.addWidget(self.names, r)
 
-        self.bb.setStandardButtons(self.bb.Close)
+        self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Close)
         if self.revert_button_msg is not None:
-            self.rvb = b = self.bb.addButton(self.revert_button_msg, self.bb.ActionRole)
+            self.rvb = b = self.bb.addButton(self.revert_button_msg, QDialogButtonBox.ButtonRole.ActionRole)
             b.setIcon(QIcon(I('edit-undo.png'))), b.setAutoDefault(False)
             b.clicked.connect(self.revert_requested)
             b.clicked.connect(self.reject)
-        self.bb.button(self.bb.Close).setDefault(True)
+        self.bb.button(QDialogButtonBox.StandardButton.Close).setDefault(True)
         self.hl.addWidget(self.bb, r)
 
         self.view.setFocus(Qt.FocusReason.OtherFocusReason)
