@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -15,6 +14,7 @@ from PyQt5.Qt import (
 from calibre.gui2 import error_dialog
 from calibre.gui2.tweak_book import actions, tprefs, editors
 from calibre.gui2.tweak_book.editor.canvas import Canvas
+from polyglot.builtins import itervalues
 
 
 class ResizeDialog(QDialog):  # {{{
@@ -47,7 +47,7 @@ class ResizeDialog(QDialog):  # {{{
         l.addRow(ar)
         self.resize(self.sizeHint())
 
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
         l.addRow(bb)
@@ -61,23 +61,21 @@ class ResizeDialog(QDialog):  # {{{
             other.setValue(oval)
             other.blockSignals(False)
 
-    @dynamic_property
+    @property
     def width(self):
-        def fget(self):
-            return self._width.value()
+        return self._width.value()
 
-        def fset(self, val):
-            self._width.setValue(val)
-        return property(fget=fget, fset=fset)
+    @width.setter
+    def width(self, val):
+        self._width.setValue(val)
 
-    @dynamic_property
+    @property
     def height(self):
-        def fget(self):
-            return self._height.value()
+        return self._height.value()
 
-        def fset(self, val):
-            self._height.setValue(val)
-        return property(fget=fget, fset=fset)
+    @height.setter
+    def height(self, val):
+        self._height.setValue(val)
 # }}}
 
 
@@ -94,7 +92,7 @@ class Editor(QMainWindow):
     def __init__(self, syntax, parent=None):
         QMainWindow.__init__(self, parent)
         if parent is None:
-            self.setWindowFlags(Qt.Widget)
+            self.setWindowFlags(Qt.WindowType.Widget)
 
         self.is_synced_to_container = False
         self.syntax = syntax
@@ -110,24 +108,22 @@ class Editor(QMainWindow):
         self.canvas.undo_redo_state_changed.connect(self.undo_redo_state_changed)
         self.canvas.selection_state_changed.connect(self.update_clipboard_actions)
 
-    @dynamic_property
+    @property
     def is_modified(self):
-        def fget(self):
-            return self._is_modified
+        return self._is_modified
 
-        def fset(self, val):
-            self._is_modified = val
-            self.modification_state_changed.emit(val)
-        return property(fget=fget, fset=fset)
+    @is_modified.setter
+    def is_modified(self, val):
+        self._is_modified = val
+        self.modification_state_changed.emit(val)
 
-    @dynamic_property
+    @property
     def current_editing_state(self):
-        def fget(self):
-            return {}
+        return {}
 
-        def fset(self, val):
-            pass
-        return property(fget=fget, fset=fset)
+    @current_editing_state.setter
+    def current_editing_state(self, val):
+        pass
 
     @property
     def undo_available(self):
@@ -137,14 +133,13 @@ class Editor(QMainWindow):
     def redo_available(self):
         return self.canvas.redo_action.isEnabled()
 
-    @dynamic_property
+    @property
     def current_line(self):
-        def fget(self):
-            return 0
+        return 0
 
-        def fset(self, val):
-            pass
-        return property(fget=fget, fset=fset)
+    @current_line.setter
+    def current_line(self, val):
+        pass
 
     @property
     def number_of_lines(self):
@@ -159,15 +154,14 @@ class Editor(QMainWindow):
     def get_raw_data(self):
         return self.canvas.get_image_data(quality=self.quality)
 
-    @dynamic_property
+    @property
     def data(self):
-        def fget(self):
-            return self.get_raw_data()
+        return self.get_raw_data()
 
-        def fset(self, val):
-            self.canvas.load_image(val)
-            self._is_modified = False  # The image_changed signal will have been triggered causing this editor to be incorrectly marked as modified
-        return property(fget=fget, fset=fset)
+    @data.setter
+    def data(self, val):
+        self.canvas.load_image(val)
+        self._is_modified = False  # The image_changed signal will have been triggered causing this editor to be incorrectly marked as modified
 
     def replace_data(self, raw, only_if_different=True):
         # We ignore only_if_different as it is useless in our case, and
@@ -192,7 +186,7 @@ class Editor(QMainWindow):
             self.restoreState(state)
 
     def set_focus(self):
-        self.canvas.setFocus(Qt.OtherFocusReason)
+        self.canvas.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def undo(self):
         self.canvas.undo_action.trigger()
@@ -273,8 +267,8 @@ class Editor(QMainWindow):
         self.action_resize = ac = b.addAction(QIcon(I('resize.png')), _('Resize image'), self.resize_image)
         b.addSeparator()
         self.action_filters = ac = b.addAction(QIcon(I('filter.png')), _('Image filters'))
-        b.widgetForAction(ac).setPopupMode(QToolButton.InstantPopup)
-        self.filters_menu = m = QMenu()
+        b.widgetForAction(ac).setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
+        self.filters_menu = m = QMenu(self)
         ac.setMenu(m)
         m.addAction(_('Auto-trim image'), self.canvas.autotrim_image)
         m.addAction(_('Sharpen image'), self.sharpen_image)
@@ -300,7 +294,7 @@ class Editor(QMainWindow):
     def toolbar_floated(self, floating):
         if not floating:
             self.save_state()
-            for ed in editors.itervalues():
+            for ed in itervalues(editors):
                 if ed is not self:
                     ed.restore_state()
 
@@ -315,7 +309,7 @@ class Editor(QMainWindow):
     def resize_image(self):
         im = self.canvas.current_image
         d = ResizeDialog(im.width(), im.height(), self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.canvas.resize_image(d.width, d.height)
 
     def sharpen_image(self):
