@@ -1,33 +1,33 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import sys, os
+import os
+import sys
 from functools import partial
-
 from PyQt5.Qt import (
-    QGridLayout, QSize, QListView, QStyledItemDelegate, QLabel, QPixmap,
-    QApplication, QSizePolicy, QAbstractListModel, Qt, QRect, QCheckBox,
-    QPainter, QSortFilterProxyModel, QLineEdit, QToolButton,
-    QIcon, QFormLayout, pyqtSignal, QTreeWidget, QTreeWidgetItem, QVBoxLayout,
-    QMenu, QInputDialog, QHBoxLayout)
+    QAbstractListModel, QApplication, QCheckBox, QFormLayout, QGridLayout, QClipboard,
+    QHBoxLayout, QIcon, QInputDialog, QLabel, QLineEdit, QListView, QMenu, QPainter,
+    QPixmap, QRect, QSize, QSizePolicy, QSortFilterProxyModel, QStyledItemDelegate,
+    Qt, QToolButton, QTreeWidget, QTreeWidgetItem, QVBoxLayout, pyqtSignal, QDialog, QDialogButtonBox
+)
 
 from calibre import fit_image
-from calibre.constants import plugins
 from calibre.ebooks.metadata import string_to_authors
 from calibre.ebooks.metadata.book.base import Metadata
-from calibre.gui2 import choose_files, error_dialog, pixmap_to_data, empty_index
+from calibre.gui2 import choose_files, empty_index, error_dialog, pixmap_to_data
 from calibre.gui2.languages import LanguagesEdit
 from calibre.gui2.tweak_book import current_container, tprefs
-from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.gui2.tweak_book.file_list import name_is_ok
+from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.ptempfile import PersistentTemporaryFile
-from calibre.utils.localization import get_lang, canonicalize_lang
 from calibre.utils.icu import numeric_sort_key
+from calibre.utils.localization import canonicalize_lang, get_lang
+from calibre_extensions.progress_indicator import set_no_activate_on_click
+from polyglot.builtins import unicode_type
 
 
 class ChooseName(Dialog):  # {{{
@@ -59,13 +59,13 @@ class ChooseName(Dialog):  # {{{
         return False
 
     def verify(self):
-        return name_is_ok(unicode(self.name_edit.text()), self.show_error)
+        return name_is_ok(unicode_type(self.name_edit.text()), self.show_error)
 
     def accept(self):
         if not self.verify():
             return error_dialog(self, _('No name specified'), _(
                 'You must specify a file name for the new file, with an extension.'), show=True)
-        n = unicode(self.name_edit.text()).replace('\\', '/')
+        n = unicode_type(self.name_edit.text()).replace('\\', '/')
         name, ext = n.rpartition('.')[0::2]
         self.filename = name + '.' + ext.lower()
         super(ChooseName, self).accept()
@@ -111,7 +111,7 @@ class ImageDelegate(QStyledItemDelegate):
 
     def paint(self, painter, option, index):
         QStyledItemDelegate.paint(self, painter, option, empty_index)  # draw the hover and selection highlights
-        name = unicode(index.data(Qt.DisplayRole) or '')
+        name = unicode_type(index.data(Qt.ItemDataRole.DisplayRole) or '')
         cover = self.cover_cache.get(name, None)
         if cover is None:
             cover = self.cover_cache[name] = QPixmap()
@@ -129,7 +129,7 @@ class ImageDelegate(QStyledItemDelegate):
                 if not cover.isNull():
                     scaled, width, height = fit_image(cover.width(), cover.height(), self.cover_size.width(), self.cover_size.height())
                     if scaled:
-                        cover = self.cover_cache[name] = cover.scaled(int(dpr*width), int(dpr*height), transformMode=Qt.SmoothTransformation)
+                        cover = self.cover_cache[name] = cover.scaled(int(dpr*width), int(dpr*height), transformMode=Qt.TransformationMode.SmoothTransformation)
 
         painter.save()
         try:
@@ -144,10 +144,10 @@ class ImageDelegate(QStyledItemDelegate):
                 painter.drawPixmap(rect, cover)
             rect = trect
             rect.setTop(rect.bottom() - self.title_height + 5)
-            painter.setRenderHint(QPainter.TextAntialiasing, True)
+            painter.setRenderHint(QPainter.RenderHint.TextAntialiasing, True)
             metrics = painter.fontMetrics()
-            painter.drawText(rect, Qt.AlignCenter|Qt.TextSingleLine,
-                                metrics.elidedText(name, Qt.ElideLeft, rect.width()))
+            painter.drawText(rect, Qt.AlignmentFlag.AlignCenter|Qt.TextFlag.TextSingleLine,
+                                metrics.elidedText(name, Qt.TextElideMode.ElideLeft, rect.width()))
         finally:
             painter.restore()
 
@@ -184,7 +184,7 @@ class Images(QAbstractListModel):
             name = self.image_names[index.row()]
         except IndexError:
             return None
-        if role in (Qt.DisplayRole, Qt.ToolTipRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.ToolTipRole):
             return name
         return None
 
@@ -208,20 +208,18 @@ class InsertImage(Dialog):
         self.setLayout(l)
 
         self.la1 = la = QLabel(_('&Existing images in the book'))
-        la.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        la.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
         l.addWidget(la, 0, 0, 1, 2)
         if self.for_browsing:
             la.setVisible(False)
 
         self.view = v = QListView(self)
-        v.setViewMode(v.IconMode)
-        v.setFlow(v.LeftToRight)
+        v.setViewMode(QListView.ViewMode.IconMode)
+        v.setFlow(QListView.Flow.LeftToRight)
         v.setSpacing(4)
-        v.setResizeMode(v.Adjust)
+        v.setResizeMode(QListView.ResizeMode.Adjust)
         v.setUniformItemSizes(True)
-        pi = plugins['progress_indicator'][0]
-        if hasattr(pi, 'set_no_activate_on_click'):
-            pi.set_no_activate_on_click(v)
+        set_no_activate_on_click(v)
         v.activated.connect(self.activated)
         v.doubleClicked.connect(self.activated)
         self.d = ImageDelegate(v)
@@ -247,18 +245,18 @@ class InsertImage(Dialog):
 
         if self.for_browsing:
             self.bb.clear()
-            self.bb.addButton(self.bb.Close)
-            b = self.refresh_button = self.bb.addButton(_('&Refresh'), self.bb.ActionRole)
+            self.bb.addButton(QDialogButtonBox.StandardButton.Close)
+            b = self.refresh_button = self.bb.addButton(_('&Refresh'), QDialogButtonBox.ButtonRole.ActionRole)
             b.clicked.connect(self.refresh)
             b.setIcon(QIcon(I('view-refresh.png')))
             b.setToolTip(_('Refresh the displayed images'))
-            self.setAttribute(Qt.WA_DeleteOnClose, False)
+            self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
         else:
-            b = self.import_button = self.bb.addButton(_('&Import image'), self.bb.ActionRole)
+            b = self.import_button = self.bb.addButton(_('&Import image'), QDialogButtonBox.ButtonRole.ActionRole)
             b.clicked.connect(self.import_image)
             b.setIcon(QIcon(I('view-image.png')))
             b.setToolTip(_('Import an image from elsewhere in your computer'))
-            b = self.paste_button = self.bb.addButton(_('&Paste image'), self.bb.ActionRole)
+            b = self.paste_button = self.bb.addButton(_('&Paste image'), QDialogButtonBox.ButtonRole.ActionRole)
             b.clicked.connect(self.paste_image)
             b.setIcon(QIcon(I('edit-paste.png')))
             b.setToolTip(_('Paste an image from the clipboard'))
@@ -274,10 +272,10 @@ class InsertImage(Dialog):
             h = QHBoxLayout()
             l.addLayout(h, 3, 0, 1, -1)
             h.addWidget(f), h.addStretch(10), h.addWidget(a)
-        b = self.bb.addButton(_('&Zoom in'), self.bb.ActionRole)
+        b = self.bb.addButton(_('&Zoom in'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.zoom_in)
         b.setIcon(QIcon(I('plus.png')))
-        b = self.bb.addButton(_('Zoom &out'), self.bb.ActionRole)
+        b = self.bb.addButton(_('Zoom &out'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.zoom_out)
         b.setIcon(QIcon(I('minus.png')))
         l.addWidget(self.bb, 4, 0, 1, 2)
@@ -302,7 +300,7 @@ class InsertImage(Dialog):
             n, e = basename.rpartition('.')[0::2]
             basename = n + '.' + e.lower()
             d = ChooseName(basename, self)
-            if d.exec_() == d.Accepted and d.filename:
+            if d.exec_() == QDialog.DialogCode.Accepted and d.filename:
                 self.accept()
                 self.chosen_image_is_external = (d.filename, path)
 
@@ -318,12 +316,12 @@ class InsertImage(Dialog):
         c = QApplication.instance().clipboard()
         img = c.image()
         if img.isNull():
-            img = c.image(c.Selection)
+            img = c.image(QClipboard.Mode.Selection)
         if img.isNull():
             return error_dialog(self, _('No image'), _(
                 'There is no image on the clipboard'), show=True)
         d = ChooseName('image.jpg', self)
-        if d.exec_() == d.Accepted and d.filename:
+        if d.exec_() == QDialog.DialogCode.Accepted and d.filename:
             fmt = d.filename.rpartition('.')[-1].lower()
             if fmt not in {'jpg', 'jpeg', 'png'}:
                 return error_dialog(self, _('Invalid file extension'), _(
@@ -335,21 +333,21 @@ class InsertImage(Dialog):
             self.accept()
 
     def pressed(self, index):
-        if QApplication.mouseButtons() & Qt.LeftButton:
+        if QApplication.mouseButtons() & Qt.MouseButton.LeftButton:
             self.activated(index)
 
     def activated(self, index):
         if self.for_browsing:
-            return self.image_activated.emit(unicode(index.data() or ''))
+            return self.image_activated.emit(unicode_type(index.data() or ''))
         self.chosen_image_is_external = False
         self.accept()
 
     def accept(self):
-        self.chosen_image = unicode(self.view.currentIndex().data() or '')
+        self.chosen_image = unicode_type(self.view.currentIndex().data() or '')
         super(InsertImage, self).accept()
 
     def filter_changed(self, *args):
-        f = unicode(self.filter.text())
+        f = unicode_type(self.filter.text())
         self.fm.setFilterFixedString(f)
 # }}}
 
@@ -357,7 +355,7 @@ class InsertImage(Dialog):
 def get_resource_data(rtype, parent):
     if rtype == 'image':
         d = InsertImage(parent)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             return d.chosen_image, d.chosen_image_is_external, d.fullpage.isChecked(), d.preserve_aspect_ratio.isChecked()
 
 
@@ -393,7 +391,7 @@ class ChooseFolder(Dialog):  # {{{
         f.setHeaderHidden(True)
         f.itemDoubleClicked.connect(self.accept)
         l.addWidget(f)
-        f.setContextMenuPolicy(Qt.CustomContextMenu)
+        f.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         f.customContextMenuRequested.connect(self.show_context_menu)
         self.root = QTreeWidgetItem(f, ('/',))
 
@@ -418,8 +416,8 @@ class ChooseFolder(Dialog):  # {{{
 
     def create_folder(self, item):
         text, ok = QInputDialog.getText(self, _('Folder name'), _('Enter a name for the new folder'))
-        if ok and unicode(text):
-            c = QTreeWidgetItem(item, (unicode(text),))
+        if ok and unicode_type(text):
+            c = QTreeWidgetItem(item, (unicode_type(text),))
             c.setIcon(0, QIcon(I('mimetypes/dir.png')))
             for item in self.folders.selectedItems():
                 item.setSelected(False)
@@ -429,7 +427,7 @@ class ChooseFolder(Dialog):  # {{{
     def folder_path(self, item):
         ans = []
         while item is not self.root:
-            ans.append(unicode(item.text(0)))
+            ans.append(unicode_type(item.text(0)))
             item = item.parent()
         return tuple(reversed(ans))
 
@@ -454,7 +452,7 @@ class NewBook(Dialog):  # {{{
 
         self.title = t = QLineEdit(self)
         l.addRow(_('&Title:'), t)
-        t.setFocus(Qt.OtherFocusReason)
+        t.setFocus(Qt.FocusReason.OtherFocusReason)
 
         self.authors = a = QLineEdit(self)
         l.addRow(_('&Authors:'), a)
@@ -467,10 +465,10 @@ class NewBook(Dialog):  # {{{
         bb = self.bb
         l.addRow(bb)
         bb.clear()
-        bb.addButton(bb.Cancel)
-        b = bb.addButton('&EPUB', bb.AcceptRole)
+        bb.addButton(QDialogButtonBox.StandardButton.Cancel)
+        b = bb.addButton('&EPUB', QDialogButtonBox.ButtonRole.AcceptRole)
         connect_lambda(b.clicked, self, lambda self: self.set_fmt('epub'))
-        b = bb.addButton('&AZW3', bb.AcceptRole)
+        b = bb.addButton('&AZW3', QDialogButtonBox.ButtonRole.AcceptRole)
         connect_lambda(b.clicked, self, lambda self: self.set_fmt('azw3'))
 
     def set_fmt(self, fmt):
@@ -478,15 +476,15 @@ class NewBook(Dialog):  # {{{
 
     def accept(self):
         with tprefs:
-            tprefs.set('previous_new_book_authors', unicode(self.authors.text()))
+            tprefs.set('previous_new_book_authors', unicode_type(self.authors.text()))
             tprefs.set('previous_new_book_lang', (self.languages.lang_codes or [get_lang()])[0])
             self.languages.update_recently_used()
         super(NewBook, self).accept()
 
     @property
     def mi(self):
-        mi = Metadata(unicode(self.title.text()).strip() or _('Unknown'))
-        mi.authors = string_to_authors(unicode(self.authors.text()).strip()) or [_('Unknown')]
+        mi = Metadata(unicode_type(self.title.text()).strip() or _('Unknown'))
+        mi.authors = string_to_authors(unicode_type(self.authors.text()).strip()) or [_('Unknown')]
         mi.languages = self.languages.lang_codes or [get_lang()]
         return mi
 
