@@ -1,11 +1,11 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
+
 class NullSmarts(object):
 
     override_tab_stop_width = None
@@ -22,7 +22,7 @@ class NullSmarts(object):
     def verify_for_spellcheck(self, cursor, highlighter):
         return False
 
-    def cursor_position_with_sourceline(self, cursor, for_position_sync=True):
+    def cursor_position_with_sourceline(self, cursor, for_position_sync=True, use_matched_tag=True):
         return None, None
 
     def goto_sourceline(self, editor, sourceline, tags, attribute=None):
