@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -19,9 +18,11 @@ get_leading_whitespace_on_block = lambda editor, previous=False: expand_tabs(lw(
 
 tw = 4  # The tab width (hardcoded to the pep8 value)
 
+
 def expand_tabs(text):
     return text.replace('\t', ' ' * tw)
 
+
 class Smarts(NullSmarts):
 
     override_tab_stop_width = tw
@@ -35,13 +36,13 @@ class Smarts(NullSmarts):
     def handle_key_press(self, ev, editor):
         key = ev.key()
 
-        if key == Qt.Key_Tab and smart_tab(editor, ev):
+        if key == Qt.Key.Key_Tab and smart_tab(editor, ev):
             return True
 
-        elif key == Qt.Key_Backspace and smart_backspace(editor, ev):
+        elif key == Qt.Key.Key_Backspace and smart_backspace(editor, ev):
             return True
 
-        elif key in (Qt.Key_Enter, Qt.Key_Return):
+        elif key in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
             ls = get_leading_whitespace_on_block(editor)
             cursor = editor.textCursor()
             line = cursor.block().text()
@@ -53,7 +54,7 @@ class Smarts(NullSmarts):
             editor.setTextCursor(cursor)
             return True
 
-        elif key == Qt.Key_Colon:
+        elif key == Qt.Key.Key_Colon:
             cursor, text = get_text_before_cursor(editor)
             if self.dedent_pat.search(text) is not None:
                 ls = get_leading_whitespace_on_block(editor)
@@ -65,9 +66,10 @@ class Smarts(NullSmarts):
                     editor.setTextCursor(cursor)
                     return True
 
-        if key == Qt.Key_Home and smart_home(editor, ev):
+        if key == Qt.Key.Key_Home and smart_home(editor, ev):
             return True
 
+
 if __name__ == '__main__':
     import os
     from calibre.gui2.tweak_book.editor.widget import launch_editor
