@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -13,6 +12,7 @@ from pygments.token import Comment, Punctuation, Number, Keyword, Text, String,
 import pygments.unistring as uni
 
 from calibre.gui2.tweak_book.editor.syntax.pygments_highlighter import create_highlighter
+from polyglot.builtins import native_string_type
 
 JS_IDENT_START = ('(?:[$_' + uni.combine('Lu', 'Ll', 'Lt', 'Lm', 'Lo', 'Nl') +
                   ']|\\\\u[a-fA-F0-9]{4})')
@@ -21,6 +21,7 @@ JS_IDENT_PART = ('(?:[$' + uni.combine('Lu', 'Ll', 'Lt', 'Lm', 'Lo', 'Nl',
                  u'\u200c\u200d]|\\\\u[a-fA-F0-9]{4})')
 JS_IDENT = JS_IDENT_START + '(?:' + JS_IDENT_PART + ')*'
 
+
 class JavascriptLexer(RegexLexer):
 
     """
@@ -32,39 +33,39 @@ class JavascriptLexer(RegexLexer):
     flags = re.UNICODE | re.MULTILINE
 
     tokens = {
-        b'commentsandwhitespace': [
+        native_string_type('commentsandwhitespace'): [
             (r'\s+', Text),
             (r'<!--', Comment),
             (r'//.*?$', Comment.Single),
-            (r'/\*', Comment.Multiline, b'comment')
+            (r'/\*', Comment.Multiline, native_string_type('comment'))
         ],
-        b'comment': [
+        native_string_type('comment'): [
             (r'[^*/]+', Comment.Multiline),
-            (r'\*/', Comment.Multiline, b'#pop'),
+            (r'\*/', Comment.Multiline, native_string_type('#pop')),
             (r'[*/]', Comment.Multiline),
         ],
-        b'slashstartsregex': [
-            include(b'commentsandwhitespace'),
+        native_string_type('slashstartsregex'): [
+            include(native_string_type('commentsandwhitespace')),
             (r'/(\\.|[^[/\\\n]|\[(\\.|[^\]\\\n])*])+/'
-             r'([gim]+\b|\B)', String.Regex, b'#pop'),
-            (r'(?=/)', Text, (b'#pop', b'badregex')),
-            default(b'#pop')
+             r'([gim]+\b|\B)', String.Regex, native_string_type('#pop')),
+            (r'(?=/)', Text, (native_string_type('#pop'), native_string_type('badregex'))),
+            default(native_string_type('#pop'))
         ],
-        b'badregex': [
-            (r'\n', Text, b'#pop')
+        native_string_type('badregex'): [
+            (r'\n', Text, native_string_type('#pop'))
         ],
-        b'root': [
+        native_string_type('root'): [
             (r'\A#! ?/.*?\n', Comment),  # shebang lines are recognized by node.js
-            (r'^(?=\s|/|<!--)', Text, b'slashstartsregex'),
-            include(b'commentsandwhitespace'),
+            (r'^(?=\s|/|<!--)', Text, native_string_type('slashstartsregex')),
+            include(native_string_type('commentsandwhitespace')),
             (r'\+\+|--|~|&&|\?|:|\|\||\\(?=\n)|'
-             r'(<<|>>>?|==?|!=?|[-<>+*%&|^/])=?', Operator, b'slashstartsregex'),
-            (r'[{(\[;,]', Punctuation, b'slashstartsregex'),
+             r'(<<|>>>?|==?|!=?|[-<>+*%&|^/])=?', Operator, native_string_type('slashstartsregex')),
+            (r'[{(\[;,]', Punctuation, native_string_type('slashstartsregex')),
             (r'[})\].]', Punctuation),
             (r'(for|in|while|do|break|return|continue|switch|case|default|if|else|'
              r'throw|try|catch|finally|new|delete|typeof|instanceof|void|yield|'
-             r'this)\b', Keyword, b'slashstartsregex'),
-            (r'(var|let|with|function)\b', Keyword.Declaration, b'slashstartsregex'),
+             r'this)\b', Keyword, native_string_type('slashstartsregex')),
+            (r'(var|let|with|function)\b', Keyword.Declaration, native_string_type('slashstartsregex')),
             (r'(abstract|boolean|byte|char|class|const|debugger|double|enum|export|'
              r'extends|final|float|goto|implements|import|int|interface|long|native|'
              r'package|private|protected|public|short|static|super|synchronized|throws|'
@@ -89,4 +90,4 @@ Highlighter = create_highlighter('JavascriptHighlighter', JavascriptLexer)
 
 if __name__ == '__main__':
     from calibre.gui2.tweak_book.editor.widget import launch_editor
-    launch_editor(P('viewer/images.js'), syntax='javascript')
+    launch_editor(P('viewer.js'), syntax='javascript')
