@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,13 +10,14 @@ from collections import namedtuple
 from PyQt5.Qt import (
     QColor, QBrush, QFont, QApplication, QPalette, QComboBox,
     QPushButton, QIcon, QFormLayout, QLineEdit, QWidget, QScrollArea,
-    QVBoxLayout, Qt, QHBoxLayout, pyqtSignal, QPixmap, QColorDialog,
-    QToolButton, QCheckBox, QSize, QLabel, QSplitter, QTextCharFormat)
+    QVBoxLayout, Qt, QHBoxLayout, pyqtSignal, QPixmap, QColorDialog, QDialog,
+    QToolButton, QCheckBox, QSize, QLabel, QSplitter, QTextCharFormat, QDialogButtonBox)
 
 from calibre.gui2 import error_dialog
 from calibre.gui2.tweak_book import tprefs
 from calibre.gui2.tweak_book.editor import syntax_text_char_format
 from calibre.gui2.tweak_book.widgets import Dialog
+from polyglot.builtins import iteritems, unicode_type, range, map
 
 underline_styles = {'single', 'dash', 'dot', 'dash_dot', 'dash_dot_dot', 'wave', 'spell'}
 
@@ -27,17 +27,18 @@ _default_theme = None
 def default_theme():
     global _default_theme
     if _default_theme is None:
-        isdark = QApplication.instance().palette().color(QPalette.WindowText).lightness() > 128
+        isdark = QApplication.instance().palette().color(QPalette.ColorRole.WindowText).lightness() > 128
         _default_theme = 'wombat-dark' if isdark else 'pyte-light'
     return _default_theme
 
+
 # The solarized themes {{{
 SLDX = {'base03':'1c1c1c', 'base02':'262626', 'base01':'585858', 'base00':'626262', 'base0':'808080', 'base1':'8a8a8a', 'base2':'e4e4e4', 'base3':'ffffd7', 'yellow':'af8700', 'orange':'d75f00', 'red':'d70000', 'magenta':'af005f', 'violet':'5f5faf', 'blue':'0087ff', 'cyan':'00afaf', 'green':'5f8700'}  # noqa
 SLD  = {'base03':'002b36', 'base02':'073642', 'base01':'586e75', 'base00':'657b83', 'base0':'839496', 'base1':'93a1a1', 'base2':'eee8d5', 'base3':'fdf6e3', 'yellow':'b58900', 'orange':'cb4b16', 'red':'dc322f', 'magenta':'d33682', 'violet':'6c71c4', 'blue':'268bd2', 'cyan':'2aa198', 'green':'859900'}  # noqa
-m = {'base%d'%n:'base%02d'%n for n in xrange(1, 4)}
-m.update({'base%02d'%n:'base%d'%n for n in xrange(1, 4)})
-SLL = {m.get(k, k) : v for k, v in SLD.iteritems()}
-SLLX = {m.get(k, k) : v for k, v in SLDX.iteritems()}
+m = {'base%d'%n:'base%02d'%n for n in range(1, 4)}
+m.update({'base%02d'%n:'base%d'%n for n in range(1, 4)})
+SLL = {m.get(k, k) : v for k, v in iteritems(SLD)}
+SLLX = {m.get(k, k) : v for k, v in iteritems(SLDX)}
 SOLARIZED = \
     '''
     CursorLine   bg={base02}
@@ -237,7 +238,7 @@ def read_theme(raw):
     return ans
 
 
-THEMES = {k:read_theme(raw) for k, raw in THEMES.iteritems()}
+THEMES = {k:read_theme(raw) for k, raw in iteritems(THEMES)}
 
 
 def u(x):
@@ -259,7 +260,7 @@ def to_highlight(data):
 
 def read_custom_theme(data):
     dt = THEMES[default_theme()].copy()
-    dt.update({k:to_highlight(v) for k, v in data.iteritems()})
+    dt.update({k:to_highlight(v) for k, v in iteritems(data)})
     return dt
 
 
@@ -278,7 +279,7 @@ def get_theme(name):
 def highlight_to_char_format(h):
     ans = syntax_text_char_format()
     if h.bold:
-        ans.setFontWeight(QFont.Bold)
+        ans.setFontWeight(QFont.Weight.Bold)
     if h.italic:
         ans.setFontItalic(True)
     if h.fg is not None:
@@ -308,11 +309,11 @@ def theme_format(theme, name):
 
 
 def custom_theme_names():
-    return tuple(tprefs['custom_themes'].iterkeys())
+    return tuple(tprefs['custom_themes'])
 
 
 def builtin_theme_names():
-    return tuple(THEMES.iterkeys())
+    return tuple(THEMES)
 
 
 def all_theme_names():
@@ -345,7 +346,7 @@ class CreateNewTheme(Dialog):
 
     @property
     def theme_name(self):
-        return unicode(self._name.text()).strip()
+        return unicode_type(self._name.text()).strip()
 
     def accept(self):
         if not self.theme_name:
@@ -374,7 +375,7 @@ class ColorButton(QPushButton):
             self.current_color = read_color(color).color()
             self.ic.fill(self.current_color)
         else:
-            self.ic.fill(Qt.transparent)
+            self.ic.fill(Qt.GlobalColor.transparent)
             self.current_color = color
         self.update_tooltip()
         self.setIcon(QIcon(self.ic))
@@ -383,13 +384,13 @@ class ColorButton(QPushButton):
     def clear(self):
         self.current_color = None
         self.update_tooltip()
-        self.ic.fill(Qt.transparent)
+        self.ic.fill(Qt.GlobalColor.transparent)
         self.setIcon(QIcon(self.ic))
         self.data[self.name] = self.value
         self.changed.emit()
 
     def choose_color(self):
-        col = QColorDialog.getColor(self.current_color or Qt.black, self, _('Choose color'))
+        col = QColorDialog.getColor(self.current_color or Qt.GlobalColor.black, self, _('Choose color'))
         if col.isValid():
             self.current_color = col
             self.update_tooltip()
@@ -424,7 +425,7 @@ class Bool(QCheckBox):
 
     @property
     def value(self):
-        return self.checkState() == Qt.Checked
+        return self.checkState() == Qt.CheckState.Checked
 
 
 class Property(QWidget):
@@ -474,7 +475,7 @@ class Property(QWidget):
         l.addStretch(1)
 
     def us_changed(self):
-        self.data['underline'] = unicode(self.underline.currentText()) or None
+        self.data['underline'] = unicode_type(self.underline.currentText()) or None
         self.changed.emit()
 
 # Help text {{{
@@ -602,7 +603,7 @@ class ThemeEditor(Dialog):
         sp.addWidget(s), sp.addWidget(p)
 
         self.bb.clear()
-        self.bb.addButton(self.bb.Close)
+        self.bb.addButton(QDialogButtonBox.StandardButton.Close)
         l.addWidget(self.bb)
 
         if self.theme.count() > 0:
@@ -610,15 +611,15 @@ class ThemeEditor(Dialog):
 
     def update_theme(self, name):
         data = tprefs['custom_themes'][name]
-        extra = set(data.iterkeys()) - set(THEMES[default_theme()].iterkeys())
-        missing = set(THEMES[default_theme()].iterkeys()) - set(data.iterkeys())
+        extra = set(data) - set(THEMES[default_theme()])
+        missing = set(THEMES[default_theme()]) - set(data)
         for k in extra:
             data.pop(k)
         for k in missing:
             data[k] = dict(THEMES[default_theme()][k]._asdict())
-            for nk, nv in data[k].iteritems():
+            for nk, nv in iteritems(data[k]):
                 if isinstance(nv, QBrush):
-                    data[k][nk] = unicode(nv.color().name())
+                    data[k][nk] = unicode_type(nv.color().name())
         if extra or missing:
             tprefs['custom_themes'][name] = data
         return data
@@ -632,7 +633,7 @@ class ThemeEditor(Dialog):
             c.setParent(None)
             c.deleteLater()
         self.properties = []
-        name = unicode(self.theme.currentText())
+        name = unicode_type(self.theme.currentText())
         if not name:
             return
         data = self.update_theme(name)
@@ -649,7 +650,7 @@ class ThemeEditor(Dialog):
 
     @property
     def theme_name(self):
-        return unicode(self.theme.currentText())
+        return unicode_type(self.theme.currentText())
 
     def changed(self):
         name = self.theme_name
@@ -658,12 +659,12 @@ class ThemeEditor(Dialog):
 
     def create_new_theme(self):
         d = CreateNewTheme(self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             name = '*' + d.theme_name
-            base = unicode(d.base.currentText())
+            base = unicode_type(d.base.currentText())
             theme = {}
-            for key, val in THEMES[base].iteritems():
-                theme[key] = {k:col_to_string(v.color()) if isinstance(v, QBrush) else v for k, v in val._asdict().iteritems()}
+            for key, val in iteritems(THEMES[base]):
+                theme[key] = {k:col_to_string(v.color()) if isinstance(v, QBrush) else v for k, v in iteritems(val._asdict())}
             tprefs['custom_themes'][name] = theme
             tprefs['custom_themes'] = tprefs['custom_themes']
             t = self.theme
