@@ -11,6 +11,7 @@ from PyQt5.Qt import QIcon
 from PyQt5.QtWebEngineCore import QWebEngineUrlScheme
 
 from calibre.constants import EDITOR_APP_UID, FAKE_PROTOCOL, islinux
+from calibre.ebooks.oeb.polish.check.css import shutdown as shutdown_css_check_pool
 from calibre.gui2 import (
     Application, decouple, set_gui_prefs, setup_gui_option_parser
 )
@@ -27,7 +28,7 @@ def option_parser():
             '''\
 %prog [opts] [path_to_ebook] [name_of_file_inside_book ...]
 
-Launch the calibre edit book tool. You can optionally also specify the names of
+Launch the calibre Edit book tool. You can optionally also specify the names of
 files inside the book which will be opened for editing automatically.
 '''
         )
@@ -55,7 +56,7 @@ def _run(args, notify=None):
     reset_base_dir()
     scheme = QWebEngineUrlScheme(FAKE_PROTOCOL.encode('ascii'))
     scheme.setSyntax(QWebEngineUrlScheme.Syntax.Host)
-    scheme.setFlags(QWebEngineUrlScheme.SecureScheme)
+    scheme.setFlags(QWebEngineUrlScheme.Flag.SecureScheme)
     QWebEngineUrlScheme.registerScheme(scheme)
 
     # The following two lines are needed to prevent circular imports causing
@@ -94,7 +95,10 @@ def _run(args, notify=None):
 
 
 def main(args=sys.argv):
-    _run(args)
+    try:
+        _run(args)
+    finally:
+        shutdown_css_check_pool()
 
 
 if __name__ == '__main__':
