@@ -12,9 +12,9 @@ from functools import partial
 from PyQt5.Qt import (
     QAbstractListModel, QAction, QApplication, QCheckBox, QComboBox, QFont, QFrame,
     QGridLayout, QHBoxLayout, QIcon, QItemSelection, QKeySequence, QLabel, QLineEdit,
-    QListView, QMenu, QMimeData, QModelIndex, QPushButton, QScrollArea, QSize,
-    QSizePolicy, QStackedLayout, QStyledItemDelegate, Qt, QTimer, QToolBar,
-    QToolButton, QVBoxLayout, QWidget, pyqtSignal
+    QListView, QMenu, QMimeData, QModelIndex, QPushButton, QScrollArea, QSize, QItemSelectionModel,
+    QSizePolicy, QStackedLayout, QStyledItemDelegate, Qt, QTimer, QToolBar, QDialog,
+    QToolButton, QVBoxLayout, QWidget, pyqtSignal, QAbstractItemView, QEvent, QDialogButtonBox
 )
 
 from calibre import prepare_string_for_xml
@@ -104,7 +104,7 @@ class HistoryBox(HistoryComboBox):
         self.set_uniform_item_sizes(False)
 
     def event(self, ev):
-        if ev.type() in (ev.ShortcutOverride, ev.KeyPress) and ev.key() == KEY and ev.modifiers() & MODIFIER:
+        if ev.type() in (QEvent.Type.ShortcutOverride, QEvent.Type.KeyPress) and ev.key() == KEY and ev.modifiers() & MODIFIER:
             if not self.ignore_snip_expansion:
                 self.ignore_snip_expansion = True
                 expand_template(self.lineEdit())
@@ -254,7 +254,7 @@ class SearchWidget(QWidget):
         l.setContentsMargins(0, 0, right, 0)
 
         self.fl = fl = QLabel(_('&Find:'))
-        fl.setAlignment(Qt.AlignRight | Qt.AlignCenter)
+        fl.setAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignCenter)
         self.find_text = ft = HistoryBox(self, _('Clear search &history'))
         ft.save_search.connect(self.save_search)
         ft.show_saved_searches.connect(self.show_saved_searches)
@@ -266,7 +266,7 @@ class SearchWidget(QWidget):
         l.setColumnStretch(1, 10)
 
         self.rl = rl = QLabel(_('&Replace:'))
-        rl.setAlignment(Qt.AlignRight | Qt.AlignCenter)
+        rl.setAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignCenter)
         self.replace_text = rt = HistoryBox(self, _('Clear replace &history'))
         rt.save_search.connect(self.save_search)
         rt.show_saved_searches.connect(self.show_saved_searches)
@@ -279,7 +279,7 @@ class SearchWidget(QWidget):
         l.addLayout(rs2, 1, 1)
 
         self.rl2 = rl2 = QLabel(_('F&unction:'))
-        rl2.setAlignment(Qt.AlignRight | Qt.AlignCenter)
+        rl2.setAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignCenter)
         self.functions = fb = FunctionBox(self, show_saved_search_actions=True)
         fb.show_saved_searches.connect(self.show_saved_searches)
         fb.save_search.connect(self.save_search)
@@ -289,7 +289,7 @@ class SearchWidget(QWidget):
         rs2.addWidget(w)
         self.fhl = fhl = QHBoxLayout(w)
         fhl.setContentsMargins(0, 0, 0, 0)
-        fhl.addWidget(fb, stretch=10, alignment=Qt.AlignVCenter)
+        fhl.addWidget(fb, stretch=10, alignment=Qt.AlignmentFlag.AlignVCenter)
         self.ae_func = b = QPushButton(_('Create/&edit'), self)
         b.clicked.connect(self.edit_function)
         b.setToolTip(_('Create a new function, or edit an existing function'))
@@ -299,7 +299,7 @@ class SearchWidget(QWidget):
         b.clicked.connect(self.remove_function)
         fhl.addWidget(b)
         self.fsep = f = QFrame(self)
-        f.setFrameShape(f.VLine)
+        f.setFrameShape(QFrame.Shape.VLine)
         fhl.addWidget(f)
 
         self.fb = fb = PushButton(_('Fin&d'), 'find', self)
@@ -313,7 +313,7 @@ class SearchWidget(QWidget):
 
         self.ml = ml = QLabel(_('&Mode:'))
         self.ol = ol = FlowLayout()
-        ml.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
+        ml.setAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
         l.addWidget(ml, 2, 0)
         l.addLayout(ol, 2, 1, 1, 3)
         self.mode_box = mb = ModeBox(self)
@@ -342,7 +342,7 @@ class SearchWidget(QWidget):
 
     def edit_function(self):
         d = FunctionEditor(func_name=self.functions.text().strip(), parent=self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.functions.setText(d.func_name)
 
     def remove_function(self):
@@ -450,6 +450,13 @@ class SearchWidget(QWidget):
         self.find = text
         self.find_text.lineEdit().setSelection(0, len(text)+10)
 
+    def paste_saved_search(self, s):
+        self.case_sensitive = s.get('case_sensitive') or False
+        self.dot_all = s.get('dot_all') or False
+        self.wrap = s.get('wrap') or False
+        self.mode = s.get('mode') or 'normal'
+        self.find = s.get('find') or ''
+        self.replace = s.get('replace') or ''
 # }}}
 
 
@@ -467,7 +474,7 @@ class SearchPanel(QWidget):  # {{{
         l.setContentsMargins(0, 0, 0, 0)
         self.t = t = QToolBar(self)
         l.addWidget(t)
-        t.setOrientation(Qt.Vertical)
+        t.setOrientation(Qt.Orientation.Vertical)
         t.setIconSize(QSize(12, 12))
         t.setMovable(False)
         t.setFloatable(False)
@@ -481,12 +488,15 @@ class SearchPanel(QWidget):  # {{{
         self.widget.show_saved_searches.connect(self.show_saved_searches)
         self.pre_fill = self.widget.pre_fill
 
+    def paste_saved_search(self, s):
+        self.widget.paste_saved_search(s)
+
     def hide_panel(self):
         self.setVisible(False)
 
     def show_panel(self):
         self.setVisible(True)
-        self.widget.find_text.setFocus(Qt.OtherFocusReason)
+        self.widget.find_text.setFocus(Qt.FocusReason.OtherFocusReason)
         le = self.widget.find_text.lineEdit()
         le.setSelection(0, le.maxLength())
 
@@ -508,7 +518,7 @@ class SearchPanel(QWidget):  # {{{
             self.where_before_marked = None
 
     def keyPressEvent(self, ev):
-        if ev.key() == Qt.Key_Escape:
+        if ev.key() == Qt.Key.Key_Escape:
             self.hide_panel()
             ev.accept()
         else:
@@ -523,7 +533,7 @@ class SearchDescription(QScrollArea):
         self.label = QLabel(' \n \n ')
         self.setWidget(self.label)
         self.setWidgetResizable(True)
-        self.label.setTextFormat(Qt.PlainText)
+        self.label.setTextFormat(Qt.TextFormat.PlainText)
         self.label.setWordWrap(True)
         self.set_text = self.label.setText
 
@@ -539,14 +549,14 @@ class SearchesModel(QAbstractListModel):
         return len(self.filtered_searches)
 
     def supportedDropActions(self):
-        return Qt.MoveAction
+        return Qt.DropAction.MoveAction
 
     def flags(self, index):
         ans = QAbstractListModel.flags(self, index)
         if index.isValid():
-            ans |= Qt.ItemIsDragEnabled
+            ans |= Qt.ItemFlag.ItemIsDragEnabled
         else:
-            ans |= Qt.ItemIsDropEnabled
+            ans |= Qt.ItemFlag.ItemIsDropEnabled
         return ans
 
     def mimeTypes(self):
@@ -564,7 +574,7 @@ class SearchesModel(QAbstractListModel):
         return ans
 
     def dropMimeData(self, data, action, row, column, parent):
-        if parent.isValid() or action != Qt.MoveAction or not data.hasFormat('x-calibre/searches-rows') or not self.filtered_searches:
+        if parent.isValid() or action != Qt.DropAction.MoveAction or not data.hasFormat('x-calibre/searches-rows') or not self.filtered_searches:
             return False
         rows = sorted(map(int, bytes(bytearray(data.data('x-calibre/searches-rows'))).decode('ascii').split(',')))
         moved_searches = [self.searches[self.filtered_searches[r]] for r in rows]
@@ -596,14 +606,14 @@ class SearchesModel(QAbstractListModel):
 
     def data(self, index, role):
         try:
-            if role == Qt.DisplayRole:
+            if role == Qt.ItemDataRole.DisplayRole:
                 search = self.searches[self.filtered_searches[index.row()]]
                 return search['name']
-            if role == Qt.ToolTipRole:
+            if role == Qt.ItemDataRole.ToolTipRole:
                 search = self.searches[self.filtered_searches[index.row()]]
                 tt = '\n'.join((search['find'], search['replace']))
                 return tt
-            if role == Qt.UserRole:
+            if role == Qt.ItemDataRole.UserRole:
                 search = self.searches[self.filtered_searches[index.row()]]
                 return (self.filtered_searches[index.row()], search)
         except IndexError:
@@ -660,7 +670,7 @@ class EditSearch(QFrame):  # {{{
 
     def __init__(self, parent=None):
         QFrame.__init__(self, parent)
-        self.setFrameShape(self.StyledPanel)
+        self.setFrameShape(QFrame.Shape.StyledPanel)
         self.search_index = -1
         self.search = {}
         self.original_name = None
@@ -722,7 +732,7 @@ class EditSearch(QFrame):  # {{{
         self.h2 = h = QHBoxLayout()
         l.addLayout(h)
         self.mode_box = m = ModeBox(self)
-        m.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+        m.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         self.la4 = la = QLabel(_('&Mode:'))
         la.setBuddy(m)
         h.addWidget(la), h.addWidget(m), h.addStretch(2)
@@ -737,7 +747,7 @@ class EditSearch(QFrame):  # {{{
 
     def edit_function(self):
         d = FunctionEditor(func_name=self.function.text().strip(), parent=self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.function.setText(d.func_name)
 
     def remove_function(self):
@@ -784,7 +794,7 @@ class EditSearch(QFrame):  # {{{
         self.done.emit(True)
 
     def keyPressEvent(self, ev):
-        if ev.key() == Qt.Key_Escape:
+        if ev.key() == Qt.Key.Key_Escape:
             self.abort_editing()
             ev.accept()
             return
@@ -865,6 +875,7 @@ class SearchDelegate(QStyledItemDelegate):
 class SavedSearches(QWidget):
 
     run_saved_searches = pyqtSignal(object, object)
+    copy_search_to_search_panel = pyqtSignal(object)
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
@@ -901,11 +912,11 @@ class SavedSearches(QWidget):
         self.model.dataChanged.connect(self.show_details)
         searches.setModel(self.model)
         searches.selectionModel().currentChanged.connect(self.show_details)
-        searches.setSelectionMode(searches.ExtendedSelection)
+        searches.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
         self.delegate = SearchDelegate(searches)
         searches.setItemDelegate(self.delegate)
         searches.setAlternatingRowColors(True)
-        searches.setDragEnabled(True), searches.setAcceptDrops(True), searches.setDragDropMode(searches.InternalMove)
+        searches.setDragEnabled(True), searches.setAcceptDrops(True), searches.setDragDropMode(QAbstractItemView.DragDropMode.InternalMove)
         searches.setDropIndicatorShown(True)
         h.addLayout(stack, stretch=10)
         self.v = v = QVBoxLayout()
@@ -918,7 +929,7 @@ class SavedSearches(QWidget):
             b.setToolTip(tooltip or '')
             if action:
                 b.setObjectName(action)
-            b.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
+            b.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
             return b
 
         mulmsg = '\n\n' + _('The entries are tried in order until the first one matches.')
@@ -936,14 +947,15 @@ class SavedSearches(QWidget):
             connect_lambda(b.clicked, self, lambda self: self.run_search(self.sender().objectName()))
 
         self.d1 = d = QFrame(self)
-        d.setFrameStyle(QFrame.HLine)
+        d.setFrameStyle(QFrame.Shape.HLine)
         v.addWidget(d)
 
         self.h3 = h = QHBoxLayout()
         self.upb = b = QToolButton(self)
         self.move_up_action = a = QAction(self)
         a.setShortcut(QKeySequence('Alt+Up'))
-        b.setIcon(QIcon(I('arrow-up.png'))), b.setToolTip(_('Move selected entries up') + ' [%s]' % a.shortcut().toString(QKeySequence.NativeText))
+        b.setIcon(QIcon(I('arrow-up.png')))
+        b.setToolTip(_('Move selected entries up') + ' [%s]' % a.shortcut().toString(QKeySequence.SequenceFormat.NativeText))
         connect_lambda(a.triggered, self, lambda self: self.move_entry(-1))
         self.searches.addAction(a)
         connect_lambda(b.clicked, self, lambda self: self.move_entry(-1))
@@ -951,7 +963,8 @@ class SavedSearches(QWidget):
         self.dnb = b = QToolButton(self)
         self.move_down_action = a = QAction(self)
         a.setShortcut(QKeySequence('Alt+Down'))
-        b.setIcon(QIcon(I('arrow-down.png'))), b.setToolTip(_('Move selected entries down') + ' [%s]' % a.shortcut().toString(QKeySequence.NativeText))
+        b.setIcon(QIcon(I('arrow-down.png')))
+        b.setToolTip(_('Move selected entries down') + ' [%s]' % a.shortcut().toString(QKeySequence.SequenceFormat.NativeText))
         connect_lambda(a.triggered, self, lambda self: self.move_entry(1))
         self.searches.addAction(a)
         connect_lambda(b.clicked, self, lambda self: self.move_entry(1))
@@ -971,7 +984,7 @@ class SavedSearches(QWidget):
         v.addWidget(b)
 
         self.d2 = d = QFrame(self)
-        d.setFrameStyle(QFrame.HLine)
+        d.setFrameStyle(QFrame.Shape.HLine)
         v.addWidget(d)
 
         self.where_box = wb = WhereBox(self, emphasize=True)
@@ -987,7 +1000,7 @@ class SavedSearches(QWidget):
         v.addWidget(wr)
 
         self.d3 = d = QFrame(self)
-        d.setFrameStyle(QFrame.HLine)
+        d.setFrameStyle(QFrame.Shape.HLine)
         v.addWidget(d)
 
         self.description = d = SearchDescription(self)
@@ -1003,9 +1016,10 @@ class SavedSearches(QWidget):
         self.em = m = QMenu(_('Export'))
         m.addAction(_('Export all'), lambda : QTimer.singleShot(0, partial(self.export_searches, all=True)))
         m.addAction(_('Export selected'), lambda : QTimer.singleShot(0, partial(self.export_searches, all=False)))
+        m.addAction(_('Copy to search panel'), lambda : QTimer.singleShot(0, self.copy_to_search_panel))
         b.setMenu(m)
 
-        self.searches.setFocus(Qt.OtherFocusReason)
+        self.searches.setFocus(Qt.FocusReason.OtherFocusReason)
 
     @property
     def state(self):
@@ -1094,7 +1108,7 @@ class SavedSearches(QWidget):
                 seen.add(index.row())
                 search = SearchWidget.DEFAULT_STATE.copy()
                 del search['mode']
-                search_index, s = index.data(Qt.UserRole)
+                search_index, s = index.data(Qt.ItemDataRole.UserRole)
                 search.update(s)
                 fill_in_search(search)
                 searches.append(search)
@@ -1125,9 +1139,9 @@ class SavedSearches(QWidget):
                 index = self.model.index_for_search(s)
                 if index.isValid() and index.row() > -1:
                     if s is current_search:
-                        sm.setCurrentIndex(index, sm.Select)
+                        sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.Select)
                     else:
-                        sm.select(index, sm.Select)
+                        sm.select(index, QItemSelectionModel.SelectionFlag.Select)
 
     def search_editing_done(self, save_changes):
         if save_changes and not self.edit_search_widget.save_changes():
@@ -1147,10 +1161,10 @@ class SavedSearches(QWidget):
             return error_dialog(self, _('Cannot edit'), _(
                 'Cannot edit search - no search selected.'), show=True)
         if not self.editing_search:
-            search_index, search = index.data(Qt.UserRole)
+            search_index, search = index.data(Qt.ItemDataRole.UserRole)
             self.edit_search_widget.show_search(search=search, search_index=search_index)
             self.stack.setCurrentIndex(1)
-            self.edit_search_widget.find.setFocus(Qt.OtherFocusReason)
+            self.edit_search_widget.find.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def remove_search(self):
         if self.editing_search:
@@ -1166,14 +1180,14 @@ class SavedSearches(QWidget):
             return
         self.edit_search_widget.show_search()
         self.stack.setCurrentIndex(1)
-        self.edit_search_widget.search_name.setFocus(Qt.OtherFocusReason)
+        self.edit_search_widget.search_name.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def _add_search(self):
         self.model.add_searches()
         index = self.model.index(self.model.rowCount() - 1)
         self.searches.scrollTo(index)
         sm = self.searches.selectionModel()
-        sm.setCurrentIndex(index, sm.ClearAndSelect)
+        sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.ClearAndSelect)
         self.show_details()
 
     def add_predefined_search(self, state):
@@ -1181,14 +1195,14 @@ class SavedSearches(QWidget):
             return
         self.edit_search_widget.show_search(state=state)
         self.stack.setCurrentIndex(1)
-        self.edit_search_widget.search_name.setFocus(Qt.OtherFocusReason)
+        self.edit_search_widget.search_name.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def show_details(self):
         self.description.set_text(' \n \n ')
         i = self.searches.currentIndex()
         if i.isValid():
             try:
-                search_index, search = i.data(Qt.UserRole)
+                search_index, search = i.data(Qt.ItemDataRole.UserRole)
             except TypeError:
                 return  # no saved searches
             cs = '✓' if search.get('case_sensitive', SearchWidget.DEFAULT_STATE['case_sensitive']) else '✗'
@@ -1225,9 +1239,15 @@ class SavedSearches(QWidget):
                 self.model.add_searches(count=count)
                 sm = self.searches.selectionModel()
                 top, bottom = self.model.index(self.model.rowCount() - count), self.model.index(self.model.rowCount() - 1)
-                sm.select(QItemSelection(top, bottom), sm.ClearAndSelect)
+                sm.select(QItemSelection(top, bottom), QItemSelectionModel.SelectionFlag.ClearAndSelect)
                 self.searches.scrollTo(bottom)
 
+    def copy_to_search_panel(self):
+        ci = self.searches.selectionModel().currentIndex()
+        if ci and ci.isValid():
+            search = ci.data(Qt.ItemDataRole.UserRole)[-1]
+            self.copy_search_to_search_panel.emit(search)
+
     def export_searches(self, all=True):
         if all:
             searches = copy.deepcopy(tprefs['saved_searches'])
@@ -1237,7 +1257,7 @@ class SavedSearches(QWidget):
         else:
             searches = []
             for index in self.searches.selectionModel().selectedIndexes():
-                search = index.data(Qt.UserRole)[-1]
+                search = index.data(Qt.ItemDataRole.UserRole)[-1]
                 searches.append(search.copy())
             if not searches:
                 return error_dialog(self, _('No searches'), _(
@@ -1466,14 +1486,14 @@ def run_search(
                 msg = _('Performed the replacement at {num} occurrences of {query}')
             else:
                 msg = _('Found {num} occurrences of {query}')
-            msg = msg.format(num=count, query=errfind)
+            msg = msg.format(num=count, query=prepare_string_for_xml(errfind))
             det_msg = ''
             if count_map is not None and count > 0 and len(count_map) > 1:
                 for k in sorted(count_map):
                     det_msg += _('{0}: {1} occurrences').format(k, count_map[k]) + '\n'
             if show_diff and count > 0:
-                d = MessageBox(MessageBox.INFO, _('Searching done'), prepare_string_for_xml(msg), parent=gui_parent, show_copy_button=False, det_msg=det_msg)
-                d.diffb = b = d.bb.addButton(_('See what &changed'), d.bb.AcceptRole)
+                d = MessageBox(MessageBox.INFO, _('Searching done'), '<p>'+msg, parent=gui_parent, show_copy_button=False, det_msg=det_msg)
+                d.diffb = b = d.bb.addButton(_('See what &changed'), QDialogButtonBox.ButtonRole.AcceptRole)
                 d.show_changes = False
                 b.setIcon(QIcon(I('diff.png'))), b.clicked.connect(d.accept)
                 connect_lambda(b.clicked, d, lambda d: setattr(d, 'show_changes', True))
