@@ -129,7 +129,7 @@ class UpdateNotification(QDialog):
 
     def __init__(self, calibre_version, plugin_updates, parent=None):
         QDialog.__init__(self, parent)
-        self.setAttribute(Qt.WA_QuitOnClose, False)
+        self.setAttribute(Qt.WidgetAttribute.WA_QuitOnClose, False)
         self.resize(400, 250)
         self.l = QGridLayout()
         self.setLayout(self.l)
@@ -156,14 +156,14 @@ class UpdateNotification(QDialog):
         self.cb.setChecked(config.get('new_version_notification'))
         self.cb.stateChanged.connect(self.show_future)
         self.bb = QDialogButtonBox(self)
-        b = self.bb.addButton(_('&Get update'), self.bb.AcceptRole)
+        b = self.bb.addButton(_('&Get update'), QDialogButtonBox.ButtonRole.AcceptRole)
         b.setDefault(True)
         b.setIcon(QIcon(I('arrow-down.png')))
         if plugin_updates > 0:
-            b = self.bb.addButton(_('Update &plugins'), self.bb.ActionRole)
+            b = self.bb.addButton(_('Update &plugins'), QDialogButtonBox.ButtonRole.ActionRole)
             b.setIcon(QIcon(I('plugins/plugin_updater.png')))
-            b.clicked.connect(self.get_plugins, type=Qt.QueuedConnection)
-        self.bb.addButton(self.bb.Cancel)
+            b.clicked.connect(self.get_plugins, type=Qt.ConnectionType.QueuedConnection)
+        self.bb.addButton(QDialogButtonBox.StandardButton.Cancel)
         self.l.addWidget(self.bb, 2, 0, 1, -1)
         self.bb.accepted.connect(self.accept)
         self.bb.rejected.connect(self.reject)
@@ -201,7 +201,7 @@ class UpdateMixin(object):
         if not opts.no_update_check:
             self.update_checker = CheckForUpdates(self)
             self.update_checker.signal.update_found.connect(self.update_found,
-                    type=Qt.QueuedConnection)
+                    type=Qt.ConnectionType.QueuedConnection)
             self.update_checker.start()
 
     def recalc_update_label(self, number_of_plugin_updates):
