@@ -9,7 +9,7 @@ import sys
 from threading import Thread
 
 from PyQt5.Qt import (
-    QCheckBox, QDoubleSpinBox, QFormLayout, QHBoxLayout, QIcon, QLabel,
+    QCheckBox, QDoubleSpinBox, QFormLayout, QHBoxLayout, QIcon, QLabel, QDialog,
     QLineEdit, QPageSize, QProgressDialog, QTimer, QToolButton, QVBoxLayout
 )
 
@@ -225,7 +225,7 @@ class Printing(QProgressDialog):
 def print_book(path_to_book, parent=None, book_title=None):
     book_title = book_title or os.path.splitext(os.path.basename(path_to_book))[0]
     d = PrintDialog(book_title, parent)
-    if d.exec_() == d.Accepted:
+    if d.exec_() == QDialog.DialogCode.Accepted:
         data = d.data
         data['input'] = path_to_book
         t = DoPrint(data)
