@@ -9,12 +9,12 @@ import re
 import sys
 from collections import defaultdict, namedtuple
 from hashlib import sha256
-from threading import Thread
-
 from PyQt5.Qt import (
-    QApplication, QCursor, QDockWidget, QEvent, QMenu, QMimeData, QModelIndex,
-    QPixmap, Qt, QTimer, QToolBar, QUrl, QVBoxLayout, QWidget, pyqtSignal
+    QApplication, QCursor, QDockWidget, QEvent, QMainWindow, QMenu, QMimeData,
+    QModelIndex, QPixmap, Qt, QTimer, QToolBar, QUrl, QVBoxLayout, QWidget,
+    pyqtSignal
 )
+from threading import Thread
 
 from calibre import prints
 from calibre.constants import DEBUG
@@ -24,6 +24,7 @@ from calibre.gui2 import choose_files, error_dialog
 from calibre.gui2.dialogs.drm_error import DRMErrorMessage
 from calibre.gui2.image_popup import ImagePopup
 from calibre.gui2.main_window import MainWindow
+from calibre.gui2.viewer import get_current_book_data
 from calibre.gui2.viewer.annotations import (
     AnnotationsSaveWorker, annotations_dir, parse_annotations
 )
@@ -97,7 +98,7 @@ class EbookViewer(MainWindow):
         connect_lambda(t.timeout, self, lambda self: self.save_annotations(in_book_file=False))
         self.pending_open_at = open_at
         self.base_window_title = _('E-book viewer')
-        self.setDockOptions(MainWindow.AnimatedDocks | MainWindow.AllowTabbedDocks | MainWindow.AllowNestedDocks)
+        self.setDockOptions(QMainWindow.DockOption.AnimatedDocks | QMainWindow.DockOption.AllowTabbedDocks | QMainWindow.DockOption.AllowNestedDocks)
         self.setWindowTitle(self.base_window_title)
         self.in_full_screen_mode = None
         self.image_popup = ImagePopup(self)
@@ -109,6 +110,7 @@ class EbookViewer(MainWindow):
         except EnvironmentError:
             pass
         self.current_book_data = {}
+        get_current_book_data(self.current_book_data)
         self.book_prepared.connect(self.load_finished, type=Qt.ConnectionType.QueuedConnection)
         self.dock_defs = dock_defs()
 
@@ -166,6 +168,7 @@ class EbookViewer(MainWindow):
         self.web_view.find_next.connect(self.search_widget.find_next_requested)
         self.search_widget.show_search_result.connect(self.web_view.show_search_result)
         self.web_view.search_result_not_found.connect(self.search_widget.search_result_not_found)
+        self.web_view.search_result_discovered.connect(self.search_widget.search_result_discovered)
         self.web_view.toggle_bookmarks.connect(self.toggle_bookmarks)
         self.web_view.toggle_highlights.connect(self.toggle_highlights)
         self.web_view.new_bookmark.connect(self.bookmarks_widget.create_new_bookmark)
@@ -295,7 +298,10 @@ class EbookViewer(MainWindow):
         self.inspector_dock.setVisible(not visible)
 
     def toggle_toc(self):
-        self.toc_dock.setVisible(not self.toc_dock.isVisible())
+        is_visible = self.toc_dock.isVisible()
+        self.toc_dock.setVisible(not is_visible)
+        if not is_visible:
+            self.toc.scroll_to_current_toc_node()
 
     def show_search(self, text, trigger=False):
         self.search_dock.setVisible(True)
@@ -457,6 +463,7 @@ class EbookViewer(MainWindow):
         self.loading_overlay(_('Loading book, please wait'))
         self.save_annotations()
         self.current_book_data = {}
+        get_current_book_data(self.current_book_data)
         self.search_widget.clear_searches()
         t = Thread(name='LoadBook', target=self._load_ebook_worker, args=(pathtoebook, open_at, reload_book or self.force_reload))
         t.daemon = True
@@ -513,6 +520,7 @@ class EbookViewer(MainWindow):
             self.load_ebook(data['pathtoebook'], open_at=data['open_at'], reload_book=True)
             return
         self.current_book_data = data
+        get_current_book_data(self.current_book_data)
         self.current_book_data['annotations_map'] = defaultdict(list)
         self.current_book_data['annotations_path_key'] = path_key(data['pathtoebook']) + '.json'
         self.load_book_data(cbd)
@@ -699,7 +707,7 @@ class EbookViewer(MainWindow):
         t.start()
 
     def eventFilter(self, obj, ev):
-        if ev.type() == ev.MouseMove:
+        if ev.type() == QEvent.Type.MouseMove:
             if self.cursor_hidden:
                 self.cursor_hidden = False
                 QApplication.instance().restoreOverrideCursor()
