@@ -248,6 +248,7 @@ class ViewerBridge(Bridge):
     toggle_lookup = from_js(object)
     show_search = from_js(object, object)
     search_result_not_found = from_js(object)
+    search_result_discovered = from_js(object)
     find_next = from_js(object)
     quit = from_js()
     update_current_toc_nodes = from_js(object)
@@ -449,6 +450,7 @@ class WebView(RestartingWebEngineView):
     toggle_toc = pyqtSignal()
     show_search = pyqtSignal(object, object)
     search_result_not_found = pyqtSignal(object)
+    search_result_discovered = pyqtSignal(object)
     find_next = pyqtSignal(object)
     toggle_bookmarks = pyqtSignal()
     toggle_highlights = pyqtSignal()
@@ -507,6 +509,7 @@ class WebView(RestartingWebEngineView):
         self.bridge.toggle_toc.connect(self.toggle_toc)
         self.bridge.show_search.connect(self.show_search)
         self.bridge.search_result_not_found.connect(self.search_result_not_found)
+        self.bridge.search_result_discovered.connect(self.search_result_discovered)
         self.bridge.find_next.connect(self.find_next)
         self.bridge.toggle_bookmarks.connect(self.toggle_bookmarks)
         self.bridge.toggle_highlights.connect(self.toggle_highlights)
