@@ -7,12 +7,12 @@ Miscellaneous widgets used in the GUI
 '''
 import re, os
 
-from PyQt5.Qt import (QIcon, QFont, QLabel, QListWidget, QAction,
+from PyQt5.Qt import (QIcon, QFont, QLabel, QListWidget, QAction, QEvent,
         QListWidgetItem, QTextCharFormat, QApplication, QSyntaxHighlighter,
         QCursor, QColor, QWidget, QPixmap, QSplitterHandle, QToolButton,
         Qt, pyqtSignal, QRegExp, QSize, QSplitter, QPainter, QPageSize, QPrinter,
-        QLineEdit, QComboBox, QPen, QGraphicsScene, QMenu, QStringListModel,
-        QCompleter, QTimer, QRect, QGraphicsView, QPagedPaintDevice)
+        QLineEdit, QComboBox, QPen, QGraphicsScene, QMenu, QStringListModel, QKeySequence,
+        QCompleter, QTimer, QRect, QGraphicsView, QPagedPaintDevice, QPalette, QClipboard)
 
 from calibre.constants import iswindows, ismacos
 from calibre.gui2 import (error_dialog, pixmap_to_data, gprefs,
@@ -38,7 +38,7 @@ class ProgressIndicator(QWidget):  # {{{
         self.pi = _ProgressIndicator(self)
         self.status = QLabel(self)
         self.status.setWordWrap(True)
-        self.status.setAlignment(Qt.AlignHCenter|Qt.AlignTop)
+        self.status.setAlignment(Qt.AlignmentFlag.AlignHCenter|Qt.AlignmentFlag.AlignTop)
         self.setVisible(False)
         self.pos = None
 
@@ -188,7 +188,7 @@ class FormatList(QListWidget):  # {{{
             event.acceptProposedAction()
 
     def dropEvent(self, event):
-        event.setDropAction(Qt.CopyAction)
+        event.setDropAction(Qt.DropAction.CopyAction)
         md = event.mimeData()
         # Now look for ebook files
         urls, filenames = dnd_get_files(md, self.DROPABBLE_EXTENSIONS, allow_all_extensions=True)
@@ -210,7 +210,7 @@ class FormatList(QListWidget):  # {{{
         event.acceptProposedAction()
 
     def keyPressEvent(self, event):
-        if event.key() == Qt.Key_Delete:
+        if event.key() == Qt.Key.Key_Delete:
             self.delete_format.emit()
         else:
             return QListWidget.keyPressEvent(self, event)
@@ -236,7 +236,7 @@ class ImageDropMixin(object):  # {{{
             event.acceptProposedAction()
 
     def dropEvent(self, event):
-        event.setDropAction(Qt.CopyAction)
+        event.setDropAction(Qt.DropAction.CopyAction)
         md = event.mimeData()
         pmap, data = dnd_get_local_image_and_pixmap(md)
         if pmap is not None:
@@ -295,7 +295,7 @@ class ImageDropMixin(object):  # {{{
         cb = QApplication.instance().clipboard()
         pmap = cb.pixmap()
         if pmap.isNull() and cb.supportsSelection():
-            pmap = cb.pixmap(cb.Selection)
+            pmap = cb.pixmap(QClipboard.Mode.Selection)
         if not pmap.isNull():
             self.set_pixmap(pmap)
             self.cover_changed.emit(
@@ -311,7 +311,7 @@ def draw_size(p, rect, w, h):
     f.setBold(True)
     p.setFont(f)
     sz = '\u00a0%d x %d\u00a0'%(w, h)
-    flags = Qt.AlignBottom|Qt.AlignRight|Qt.TextSingleLine
+    flags = Qt.AlignmentFlag.AlignBottom|Qt.AlignmentFlag.AlignRight|Qt.TextFlag.TextSingleLine
     szrect = p.boundingRect(rect, flags, sz)
     p.fillRect(szrect.adjusted(0, 0, 0, 4), QColor(0, 0, 0, 200))
     p.setPen(QPen(QColor(255,255,255)))
@@ -372,14 +372,14 @@ class ImageView(QWidget, ImageDropMixin):
         cw, ch = self.rect().width(), self.rect().height()
         scaled, nw, nh = fit_image(w, h, cw, ch)
         if scaled:
-            pmap = pmap.scaled(int(nw*pmap.devicePixelRatio()), int(nh*pmap.devicePixelRatio()), Qt.IgnoreAspectRatio,
-                    Qt.SmoothTransformation)
+            pmap = pmap.scaled(int(nw*pmap.devicePixelRatio()), int(nh*pmap.devicePixelRatio()), Qt.AspectRatioMode.IgnoreAspectRatio,
+                    Qt.TransformationMode.SmoothTransformation)
         w, h = int(pmap.width()/pmap.devicePixelRatio()), int(pmap.height()/pmap.devicePixelRatio())
         x = int(abs(cw - w)/2)
         y = int(abs(ch - h)/2)
         target = QRect(x, y, w, h)
         p = QPainter(self)
-        p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
+        p.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
         p.drawPixmap(target, pmap)
         if self.draw_border:
             pen = QPen()
@@ -402,7 +402,7 @@ class CoverView(QGraphicsView, ImageDropMixin):  # {{{
         ImageDropMixin.__init__(self)
         self.pixmap_size = 0, 0
         if self.show_size:
-            self.setViewportUpdateMode(self.FullViewportUpdate)
+            self.setViewportUpdateMode(QGraphicsView.ViewportUpdateMode.FullViewportUpdate)
 
     def get_pixmap(self):
         for item in self.scene.items():
@@ -523,8 +523,9 @@ class EnLineEdit(LineEditECM, QLineEdit):  # {{{
 
     def event(self, ev):
         # See https://bugreports.qt.io/browse/QTBUG-46911
-        if ev.type() == ev.ShortcutOverride and (
-                hasattr(ev, 'key') and ev.key() in (Qt.Key_Left, Qt.Key_Right) and (ev.modifiers() & ~Qt.KeypadModifier) == Qt.ControlModifier):
+        if ev.type() == QEvent.Type.ShortcutOverride and (
+                hasattr(ev, 'key') and ev.key() in (Qt.Key.Key_Left, Qt.Key.Key_Right) and (
+                    ev.modifiers() & ~Qt.KeyboardModifier.KeypadModifier) == Qt.KeyboardModifier.ControlModifier):
             ev.accept()
         return QLineEdit.event(self, ev)
 
@@ -574,7 +575,7 @@ class CompleteLineEdit(EnLineEdit):  # {{{
         self.textChanged.connect(self.text_changed)
 
         self.completer = ItemsCompleter(self, complete_items)
-        self.completer.setCaseSensitivity(Qt.CaseInsensitive)
+        self.completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
 
         self.completer.activated[native_string_type].connect(self.complete_text)
 
@@ -630,14 +631,14 @@ class EnComboBox(QComboBox):  # {{{
     def __init__(self, *args):
         QComboBox.__init__(self, *args)
         self.setLineEdit(EnLineEdit(self))
-        self.completer().setCaseSensitivity(Qt.CaseInsensitive)
+        self.completer().setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
         self.setMinimumContentsLength(20)
 
     def text(self):
         return unicode_type(self.currentText())
 
     def setText(self, text):
-        idx = self.findText(text, Qt.MatchFixedString|Qt.MatchCaseSensitive)
+        idx = self.findText(text, Qt.MatchFlag.MatchFixedString|Qt.MatchFlag.MatchCaseSensitive)
         if idx == -1:
             self.insertItem(0, text)
             idx = 0
@@ -671,7 +672,7 @@ class HistoryLineEdit(QComboBox):  # {{{
     def __init__(self, *args):
         QComboBox.__init__(self, *args)
         self.setEditable(True)
-        self.setInsertPolicy(self.NoInsert)
+        self.setInsertPolicy(QComboBox.InsertPolicy.NoInsert)
         self.setMaxCount(10)
         self.setClearButtonEnabled = self.lineEdit().setClearButtonEnabled
         self.textChanged = self.editTextChanged
@@ -867,9 +868,9 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
         p = QApplication.instance().palette()
         for name, color, bold, italic in (
                 ("normal", None, False, False),
-                ("keyword", p.color(p.Link).name(), True, False),
-                ("builtin", p.color(p.Link).name(), False, False),
-                ("constant", p.color(p.Link).name(), False, False),
+                ("keyword", p.color(QPalette.ColorRole.Link).name(), True, False),
+                ("builtin", p.color(QPalette.ColorRole.Link).name(), False, False),
+                ("constant", p.color(QPalette.ColorRole.Link).name(), False, False),
                 ("decorator", "#0000E0", False, False),
                 ("comment", "#007F00", False, True),
                 ("string", "#808000", False, False),
@@ -881,7 +882,7 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
             if color is not None:
                 fmt.setForeground(QColor(color))
             if bold:
-                fmt.setFontWeight(QFont.Bold)
+                fmt.setFontWeight(QFont.Weight.Bold)
             if italic:
                 fmt.setFontItalic(italic)
             cls.Formats[name] = fmt
@@ -957,7 +958,7 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
                                PythonHighlighter.Formats["string"])
 
     def rehighlight(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
         QSyntaxHighlighter.rehighlight(self)
         QApplication.restoreOverrideCursor()
 
@@ -974,9 +975,9 @@ class SplitterHandle(QSplitterHandle):
     def __init__(self, orientation, splitter):
         QSplitterHandle.__init__(self, orientation, splitter)
         splitter.splitterMoved.connect(self.splitter_moved,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.double_clicked.connect(splitter.double_clicked,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.highlight = False
         self.setToolTip(_('Drag to resize')+' '+splitter.label)
 
@@ -1002,13 +1003,13 @@ class LayoutButton(QToolButton):
         self.splitter = splitter
         if splitter is not None:
             splitter.state_changed.connect(self.update_state)
-        self.setCursor(Qt.PointingHandCursor)
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
         self.shortcut = shortcut or ''
 
     def update_shortcut(self, action_toggle=None):
         action_toggle = action_toggle or getattr(self, 'action_toggle', None)
         if action_toggle:
-            sc = ', '.join(sc.toString(sc.NativeText)
+            sc = ', '.join(sc.toString(QKeySequence.SequenceFormat.NativeText)
                                 for sc in action_toggle.shortcuts())
             self.shortcut = sc or ''
             self.update_text()
@@ -1035,7 +1036,7 @@ class LayoutButton(QToolButton):
             self.set_state_to_hide()
 
     def mouseReleaseEvent(self, ev):
-        if ev.button() == Qt.RightButton:
+        if ev.button() == Qt.MouseButton.RightButton:
             from calibre.gui2.ui import get_gui
             gui = get_gui()
             if self.icname == 'search':
@@ -1055,12 +1056,15 @@ class LayoutButton(QToolButton):
 class Splitter(QSplitter):
 
     state_changed = pyqtSignal(object)
+    reapply_sizes = pyqtSignal(object)
 
     def __init__(self, name, label, icon, initial_show=True,
             initial_side_size=120, connect_button=True,
-            orientation=Qt.Horizontal, side_index=0, parent=None,
+            orientation=Qt.Orientation.Horizontal, side_index=0, parent=None,
             shortcut=None, hide_handle_on_single_panel=True):
         QSplitter.__init__(self, parent)
+        self.reapply_sizes.connect(self.setSizes, type=Qt.ConnectionType.QueuedConnection)
+        self.hide_handle_on_single_panel = hide_handle_on_single_panel
         if hide_handle_on_single_panel:
             self.state_changed.connect(self.update_handle_width)
         self.original_handle_width = self.handleWidth()
@@ -1076,7 +1080,7 @@ class Splitter(QSplitter):
         self.label = label
         self.initial_side_size = initial_side_size
         self.initial_show = initial_show
-        self.splitterMoved.connect(self.splitter_moved, type=Qt.QueuedConnection)
+        self.splitterMoved.connect(self.splitter_moved, type=Qt.ConnectionType.QueuedConnection)
         self.button = LayoutButton(icon, label, self, shortcut=shortcut)
         if connect_button:
             self.button.clicked.connect(self.double_clicked)
@@ -1130,7 +1134,7 @@ class Splitter(QSplitter):
 
     @property
     def save_name(self):
-        ori = 'horizontal' if self.orientation() == Qt.Horizontal \
+        ori = 'horizontal' if self.orientation() == Qt.Orientation.Horizontal \
                 else 'vertical'
         return self._name + '_' + ori
 
@@ -1149,18 +1153,26 @@ class Splitter(QSplitter):
     def side_index_size(self, val):
         if self.count() < 2:
             return
-        if val == 0 and not self.is_side_index_hidden:
+        side_index_hidden = self.is_side_index_hidden
+        if val == 0 and not side_index_hidden:
             self.save_state()
         sizes = list(self.sizes())
         for i in range(len(sizes)):
             sizes[i] = val if i == self.side_index else 10
         self.setSizes(sizes)
-        total = sum(self.sizes())
         sizes = list(self.sizes())
+        total = sum(sizes)
+        total_needs_adjustment = self.hide_handle_on_single_panel and side_index_hidden
+        if total_needs_adjustment:
+            total -= self.original_handle_width
         for i in range(len(sizes)):
             sizes[i] = val if i == self.side_index else total-val
         self.setSizes(sizes)
         self.initialize()
+        if total_needs_adjustment:
+            # the handle visibility and therefore size distribution will change
+            # when the event loop ticks
+            self.reapply_sizes.emit(sizes)
 
     def do_resize(self, *args):
         orig = self.desired_side_size
@@ -1252,14 +1264,14 @@ class PaperSizes(QComboBox):  # {{{
             if iswindows or ismacos:
                 # On Linux, this can cause Qt to load the system cups plugin
                 # which can crash: https://bugs.launchpad.net/calibre/+bug/1861741
-                PaperSizes.system_default_paper_size = 'letter' if QPrinter().pageSize() == QPagedPaintDevice.Letter else 'a4'
+                PaperSizes.system_default_paper_size = 'letter' if QPrinter().pageSize() == QPagedPaintDevice.PageSize.Letter else 'a4'
         if not choices:
             from calibre.ebooks.conversion.plugins.pdf_output import PAPER_SIZES
             choices = PAPER_SIZES
         for a in sorted(choices, key=numeric_sort_key):
             s = getattr(QPageSize, a.capitalize())
             sz = QPageSize.definitionSize(s)
-            unit = {QPageSize.Millimeter: 'mm', QPageSize.Inch: 'inch'}[QPageSize.definitionUnits(s)]
+            unit = {QPageSize.Unit.Millimeter: 'mm', QPageSize.Unit.Inch: 'inch'}[QPageSize.definitionUnits(s)]
             name = '{} ({:g} x {:g} {})'.format(QPageSize.name(s), sz.width(), sz.height(), unit)
             self.addItem(name, a)
 
