@@ -117,9 +117,9 @@ class RelaySetup(QDialog):
                 self.ptoggle = QCheckBox(_('&Show password'), self)
                 l.addWidget(self.ptoggle, r, 2)
                 self.ptoggle.stateChanged.connect(
-                        lambda s: self.password.setEchoMode(self.password.Normal if s == Qt.CheckState.Checked else self.password.Password))
+                        lambda s: self.password.setEchoMode(QLineEdit.EchoMode.Normal if s == Qt.CheckState.Checked else QLineEdit.EchoMode.Password))
         self.username.setText(service['username'])
-        self.password.setEchoMode(self.password.Password)
+        self.password.setEchoMode(QLineEdit.EchoMode.Password)
         self.bl = QLabel('<p>' + _(
             'If you plan to use email to send books to your Kindle, remember to'
             ' add your %s email address to the allowed email addresses in your '
@@ -174,8 +174,8 @@ class SendEmail(QWidget, Ui_Form):
             button.clicked.connect(partial(self.create_service_relay, x))
         self.relay_show_password.stateChanged.connect(
          lambda state : self.relay_password.setEchoMode(
-             self.relay_password.Password if
-             state == 0 else self.relay_password.Normal))
+             QLineEdit.EchoMode.Password if
+             state == 0 else QLineEdit.EchoMode.Normal))
         self.test_email_button.clicked.connect(self.test_email)
 
     def changed(self, *args):
@@ -238,7 +238,7 @@ class SendEmail(QWidget, Ui_Form):
                         'Google recently deliberately broke their email sending protocol (SMTP) support in'
                         ' an attempt to force everyone to use their web interface so they can'
                         ' show you more ads. They are trying to claim that SMTP is insecure,'
-                        ' that is incorrect and simply an excuse. To use a gmail account'
+                        ' that is incorrect and simply an excuse. To use a Gmail account'
                         ' you will need to "allow less secure apps" as described'
                         ' <a href="https://support.google.com/accounts/answer/6010255">here</a>.'),
                     'at_in_username': True,
@@ -250,7 +250,7 @@ class SendEmail(QWidget, Ui_Form):
                     'username': '',
                     'url': 'www.hotmail.com',
                     'extra': _('If you are setting up a new'
-                        ' hotmail account, Microsoft requires that you '
+                        ' Hotmail account, Microsoft requires that you '
                         ' verify your account periodically, before it'
                         ' will let calibre send email.'),
                     'at_in_username': True,
