@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -26,8 +25,8 @@ def do_detach(fork=True, setsid=True, redirect=True):
     if setsid:
         os.setsid()
     if redirect:
-        from calibre.constants import plugins
-        plugins['speedup'][0].detach(os.devnull)
+        from calibre_extensions.speedup import detach
+        detach(os.devnull)
     is_detached = True
 
 
@@ -82,6 +81,30 @@ def ebook_viewer(args=sys.argv):
         main(args)
 
 
+def store_dialog(args=sys.argv):
+    detach_gui()
+    init_dbus()
+    from calibre.gui2.store.web_store import main
+    main(args)
+
+
+def webengine_dialog(**kw):
+    detach_gui()
+    init_dbus()
+    from calibre.debug import load_user_plugins
+    load_user_plugins()
+    import importlib
+    m = importlib.import_module(kw.pop('module'))
+    getattr(m, kw.pop('entry_func', 'main'))(**kw)
+
+
+def toc_dialog(**kw):
+    detach_gui()
+    init_dbus()
+    from calibre.gui2.toc.main import main
+    main(**kw)
+
+
 def gui_ebook_edit(path=None, notify=None):
     ' For launching the editor from inside calibre '
     init_dbus()
