@@ -1,6 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -29,7 +30,7 @@ def find_folders_under(root, db, add_root=True,  # {{{
 
     root = os.path.abspath(root)
 
-    ans = set([])
+    ans = set()
     for dirpath, dirnames, __ in os.walk(root, topdown=True, followlinks=follow_links):
         if cancel_callback():
             break
