@@ -1,13 +1,13 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import itertools, time, traceback, locale
-from itertools import repeat, izip, imap
+import time, traceback, locale
+from itertools import repeat
 from datetime import timedelta
 from threading import Thread
 
@@ -19,7 +19,10 @@ from calibre.utils.localization import (canonicalize_lang, lang_map, get_udc)
 from calibre.db.search import CONTAINS_MATCH, EQUALS_MATCH, REGEXP_MATCH, _match
 from calibre.ebooks.metadata import title_sort, author_to_author_sort
 from calibre.ebooks.metadata.opf2 import metadata_to_opf
-from calibre import prints
+from calibre import prints, force_unicode
+from polyglot.builtins import (iteritems, itervalues, map,
+        unicode_type, string_or_bytes, zip, cmp)
+
 
 class MetadataBackup(Thread):  # {{{
     '''
@@ -122,17 +125,22 @@ class MetadataBackup(Thread):  # {{{
 # This is a global for performance
 pref_use_primary_find_in_search = False
 
+
 def set_use_primary_find_in_search(toWhat):
     global pref_use_primary_find_in_search
     pref_use_primary_find_in_search = toWhat
 
+
 y, c, n, u = map(icu_lower, (_('yes'), _('checked'), _('no'), _('unchecked')))
 yes_vals = {y, c, 'true'}
 no_vals = {n, u, 'false'}
 del y, c, n, u
 
+
 def force_to_bool(val):
-    if isinstance(val, (str, unicode)):
+    if isinstance(val, (bytes, unicode_type)):
+        if isinstance(val, bytes):
+            val = force_unicode(val)
         try:
             val = icu_lower(val)
             if not val:
@@ -147,11 +155,17 @@ def force_to_bool(val):
             val = None
     return val
 
+
 class CacheRow(list):  # {{{
 
-    def __init__(self, db, composites, val, series_col, series_sort_col):
+    def __init__(self, db, composites, datetimes, val, series_col, series_sort_col):
+        from calibre.db.tables import c_parse
         self.db = db
         self._composites = composites
+        for num in datetimes:
+            val[num] = c_parse(val[num])
+            if val[num] is UNDEFINED_DATE:
+                val[num] = None
         list.__init__(self, val)
         self._must_do = len(composites) > 0
         self._series_col = series_col
@@ -196,19 +210,25 @@ class CacheRow(list):  # {{{
 
 # }}}
 
+
 class ResultCache(SearchQueryParser):  # {{{
 
     '''
     Stores sorted and filtered metadata in memory.
     '''
+
     def __init__(self, FIELD_MAP, field_metadata, db_prefs=None):
         self.FIELD_MAP = FIELD_MAP
         self.db_prefs = db_prefs
         self.composites = {}
+        self.datetimes = set()
         self.udc = get_udc()
         for key in field_metadata:
-            if field_metadata[key]['datatype'] == 'composite':
+            dt = field_metadata[key]['datatype']
+            if dt == 'composite':
                 self.composites[field_metadata[key]['rec_index']] = key
+            elif dt == 'datetime':
+                self.datetimes.add(field_metadata[key]['rec_index'])
         self.series_col = field_metadata['series']['rec_index']
         self.series_sort_col = field_metadata['series_sort']['rec_index']
         self._data = []
@@ -258,7 +278,7 @@ class ResultCache(SearchQueryParser):  # {{{
     # Search functions {{{
 
     def universal_set(self):
-        return set([i[0] for i in self._data if i is not None])
+        return {i[0] for i in self._data if i is not None}
 
     def change_search_locations(self, locations):
         self.sqp_change_locations(locations)
@@ -326,7 +346,7 @@ class ResultCache(SearchQueryParser):  # {{{
     untrans_daysago_len = len('_daysago')
 
     def get_dates_matches(self, location, query, candidates):
-        matches = set([])
+        matches = set()
         if len(query) < 2:
             return matches
 
@@ -340,7 +360,7 @@ class ResultCache(SearchQueryParser):  # {{{
                 if item is None:
                     continue
                 v = item[loc]
-                if isinstance(v, (str, unicode)):
+                if isinstance(v, (bytes, unicode_type)):
                     v = parse_date(v)
                 if v is None or v <= UNDEFINED_DATE:
                     matches.add(item[0])
@@ -351,7 +371,7 @@ class ResultCache(SearchQueryParser):  # {{{
                 if item is None:
                     continue
                 v = item[loc]
-                if isinstance(v, (str, unicode)):
+                if isinstance(v, (bytes, unicode_type)):
                     v = parse_date(v)
                 if v is not None and v > UNDEFINED_DATE:
                     matches.add(item[0])
@@ -363,7 +383,7 @@ class ResultCache(SearchQueryParser):  # {{{
                 (p, relop) = self.date_search_relops[k]
                 query = query[p:]
         if relop is None:
-                (p, relop) = self.date_search_relops['=']
+            (p, relop) = self.date_search_relops['=']
 
         if query in self.local_today:
             qd = now()
@@ -375,7 +395,7 @@ class ResultCache(SearchQueryParser):  # {{{
             qd = now()
             field_count = 2
         elif query.endswith(self.local_daysago) or query.endswith(self.untrans_daysago):
-            num = query[0:-(self.local_daysago_len if query.endswith(self.local_daysago) else self.untrans_daysago_len)]
+            num = query[0:-(self.untrans_daysago_len if query.endswith(self.untrans_daysago) else self.local_daysago_len)]
             try:
                 qd = now() - timedelta(int(num))
             except:
@@ -395,7 +415,7 @@ class ResultCache(SearchQueryParser):  # {{{
             if item is None or item[loc] is None:
                 continue
             v = item[loc]
-            if isinstance(v, (str, unicode)):
+            if isinstance(v, (bytes, unicode_type)):
                 v = parse_date(v)
             if relop(v, qd, field_count):
                 matches.add(item[0])
@@ -412,7 +432,7 @@ class ResultCache(SearchQueryParser):  # {{{
                     }
 
     def get_numeric_matches(self, location, query, candidates, val_func=None):
-        matches = set([])
+        matches = set()
         if len(query) == 0:
             return matches
 
@@ -440,13 +460,13 @@ class ResultCache(SearchQueryParser):  # {{{
                     (p, relop) = self.numeric_search_relops[k]
                     query = query[p:]
             if relop is None:
-                    (p, relop) = self.numeric_search_relops['=']
+                (p, relop) = self.numeric_search_relops['=']
 
             if dt == 'int':
                 cast = lambda x: int(x)
             elif dt == 'rating':
                 cast = lambda x: 0 if x is None else int(x)
-                adjust = lambda x: x/2
+                adjust = lambda x: x//2
             elif dt in ('float', 'composite'):
                 cast = lambda x : float(x)
             else:  # count operation
@@ -479,7 +499,7 @@ class ResultCache(SearchQueryParser):  # {{{
         return matches
 
     def get_user_category_matches(self, location, query, candidates):
-        matches = set([])
+        matches = set()
         if self.db_prefs is None or len(query) < 2:
             return matches
         user_cats = self.db_prefs.get('user_categories', [])
@@ -502,7 +522,7 @@ class ResultCache(SearchQueryParser):  # {{{
         return matches
 
     def get_keypair_matches(self, location, query, candidates):
-        matches = set([])
+        matches = set()
         if query.find(':') >= 0:
             q = [q.strip() for q in query.split(':')]
             if len(q) != 2:
@@ -620,7 +640,7 @@ class ResultCache(SearchQueryParser):  # {{{
             allow_recursion=True):
         # If candidates is not None, it must not be modified. Changing its
         # value will break query optimization in the search parser
-        matches = set([])
+        matches = set()
         if candidates is None:
             candidates = self.universal_set()
         if len(candidates) == 0:
@@ -661,7 +681,7 @@ class ResultCache(SearchQueryParser):  # {{{
             # apply the limit if appropriate
             if location == 'all' and prefs['limit_search_columns'] and \
                             prefs['limit_search_columns_to']:
-                terms = set([])
+                terms = set()
                 for l in prefs['limit_search_columns_to']:
                     l = icu_lower(l.strip())
                     if l and l != 'all' and l in self.all_search_locations:
@@ -722,7 +742,7 @@ class ResultCache(SearchQueryParser):  # {{{
             # everything else, or 'all' matches
             matchkind, query = self._matchkind(query)
 
-            if not isinstance(query, unicode):
+            if not isinstance(query, unicode_type):
                 query = query.decode('utf-8')
 
             db_col = {}
@@ -764,7 +784,7 @@ class ResultCache(SearchQueryParser):  # {{{
                     q = canonicalize_lang(query)
                     if q is None:
                         lm = lang_map()
-                        rm = {v.lower():k for k,v in lm.iteritems()}
+                        rm = {v.lower():k for k,v in iteritems(lm)}
                         q = rm.get(query, query)
                 else:
                     q = query
@@ -783,7 +803,7 @@ class ResultCache(SearchQueryParser):  # {{{
                         continue
 
                     if q == 'true' and matchkind == CONTAINS_MATCH:
-                        if isinstance(item[loc], basestring):
+                        if isinstance(item[loc], string_or_bytes):
                             if item[loc].strip() == '':
                                 continue
                         matches.add(item[0])
@@ -831,7 +851,7 @@ class ResultCache(SearchQueryParser):  # {{{
     def _build_restriction_string(self, restriction):
         if self.base_restriction:
             if restriction:
-                return u'(%s) and (%s)' % (self.base_restriction, restriction)
+                return '(%s) and (%s)' % (self.base_restriction, restriction)
             else:
                 return self.base_restriction
         else:
@@ -847,13 +867,13 @@ class ResultCache(SearchQueryParser):  # {{{
         else:
             q = query
             if search_restriction:
-                q = u'(%s) and (%s)' % (search_restriction, query)
+                q = '(%s) and (%s)' % (search_restriction, query)
         if not q:
             if set_restriction_count:
                 self.search_restriction_book_count = len(self._map)
             return list(self._map)
         matches = self.parse(q)
-        tmap = list(itertools.repeat(False, len(self._data)))
+        tmap = list(repeat(False, len(self._data)))
         for x in matches:
             tmap[x] = True
         rv = [x for x in self._map if tmap[x]]
@@ -904,18 +924,18 @@ class ResultCache(SearchQueryParser):  # {{{
         '''
         if not hasattr(id_dict, 'items'):
             # Simple list. Make it a dict of string 'true'
-            self.marked_ids_dict = dict.fromkeys(id_dict, u'true')
+            self.marked_ids_dict = dict.fromkeys(id_dict, 'true')
         else:
             # Ensure that all the items in the dict are text
-            self.marked_ids_dict = dict(izip(id_dict.iterkeys(), imap(unicode,
-                id_dict.itervalues())))
+            self.marked_ids_dict = dict(zip(iter(id_dict), map(unicode_type,
+                itervalues(id_dict))))
 
         # Set the values in the cache
         marked_col = self.FIELD_MAP['marked']
         for r in self.iterall():
             r[marked_col] = None
 
-        for id_, val in self.marked_ids_dict.iteritems():
+        for id_, val in iteritems(self.marked_ids_dict):
             try:
                 self._data[id_][marked_col] = val
             except:
@@ -980,7 +1000,7 @@ class ResultCache(SearchQueryParser):  # {{{
         '''
         for id in ids:
             try:
-                self._data[id] = CacheRow(db, self.composites,
+                self._data[id] = CacheRow(db, self.composites, self.datetimes,
                         db.conn.get('SELECT * from meta2 WHERE id=?', (id,))[0],
                         self.series_col, self.series_sort_col)
                 self._data[id].append(db.book_on_device_string(id))
@@ -990,7 +1010,7 @@ class ResultCache(SearchQueryParser):  # {{{
             except IndexError:
                 return None
         try:
-            return map(self.row, ids)
+            return list(map(self.row, ids))
         except ValueError:
             pass
         return None
@@ -1000,7 +1020,7 @@ class ResultCache(SearchQueryParser):  # {{{
             return
         self._data.extend(repeat(None, max(ids)-len(self._data)+2))
         for id in ids:
-            self._data[id] = CacheRow(db, self.composites,
+            self._data[id] = CacheRow(db, self.composites, self.datetimes,
                         db.conn.get('SELECT * from meta2 WHERE id=?', (id,))[0],
                         self.series_col, self.series_sort_col)
             self._data[id].append(db.book_on_device_string(id))
@@ -1029,9 +1049,9 @@ class ResultCache(SearchQueryParser):  # {{{
         db.initialize_template_cache()
 
         temp = db.conn.get('SELECT * FROM meta2')
-        self._data = list(itertools.repeat(None, temp[-1][0]+2)) if temp else []
+        self._data = list(repeat(None, temp[-1][0]+2)) if temp else []
         for r in temp:
-            self._data[r[0]] = CacheRow(db, self.composites, r,
+            self._data[r[0]] = CacheRow(db, self.composites, self.datetimes, r,
                                         self.series_col, self.series_sort_col)
             self._uuid_map[self._data[r[0]][self._uuid_column_index]] = r[0]
 
@@ -1042,7 +1062,7 @@ class ResultCache(SearchQueryParser):  # {{{
                 item.extend((None, None))
 
         marked_col = self.FIELD_MAP['marked']
-        for id_,val in self.marked_ids_dict.iteritems():
+        for id_,val in iteritems(self.marked_ids_dict):
             try:
                 self._data[id_][marked_col] = val
             except:
@@ -1089,25 +1109,45 @@ class ResultCache(SearchQueryParser):  # {{{
         if only_ids is None:
             self._map.sort(key=keyg)
 
-            tmap = list(itertools.repeat(False, len(self._data)))
+            tmap = list(repeat(False, len(self._data)))
             for x in self._map_filtered:
                 tmap[x] = True
             self._map_filtered = [x for x in self._map if tmap[x]]
         else:
             only_ids.sort(key=keyg)
 
+
 class SortKey(object):
 
     def __init__(self, orders, values):
         self.orders, self.values = orders, values
 
-    def __cmp__(self, other):
+    def compare_to_other(self, other):
         for i, ascending in enumerate(self.orders):
             ans = cmp(self.values[i], other.values[i])
             if ans != 0:
                 return ans * ascending
         return 0
 
+    def __eq__(self, other):
+        return self.compare_to_other(other) == 0
+
+    def __ne__(self, other):
+        return self.compare_to_other(other) != 0
+
+    def __lt__(self, other):
+        return self.compare_to_other(other) < 0
+
+    def __le__(self, other):
+        return self.compare_to_other(other) <= 0
+
+    def __gt__(self, other):
+        return self.compare_to_other(other) > 0
+
+    def __ge__(self, other):
+        return self.compare_to_other(other) >= 0
+
+
 class SortKeyGenerator(object):
 
     def __init__(self, fields, field_metadata, data, db_prefs):
@@ -1174,7 +1214,7 @@ class SortKeyGenerator(object):
                 else:
                     if self.library_order:
                         try:
-                            lang = record[self.lang_idx].partition(u',')[0]
+                            lang = record[self.lang_idx].partition(',')[0]
                         except (AttributeError, ValueError, KeyError,
                                 IndexError, TypeError):
                             lang = None
@@ -1207,5 +1247,3 @@ class SortKeyGenerator(object):
     # }}}
 
 # }}}
-
-
