@@ -13,7 +13,6 @@ from calibre import strftime
 from calibre.customize import CatalogPlugin
 from calibre.library.catalogs import FIELDS, TEMPLATE_ALLOWED_FIELDS
 from calibre.customize.conversion import DummyReporter
-from calibre.constants import preferred_encoding
 from calibre.ebooks.metadata import format_isbn
 from polyglot.builtins import filter, string_or_bytes, unicode_type
 
@@ -113,7 +112,6 @@ class BIBTEX(CatalogPlugin):
         from calibre.utils.html2text import html2text
         from calibre.utils.bibtex import BibTeX
         from calibre.library.save_to_disk import preprocess_template
-        from calibre.utils.date import now as nowf
         from calibre.utils.logging import default_log as log
         from calibre.utils.filenames import ascii_text
 
@@ -395,9 +393,9 @@ class BIBTEX(CatalogPlugin):
                 for entry in data:
                     entry['ondevice'] = db.catalog_plugin_on_device_temp_mapping[entry['id']]['ondevice']
 
-            outfile.write('%%%Calibre catalog\n%%%{0} entries in catalog\n\n'.format(nb_entries))
+            # outfile.write('%%%Calibre catalog\n%%%{0} entries in catalog\n\n'.format(nb_entries))
             outfile.write('@preamble{"This catalog of %d entries was generated by calibre on %s"}\n\n'
-                % (nb_entries, nowf().strftime("%A, %d. %B %Y %H:%M").decode(preferred_encoding)))
+                % (nb_entries, strftime("%A, %d. %B %Y %H:%M")))
 
             for entry in data:
                 outfile.write(create_bibtex_entry(entry, fields, bib_entry, template_citation,
