@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -17,6 +16,7 @@ from calibre.library import current_library_name
 from calibre.library.catalogs import AuthorSortMismatchException, EmptyCatalogException
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.localization import calibre_langcode_to_name, canonicalize_lang, get_lang
+from polyglot.builtins import unicode_type
 
 Option = namedtuple('Option', 'option, default, dest, action, help')
 
@@ -61,7 +61,7 @@ class EPUB_MOBI(CatalogPlugin):
                            "Default: '%default'\n"
                            "Applies to: AZW3, EPUB, MOBI output formats")),
                    Option('--exclude-genre',
-                          default='\\[.+\\]|^\\+$',
+                          default=r'\[.+\]|^\+$',
                           dest='exclude_genre',
                           action=None,
                           help=_("Regex describing tags to exclude as genres.\n"
@@ -141,9 +141,9 @@ class EPUB_MOBI(CatalogPlugin):
                           dest='merge_comments_rule',
                           action=None,
                           help=_("#<custom field>:[before|after]:[True|False] specifying:\n"
-                          " <custom field> Custom field containing notes to merge with Comments\n"
-                          " [before|after] Placement of notes with respect to Comments\n"
-                          " [True|False] - A horizontal rule is inserted between notes and Comments\n"
+                          " <custom field> Custom field containing notes to merge with comments\n"
+                          " [before|after] Placement of notes with respect to comments\n"
+                          " [True|False] - A horizontal rule is inserted between notes and comments\n"
                           "Default: '%default'\n"
                           "Applies to: AZW3, EPUB, MOBI output formats")),
                    Option('--output-profile',
@@ -265,7 +265,7 @@ class EPUB_MOBI(CatalogPlugin):
 
         build_log = []
 
-        build_log.append(u"%s('%s'): Generating %s %sin %s environment, locale: '%s'" %
+        build_log.append("%s('%s'): Generating %s %sin %s environment, locale: '%s'" %
             (self.name,
              current_library_name(),
              self.fmt,
@@ -283,23 +283,23 @@ class EPUB_MOBI(CatalogPlugin):
         if opts.connected_device['is_device_connected'] and \
            opts.connected_device['kind'] == 'device':
             if opts.connected_device['serial']:
-                build_log.append(u" connected_device: '%s' #%s%s " %
+                build_log.append(" connected_device: '%s' #%s%s " %
                     (opts.connected_device['name'],
                      opts.connected_device['serial'][0:4],
                      'x' * (len(opts.connected_device['serial']) - 4)))
                 for storage in opts.connected_device['storage']:
                     if storage:
-                        build_log.append(u"  mount point: %s" % storage)
+                        build_log.append("  mount point: %s" % storage)
             else:
-                build_log.append(u" connected_device: '%s'" % opts.connected_device['name'])
+                build_log.append(" connected_device: '%s'" % opts.connected_device['name'])
                 try:
                     for storage in opts.connected_device['storage']:
                         if storage:
-                            build_log.append(u"  mount point: %s" % storage)
+                            build_log.append("  mount point: %s" % storage)
                 except:
-                    build_log.append(u"  (no mount points)")
+                    build_log.append("  (no mount points)")
         else:
-            build_log.append(u" connected_device: '%s'" % opts.connected_device['name'])
+            build_log.append(" connected_device: '%s'" % opts.connected_device['name'])
 
         opts_dict = vars(opts)
         if opts_dict['ids']:
@@ -338,7 +338,7 @@ class EPUB_MOBI(CatalogPlugin):
             sections_list.insert(0, 'Authors')
             opts.generate_authors = True
 
-        opts.log(u" Sections: %s" % ', '.join(sections_list))
+        opts.log(" Sections: %s" % ', '.join(sections_list))
         opts.section_list = sections_list
 
         # Limit thumb_width to 1.0" - 2.0"
@@ -398,7 +398,7 @@ class EPUB_MOBI(CatalogPlugin):
 
         if opts.verbose:
             log.info(" Begin catalog source generation (%s)" %
-                     str(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
+                     unicode_type(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
 
         # Launch the Catalog builder
         catalog = CatalogBuilder(db, opts, self, report_progress=notification)
@@ -407,7 +407,7 @@ class EPUB_MOBI(CatalogPlugin):
             catalog.build_sources()
             if opts.verbose:
                 log.info(" Completed catalog source generation (%s)\n"  %
-                         str(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
+                         unicode_type(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
         except (AuthorSortMismatchException, EmptyCatalogException) as e:
             log.error(" *** Terminated catalog generation: %s ***" % e)
         except:
@@ -500,7 +500,7 @@ class EPUB_MOBI(CatalogPlugin):
 
             if opts.verbose:
                 log.info(" Catalog creation complete (%s)\n" %
-                     str(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
+                     unicode_type(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
 
         # returns to gui2.actions.catalog:catalog_generated()
         return catalog.error
