@@ -1,8 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 # License: GPLv3 Copyright: 2010, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 import re
 
@@ -95,6 +94,7 @@ def comments_to_html(comments):
     open_pTag = False
 
     all_tokens = list(soup.contents)
+    inline_tags = ('br', 'b', 'i', 'em', 'strong', 'span', 'font', 'a', 'hr')
     for token in all_tokens:
         if isinstance(token,  (CData, Comment, Declaration, ProcessingInstruction)):
             continue
@@ -105,8 +105,7 @@ def comments_to_html(comments):
                 ptc = 0
             pTag.insert(ptc, token)
             ptc += 1
-        elif token.name in ['br', 'b', 'i', 'em', 'strong', 'span', 'font', 'a',
-                'hr']:
+        elif token.name in inline_tags:
             if not open_pTag:
                 pTag = result.new_tag('p')
                 open_pTag = True
