@@ -1,16 +1,21 @@
-from __future__ import print_function
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
+
 '''
 Backend that implements storage of ebooks in an sqlite database.
 '''
+
 import sqlite3 as sqlite
-import datetime, re, cPickle, sre_constants
+import datetime, re, sre_constants
 from zlib import compress, decompress
 
 from calibre.ebooks.metadata import MetaInformation
 from calibre.ebooks.metadata import string_to_authors
+from calibre.utils.serialize import pickle_loads, pickle_dumps
 from calibre import isbytestring
+from polyglot.builtins import unicode_type, filter, map
 
 
 class Concatenate(object):
@@ -25,11 +30,16 @@ class Concatenate(object):
             self.ans += value + self.sep
 
     def finalize(self):
-        if not self.ans:
-            return None
-        if self.sep:
-            return self.ans[:-len(self.sep)]
-        return self.ans
+        try:
+            if not self.ans:
+                return None
+            if self.sep:
+                return self.ans[:-len(self.sep)]
+            return self.ans
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            raise
 
 
 class Connection(sqlite.Connection):
@@ -45,12 +55,12 @@ class Connection(sqlite.Connection):
 
 
 def _connect(path):
-    if isinstance(path, unicode):
+    if isinstance(path, unicode_type):
         path = path.encode('utf-8')
     conn =  sqlite.connect(path, factory=Connection, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
     conn.row_factory = lambda cursor, row : list(row)
     conn.create_aggregate('concat', 1, Concatenate)
-    title_pat = re.compile('^(A|The|An)\\s+', re.IGNORECASE)
+    title_pat = re.compile(r'^(A|The|An)\s+', re.IGNORECASE)
 
     def title_sort(title):
         match = title_pat.search(title)
@@ -820,13 +830,10 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         #        _lock_file = None
         self.conn.close()
 
-    @dynamic_property
+    @property
     def user_version(self):
-        doc = 'The user version of this database'
-
-        def fget(self):
-            return self.conn.get('pragma user_version;', all=False)
-        return property(doc=doc, fget=fget)
+        'The user version of this database'
+        return self.conn.get('pragma user_version;', all=False)
 
     def is_empty(self):
         return not self.conn.get('SELECT id FROM books LIMIT 1', all=False)
@@ -864,7 +871,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         self.conn.commit()
 
     def refresh_ids(self, ids):
-        indices = map(self.index, ids)
+        indices = list(map(self.index, ids))
         for id, idx in zip(ids, indices):
             row = self.conn.get('SELECT * from meta WHERE id=?', (id,), all=False)
             self.data[idx] = row
@@ -1016,7 +1023,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         if not ans:
             return []
         ans = [id[0] for id in ans]
-        ans.sort(cmp=lambda x, y: cmp(self.series_index(x, True), self.series_index(y, True)))
+        ans.sort(key=lambda x: self.series_index(x, True))
         return ans
 
     def books_in_series_of(self, index, index_is_id=False):
@@ -1068,8 +1075,8 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
                 self.conn.get('SELECT id, name FROM authors')]
 
     def all_author_names(self):
-        return filter(None, [i[0].strip().replace('|', ',') for i in self.conn.get(
-            'SELECT name FROM authors')])
+        return list(filter(None, [i[0].strip().replace('|', ',') for i in self.conn.get(
+            'SELECT name FROM authors')]))
 
     def all_publishers(self):
         return [(i[0], i[1]) for i in
@@ -1089,7 +1096,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
     def conversion_options(self, id, format):
         data = self.conn.get('SELECT data FROM conversion_options WHERE book=? AND format=?', (id, format.upper()), all=False)
         if data:
-            return cPickle.loads(str(data))
+            return pickle_loads(bytes(data))
         return None
 
     def has_conversion_options(self, ids, format='PIPE'):
@@ -1165,7 +1172,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
             self.set_tags(id, val.split(','), append=False)
 
     def set_conversion_options(self, id, format, options):
-        data = sqlite.Binary(cPickle.dumps(options, -1))
+        data = sqlite.Binary(pickle_dumps(options))
         oid = self.conn.get('SELECT id FROM conversion_options WHERE book=? AND format=?', (id, format.upper()), all=False)
         if oid:
             self.conn.execute('UPDATE conversion_options SET data=? WHERE id=?', (data, oid))
@@ -1335,10 +1342,10 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         formats, metadata, uris = iter(formats), iter(metadata), iter(uris)
         duplicates = []
         for path in paths:
-            mi = metadata.next()
-            format = formats.next()
+            mi = next(metadata)
+            format = next(formats)
             try:
-                uri = uris.next()
+                uri = next(uris)
             except StopIteration:
                 uri = None
             if not add_duplicates and self.has_book(mi):
