@@ -1,8 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os
 from functools import partial
@@ -57,8 +56,11 @@ def cdb_run(ctx, rd, which, version):
     try:
         result = m.implementation(db, partial(ctx.notify_changes, db.backend.library_path), *args)
     except Exception as err:
-        import traceback
-        return {'err': as_unicode(err), 'tb': traceback.format_exc()}
+        tb = ''
+        if not getattr(err, 'suppress_traceback', False):
+            import traceback
+            tb = traceback.format_exc()
+        return {'err': as_unicode(err), 'tb': tb}
     return {'result': result}
 
 
@@ -69,9 +71,12 @@ def cdb_add_book(ctx, rd, job_id, add_duplicates, filename, library_id):
     Add a file as a new book. The file contents must be in the body of the request.
 
     The response will also have the title/authors/languages read from the
-    metadata of the file/filename. It will contain a `book_id` field specifying the id of the newly added book,
-    or if add_duplicates is not specified and a duplicate was found, no book_id will be present. It will also
-    return the value of `job_id` as the `id` field and `filename` as the `filename` field.
+    metadata of the file/filename. It will contain a `book_id` field specifying
+    the id of the newly added book, or if add_duplicates is not specified and a
+    duplicate was found, no book_id will be present, instead there will be a
+    `duplicates` field specifying the title and authors for all duplicate
+    matches. It will also return the value of `job_id` as the `id` field and
+    `filename` as the `filename` field.
     '''
     db = get_db(ctx, rd, library_id)
     if ctx.restriction_for(rd, db):
@@ -96,6 +101,8 @@ def cdb_add_book(ctx, rd, job_id, add_duplicates, filename, library_id):
     if ids:
         ans['book_id'] = ids[0]
         ctx.notify_changes(db.backend.library_path, books_added(ids))
+    else:
+        ans['duplicates'] = [{'title': m.title, 'authors': m.authors} for m, _ in duplicates]
     return ans
 
 
