@@ -1,31 +1,34 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os
-from binascii import hexlify
+import os, errno
 from io import BytesIO
 from threading import Lock
+from polyglot.builtins import map, unicode_type
+from functools import partial
 
-from calibre import fit_image
+from calibre import fit_image, sanitize_file_name
 from calibre.constants import config_dir, iswindows
 from calibre.db.errors import NoSuchFormat
+from calibre.ebooks.covers import cprefs, override_prefs, scale_cover, generate_cover, set_use_roman
 from calibre.ebooks.metadata import authors_to_string
 from calibre.ebooks.metadata.meta import set_metadata
 from calibre.ebooks.metadata.opf2 import metadata_to_opf
 from calibre.library.save_to_disk import find_plugboard
-from calibre.srv.errors import HTTPNotFound
+from calibre.srv.errors import HTTPNotFound, BookNotFound
 from calibre.srv.routes import endpoint, json
-from calibre.srv.utils import http_date
+from calibre.srv.utils import http_date, get_db, get_use_roman
 from calibre.utils.config_base import tweaks
 from calibre.utils.date import timestampfromdt
+from calibre.utils.img import scale_image, image_from_data
 from calibre.utils.filenames import ascii_filename, atomic_rename
-from calibre.utils.magick.draw import thumbnail, Image
 from calibre.utils.shared_file import share_open
+from polyglot.urllib import quote
+from polyglot.binary import as_hex_unicode
 
 plugboard_content_server_value = 'content_server'
 plugboard_content_server_formats = ['epub', 'mobi', 'azw3']
@@ -39,6 +42,22 @@ lock = Lock()
 mtimes = {}
 rename_counter = 0
 
+
+def reset_caches():
+    mtimes.clear()
+
+
+def open_for_write(fname):
+    try:
+        return share_open(fname, 'w+b')
+    except EnvironmentError:
+        try:
+            os.makedirs(os.path.dirname(fname))
+        except EnvironmentError:
+            pass
+    return share_open(fname, 'w+b')
+
+
 def create_file_copy(ctx, rd, prefix, library_id, book_id, ext, mtime, copy_func, extra_etag_data=''):
     ''' We cannot copy files directly from the library folder to the output
     socket, as this can potentially lock the library for an extended period. So
@@ -74,43 +93,86 @@ def create_file_copy(ctx, rd, prefix, library_id, book_id, ext, mtime, copy_func
                     os.remove(dname)
                 else:
                     os.remove(fname)
-            try:
-                ans = share_open(fname, 'w+b')
-            except EnvironmentError:
-                try:
-                    os.makedirs(base)
-                except EnvironmentError:
-                    pass
-                ans = share_open(fname, 'w+b')
+            ans = open_for_write(fname)
             mtimes[bname] = mtime
             copy_func(ans)
             ans.seek(0)
         else:
-            ans = share_open(fname, 'rb')
-            used_cache = 'yes'
+            try:
+                ans = share_open(fname, 'rb')
+                used_cache = 'yes'
+            except EnvironmentError as err:
+                if err.errno != errno.ENOENT:
+                    raise
+                ans = open_for_write(fname)
+                mtimes[bname] = mtime
+                copy_func(ans)
+                ans.seek(0)
         if ctx.testing:
             rd.outheaders['Used-Cache'] = used_cache
-            rd.outheaders['Tempfile'] = hexlify(fname.encode('utf-8'))
+            rd.outheaders['Tempfile'] = as_hex_unicode(fname)
         return rd.filesystem_file_with_custom_etag(ans, prefix, library_id, book_id, mtime, extra_etag_data)
 
+
+def write_generated_cover(db, book_id, width, height, destf):
+    mi = db.get_metadata(book_id)
+    set_use_roman(get_use_roman())
+    if height is None:
+        prefs = cprefs
+    else:
+        ratio = height / float(cprefs['cover_height'])
+        prefs = override_prefs(cprefs)
+        scale_cover(prefs, ratio)
+    cdata = generate_cover(mi, prefs=prefs)
+    destf.write(cdata)
+
+
+def generated_cover(ctx, rd, library_id, db, book_id, width=None, height=None):
+    prefix = 'generated-cover'
+    if height is not None:
+        prefix += '-%sx%s' % (width, height)
+
+    mtime = timestampfromdt(db.field_for('last_modified', book_id))
+    return create_file_copy(ctx, rd, prefix, library_id, book_id, 'jpg', mtime, partial(write_generated_cover, db, book_id, width, height))
+
+
 def cover(ctx, rd, library_id, db, book_id, width=None, height=None):
     mtime = db.cover_last_modified(book_id)
     if mtime is None:
-        raise HTTPNotFound('No cover for book: %r' % book_id)
+        return generated_cover(ctx, rd, library_id, db, book_id, width, height)
     prefix = 'cover'
     if width is None and height is None:
         def copy_func(dest):
             db.copy_cover_to(book_id, dest)
     else:
         prefix += '-%sx%s' % (width, height)
+
         def copy_func(dest):
             buf = BytesIO()
             db.copy_cover_to(book_id, buf)
             quality = min(99, max(50, tweaks['content_server_thumbnail_compression_quality']))
-            w, h, data = thumbnail(buf.getvalue(), width=width, height=height, compression_quality=quality)
+            data = scale_image(buf.getvalue(), width=width, height=height, compression_quality=quality)[-1]
             dest.write(data)
     return create_file_copy(ctx, rd, prefix, library_id, book_id, 'jpg', mtime, copy_func)
 
+
+def book_filename(rd, book_id, mi, fmt, as_encoded_unicode=False):
+    au = authors_to_string(mi.authors or [_('Unknown')])
+    title = mi.title or _('Unknown')
+    ext = (fmt or '').lower()
+    fname = '%s - %s_%s.%s' % (title[:30], au[:30], book_id, ext)
+    if as_encoded_unicode:
+        # See https://tools.ietf.org/html/rfc6266
+        fname = sanitize_file_name(fname).encode('utf-8')
+        fname = unicode_type(quote(fname))
+    else:
+        fname = ascii_filename(fname).replace('"', '_')
+    if ext == 'kepub' and 'Kobo Touch' in rd.inheaders.get('User-Agent', ''):
+        fname = fname.replace('!', '_')
+        fname += '.epub'
+    return fname
+
+
 def book_fmt(ctx, rd, library_id, db, book_id, fmt):
     mdata = db.format_metadata(book_id, fmt)
     if not mdata:
@@ -122,7 +184,7 @@ def book_fmt(ctx, rd, library_id, db, book_id, fmt):
     if update_metadata:
         mi = db.get_metadata(book_id)
         mtime = max(mtime, mi.last_modified)
-        # Get any plugboards for the content server
+        # Get any plugboards for the Content server
         plugboards = db.pref('plugboards')
         if plugboards:
             cpb = find_plugboard(plugboard_content_server_value, fmt, plugboards)
@@ -138,18 +200,20 @@ def book_fmt(ctx, rd, library_id, db, book_id, fmt):
     def copy_func(dest):
         db.copy_format_to(book_id, fmt, dest)
         if update_metadata:
+            if not mi.cover_data or not mi.cover_data[-1]:
+                cdata = db.cover(book_id)
+                if cdata:
+                    mi.cover_data = ('jpeg', cdata)
             set_metadata(dest, mi, fmt)
             dest.seek(0)
 
-    au = authors_to_string(mi.authors or [_('Unknown')])
-    title = mi.title or _('Unknown')
-    fname = '%s - %s_%s.%s' % (title[:30], au[:30], book_id, fmt)
-    fname = ascii_filename(fname).replace('"', '_')
-    rd.outheaders['Content-Disposition'] = 'attachment; filename="%s"' % fname
+    rd.outheaders['Content-Disposition'] = '''attachment; filename="%s"; filename*=utf-8''%s''' % (
+        book_filename(rd, book_id, mi, fmt), book_filename(rd, book_id, mi, fmt, as_encoded_unicode=True))
 
     return create_file_copy(ctx, rd, 'fmt', library_id, book_id, fmt, mtime, copy_func, extra_etag_data=extra_etag_data)
 # }}}
 
+
 @endpoint('/static/{+what}', auth_required=False, cache_control=24)
 def static(ctx, rd, what):
     if not what:
@@ -165,10 +229,17 @@ def static(ctx, rd, what):
     except EnvironmentError:
         raise HTTPNotFound()
 
+
 @endpoint('/favicon.png', auth_required=False, cache_control=24)
 def favicon(ctx, rd):
     return share_open(I('lt.png'), 'rb')
 
+
+@endpoint('/apple-touch-icon.png', auth_required=False, cache_control=24)
+def apple_touch_icon(ctx, rd):
+    return share_open(I('apple-touch-icon.png'), 'rb')
+
+
 @endpoint('/icon/{+which}', auth_required=False, cache_control=24)
 def icon(ctx, rd, which):
     sz = rd.query.get('sz')
@@ -197,8 +268,7 @@ def icon(ctx, rd, which):
         except EnvironmentError:
             raise HTTPNotFound()
     with lock:
-        tdir = os.path.join(rd.tdir, 'icons')
-        cached = os.path.join(tdir, '%d-%s.png' % (sz, which))
+        cached = os.path.join(rd.tdir, 'icons', '%d-%s.png' % (sz, which))
         try:
             return share_open(cached, 'rb')
         except EnvironmentError:
@@ -208,39 +278,40 @@ def icon(ctx, rd, which):
         except EnvironmentError:
             raise HTTPNotFound()
         with src:
-            img = Image()
-            img.load(src.read())
-        width, height = img.size
-        scaled, width, height = fit_image(width, height, sz, sz)
+            idata = src.read()
+            img = image_from_data(idata)
+        scaled, width, height = fit_image(img.width(), img.height(), sz, sz)
         if scaled:
-            img.size = (width, height)
-        try:
-            ans = share_open(cached, 'w+b')
-        except EnvironmentError:
-            try:
-                os.mkdir(tdir)
-            except EnvironmentError:
-                pass
-            ans = share_open(cached, 'w+b')
-        ans.write(img.export('png'))
+            idata = scale_image(img, width, height, as_png=True)[-1]
+        ans = open_for_write(cached)
+        ans.write(idata)
         ans.seek(0)
         return ans
 
 
-@endpoint('/get/{what}/{book_id}/{library_id=None}', types={'book_id':int})
+@endpoint('/get/{what}/{book_id}/{library_id=None}', android_workaround=True)
 def get(ctx, rd, what, book_id, library_id):
-    db = ctx.get_library(library_id)
+    book_id, rest = book_id.partition('_')[::2]
+    try:
+        book_id = int(book_id)
+    except Exception:
+        raise HTTPNotFound('Book with id %r does not exist' % book_id)
+    db = get_db(ctx, rd, library_id)
     if db is None:
         raise HTTPNotFound('Library %r not found' % library_id)
     with db.safe_read_lock:
-        if book_id not in ctx.allowed_book_ids(rd, db):
-            raise HTTPNotFound('Book with id %r does not exist' % book_id)
+        if not ctx.has_id(rd, db, book_id):
+            raise BookNotFound(book_id, db)
         library_id = db.server_library_id  # in case library_id was None
         if what == 'thumb':
             sz = rd.query.get('sz')
             w, h = 60, 80
             if sz is None:
-                pass
+                if rest:
+                    try:
+                        w, h = map(int, rest.split('_'))
+                    except Exception:
+                        pass
             elif sz == 'full':
                 w = h = None
             elif 'x' in sz:
@@ -270,4 +341,4 @@ def get(ctx, rd, what, book_id, library_id):
             try:
                 return book_fmt(ctx, rd, library_id, db, book_id, what.lower())
             except NoSuchFormat:
-                raise HTTPNotFound('No %r format for the book %r' % (what.lower(), book_id))
+                raise HTTPNotFound('No %s format for the book %r' % (what.lower(), book_id))
