@@ -5,28 +5,33 @@
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, hashlib, uuid, struct
+import errno
+import hashlib
+import os
+import struct
+import uuid
 from collections import namedtuple
-from io import BytesIO, DEFAULT_BUFFER_SIZE
+from functools import wraps
+from io import DEFAULT_BUFFER_SIZE, BytesIO
 from itertools import chain, repeat
 from operator import itemgetter
-from functools import wraps
 
-from polyglot.builtins import iteritems, itervalues, reraise, map, unicode_type, string_or_bytes
-
-from calibre import guess_type, force_unicode
+from calibre import force_unicode, guess_type
 from calibre.constants import __version__
-from calibre.srv.loop import WRITE
 from calibre.srv.errors import HTTPSimpleResponse
 from calibre.srv.http_request import HTTPRequest, read_headers
-from calibre.srv.sendfile import file_metadata, sendfile_to_socket_async, CannotSendfile, SendfileInterrupted
+from calibre.srv.loop import WRITE
 from calibre.srv.utils import (
-    MultiDict, http_date, HTTP1, HTTP11, socket_errors_socket_closed,
-    sort_q_values, get_translator_for_lang, Cookie, fast_now_strftime)
-from calibre.utils.speedups import ReadOnlyFileBuffer
+    HTTP1, HTTP11, Cookie, MultiDict, fast_now_strftime, get_translator_for_lang,
+    http_date, socket_errors_socket_closed, sort_q_values
+)
 from calibre.utils.monotonic import monotonic
+from calibre.utils.speedups import ReadOnlyFileBuffer
 from polyglot import http_client, reprlib
-from polyglot.builtins import error_message
+from polyglot.builtins import (
+    error_message, iteritems, itervalues, map, reraise, string_or_bytes,
+    unicode_type
+)
 
 Range = namedtuple('Range', 'start stop size')
 MULTIPART_SEPARATOR = uuid.uuid4().hex
@@ -37,6 +42,14 @@ import zlib
 from itertools import zip_longest
 
 
+def file_metadata(fileobj):
+    try:
+        fd = fileobj.fileno()
+        return os.fstat(fd)
+    except Exception:
+        pass
+
+
 def header_list_to_file(buf):  # {{{
     buf.append('')
     return ReadOnlyFileBuffer(b''.join((x + '\r\n').encode('ascii') for x in buf))
@@ -374,17 +387,15 @@ class HTTPConnection(HTTPRequest):
         if limit <= 0:
             return True
         if self.use_sendfile and not isinstance(buf, (BytesIO, ReadOnlyFileBuffer)):
+            limit = min(limit, 2 ** 30)
             try:
-                sent = sendfile_to_socket_async(buf, pos, limit, self.socket)
-            except CannotSendfile:
-                self.use_sendfile = False
-                return False
-            except SendfileInterrupted:
-                return False
-            except IOError as e:
+                sent = os.sendfile(self.socket.fileno(), buf.fileno(), pos, limit)
+            except OSError as e:
                 if e.errno in socket_errors_socket_closed:
                     self.ready = self.use_sendfile = False
                     return False
+                if e.errno in (errno.EAGAIN, errno.EINTR):
+                    return False
                 raise
             finally:
                 self.last_activity = monotonic()
@@ -561,7 +572,7 @@ class HTTPConnection(HTTPRequest):
             self.reset_state()
             return
         if isinstance(output, ReadableOutput):
-            self.use_sendfile = output.use_sendfile and self.opts.use_sendfile and sendfile_to_socket_async is not None and self.ssl_context is None
+            self.use_sendfile = output.use_sendfile and self.opts.use_sendfile and hasattr(os, 'sendfile') and self.ssl_context is None
             # sendfile() does not work with SSL sockets since encryption has to
             # be done in userspace
             if output.ranges is not None:
