@@ -19,7 +19,7 @@ from css_parser.css import CSSRule
 from lxml.etree import Comment
 
 from calibre import detect_ncpus, force_unicode, prepare_string_for_xml
-from calibre.constants import iswindows, plugins
+from calibre.constants import iswindows
 from calibre.customize.ui import plugin_for_input_format
 from calibre.ebooks import parse_css_length
 from calibre.ebooks.css_transform_rules import StyleDeclaration
@@ -41,7 +41,6 @@ from calibre.srv.opts import grouper
 from calibre.utils.date import EPOCH
 from calibre.utils.filenames import rmtree
 from calibre.utils.ipc.simple_worker import start_pipe_worker
-from calibre.utils.iso8601 import parse_iso8601
 from calibre.utils.logging import default_log
 from calibre.utils.serialize import (
     json_dumps, json_loads, msgpack_dumps, msgpack_loads
@@ -53,11 +52,11 @@ from polyglot.binary import (
 )
 from polyglot.builtins import as_bytes, iteritems, map, unicode_type
 from polyglot.urllib import quote, urlparse
+from calibre_extensions import speedup
 
 RENDER_VERSION = 1
 
 BLANK_JPEG = b'\xff\xd8\xff\xdb\x00C\x00\x03\x02\x02\x02\x02\x02\x03\x02\x02\x02\x03\x03\x03\x03\x04\x06\x04\x04\x04\x04\x04\x08\x06\x06\x05\x06\t\x08\n\n\t\x08\t\t\n\x0c\x0f\x0c\n\x0b\x0e\x0b\t\t\r\x11\r\x0e\x0f\x10\x10\x11\x10\n\x0c\x12\x13\x12\x10\x13\x0f\x10\x10\x10\xff\xc9\x00\x0b\x08\x00\x01\x00\x01\x01\x01\x11\x00\xff\xcc\x00\x06\x00\x10\x10\x05\xff\xda\x00\x08\x01\x01\x00\x00?\x00\xd2\xcf \xff\xd9'  # noqa
-speedup = plugins['speedup'][0]
 
 
 def XPath(expr):
@@ -101,11 +100,14 @@ def create_link_replacer(container, link_uid, changed):
     def link_replacer(base, url):
         if url.startswith('#'):
             frag = urlunquote(url[1:])
-            if not frag:
-                return url
             changed.add(base)
+            if not frag:
+                return link_uid
             return resource_template.format(encode_url(base, frag))
-        purl = urlparse(url)
+        try:
+            purl = urlparse(url)
+        except Exception:
+            return url
         if purl.netloc or purl.query:
             return url
         if purl.scheme and purl.scheme != 'file':
@@ -129,11 +131,20 @@ def create_link_replacer(container, link_uid, changed):
 
 
 page_break_properties = ('page-break-before', 'page-break-after', 'page-break-inside')
+absolute_font_sizes = {
+    'xx-small': '0.5rem', 'x-small': '0.625rem', 'small': '0.8rem',
+    'medium': '1rem',
+    'large': '1.125rem', 'x-large': '1.5rem', 'xx-large': '2rem', 'xxx-large': '2.55rem'
+}
+nonstandard_writing_mode_property_names = ('-webkit-writing-mode', '-epub-writing-mode')
 
 
 def transform_declaration(decl):
     decl = StyleDeclaration(decl)
     changed = False
+    nonstandard_writing_mode_props = {}
+    standard_writing_mode_props = {}
+
     for prop, parent_prop in tuple(decl):
         if prop.name in page_break_properties:
             changed = True
@@ -144,11 +155,29 @@ def transform_declaration(decl):
                 decl.set_property(prefix + name, prop.value, prop.priority)
             decl.remove_property(prop, parent_prop)
         elif prop.name == 'font-size':
-            l, unit = parse_css_length(prop.value)
+            raw = prop.value
+            afs = absolute_font_sizes.get(raw)
+            if afs is not None:
+                changed = True
+                decl.change_property(prop, parent_prop, afs)
+                continue
+            l, unit = parse_css_length(raw)
             if unit in absolute_units:
                 changed = True
                 l = convert_fontsize(l, unit)
                 decl.change_property(prop, parent_prop, unicode_type(l) + 'rem')
+        elif prop.name in nonstandard_writing_mode_property_names:
+            nonstandard_writing_mode_props[prop.value] = prop.priority
+        elif prop.name == 'writing-mode':
+            standard_writing_mode_props[prop.value] = True
+
+    # Add standard writing-mode properties if they don't exist so that
+    # all of the browsers supported by the viewer work in vertical modes
+    for value, priority in nonstandard_writing_mode_props.items():
+        if value not in standard_writing_mode_props:
+            decl.set_property('writing-mode', value, priority)
+            changed = True
+
     return changed
 
 
@@ -411,10 +440,12 @@ def transform_html(container, name, virtualize_resources, link_uid, link_to_map,
                 href = link_replacer(name, href)
             if href and href.startswith(link_uid):
                 a.set(attr, 'javascript:void(0)')
-                parts = decode_url(href.split('|')[1])
-                lname, lfrag = parts[0], parts[1]
-                link_to_map.setdefault(lname, {}).setdefault(lfrag or '', set()).add(name)
-                a.set('data-' + link_uid, json.dumps({'name':lname, 'frag':lfrag}, ensure_ascii=False))
+                parts = href.split('|')
+                if len(parts) > 1:
+                    parts = decode_url(parts[1])
+                    lname, lfrag = parts[0], parts[1]
+                    link_to_map.setdefault(lname, {}).setdefault(lfrag or '', set()).add(name)
+                    a.set('data-' + link_uid, json.dumps({'name':lname, 'frag':lfrag}, ensure_ascii=False))
 
         for a in link_xpath(root):
             handle_link(a)
@@ -626,6 +657,12 @@ def process_exploded_book(
     spineq = frozenset(spine)
     landmarks = [l for l in get_landmarks(container) if l['dest'] in spineq]
 
+    page_progression_direction = None
+    try:
+        page_progression_direction = container.opf_xpath('//opf:spine/@page-progression-direction')[0]
+    except IndexError:
+        pass
+
     book_render_data = {
         'version': RENDER_VERSION,
         'toc':toc,
@@ -642,6 +679,7 @@ def process_exploded_book(
         'toc_anchor_map': toc_anchor_map(toc),
         'landmarks': landmarks,
         'link_to_map': {},
+        'page_progression_direction': page_progression_direction,
     }
 
     names = sorted(
@@ -740,17 +778,11 @@ def ensure_body(root):
 
 
 def html_as_json(root):
+    from calibre_extensions.html_as_json import serialize
     ns, name = split_name(root.tag)
     if ns not in (None, XHTML_NS):
         raise ValueError('HTML tag must be in empty or XHTML namespace')
     ensure_body(root)
-    pl, err = plugins['html_as_json']
-    if err:
-        raise SystemExit('Failed to load html_as_json plugin with error: {}'.format(err))
-    try:
-        serialize = pl.serialize
-    except AttributeError:
-        raise SystemExit('You are running calibre from source, you need to also update the main calibre installation to version >=4.3')
     for child in tuple(root.iterchildren('*')):
         if child.tag.partition('}')[-1] not in ('head', 'body'):
             root.remove(child)
@@ -769,25 +801,13 @@ def serialize_datetimes(d):
 EPUB_FILE_TYPE_MAGIC = b'encoding=json+base64:\n'
 
 
-def parse_annotation(annot):
-    ts = annot['timestamp']
-    if hasattr(ts, 'rstrip'):
-        annot['timestamp'] = parse_iso8601(ts, assume_utc=True)
-    return annot
-
-
-def parse_annotations(raw):
-    for annot in json_loads(raw):
-        yield parse_annotation(annot)
-
-
 def get_stored_annotations(container, bookmark_data):
     raw = bookmark_data or b''
     if not raw:
         return
     if raw.startswith(EPUB_FILE_TYPE_MAGIC):
         raw = raw[len(EPUB_FILE_TYPE_MAGIC):].replace(b'\n', b'')
-        for annot in parse_annotations(from_base64_bytes(raw)):
+        for annot in json_loads(from_base64_bytes(raw)):
             yield annot
         return
 
@@ -878,5 +898,15 @@ def profile():
         )
 
 
+def develop():
+    from calibre.ptempfile import TemporaryDirectory
+    path = sys.argv[-1]
+    with TemporaryDirectory() as tdir:
+        return render(
+            path, tdir, serialize_metadata=True,
+            extract_annotations=True, virtualize_resources=False, max_workers=1
+        )
+
+
 if __name__ == '__main__':
-    profile()
+    develop()
