@@ -5,13 +5,11 @@
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os
-from calibre.utils.run_tests import find_tests_in_dir, run_tests
+from calibre.utils.run_tests import find_tests_in_package, run_tests
 
 
 def find_tests():
-    base = os.path.dirname(os.path.abspath(__file__))
-    return find_tests_in_dir(base)
+    return find_tests_in_package('calibre.srv.tests')
 
 
 if __name__ == '__main__':
