@@ -1,8 +1,8 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 import socket, os, struct, errno, numbers
 from collections import deque, namedtuple
 from functools import partial
@@ -132,7 +132,7 @@ class WSClient(object):
     def write_message(self, msg, chunk_size=None):
         if isinstance(msg, tuple):
             opcode, msg = msg
-            if isinstance(msg, type('')):
+            if isinstance(msg, unicode_type):
                 msg = msg.encode('utf-8')
             return self.write_frame(1, opcode, msg)
         w = MessageWriter(msg, self.mask, chunk_size)
@@ -147,7 +147,7 @@ class WSClient(object):
         self.socket.sendall(frame)
 
     def write_close(self, code, reason=b''):
-        if isinstance(reason, type('')):
+        if isinstance(reason, unicode_type):
             reason = reason.encode('utf-8')
         self.write_frame(1, CLOSE, struct.pack(b'!H', code) + reason)
 
@@ -195,7 +195,12 @@ class WebSocketTest(BaseTest):
                 expected_messages.append(ex)
         if send_close:
             client.write_close(close_code, close_reason)
-        messages, control_frames = client.read_messages()
+        try:
+            messages, control_frames = client.read_messages()
+        except ConnectionAbortedError:
+            if expected_messages or expected_controls or send_close:
+                raise
+            return
         self.ae(expected_messages, messages)
         self.assertGreaterEqual(len(control_frames), 1)
         self.ae(expected_controls, control_frames[:-1])
@@ -217,8 +222,7 @@ class WebSocketTest(BaseTest):
             for payload in [b'', b'ping', b'\x00\xff\xfe\xfd\xfc\xfb\x00\xff', b"\xfe" * 125]:
                 simple_test([(PING, payload)], [(PONG, payload)])
 
-            with server.silence_log:
-                simple_test([(PING, 'a'*126)], close_code=PROTOCOL_ERROR, send_close=False)
+            simple_test([(PING, 'a'*126)], close_code=PROTOCOL_ERROR, send_close=False)
 
             for payload in (b'', b'pong'):
                 simple_test([(PONG, payload)], [])
@@ -226,46 +230,45 @@ class WebSocketTest(BaseTest):
             fragments = 'Hello-µ@ßöä üàá-UTF-8!!'.split()
             nc = struct.pack(b'!H', NORMAL_CLOSE)
 
-            with server.silence_log:
-                # It can happen that the server detects bad data and closes the
-                # connection before the client has finished sending all
-                # messages, so ignore failures to send packets.
-                isf_test = partial(simple_test, ignore_send_failures=True)
-                for rsv in range(1, 7):
-                    isf_test([{'rsv':rsv, 'opcode':BINARY}], [], close_code=PROTOCOL_ERROR, send_close=False)
-                for opcode in (3, 4, 5, 6, 7, 11, 12, 13, 14, 15):
-                    isf_test([{'opcode':opcode}], [], close_code=PROTOCOL_ERROR, send_close=False)
-
-                for opcode in (PING, PONG):
-                    isf_test([
-                        {'opcode':opcode, 'payload':'f1', 'fin':0}, {'opcode':opcode, 'payload':'f2'}
-                    ], close_code=PROTOCOL_ERROR, send_close=False)
-                isf_test([(CLOSE, nc + b'x'*124)], send_close=False, close_code=PROTOCOL_ERROR)
-
-                for fin in (0, 1):
-                    isf_test([{'opcode':0, 'fin': fin, 'payload':b'non-continuation frame'}, 'some text'], close_code=PROTOCOL_ERROR, send_close=False)
+            # It can happen that the server detects bad data and closes the
+            # connection before the client has finished sending all
+            # messages, so ignore failures to send packets.
+            isf_test = partial(simple_test, ignore_send_failures=True)
+            for rsv in range(1, 7):
+                isf_test([{'rsv':rsv, 'opcode':BINARY}], [], close_code=PROTOCOL_ERROR, send_close=False)
+            for opcode in (3, 4, 5, 6, 7, 11, 12, 13, 14, 15):
+                isf_test([{'opcode':opcode}], [], close_code=PROTOCOL_ERROR, send_close=False)
 
+            for opcode in (PING, PONG):
                 isf_test([
-                    {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':CONTINUATION, 'payload':fragments[1]}, {'opcode':0, 'fin':0}
-                ], [''.join(fragments)], close_code=PROTOCOL_ERROR, send_close=False)
-
-                isf_test([
-                    {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':TEXT, 'payload':fragments[1]},
+                    {'opcode':opcode, 'payload':'f1', 'fin':0}, {'opcode':opcode, 'payload':'f2'}
                 ], close_code=PROTOCOL_ERROR, send_close=False)
+            isf_test([(CLOSE, nc + b'x'*124)], send_close=False, close_code=PROTOCOL_ERROR)
 
-                frags = []
-                for payload in (b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5', b'\xed\xa0\x80', b'\x80\x65\x64\x69\x74\x65\x64'):
-                    frags.append({'opcode':(CONTINUATION if frags else TEXT), 'fin':1 if len(frags) == 2 else 0, 'payload':payload})
-                isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False)
+            for fin in (0, 1):
+                isf_test([{'opcode':0, 'fin': fin, 'payload':b'non-continuation frame'}, 'some text'], close_code=PROTOCOL_ERROR, send_close=False)
 
-                frags, q = [], b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5\xed\xa0\x80\x80\x65\x64\x69\x74\x65\x64'
-                for i in range(len(q)):
-                    b = q[i:i+1]
-                    frags.append({'opcode':(TEXT if i == 0 else CONTINUATION), 'fin':1 if i == len(q)-1 else 0, 'payload':b})
-                isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False, ignore_send_failures=True)
+            isf_test([
+                {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':CONTINUATION, 'payload':fragments[1]}, {'opcode':0, 'fin':0}
+            ], [''.join(fragments)], close_code=PROTOCOL_ERROR, send_close=False)
 
-                for q in (b'\xce', b'\xce\xba\xe1'):
-                    isf_test([{'opcode':TEXT, 'payload':q}], close_code=INCONSISTENT_DATA, send_close=False)
+            isf_test([
+                {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':TEXT, 'payload':fragments[1]},
+            ], close_code=PROTOCOL_ERROR, send_close=False)
+
+            frags = []
+            for payload in (b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5', b'\xed\xa0\x80', b'\x80\x65\x64\x69\x74\x65\x64'):
+                frags.append({'opcode':(CONTINUATION if frags else TEXT), 'fin':1 if len(frags) == 2 else 0, 'payload':payload})
+            isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False)
+
+            frags, q = [], b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5\xed\xa0\x80\x80\x65\x64\x69\x74\x65\x64'
+            for i in range(len(q)):
+                b = q[i:i+1]
+                frags.append({'opcode':(TEXT if i == 0 else CONTINUATION), 'fin':1 if i == len(q)-1 else 0, 'payload':b})
+            isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False, ignore_send_failures=True)
+
+            for q in (b'\xce', b'\xce\xba\xe1'):
+                isf_test([{'opcode':TEXT, 'payload':q}], close_code=INCONSISTENT_DATA, send_close=False)
 
             simple_test([
                 {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':CONTINUATION, 'payload':fragments[1]}
@@ -317,3 +320,8 @@ class WebSocketTest(BaseTest):
                 sz *= 1024
                 t, b = 'a'*sz, b'a'*sz
                 simple_test([t, b], [t, b])
+
+
+def find_tests():
+    import unittest
+    return unittest.defaultTestLoader.loadTestsFromTestCase(WebSocketTest)
