@@ -463,7 +463,7 @@ def test_glyph_ids():
 
 def test_supports_text():
     data = P('fonts/calibreSymbols.otf', data=True)
-    if not supports_text(data, '.★½'):
+    if not supports_text(data, '.★½⯨'):
         raise RuntimeError('Incorrectly returning that text is not supported')
     if supports_text(data, 'abc'):
         raise RuntimeError('Incorrectly claiming that text is supported')
