@@ -31,6 +31,10 @@ class Node(object):
     NODE_CALL = 10
     NODE_ARGUMENTS = 11
     NODE_FIRST_NON_EMPTY = 12
+    NODE_FOR = 13
+    NODE_GLOBALS = 14
+    NODE_SET_GLOBALS = 15
+    NODE_CONTAINS = 16
 
 
 class IfNode(Node):
@@ -42,6 +46,16 @@ class IfNode(Node):
         self.else_part = else_part
 
 
+class ForNode(Node):
+    def __init__(self, variable, list_field_expr, separator, block):
+        Node.__init__(self)
+        self.node_type = self.NODE_FOR
+        self.variable = variable
+        self.list_field_expr = list_field_expr
+        self.separator = separator
+        self.block = block
+
+
 class AssignNode(Node):
     def __init__(self, left, right):
         Node.__init__(self)
@@ -73,6 +87,20 @@ class ArgumentsNode(Node):
         self.expression_list = expression_list
 
 
+class GlobalsNode(Node):
+    def __init__(self, expression_list):
+        Node.__init__(self)
+        self.node_type = self.NODE_GLOBALS
+        self.expression_list = expression_list
+
+
+class SetGlobalsNode(Node):
+    def __init__(self, expression_list):
+        Node.__init__(self)
+        self.node_type = self.NODE_SET_GLOBALS
+        self.expression_list = expression_list
+
+
 class StringInfixNode(Node):
     def __init__(self, operator, left, right):
         Node.__init__(self)
@@ -113,10 +141,11 @@ class FieldNode(Node):
 
 
 class RawFieldNode(Node):
-    def __init__(self, expression):
+    def __init__(self, expression, default=None):
         Node.__init__(self)
         self.node_type = self.NODE_RAW_FIELD
         self.expression = expression
+        self.default = default
 
 
 class FirstNonEmptyNode(Node):
@@ -126,6 +155,16 @@ class FirstNonEmptyNode(Node):
         self.expression_list = expression_list
 
 
+class ContainsNode(Node):
+    def __init__(self, arguments):
+        Node.__init__(self)
+        self.node_type = self.NODE_CONTAINS
+        self.value_expression = arguments[0]
+        self.test_expression = arguments[1]
+        self.match_expression = arguments[2]
+        self.not_match_expression = arguments[3]
+
+
 class _Parser(object):
     LEX_OP = 1
     LEX_ID = 2
@@ -138,16 +177,15 @@ class _Parser(object):
     def error(self, message):
         try:
             tval = "'" + self.prog[self.lex_pos-1][1] + "'"
-        except:
+        except Exception:
             tval = _('Unknown')
-        m = 'Formatter: ' + message + _(' near')
         if self.lex_pos > 0:
-            m = '{0} {1}'.format(m, tval)
+            location = tval
         elif self.lex_pos < self.prog_len:
-            m = '{0} {1}'.format(m, tval)
+            location = tval
         else:
-            m = '{0} {1}'.format(m, _('end of program'))
-        raise ValueError(m)
+            location = _('the end of the program')
+        raise ValueError(_('{0}: {1} near {2}').format('Formatter', message, location))
 
     def token(self):
         try:
@@ -207,6 +245,13 @@ class _Parser(object):
         except:
             return False
 
+    def token_op_is_colon(self):
+        try:
+            token = self.prog[self.lex_pos]
+            return token[1] == ':' and token[0] == self.LEX_OP
+        except:
+            return False
+
     def token_is_id(self):
         try:
             return self.prog[self.lex_pos][0] == self.LEX_ID
@@ -255,6 +300,34 @@ class _Parser(object):
         except:
             return False
 
+    def token_is_for(self):
+        try:
+            token = self.prog[self.lex_pos]
+            return token[1] == 'for' and token[0] == self.LEX_KEYWORD
+        except:
+            return False
+
+    def token_is_in(self):
+        try:
+            token = self.prog[self.lex_pos]
+            return token[1] == 'in' and token[0] == self.LEX_KEYWORD
+        except:
+            return False
+
+    def token_is_rof(self):
+        try:
+            token = self.prog[self.lex_pos]
+            return token[1] == 'rof' and token[0] == self.LEX_KEYWORD
+        except:
+            return False
+
+    def token_is_separator(self):
+        try:
+            token = self.prog[self.lex_pos]
+            return token[1] == 'separator' and token[0] == self.LEX_ID
+        except:
+            return False
+
     def token_is_constant(self):
         try:
             return self.prog[self.lex_pos][0] == self.LEX_CONST
@@ -271,7 +344,7 @@ class _Parser(object):
         self.lex_pos = 0
         self.parent = parent
         self.funcs = funcs
-        self.func_names = frozenset(set(self.funcs.keys()) | {'arguments',})
+        self.func_names = frozenset(set(self.funcs.keys()))
         self.prog = prog[0]
         self.prog_len = len(self.prog)
         if prog[1] != '':
@@ -309,9 +382,32 @@ class _Parser(object):
         self.consume()
         return IfNode(condition, then_part, else_part)
 
+    def for_expression(self):
+        self.consume()
+        if not self.token_is_id():
+            self.error(_("Missing identifier in for statement"))
+        variable = self.token()
+        if not self.token_is_in():
+            self.error(_("Missing 'in' in for statement"))
+        self.consume()
+        list_expr = self.infix_expr()
+        if self.token_is_separator():
+            self.consume()
+            separator = self.expr()
+        else:
+            separator = None
+        if not self.token_op_is_colon():
+            self.error(_("Missing colon (':') in for statement"))
+        self.consume()
+        block = self.expression_list()
+        if not self.token_is_rof():
+            self.error(_("Missing 'rof' in for statement"))
+        self.consume()
+        return ForNode(variable, list_expr, separator, block)
+
     def infix_expr(self):
         left = self.expr()
-        if self.token_op_is_string_infix_compare():
+        if self.token_op_is_string_infix_compare() or self.token_is_in():
             operator = self.token()
             return StringInfixNode(operator, left, self.expr())
         if self.token_op_is_numeric_infix_compare():
@@ -334,6 +430,8 @@ class _Parser(object):
     def expr(self):
         if self.token_is_if():
             return self.if_expression()
+        if self.token_is_for():
+            return self.for_expression()
         if self.token_is_id():
             # We have an identifier. Determine if it is a function
             id_ = self.token()
@@ -363,24 +461,30 @@ class _Parser(object):
                 self.error(_('Missing closing parenthesis'))
             if id_ == 'field' and len(arguments) == 1:
                 return FieldNode(arguments[0])
-            if id_ == 'raw_field' and len(arguments) == 1:
-                return RawFieldNode(arguments[0])
+            if id_ == 'raw_field' and (len(arguments) in (1, 2)):
+                return RawFieldNode(*arguments)
             if id_ == 'test' and len(arguments) == 3:
                 return IfNode(arguments[0], (arguments[1],), (arguments[2],))
             if id_ == 'first_non_empty' and len(arguments) > 0:
                 return FirstNonEmptyNode(arguments)
             if (id_ == 'assign' and len(arguments) == 2 and arguments[0].node_type == Node.NODE_RVALUE):
                 return AssignNode(arguments[0].name, arguments[1])
-            if id_ == 'arguments':
+            if id_ == 'arguments' or id_ == 'globals' or id_ == 'set_globals':
                 new_args = []
                 for arg in arguments:
                     if arg.node_type not in (Node.NODE_ASSIGN, Node.NODE_RVALUE):
-                        self.error(_("Parameters to 'arguments' must be "
-                                     "variables or assignments"))
+                        self.error(_("Parameters to '{}' must be "
+                                     "variables or assignments").format(id_))
                     if arg.node_type == Node.NODE_RVALUE:
                         arg = AssignNode(arg.name, ConstantNode(''))
                     new_args.append(arg)
-                return ArgumentsNode(new_args)
+                if id_ == 'arguments':
+                    return ArgumentsNode(new_args)
+                if id_ == 'set_globals':
+                    return SetGlobalsNode(new_args)
+                return GlobalsNode(new_args)
+            if id_ == 'contains' and len(arguments) == 4:
+                return ContainsNode(arguments)
             if id_ in self.func_names and not self.funcs[id_].is_python:
                 return self.call_expression(id_, arguments)
             cls = self.funcs[id_]
@@ -399,12 +503,13 @@ class _Interpreter(object):
         m = 'Interpreter: ' + message
         raise ValueError(m)
 
-    def program(self, funcs, parent, prog, val, is_call=False, args=None):
+    def program(self, funcs, parent, prog, val, is_call=False, args=None, global_vars=None):
         self.parent = parent
         self.parent_kwargs = parent.kwargs
         self.parent_book = parent.book
         self.funcs = funcs
         self.locals = {'$':val}
+        self.global_vars = global_vars if isinstance(global_vars, dict) else {}
         if is_call:
             return self.do_node_call(CallNode(prog, None), args=args)
         return self.expression_list(prog)
@@ -422,6 +527,7 @@ class _Interpreter(object):
         "<=": lambda x, y: strcmp(x, y) <= 0,
         ">": lambda x, y: strcmp(x, y) > 0,
         ">=": lambda x, y: strcmp(x, y) >= 0,
+        "in": lambda x, y: re.search(x, y, flags=re.I),
         }
 
     def do_node_string_infix(self, prog):
@@ -498,6 +604,18 @@ class _Interpreter(object):
             self.locals[arg.left] = self.locals.get('*arg_'+ str(dex), self.expr(arg.right))
         return ''
 
+    def do_node_globals(self, prog):
+        res = ''
+        for arg in prog.expression_list:
+            res = self.locals[arg.left] = self.global_vars.get(arg.left, self.expr(arg.right))
+        return res
+
+    def do_node_set_globals(self, prog):
+        res = ''
+        for arg in prog.expression_list:
+            res = self.global_vars[arg.left] = self.locals.get(arg.left, self.expr(arg.right))
+        return res
+
     def do_node_constant(self, prog):
         return prog.value
 
@@ -517,6 +635,8 @@ class _Interpreter(object):
         try:
             name = self.expr(prog.expression)
             res = getattr(self.parent_book, name, None)
+            if res is None and prog.default is not None:
+                return self.expr(prog.default)
             if res is not None:
                 if isinstance(res, list):
                     fm = self.parent_book.metadata_for_field(name)
@@ -540,6 +660,33 @@ class _Interpreter(object):
                 return v
         return ''
 
+    def do_node_for(self, prog):
+        try:
+            separator = ',' if prog.separator is None else self.expr(prog.separator)
+            v = prog.variable
+            f = self.expr(prog.list_field_expr)
+            res = getattr(self.parent_book, f, f)
+            if res is not None:
+                if not isinstance(res, list):
+                    res = [r.strip() for r in res.split(separator) if r.strip()]
+                ret = ''
+                for x in res:
+                    self.locals[v] = x
+                    ret = self.expression_list(prog.block)
+                return ret
+            self.error(_('The field {0} is not a list').format(f))
+        except ValueError as e:
+            raise e
+        except Exception as e:
+            self.error(_('Unhandled exception {0}').format(e))
+
+    def do_node_contains(self, prog):
+        v = self.expr(prog.value_expression)
+        t = self.expr(prog.test_expression)
+        if re.search(t, v, flags=re.I):
+            return self.expr(prog.match_expression)
+        return self.expr(prog.not_match_expression)
+
     NODE_OPS = {
         Node.NODE_IF:             do_node_if,
         Node.NODE_ASSIGN:         do_node_assign,
@@ -553,6 +700,10 @@ class _Interpreter(object):
         Node.NODE_ARGUMENTS:      do_node_arguments,
         Node.NODE_CALL:           do_node_call,
         Node.NODE_FIRST_NON_EMPTY:do_node_first_non_empty,
+        Node.NODE_FOR:            do_node_for,
+        Node.NODE_GLOBALS:        do_node_globals,
+        Node.NODE_SET_GLOBALS:    do_node_set_globals,
+        Node.NODE_CONTAINS:       do_node_contains,
         }
 
     def expr(self, prog):
@@ -636,7 +787,8 @@ class TemplateFormatter(string.Formatter):
             (r'(==#|!=#|<=#|<#|>=#|>#)', lambda x,t: (_Parser.LEX_NUMERIC_INFIX, t)),
             (r'(==|!=|<=|<|>=|>)',       lambda x,t: (_Parser.LEX_STRING_INFIX, t)),  # noqa
             (r'(if|then|else|elif|fi)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
-            (r'[(),=;]',                 lambda x,t: (_Parser.LEX_OP, t)),  # noqa
+            (r'(for|in|rof)\b',          lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
+            (r'[(),=;:]',                lambda x,t: (_Parser.LEX_OP, t)),  # noqa
             (r'-?[\d\.]+',               lambda x,t: (_Parser.LEX_CONST, t)),  # noqa
             (r'\$',                      lambda x,t: (_Parser.LEX_ID, t)),  # noqa
             (r'\w+',                     lambda x,t: (_Parser.LEX_ID, t)),  # noqa
@@ -646,7 +798,7 @@ class TemplateFormatter(string.Formatter):
             (r'\s',                      None),
         ], flags=re.DOTALL)
 
-    def _eval_program(self, val, prog, column_name):
+    def _eval_program(self, val, prog, column_name, global_vars):
         if column_name is not None and self.template_cache is not None:
             tree = self.template_cache.get(column_name, None)
             if not tree:
@@ -654,9 +806,9 @@ class TemplateFormatter(string.Formatter):
                 self.template_cache[column_name] = tree
         else:
             tree = self.gpm_parser.program(self, self.funcs, self.lex_scanner.scan(prog))
-        return self.gpm_interpreter.program(self.funcs, self, tree, val)
+        return self.gpm_interpreter.program(self.funcs, self, tree, val, global_vars=global_vars)
 
-    def _eval_sfm_call(self, template_name, args):
+    def _eval_sfm_call(self, template_name, args, global_vars):
         func = self.funcs[template_name]
         tree = func.cached_parse_tree
         if tree is None:
@@ -664,7 +816,8 @@ class TemplateFormatter(string.Formatter):
                            self.lex_scanner.scan(func.program_text[len('program:'):]))
             func.cached_parse_tree = tree
         return self.gpm_interpreter.program(self.funcs, self, tree, None,
-                                            is_call=True, args=args)
+                                            is_call=True, args=args,
+                                            global_vars=global_vars)
     # ################# Override parent classes methods #####################
 
     def get_value(self, key, args, kwargs):
@@ -689,7 +842,7 @@ class TemplateFormatter(string.Formatter):
             if p >= 0:
                 p += 1
         if p >= 0 and fmt[-1] == '\'':
-            val = self._eval_program(val, fmt[p+1:-1], None)
+            val = self._eval_program(val, fmt[p+1:-1], None, self.global_vars)
             colon = fmt[0:p].find(':')
             if colon < 0:
                 dispfmt = ''
@@ -720,7 +873,7 @@ class TemplateFormatter(string.Formatter):
                         args = [self.backslash_comma_to_comma.sub(',', a) for a in args]
                     if not func.is_python:
                         args.insert(0, val)
-                        val = self._eval_sfm_call(fname, args)
+                        val = self._eval_sfm_call(fname, args, self.global_vars)
                     else:
                         if (func.arg_count == 1 and (len(args) != 1 or args[0])) or \
                                 (func.arg_count > 1 and func.arg_count != len(args)+1):
@@ -742,9 +895,10 @@ class TemplateFormatter(string.Formatter):
             return ''
         return prefix + val + suffix
 
-    def evaluate(self, fmt, args, kwargs):
+    def evaluate(self, fmt, args, kwargs, global_vars):
         if fmt.startswith('program:'):
-            ans = self._eval_program(kwargs.get('$', None), fmt[8:], self.column_name)
+            ans = self._eval_program(kwargs.get('$', None), fmt[8:],
+                                     self.column_name, global_vars)
         else:
             ans = self.vformat(fmt, args, kwargs)
         if self.strip_results:
@@ -753,25 +907,28 @@ class TemplateFormatter(string.Formatter):
 
     # ######### a formatter that throws exceptions ############
 
-    def unsafe_format(self, fmt, kwargs, book, strip_results=True):
+    def unsafe_format(self, fmt, kwargs, book, strip_results=True, global_vars=None):
         self.strip_results = strip_results
         self.column_name = self.template_cache = None
         self.kwargs = kwargs
         self.book = book
         self.composite_values = {}
         self.locals = {}
-        return self.evaluate(fmt, [], kwargs)
+        self.global_vars = global_vars if isinstance(global_vars, dict) else {}
+        return self.evaluate(fmt, [], kwargs, self.global_vars)
 
     # ######### a formatter guaranteed not to throw an exception ############
 
     def safe_format(self, fmt, kwargs, error_value, book,
                     column_name=None, template_cache=None,
-                    strip_results=True, template_functions=None):
+                    strip_results=True, template_functions=None,
+                    global_vars=None):
         self.strip_results = strip_results
         self.column_name = column_name
         self.template_cache = template_cache
         self.kwargs = kwargs
         self.book = book
+        self.global_vars = global_vars if isinstance(global_vars, dict) else {}
         if template_functions:
             self.funcs = template_functions
         else:
@@ -779,7 +936,7 @@ class TemplateFormatter(string.Formatter):
         self.composite_values = {}
         self.locals = {}
         try:
-            ans = self.evaluate(fmt, [], kwargs)
+            ans = self.evaluate(fmt, [], kwargs, self.global_vars)
         except Exception as e:
             if DEBUG:  # and getattr(e, 'is_locking_error', False):
                 traceback.print_exc()
