@@ -146,7 +146,7 @@ class BuiltinFormatterFunction(FormatterFunction):
     def __init__(self):
         formatter_functions().register_builtin(self)
         eval_func = inspect.getmembers(self.__class__,
-                        lambda x: inspect.ismethod(x) and x.__name__ == 'evaluate')
+                        lambda x: inspect.isfunction(x) and x.__name__ == 'evaluate')
         try:
             lines = [l[4:] for l in inspect.getsourcelines(eval_func[0][1])[0]]
         except:
@@ -241,14 +241,17 @@ class BuiltinStrlen(BuiltinFormatterFunction):
 
 class BuiltinAdd(BuiltinFormatterFunction):
     name = 'add'
-    arg_count = 2
+    arg_count = -1
     category = 'Arithmetic'
-    __doc__ = doc = _('add(x, y) -- returns x + y. Throws an exception if either x or y are not numbers.')
+    __doc__ = doc = _('add(x, y, ...) -- returns the sum of its arguments. '
+                      'Throws an exception if an argument is not a number.')
 
-    def evaluate(self, formatter, kwargs, mi, locals, x, y):
-        x = float(x if x and x != 'None' else 0)
-        y = float(y if y and y != 'None' else 0)
-        return unicode_type(x + y)
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        res = 0
+        for v in args:
+            v = float(v if v and v != 'None' else 0)
+            res += v
+        return unicode_type(res)
 
 
 class BuiltinSubtract(BuiltinFormatterFunction):
@@ -265,14 +268,17 @@ class BuiltinSubtract(BuiltinFormatterFunction):
 
 class BuiltinMultiply(BuiltinFormatterFunction):
     name = 'multiply'
-    arg_count = 2
+    arg_count = -1
     category = 'Arithmetic'
-    __doc__ = doc = _('multiply(x, y) -- returns x * y. Throws an exception if either x or y are not numbers.')
+    __doc__ = doc = _('multiply(x, y, ...) -- returns the product of its arguments. '
+                      'Throws an exception if any argument is not a number.')
 
-    def evaluate(self, formatter, kwargs, mi, locals, x, y):
-        x = float(x if x and x != 'None' else 0)
-        y = float(y if y and y != 'None' else 0)
-        return unicode_type(x * y)
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        res = 1
+        for v in args:
+            v = float(v if v and v != 'None' else 0)
+            res *= v
+        return unicode_type(res)
 
 
 class BuiltinDivide(BuiltinFormatterFunction):
@@ -404,6 +410,26 @@ class BuiltinAssign(BuiltinFormatterFunction):
         return value
 
 
+class BuiltinListSplit(BuiltinFormatterFunction):
+    name = 'list_split'
+    arg_count = 3
+    category = 'List manipulation'
+    __doc__ = doc = _('split(list_val, sep, id_prefix) -- splits the list_val '
+                    "into separate values using 'sep', then assigns the values "
+                    "to variables named 'id_prefix_N' where N is the position "
+                    "of the value in the list. The first item has position 0 (zero). "
+                    "The function returns the last element in the list. "
+                    "Example: split('one, two, foo', ',', 'var') is equivalent "
+                    "to var_0 = 'one'; var_1 = 'two'; var_3 = 'foo'.")
+
+    def evaluate(self, formatter, kwargs, mi, locals, list_val, sep, id_prefix):
+        l = [v.strip() for v in list_val.split(sep)]
+        res = ''
+        for i,v in enumerate(l):
+            res = locals[id_prefix+'_'+unicode_type(i)] = v
+        return res
+
+
 class BuiltinPrint(BuiltinFormatterFunction):
     name = 'print'
     arg_count = -1
@@ -429,13 +455,17 @@ class BuiltinField(BuiltinFormatterFunction):
 
 class BuiltinRawField(BuiltinFormatterFunction):
     name = 'raw_field'
-    arg_count = 1
+    arg_count = -1
     category = 'Get values from metadata'
-    __doc__ = doc = _('raw_field(name) -- returns the metadata field named by name '
-            'without applying any formatting.')
+    __doc__ = doc = _('raw_field(name [, optional_default]) -- returns the '
+            'metadata field named by name without applying any formatting. '
+            'It evaluates and returns the optional second argument '
+            "'default' if the field is undefined ('None').")
 
-    def evaluate(self, formatter, kwargs, mi, locals, name):
+    def evaluate(self, formatter, kwargs, mi, locals, name, default=None):
         res = getattr(mi, name, None)
+        if res is None and default is not None:
+            return default
         if isinstance(res, list):
             fm = mi.metadata_for_field(name)
             if fm is None:
@@ -602,7 +632,8 @@ class BuiltinInList(BuiltinFormatterFunction):
             'not_found_val. The pattern and found_value can be repeated as '
             'many times as desired, permitting returning different values '
             'depending on the search. The patterns are checked in order. The '
-            'first match is returned.')
+            'first match is returned. Aliases: in_list(), list_contains()')
+    aliases = ['list_contains']
 
     def evaluate(self, formatter, kwargs, mi, locals, val, sep, *args):
         if (len(args) % 2) != 1:
@@ -787,15 +818,37 @@ class BuiltinCount(BuiltinFormatterFunction):
     name = 'count'
     arg_count = 2
     category = 'List manipulation'
+    aliases = ['list_count']
+
     __doc__ = doc = _('count(val, separator) -- interprets the value as a list of items '
             'separated by `separator`, returning the number of items in the '
             'list. Most lists use a comma as the separator, but authors '
-            'uses an ampersand. Examples: {tags:count(,)}, {authors:count(&)}')
+            'uses an ampersand. Examples: {tags:count(,)}, {authors:count(&)}. '
+            'Aliases: count(), list_count()')
 
     def evaluate(self, formatter, kwargs, mi, locals, val, sep):
         return unicode_type(len([v for v in val.split(sep) if v]))
 
 
+class BuiltinListCountMatching(BuiltinFormatterFunction):
+    name = 'list_count_matching'
+    arg_count = 3
+    category = 'List manipulation'
+    aliases = ['count_matching']
+
+    __doc__ = doc = _('list_count_matching(list, pattern, separator) -- '
+            "interprets 'list' as a list of items separated by 'separator', "
+            'returning the number of items in the list that match the regular '
+            "expression 'pattern'. Aliases: list_count_matching(), count_matching()")
+
+    def evaluate(self, formatter, kwargs, mi, locals, list_, pattern, sep):
+        res = 0
+        for v in [x.strip() for x in list_.split(sep) if x.strip()]:
+            if re.search(pattern, v, flags=re.I):
+                res += 1
+        return unicode_type(res)
+
+
 class BuiltinListitem(BuiltinFormatterFunction):
     name = 'list_item'
     arg_count = 3
@@ -1052,9 +1105,11 @@ class BuiltinSubitems(BuiltinFormatterFunction):
                 components = [item]
             try:
                 if ei == 0:
-                    rv.add('.'.join(components[si:]))
+                    t = '.'.join(components[si:]).strip()
                 else:
-                    rv.add('.'.join(components[si:ei]))
+                    t = '.'.join(components[si:ei]).strip()
+                if t:
+                    rv.add(t)
             except:
                 pass
         return ', '.join(sorted(rv, key=sort_key))
@@ -1201,6 +1256,27 @@ class BuiltinAnnotationCount(BuiltinFormatterFunction):
         return _('This function can be used only in the GUI')
 
 
+class BuiltinIsMarked(BuiltinFormatterFunction):
+    name = 'is_marked'
+    arg_count = 0
+    category = 'Get values from metadata'
+    __doc__ = doc = _("is_marked() -- check whether the book is 'marked' in "
+                      "calibre. If it is then return the value of the mark, "
+                      "either 'true' or the comma-separated list of named "
+                      "marks. Returns '' if the book is not marked.")
+
+    def evaluate(self, formatter, kwargs, mi, locals):
+        if hasattr(mi, '_proxy_metadata'):
+            try:
+                from calibre.gui2.ui import get_gui
+                c = get_gui().current_db.data.get_marked(mi.id)
+                return c if c else ''
+            except:
+                return _('Failed to get marked status')
+            return ''
+        return _('This function can be used only in the GUI')
+
+
 class BuiltinSeriesSort(BuiltinFormatterFunction):
     name = 'series_sort'
     arg_count = 0
@@ -1302,20 +1378,32 @@ class BuiltinListUnion(BuiltinFormatterFunction):
             'removing duplicate items using a case-insensitive comparison. If '
             'items differ in case, the one in list1 is used. '
             'The items in list1 and list2 are separated by separator, as are '
-            'the items in the returned list.')
+            'the items in the returned list. Aliases: list_union(), merge_lists()')
     aliases = ['merge_lists']
 
     def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
-        res = [l.strip() for l in list1.split(separator) if l.strip()]
-        l2 = [l.strip() for l in list2.split(separator) if l.strip()]
-        lcl1 = {icu_lower(l) for l in res}
-
-        for i in l2:
-            if icu_lower(i) not in lcl1 and i not in res:
-                res.append(i)
+        res = {icu_lower(l.strip()): l.strip() for l in list2.split(separator) if l.strip()}
+        res.update({icu_lower(l.strip()): l.strip() for l in list1.split(separator) if l.strip()})
         if separator == ',':
-            return ', '.join(res)
-        return separator.join(res)
+            separator = ', '
+        return separator.join(res.values())
+
+
+class BuiltinListRemoveDuplicates(BuiltinFormatterFunction):
+    name = 'list_remove_duplicates'
+    arg_count = 2
+    category = 'List manipulation'
+    __doc__ = doc = _('list_remove_duplicates(list, separator) -- '
+            'return a list made by removing duplicate items in the source list. '
+            'If items differ only in case, the last of them is returned. '
+            'The items in source list are separated by separator, as are '
+            'the items in the returned list.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, list_, separator):
+        res = {icu_lower(l.strip()): l.strip() for l in list_.split(separator) if l.strip()}
+        if separator == ',':
+            separator = ', '
+        return separator.join(res.values())
 
 
 class BuiltinListDifference(BuiltinFormatterFunction):
@@ -1722,6 +1810,40 @@ class BuiltinConnectedDeviceName(BuiltinFormatterFunction):
         return _('This function can be used only in the GUI')
 
 
+class BuiltinConnectedDeviceUUID(BuiltinFormatterFunction):
+    name = 'connected_device_uuid'
+    arg_count = 1
+    category = 'Get values from metadata'
+    __doc__ = doc = _("connected_device_uuid(storage_location) -- if a device is "
+                      "connected then return the device uuid (unique id), "
+                      "otherwise return the empty string. Each storage location "
+                      "on a device has a different uuid. The location names are "
+                      "'main', 'carda' and 'cardb'. This function works only in "
+                      "the GUI.")
+
+    def evaluate(self, formatter, kwargs, mi, locals, storage_location):
+        if hasattr(mi, '_proxy_metadata'):
+            # Do the import here so that we don't entangle the GUI when using
+            # command line functions
+            from calibre.gui2.ui import get_gui
+            info = get_gui().device_manager.get_current_device_information()
+            if info is None:
+                return ''
+            try:
+                if storage_location not in {'main', 'carda', 'cardb'}:
+                    raise ValueError(
+                         _('connected_device_name: invalid storage location "{0}"'
+                                    .format(storage_location)))
+                info = info['info'][4]
+                if storage_location not in info:
+                    return ''
+                return info[storage_location]['device_store_uuid']
+            except:
+                traceback.print_exc()
+                raise
+        return _('This function can be used only in the GUI')
+
+
 class BuiltinCheckYesNo(BuiltinFormatterFunction):
     name = 'check_yes_no'
     arg_count = 4
@@ -1804,25 +1926,100 @@ class BuiltinSwapAroundArticles(BuiltinFormatterFunction):
         return separator.join(sorted(result, key=sort_key))
 
 
+class BuiltinArguments(BuiltinFormatterFunction):
+    name = 'arguments'
+    arg_count = -1
+    category = 'other'
+    __doc__ = doc = _('arguments(id[=expression] [, id[=expression]]*) '
+                      '-- Used in a stored template to retrieve the arguments '
+                      'passed in the call. It both declares and initializes '
+                      'local variables, effectively parameters. The variables '
+                      'are positional; they get the value of the value given '
+                      'in the call in the same position. If the corresponding '
+                      'parameter is not provided in the call then arguments '
+                      'assigns that variable the provided default value. If '
+                      'there is no default value then the variable is set to '
+                      'the empty string.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        # The arguments function is implemented in-line in the formatter
+        raise NotImplementedError()
+
+
+class BuiltinGlobals(BuiltinFormatterFunction):
+    name = 'globals'
+    arg_count = -1
+    category = 'other'
+    __doc__ = doc = _('globals(id[=expression] [, id[=expression]]*) '
+                      '-- Retrieves "global variables" that can be passed into '
+                      'the formatter. It both declares and initializes local '
+                      'variables with the names of the global variables passed '
+                      'in. If the corresponding variable is not provided in '
+                      'the passed-in globals then it assigns that variable the '
+                      'provided default value. If there is no default value '
+                      'then the variable is set to the empty string.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        # The globals function is implemented in-line in the formatter
+        raise NotImplementedError()
+
+
+class BuiltinSetGlobals(BuiltinFormatterFunction):
+    name = 'set_globals'
+    arg_count = -1
+    category = 'other'
+    __doc__ = doc = _('globals(id[=expression] [, id[=expression]]*) '
+                      '-- Retrieves "global variables" that can be passed into '
+                      'the formatter. It both declares and initializes local '
+                      'variables with the names of the global variables passed '
+                      'in. If the corresponding variable is not provided in '
+                      'the passed-in globals then it assigns that variable the '
+                      'provided default value. If there is no default value '
+                      'then the variable is set to the empty string.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        # The globals function is implemented in-line in the formatter
+        raise NotImplementedError()
+
+
+class BuiltinFieldExists(BuiltinFormatterFunction):
+    name = 'field_exists'
+    arg_count = 1
+    category = 'If-then-else'
+    __doc__ = doc = _('field_exists(field_name) -- checks if a field '
+                      '(column) named field_name exists, returning '
+                      "'1' if so and '' if not.")
+
+    def evaluate(self, formatter, kwargs, mi, locals, field_name):
+        if field_name.lower() in mi.all_field_keys():
+            return '1'
+        return ''
+
+
 _formatter_builtins = [
-    BuiltinAdd(), BuiltinAnd(), BuiltinApproximateFormats(), BuiltinAssign(),
+    BuiltinAdd(), BuiltinAnd(), BuiltinApproximateFormats(), BuiltinArguments(),
+    BuiltinAssign(),
     BuiltinAuthorLinks(), BuiltinAuthorSorts(), BuiltinBooksize(),
     BuiltinCapitalize(), BuiltinCheckYesNo(), BuiltinCeiling(),
-    BuiltinCmp(), BuiltinConnectedDeviceName(), BuiltinContains(),
+    BuiltinCmp(), BuiltinConnectedDeviceName(), BuiltinConnectedDeviceUUID(), BuiltinContains(),
     BuiltinCount(), BuiltinCurrentLibraryName(), BuiltinCurrentLibraryPath(),
     BuiltinDaysBetween(), BuiltinDivide(), BuiltinEval(), BuiltinFirstNonEmpty(),
-    BuiltinField(), BuiltinFinishFormatting(), BuiltinFirstMatchingCmp(), BuiltinFloor(),
+    BuiltinField(), BuiltinFieldExists(),
+    BuiltinFinishFormatting(), BuiltinFirstMatchingCmp(), BuiltinFloor(),
     BuiltinFormatDate(), BuiltinFormatNumber(), BuiltinFormatsModtimes(),
     BuiltinFormatsPaths(), BuiltinFormatsSizes(), BuiltinFractionalPart(),
+    BuiltinGlobals(),
     BuiltinHasCover(), BuiltinHumanReadable(), BuiltinIdentifierInList(),
     BuiltinIfempty(), BuiltinLanguageCodes(), BuiltinLanguageStrings(),
-    BuiltinInList(), BuiltinListDifference(), BuiltinListEquals(),
+    BuiltinInList(), BuiltinIsMarked(), BuiltinListCountMatching(),
+    BuiltinListDifference(), BuiltinListEquals(),
     BuiltinListIntersection(), BuiltinListitem(), BuiltinListRe(),
-    BuiltinListReGroup(), BuiltinListSort(), BuiltinListUnion(), BuiltinLookup(),
+    BuiltinListReGroup(), BuiltinListRemoveDuplicates(), BuiltinListSort(),
+    BuiltinListSplit(), BuiltinListUnion(),BuiltinLookup(),
     BuiltinLowercase(), BuiltinMod(), BuiltinMultiply(), BuiltinNot(), BuiltinOndevice(),
     BuiltinOr(), BuiltinPrint(), BuiltinRatingToStars(), BuiltinRawField(), BuiltinRawList(),
     BuiltinRe(), BuiltinReGroup(), BuiltinRound(), BuiltinSelect(), BuiltinSeriesSort(),
-    BuiltinShorten(), BuiltinStrcat(), BuiltinStrcatMax(),
+    BuiltinSetGlobals(), BuiltinShorten(), BuiltinStrcat(), BuiltinStrcatMax(),
     BuiltinStrcmp(), BuiltinStrInList(), BuiltinStrlen(), BuiltinSubitems(),
     BuiltinSublist(),BuiltinSubstr(), BuiltinSubtract(), BuiltinSwapAroundArticles(),
     BuiltinSwapAroundComma(), BuiltinSwitch(),
