@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,6 +8,7 @@ __docformat__ = 'restructuredtext en'
 
 import os, re, sys
 from calibre.constants import iswindows, cache_dir, get_version
+from polyglot.builtins import exec_path
 
 ipydir = os.path.join(cache_dir(), 'ipython')
 
@@ -19,7 +19,6 @@ def setup_pyreadline():
     config = '''
 #Bind keys for exit (keys only work on empty lines
 #disable_readline(True)		#Disable pyreadline completely.
-from __future__ import print_function, unicode_literals, absolute_import
 debug_output("off")             #"on" saves log info to./pyreadline_debug_log.txt
                                 #"on_nologfile" only enables print warning messages
 bind_exit_key("Control-d")
@@ -143,6 +142,27 @@ history_length(2000) #value of -1 means no limit
         del readline, rlcompleter, atexit
 
 
+class Exit:
+
+    def __repr__(self):
+        raise SystemExit(0)
+    __str__ = __repr__
+
+    def __call__(self):
+        raise SystemExit(0)
+
+
+class Helper(object):
+
+    def __repr__(self):
+        return "Type help() for interactive help, " \
+               "or help(object) for help about object."
+
+    def __call__(self, *args, **kwds):
+        import pydoc
+        return pydoc.help(*args, **kwds)
+
+
 def simple_repl(user_ns={}):
     if iswindows:
         setup_pyreadline()
@@ -158,8 +178,11 @@ def simple_repl(user_ns={}):
     import sys, re  # noqa
     for x in ('os', 'sys', 're'):
         user_ns[x] = user_ns.get(x, globals().get(x, locals().get(x)))
-    import code
-    code.interact(BANNER, raw_input, user_ns)
+    user_ns['exit'] = Exit()
+    user_ns['help'] = Helper()
+    from code import InteractiveConsole
+    console = InteractiveConsole(user_ns)
+    console.interact(BANNER + 'Use exit to quit')
 
 
 def ipython(user_ns=None):
@@ -187,6 +210,9 @@ def ipython(user_ns=None):
     defns.update(user_ns or {})
 
     c = Config()
+    user_conf = os.path.expanduser('~/.ipython/profile_default/ipython_config.py')
+    if os.path.exists(user_conf):
+        exec_path(user_conf, {'get_config': lambda: c})
     c.TerminalInteractiveShell.prompts_class = CustomPrompt
     c.InteractiveShellApp.exec_lines = [
         'from __future__ import division, absolute_import, unicode_literals, print_function',
