@@ -1,9 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import atexit
 import errno
 import os
@@ -13,17 +11,19 @@ import time
 from functools import partial
 
 from calibre.constants import (
-    __appname__, fcntl, filesystem_encoding, islinux, isosx, iswindows, plugins
+    __appname__, filesystem_encoding, islinux, ismacos, iswindows
 )
+from calibre_extensions import speedup
 from calibre.utils.monotonic import monotonic
 
-speedup = plugins['speedup'][0]
 if iswindows:
-    import msvcrt, win32file, pywintypes, winerror, win32api, win32event
+    import msvcrt
     from calibre.constants import get_windows_username
+    from calibre_extensions import winutil
     excl_file_mode = stat.S_IREAD | stat.S_IWRITE
 else:
     excl_file_mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+    import fcntl
 
 
 def unix_open(path):
@@ -50,27 +50,24 @@ def unix_retry(err):
 
 def windows_open(path):
     if isinstance(path, bytes):
-        path = path.decode('mbcs')
-    try:
-        h = win32file.CreateFileW(
-            path,
-            win32file.GENERIC_READ |
-            win32file.GENERIC_WRITE,  # Open for reading and writing
-            0,  # Open exclusive
-            None,  # No security attributes, ensures handle is not inherited by children
-            win32file.OPEN_ALWAYS,  # If file does not exist, create it
-            win32file.FILE_ATTRIBUTE_NORMAL,  # Normal attributes
-            None,  # No template file
-        )
-    except pywintypes.error as err:
-        raise WindowsError(err[0], err[2], path)
-    fd = msvcrt.open_osfhandle(h.Detach(), 0)
-    return os.fdopen(fd, 'r+b')
+        path = os.fsdecode(path)
+    h = winutil.create_file(
+        path,
+        winutil.GENERIC_READ |
+        winutil.GENERIC_WRITE,  # Open for reading and writing
+        0,  # Open exclusive
+        winutil.OPEN_ALWAYS,  # If file does not exist, create it
+        winutil.FILE_ATTRIBUTE_NORMAL,  # Normal attributes
+    )
+    fd = msvcrt.open_osfhandle(int(h), 0)
+    ans = os.fdopen(fd, 'r+b')
+    h.detach()
+    return ans
 
 
 def windows_retry(err):
     return err.winerror in (
-        winerror.ERROR_SHARING_VIOLATION, winerror.ERROR_LOCK_VIOLATION
+        winutil.ERROR_SHARING_VIOLATION, winutil.ERROR_LOCK_VIOLATION
     )
 
 
@@ -85,6 +82,19 @@ def retry_for_a_time(timeout, sleep_time, func, error_retry, *args):
         time.sleep(sleep_time)
 
 
+def lock_file(path, timeout=15, sleep_time=0.2):
+    if iswindows:
+        return retry_for_a_time(
+            timeout, sleep_time, windows_open, windows_retry, path
+        )
+    f = unix_open(path)
+    retry_for_a_time(
+        timeout, sleep_time, fcntl.flock, unix_retry,
+        f.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB
+    )
+    return f
+
+
 class ExclusiveFile(object):
 
     def __init__(self, path, timeout=15, sleep_time=0.2):
@@ -95,17 +105,7 @@ class ExclusiveFile(object):
         self.sleep_time = sleep_time
 
     def __enter__(self):
-        if iswindows:
-            self.file = retry_for_a_time(
-                self.timeout, self.sleep_time, windows_open, windows_retry, self.path
-            )
-        else:
-            f = unix_open(self.path)
-            retry_for_a_time(
-                self.timeout, self.sleep_time, fcntl.flock, unix_retry,
-                f.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB
-            )
-            self.file = f
+        self.file = lock_file(self.path, self.timeout, self.sleep_time)
         return self.file
 
     def __exit__(self, type, value, traceback):
@@ -124,21 +124,15 @@ def _clean_lock_file(file_obj):
 
 
 if iswindows:
-
     def create_single_instance_mutex(name, per_user=True):
         mutexname = '{}-singleinstance-{}-{}'.format(
             __appname__, (get_windows_username() if per_user else ''), name
         )
-        mutex = win32event.CreateMutex(None, False, mutexname)
-        if not mutex:
+        try:
+            mutex = winutil.create_mutex(mutexname, False)
+        except FileExistsError:
             return
-        err = win32api.GetLastError()
-        if err == winerror.ERROR_ALREADY_EXISTS:
-            # Close this handle other wise this handle will prevent the mutex
-            # from being deleted when the process that created it exits.
-            win32api.CloseHandle(mutex)
-            return
-        return partial(win32api.CloseHandle, mutex)
+        return mutex.close
 
 elif islinux:
 
@@ -148,8 +142,8 @@ elif islinux:
         name = '%s-singleinstance-%s-%s' % (
             __appname__, (os.geteuid() if per_user else ''), name
         )
-        name = name.encode('utf-8')
-        address = b'\0' + name.replace(b' ', b'_')
+        name = name
+        address = '\0' + name.replace(' ', '_')
         sock = socket.socket(family=socket.AF_UNIX)
         try:
             eintr_retry_call(sock.bind, address)
@@ -170,7 +164,7 @@ else:
         )
         home = os.path.expanduser('~')
         locs = ['/var/lock', home, tempfile.gettempdir()]
-        if isosx:
+        if ismacos:
             locs.insert(0, '/Library/Caches')
         for loc in locs:
             if os.access(loc, os.W_OK | os.R_OK | os.X_OK):
@@ -191,6 +185,22 @@ else:
                 raise
 
 
+class SingleInstance:
+
+    def __init__(self, name):
+        self.name = name
+        self.release_mutex = None
+
+    def __enter__(self):
+        self.release_mutex = create_single_instance_mutex(self.name)
+        return self.release_mutex is not None
+
+    def __exit__(self, *a):
+        if self.release_mutex is not None:
+            self.release_mutex()
+            self.release_mutex = None
+
+
 def singleinstance(name):
     ' Ensure that only a single process holding exists with the specified mutex key '
     release_mutex = create_single_instance_mutex(name)
