@@ -7,7 +7,7 @@ __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import re
 import sys
-from PyQt5.Qt import QBuffer, QByteArray, QPixmap, Qt, QtWin
+from PyQt5.Qt import QBuffer, QByteArray, QPixmap, Qt, QtWin, QIODevice
 
 from calibre.gui2 import must_use_qt
 from calibre.utils.winreg.default_programs import split_commandline
@@ -24,7 +24,7 @@ def hicon_to_pixmap(hicon):
 def pixmap_to_data(pixmap):
     ba = QByteArray()
     buf = QBuffer(ba)
-    buf.open(QBuffer.WriteOnly)
+    buf.open(QIODevice.OpenModeFlag.WriteOnly)
     pixmap.save(buf, 'PNG')
     return bytes(bytearray(ba.data()))
 
@@ -63,8 +63,8 @@ def load_icon_resource_as_pixmap(icon_resource, size=ICON_SIZE):
         if area(pmap) >= q:
             if area(pmap) == q:
                 return pmap
-            return pmap.scaled(size, size, aspectRatioMode=Qt.KeepAspectRatio, transformMode=Qt.SmoothTransformation)
-    return pixmaps[-1].scaled(size, size, aspectRatioMode=Qt.KeepAspectRatio, transformMode=Qt.SmoothTransformation)
+            return pmap.scaled(size, size, aspectRatioMode=Qt.AspectRatioMode.KeepAspectRatio, transformMode=Qt.TransformationMode.SmoothTransformation)
+    return pixmaps[-1].scaled(size, size, aspectRatioMode=Qt.AspectRatioMode.KeepAspectRatio, transformMode=Qt.TransformationMode.SmoothTransformation)
 
 
 def load_icon_resource(icon_resource, as_data=False, size=ICON_SIZE):
@@ -84,7 +84,7 @@ def load_icon_for_file(path: str, as_data=False, size=ICON_SIZE):
     pmap = hicon_to_pixmap(hicon)
     if not pmap.isNull():
         if pmap.width() != size:
-            pmap = pmap.scaled(size, size, aspectRatioMode=Qt.KeepAspectRatio, transformMode=Qt.SmoothTransformation)
+            pmap = pmap.scaled(size, size, aspectRatioMode=Qt.AspectRatioMode.KeepAspectRatio, transformMode=Qt.TransformationMode.SmoothTransformation)
         return pixmap_to_data(pmap) if as_data else pmap
 
 
