@@ -1,7 +1,6 @@
-#!/usr/bin/python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 __author__ = "Chad Miller <smartypantspy@chad.org>, Kovid Goyal <kovid at kovidgoyal.net>"
 __description__ = "Smart-quotes, smart-ellipses, and smart-dashes for weblog entries in pyblosxom"
