@@ -8,13 +8,13 @@ __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 import os, sys, time, traceback
 from threading import Thread
 
-import winerror
 
 from calibre import guess_type, prints
-from calibre.constants import is64bit, isportable, isfrozen, __version__, DEBUG, plugins
+from calibre.constants import is64bit, isportable, isfrozen, __version__, DEBUG
 from calibre.utils.winreg.lib import Key, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE
 from calibre.utils.lock import singleinstance
 from polyglot.builtins import iteritems, itervalues
+from calibre_extensions import winutil
 
 # See https://msdn.microsoft.com/en-us/library/windows/desktop/cc144154(v=vs.85).aspx
 
@@ -131,7 +131,6 @@ def register():
         with Key(r'Software\RegisteredApplications') as key:
             key.set(data['name'], capabilities_path)
 
-    winutil = plugins['winutil'][0]
     winutil.notify_associations_changed()
 
 
@@ -197,7 +196,7 @@ def get_prog_id_map(base, key_path):
     try:
         k = Key(open_at=key_path, root=base)
     except OSError as err:
-        if err.winerror == winerror.ERROR_FILE_NOT_FOUND:
+        if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
             return desc, ans
         raise
     with k:
@@ -213,7 +212,7 @@ def get_open_data(base, prog_id):
     try:
         k = Key(open_at=r'Software\Classes\%s' % prog_id, root=base)
     except OSError as err:
-        if err.winerror == winerror.ERROR_FILE_NOT_FOUND:
+        if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
             return None, None, None
     with k:
         cmd = k.get(sub_key=r'shell\open\command')
@@ -228,12 +227,12 @@ def split_commandline(commandline):
     # CommandLineToArgvW returns path to executable if called with empty string.
     if not commandline.strip():
         return []
-    return list(plugins['winutil'][0].parse_cmdline(commandline))
+    return list(winutil.parse_cmdline(commandline))
 
 
 def friendly_app_name(prog_id=None, exe=None):
     try:
-        return plugins['winutil'][0].friendly_name(prog_id, exe)
+        return winutil.friendly_name(prog_id, exe)
     except Exception:
         traceback.print_exc()
 
@@ -249,7 +248,7 @@ def find_programs(extensions):
         try:
             k = Key(open_at=r'Software\RegisteredApplications', root=base)
         except OSError as err:
-            if err.winerror == winerror.ERROR_FILE_NOT_FOUND:
+            if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
                 continue
             raise
         with k:
@@ -274,7 +273,7 @@ def find_programs(extensions):
         try:
             k = Key(open_at=r'Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.%s\OpenWithProgIDs' % ext, root=HKEY_CURRENT_USER)
         except OSError as err:
-            if err.winerror == winerror.ERROR_FILE_NOT_FOUND:
+            if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
                 continue
         for prog_id in itervalues(k):
             if prog_id and prog_id not in seen_prog_ids:
