@@ -18,7 +18,7 @@ from calibre import sanitize_file_name
 from calibre.constants import filesystem_encoding
 from calibre.ebooks.chardet import detect
 from calibre.ptempfile import SpooledTemporaryFile
-from polyglot.builtins import getcwd, map, string_or_bytes, unicode_type
+from polyglot.builtins import getcwd, map, string_or_bytes, unicode_type, as_bytes
 
 try:
     import zlib  # We may need its compression method
@@ -339,8 +339,12 @@ class ZipInfo (object):
         # This is used to ensure paths in generated ZIP files always use
         # forward slashes as the directory separator, as required by the
         # ZIP format specification.
-        if os.sep != "/" and os.sep in filename:
-            filename = filename.replace(os.sep, "/")
+        if os.sep != '/':
+            os_sep, sep = os.sep, '/'
+            if isinstance(filename, bytes):
+                os_sep, sep = as_bytes(os_sep), b'/'
+            if os_sep in filename:
+                filename = filename.replace(os_sep, sep)
 
         self.filename = filename        # Normalized file name
         self.date_time = date_time      # year, month, day, hour, min, sec
@@ -744,7 +748,6 @@ class ZipFile:
         self.debug = 0  # Level of printing: 0 through 3
         self.NameToInfo = {}    # Find file info given name
         self.filelist = []      # List of ZipInfo instances for archive
-        self.extract_mapping = {}
         self.compression = compression  # Method of compression
         self.mode = key = mode.replace('b', '')[0]
         self.pwd = None
@@ -1151,8 +1154,11 @@ class ZipFile:
 
         if member.filename[-1] == '/':
             if not os.path.isdir(targetpath):
-                os.mkdir(targetpath)
-            self.extract_mapping[member.filename] = targetpath
+                try:
+                    os.mkdir(targetpath)
+                except Exception:  # Added by Kovid
+                    targetpath = os.path.join(base_target, sanitize_file_name(fname))
+                    os.mkdir(targetpath)
             return targetpath
 
         if not os.path.exists(targetpath):
@@ -1176,7 +1182,6 @@ class ZipFile:
             os.utime(targetpath, (mtime, mtime))
         except:
             pass
-        self.extract_mapping[member.filename] = targetpath
         return targetpath
 
     def _writecheck(self, zinfo):
