@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -14,13 +14,17 @@ from lxml import etree
 from lxml.builder import ElementMaker
 
 from calibre import force_unicode
-from calibre.utils.date import parse_date, now as nowf, utcnow, tzlocal, \
-        isoformat, fromordinal
+from calibre.utils.xml_parse import safe_xml_fromstring
+from calibre.constants import numeric_version
+from calibre.utils.iso8601 import parse_iso8601
+from calibre.utils.date import now as nowf, utcnow, local_tz, isoformat, EPOCH, UNDEFINED_DATE
 from calibre.utils.recycle_bin import delete_file
+from polyglot.builtins import iteritems, unicode_type
 
 NS = 'http://calibre-ebook.com/recipe_collection'
 E = ElementMaker(namespace=NS, nsmap={None:NS})
 
+
 def iterate_over_builtin_recipe_files():
     exclude = ['craigslist', 'toronto_sun']
     d = os.path.dirname
@@ -39,7 +43,7 @@ def serialize_recipe(urn, recipe_class):
 
     def attr(n, d):
         ans = getattr(recipe_class, n, d)
-        if isinstance(ans, str):
+        if isinstance(ans, bytes):
             ans = ans.decode('utf-8', 'replace')
         return ans
 
@@ -50,7 +54,7 @@ def serialize_recipe(urn, recipe_class):
     if ns is True:
         ns = 'yes'
     return E.recipe({
-        'id'                 : str(urn),
+        'id'                 : unicode_type(urn),
         'title'              : attr('title', _('Unknown')),
         'author'             : attr('__author__', default_author),
         'language'           : attr('language', 'und'),
@@ -58,11 +62,12 @@ def serialize_recipe(urn, recipe_class):
         'description'        : attr('description', '')
         })
 
+
 def serialize_collection(mapping_of_recipe_classes):
     collection = E.recipe_collection()
     '''for u, x in mapping_of_recipe_classes.items():
         print 11111, u, repr(x.title)
-        if isinstance(x.title, str):
+        if isinstance(x.title, bytes):
             x.title.decode('ascii')
     '''
     for urn in sorted(mapping_of_recipe_classes.keys(),
@@ -76,10 +81,11 @@ def serialize_collection(mapping_of_recipe_classes):
             traceback.print_exc()
             continue
         collection.append(recipe)
-    collection.set('count', str(len(collection)))
+    collection.set('count', unicode_type(len(collection)))
     return etree.tostring(collection, encoding='utf-8', xml_declaration=True,
             pretty_print=True)
 
+
 def serialize_builtin_recipes():
     from calibre.web.feeds.recipes import compile_recipe
     recipe_mapping = {}
@@ -88,40 +94,44 @@ def serialize_builtin_recipes():
             try:
                 recipe_class = compile_recipe(stream.read())
             except:
-                print ('Failed to compile: %s'%f)
+                print('Failed to compile: %s'%f)
                 raise
         if recipe_class is not None:
             recipe_mapping['builtin:'+rid] = recipe_class
 
     return serialize_collection(recipe_mapping)
 
+
 def get_builtin_recipe_collection():
     return etree.parse(P('builtin_recipes.xml', allow_user_override=False)).getroot()
 
+
 def get_custom_recipe_collection(*args):
     from calibre.web.feeds.recipes import compile_recipe, \
             custom_recipes
     bdir = os.path.dirname(custom_recipes.file_path)
     rmap = {}
-    for id_, x in custom_recipes.iteritems():
+    for id_, x in iteritems(custom_recipes):
         title, fname = x
         recipe = os.path.join(bdir, fname)
         try:
-            recipe = open(recipe, 'rb').read().decode('utf-8')
+            with open(recipe, 'rb') as f:
+                recipe = f.read().decode('utf-8')
             recipe_class = compile_recipe(recipe)
             if recipe_class is not None:
                 rmap['custom:%s'%id_] = recipe_class
         except:
-            print 'Failed to load recipe from: %r'%fname
+            print('Failed to load recipe from: %r'%fname)
             import traceback
             traceback.print_exc()
             continue
-    return etree.fromstring(serialize_collection(rmap))
+    return safe_xml_fromstring(serialize_collection(rmap), recover=False)
 
 
 def update_custom_recipe(id_, title, script):
     update_custom_recipes([(id_, title, script)])
 
+
 def update_custom_recipes(script_ids):
     from calibre.web.feeds.recipes import custom_recipes, \
             custom_recipe_filename
@@ -129,14 +139,14 @@ def update_custom_recipes(script_ids):
     bdir = os.path.dirname(custom_recipes.file_path)
     for id_, title, script in script_ids:
 
-        id_ = str(int(id_))
+        id_ = unicode_type(int(id_))
         existing = custom_recipes.get(id_, None)
 
         if existing is None:
             fname = custom_recipe_filename(id_, title)
         else:
             fname = existing[1]
-        if isinstance(script, unicode):
+        if isinstance(script, unicode_type):
             script = script.encode('utf-8')
 
         custom_recipes[id_] = (title, fname)
@@ -151,20 +161,21 @@ def update_custom_recipes(script_ids):
 def add_custom_recipe(title, script):
     add_custom_recipes({title:script})
 
+
 def add_custom_recipes(script_map):
     from calibre.web.feeds.recipes import custom_recipes, \
             custom_recipe_filename
     id_ = 1000
-    keys = tuple(map(int, custom_recipes.iterkeys()))
+    keys = tuple(map(int, custom_recipes))
     if keys:
         id_ = max(keys)+1
     bdir = os.path.dirname(custom_recipes.file_path)
     with custom_recipes:
-        for title, script in script_map.iteritems():
-            fid = str(id_)
+        for title, script in iteritems(script_map):
+            fid = unicode_type(id_)
 
             fname = custom_recipe_filename(fid, title)
-            if isinstance(script, unicode):
+            if isinstance(script, unicode_type):
                 script = script.encode('utf-8')
 
             custom_recipes[fid] = (title, fname)
@@ -179,7 +190,7 @@ def add_custom_recipes(script_map):
 
 def remove_custom_recipe(id_):
     from calibre.web.feeds.recipes import custom_recipes
-    id_ = str(int(id_))
+    id_ = unicode_type(int(id_))
     existing = custom_recipes.get(id_, None)
     if existing is not None:
         bdir = os.path.dirname(custom_recipes.file_path)
@@ -190,9 +201,10 @@ def remove_custom_recipe(id_):
         except:
             pass
 
+
 def get_custom_recipe(id_):
     from calibre.web.feeds.recipes import custom_recipes
-    id_ = str(int(id_))
+    id_ = unicode_type(int(id_))
     existing = custom_recipes.get(id_, None)
     if existing is not None:
         bdir = os.path.dirname(custom_recipes.file_path)
@@ -200,19 +212,31 @@ def get_custom_recipe(id_):
         with open(os.path.join(bdir, fname), 'rb') as f:
             return f.read().decode('utf-8')
 
+
 def get_builtin_recipe_titles():
     return [r.get('title') for r in get_builtin_recipe_collection()]
 
+
 def download_builtin_recipe(urn):
     from calibre.utils.config_base import prefs
     from calibre.utils.https import get_https_resource_securely
     import bz2
-    return bz2.decompress(get_https_resource_securely(
+    recipe_source = bz2.decompress(get_https_resource_securely(
         'https://code.calibre-ebook.com/recipe-compressed/'+urn, headers={'CALIBRE-INSTALL-UUID':prefs['installation_uuid']}))
+    recipe_source = recipe_source.decode('utf-8')
+    from calibre.web.feeds.recipes import compile_recipe
+    recipe = compile_recipe(recipe_source)  # ensure the downloaded recipe is at least compile-able
+    if recipe is None:
+        raise ValueError('Failed to find recipe object in downloaded recipe: ' + urn)
+    if recipe.requires_version > numeric_version:
+        raise ValueError('Downloaded recipe for {} requires calibre >= {}'.format(urn, recipe.requires_version))
+    return recipe_source
+
 
 def get_builtin_recipe(urn):
     with zipfile.ZipFile(P('builtin_recipes.zip', allow_user_override=False), 'r') as zf:
-        return zf.read(urn+'.recipe')
+        return zf.read(urn+'.recipe').decode('utf-8')
+
 
 def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
     for x in get_builtin_recipe_collection():
@@ -232,6 +256,7 @@ def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
                         'Failed to download recipe, using builtin version')
             return get_builtin_recipe(urn)
 
+
 def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
     for x in get_builtin_recipe_collection():
         if x.get('id') == id_:
@@ -250,6 +275,7 @@ def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
                         'Failed to download recipe, using builtin version')
             return get_builtin_recipe(urn)
 
+
 class SchedulerConfig(object):
 
     def __init__(self):
@@ -262,9 +288,9 @@ class SchedulerConfig(object):
         if os.access(self.conf_path, os.R_OK):
             with ExclusiveFile(self.conf_path) as f:
                 try:
-                    self.root = etree.fromstring(f.read())
+                    self.root = safe_xml_fromstring(f.read(), recover=False)
                 except:
-                    print 'Failed to read recipe scheduler config'
+                    print('Failed to read recipe scheduler config')
                     import traceback
                     traceback.print_exc()
         elif os.path.exists(old_conf_path):
@@ -292,13 +318,13 @@ class SchedulerConfig(object):
                     ld = x.get('last_downloaded', None)
                     if ld and last_downloaded is None:
                         try:
-                            last_downloaded = parse_date(ld)
-                        except:
+                            last_downloaded = parse_iso8601(ld)
+                        except Exception:
                             pass
                     self.root.remove(x)
                     break
             if last_downloaded is None:
-                last_downloaded = fromordinal(1)
+                last_downloaded = EPOCH
             sr = E.scheduled_recipe({
                 'id' : recipe.get('id'),
                 'title': recipe.get('title'),
@@ -377,7 +403,7 @@ class SchedulerConfig(object):
         elif typ == 'day/time':
             text = '%d:%d:%d'%schedule
         elif typ in ('days_of_week', 'days_of_month'):
-            dw = ','.join(map(str, map(int, schedule[0])))
+            dw = ','.join(map(unicode_type, map(int, schedule[0])))
             text = '%s:%d:%d'%(dw, schedule[1], schedule[2])
         else:
             raise ValueError('Unknown schedule type: %r'%typ)
@@ -397,7 +423,11 @@ class SchedulerConfig(object):
                     days = list(map(int, [x.strip() for x in
                         parts[0].split(',')]))
                     sch = [days, int(parts[1]), int(parts[2])]
-                return typ, sch, parse_date(recipe.get('last_downloaded'))
+                try:
+                    ld = parse_iso8601(recipe.get('last_downloaded'))
+                except Exception:
+                    ld = UNDEFINED_DATE
+                return typ, sch, ld
 
     def recipe_needs_to_be_downloaded(self, recipe):
         try:
@@ -420,14 +450,20 @@ class SchedulerConfig(object):
             return utcnow() - ld > timedelta(sch)
         elif typ == 'day/time':
             now = nowf()
-            ld_local = ld.astimezone(tzlocal())
+            try:
+                ld_local = ld.astimezone(local_tz)
+            except Exception:
+                return False
             day, hour, minute = sch
             return is_weekday(day, now) and \
                     not was_downloaded_already_today(ld_local, now) and \
                     is_time(now, hour, minute)
         elif typ == 'days_of_week':
             now = nowf()
-            ld_local = ld.astimezone(tzlocal())
+            try:
+                ld_local = ld.astimezone(local_tz)
+            except Exception:
+                return False
             days, hour, minute = sch
             have_day = False
             for day in days:
@@ -439,7 +475,10 @@ class SchedulerConfig(object):
                     is_time(now, hour, minute)
         elif typ == 'days_of_month':
             now = nowf()
-            ld_local = ld.astimezone(tzlocal())
+            try:
+                ld_local = ld.astimezone(local_tz)
+            except Exception:
+                return False
             days, hour, minute = sch
             have_day = now.day in days
             return have_day and \
@@ -513,8 +552,8 @@ class SchedulerConfig(object):
                         username, password = c[k]
                     except:
                         username = password = ''
-                    self.set_account_info(urn, unicode(username),
-                            unicode(password))
+                    self.set_account_info(urn, unicode_type(username),
+                            unicode_type(password))
                 except:
                     continue
         del c
