@@ -1,12 +1,13 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import copy, zipfile
+from functools import total_ordering
 
 from PyQt5.Qt import QAbstractItemModel, Qt, QColor, QFont, QIcon, \
         QModelIndex, pyqtSignal, QPixmap
@@ -18,7 +19,11 @@ from calibre.web.feeds.recipes.collection import \
         SchedulerConfig, download_builtin_recipe, update_custom_recipe, \
         update_custom_recipes, add_custom_recipe, add_custom_recipes, \
         remove_custom_recipe, get_custom_recipe, get_builtin_recipe
+from calibre import force_unicode
+from calibre.utils.icu import primary_sort_key
 from calibre.utils.search_query_parser import ParseException
+from polyglot.builtins import iteritems, unicode_type
+
 
 class NewsTreeItem(object):
 
@@ -43,7 +48,7 @@ class NewsTreeItem(object):
         return None
 
     def flags(self):
-        return Qt.ItemIsEnabled|Qt.ItemIsSelectable
+        return Qt.ItemFlag.ItemIsEnabled|Qt.ItemFlag.ItemIsSelectable
 
     def sort(self):
         self.children.sort()
@@ -56,47 +61,55 @@ class NewsTreeItem(object):
                 self.children.remove(child)
                 child.parent = None
 
+
+@total_ordering
 class NewsCategory(NewsTreeItem):
 
     def __init__(self, category, builtin, custom, scheduler_config, parent):
         NewsTreeItem.__init__(self, builtin, custom, scheduler_config, parent)
         self.category = category
         self.cdata = get_language(self.category)
+        if self.category == _('Scheduled'):
+            self.sortq = 0, ''
+        elif self.category == _('Custom'):
+            self.sortq = 1, ''
+        else:
+            self.sortq = 2, self.cdata
         self.bold_font = QFont()
         self.bold_font.setBold(True)
         self.bold_font = (self.bold_font)
 
     def data(self, role):
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return (self.cdata + ' [%d]'%len(self.children))
-        elif role == Qt.FontRole:
+        elif role == Qt.ItemDataRole.FontRole:
             return self.bold_font
-        elif role == Qt.ForegroundRole and self.category == _('Scheduled'):
+        elif role == Qt.ItemDataRole.ForegroundRole and self.category == _('Scheduled'):
             return (QColor(0, 255, 0))
+        elif role == Qt.ItemDataRole.UserRole:
+            return '::category::{}'.format(self.sortq[0])
         return None
 
     def flags(self):
-        return Qt.ItemIsEnabled
+        return Qt.ItemFlag.ItemIsEnabled
 
-    def __cmp__(self, other):
-        def decorate(x):
-            if x == _('Scheduled'):
-                x = '0' + x
-            elif x == _('Custom'):
-                x = '1' + x
-            else:
-                x = '2' + x
-            return x
+    def __eq__(self, other):
+        return self.cdata == other.cdata
 
-        return cmp(decorate(self.cdata), decorate(getattr(other, 'cdata', '')))
+    def __lt__(self, other):
+        return self.sortq < getattr(other, 'sortq', (3, ''))
 
 
+@total_ordering
 class NewsItem(NewsTreeItem):
 
     def __init__(self, urn, title, default_icon, custom_icon, favicons, zf,
             builtin, custom, scheduler_config, parent):
         NewsTreeItem.__init__(self, builtin, custom, scheduler_config, parent)
         self.urn, self.title = urn, title
+        if isinstance(self.title, bytes):
+            self.title = force_unicode(self.title)
+        self.sortq = primary_sort_key(self.title)
         self.icon = self.default_icon = None
         self.default_icon = default_icon
         self.favicons, self.zf = favicons, zf
@@ -104,9 +117,9 @@ class NewsItem(NewsTreeItem):
             self.icon = custom_icon
 
     def data(self, role):
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return (self.title)
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             if self.icon is None:
                 icon = '%s.png'%self.urn[8:]
                 p = QPixmap()
@@ -114,23 +127,29 @@ class NewsItem(NewsTreeItem):
                     try:
                         with zipfile.ZipFile(self.zf, 'r') as zf:
                             p.loadFromData(zf.read(self.favicons[icon]))
-                    except:
+                    except Exception:
                         pass
                 if not p.isNull():
                     self.icon = (QIcon(p))
                 else:
                     self.icon = self.default_icon
             return self.icon
-        return None
+        if role == Qt.ItemDataRole.UserRole:
+            return self.urn
+
+    def __eq__(self, other):
+        return self.urn == other.urn
+
+    def __lt__(self, other):
+        return self.sortq < other.sortq
 
-    def __cmp__(self, other):
-        return cmp(self.title.lower(), getattr(other, 'title', '').lower())
 
 class AdaptSQP(SearchQueryParser):
 
     def __init__(self, *args, **kwargs):
         pass
 
+
 class RecipeModel(QAbstractItemModel, AdaptSQP):
 
     LOCATIONS = ['all']
@@ -177,7 +196,7 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
 
     def update_custom_recipes(self, script_urn_map):
         script_ids = []
-        for urn, title_script in script_urn_map.iteritems():
+        for urn, title_script in iteritems(script_urn_map):
             id_ = int(urn[len('custom:'):])
             (title, script) = title_script
             script_ids.append((id_, title, script))
@@ -199,7 +218,7 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
             remove_custom_recipe(id_)
         self.custom_recipe_collection = get_custom_recipe_collection()
 
-    def do_refresh(self, restrict_to_urns=set([])):
+    def do_refresh(self, restrict_to_urns=frozenset()):
         self.custom_recipe_collection = get_custom_recipe_collection()
         zf = P('builtin_recipes.zip', allow_user_override=False)
 
@@ -222,7 +241,7 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
         scheduled = factory(NewsCategory, new_root, _('Scheduled'))
         custom = factory(NewsCategory, new_root, _('Custom'))
         lang_map = {}
-        self.all_urns = set([])
+        self.all_urns = set()
         self.showing_count = 0
         self.builtin_count = 0
         for x in self.custom_recipe_collection:
@@ -281,7 +300,7 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
         query = query.strip().lower()
         if not query:
             return self.universal_set()
-        results = set([])
+        results = set()
         for urn in self.universal_set():
             recipe = self.recipe_from_urn(urn)
             if query in recipe.get('title', '').lower() or \
@@ -292,7 +311,7 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
     def search(self, query):
         results = []
         try:
-            query = unicode(query).strip()
+            query = unicode_type(query).strip()
             if query:
                 results = self.parse(query)
                 if not results:
@@ -316,7 +335,7 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
 
     def flags(self, index):
         if not index.isValid():
-            return Qt.ItemIsEnabled|Qt.ItemIsSelectable
+            return Qt.ItemFlag.ItemIsEnabled|Qt.ItemFlag.ItemIsSelectable
         item = index.internalPointer()
         return item.flags()
 
@@ -409,6 +428,3 @@ class RecipeModel(QAbstractItemModel, AdaptSQP):
             for recipe in self.scheduler_config.iter_recipes():
                 ans.append(recipe.get('id'))
         return ans
-
-
-
